#!/bin/bash

t100s_LEVEL_DEBUG=0
t100s_LEVEL_INFO=1
t100s_LEVEL_WARN=2
t100s_LEVEL_ERROR=3
t100s_LEVEL_FATAL=4
t100s_OPER_TYPE=1396789536

t100s_PROC_NAME_FILTER=",UltraPathScsi,mpp,vsc,huasy_vbd,megaraid_sas,usb-storage,"
t100s_phyScsiHosts=","
t100s_needScanChans=","
t100s_needScanTgts=","
t100s_thisProcName="unknown"
t100s_reportLunResult=""
t100s_sgDevs=","
t100s_needScanVirtualHost="1"
readonly t100s_nxupScsiDir="/sys/bus/scsi/drivers/NXUP"
readonly t100s_ScsiSdDir="/sys/bus/scsi/drivers/sd"
t100s_LUNIDS=""

readonly t100s_virtualHostName="UltraPathScsi"
t100s_OSVersion=""
t100s_VlunCheck="0"
readonly t100s_nxupLunMap_a="/proc/nxup_lun_map_a"
readonly t100s_nxupLunMap_b="/proc/nxup_lun_map_b"
t100s_nxupLunMap=""
readonly t100s_nxupDismtachMap="/proc/nxup_pia"
t100s_tmpLunMap="/nxup_lun_map"
t100s_NotManagedLuns=0
t100s_VLunNum=0
t100s_ForceDelVLun=0
t100s_ForceDelNoPathVLun=0
t100s_conflictVlunId=","
t100s_nopathVlunId=","
t100s_vlunNeeDWaitToDeleted=","

t100s_tmpDir="/UltraPath/tmp/"
t100s_tmpLunMap="${t100s_tmpDir}t100s_nxup_lun_map"
t100s_vlunRemoving="${t100s_tmpDir}t100s_nxup_vluns_removing"
t100s_vlunActive="${t100s_tmpDir}t100s_nxup_vluns_active"
t100s_vlunDeleted="${t100s_tmpDir}t100s_nxup_vluns_deleted"
t100s_pPathInfo=""
t100s_vlunInfo=""
declare -a t100s_vlunInfoResult
declare -a t100s_arr

t100s_VPWhiteList="${t100s_tmpDir}t100s_vpwhitelist"
t100s_tmpDiskByIDLink="${t100s_tmpDir}t100s_disk_by-id_link"
t100s_handleLun0only=0

t100s_notManageLunNumPerPath=""
t100s_lastDisksPerPhyPath=""

g10s_debugon=${g10s_debugon="0"}

if [ ! -d ${t100s_tmpDir} ]
then
    mkdir -m 600 -p "${t100s_tmpDir}"
fi

if [ -f /etc/UVP-release  ] && [ `cat /etc/UVP-release | grep -c "UVP"` -ne 0 ]
then
    g10s_logFile="/var/log/UltraPath/ultrapathTool.log"
    g10s_operFile="/var/log/UltraPath/oper_log"
    g10s_seriFile="/var/log/UltraPath/serial_oper"
elif [ -f /etc/uvp_version  ] && [ `cat /etc/uvp_version | grep -c "UVP"` -ne 0 ]
then
    g10s_logFile="/var/log/UltraPath/ultrapathTool.log"
    g10s_operFile="/var/log/UltraPath/oper_log"
    g10s_seriFile="/var/log/UltraPath//serial_oper"
elif [ -f /etc/EulerLinux.conf ]
then
    g10s_logFile="/var/log/UltraPath/ultrapathTool.log"
    g10s_operFile="/var/log/UltraPath/oper_log"
    g10s_seriFile="/var/log/UltraPath//serial_oper"   
else
    g10s_logFile="/opt/UltraPath/log/ultrapathTool.log"
    g10s_operFile="/opt/UltraPath/log/oper_log"
    g10s_seriFile="/opt/UltraPath/log/serial_oper"
fi


g10s_closeMsgHead=${g10s_closeMsgHead:="1"}

t100s_isQuiet="0"

t100s_lockFile="/var/lock/t100s_shell.lock"
t300s_lockFile="/var/lock/t300s_shell.lock"
i10s_lockFile="/var/lock/i10s_install.shell.lock"

function t100s_useage()
{
    echo "USAGE: $0 [-m] [-q] [-f] [-force]
    -m         Delete Luns which had not been managed by UltraPath without mutual information.
    -q         Delete Luns which had not been managed by UltraPath with mutual information.
    -f         Delete no-mapped virtual disks.
    -force     Delete no path virtual disks.
    -s         Delete Luns which had not been managed by UltraPath with mutual information.
    -su        Scan without deleting Luns managed by UltraPath."
    t100s_exit 1
}

function print_debug()
{
    local fileFlag=""

    if [ "$2" != "" ]
    then
        fileFlag=$2
    else
        fileFlag=${0##*/}
        fileFlag=${fileFlag%%_*}
    fi
    if [ "${g10s_debugon}" = "1" ]
    then
        if [ "${g10s_closeMsgHead}" = "0" ]
        then
            echo -e -n "\033[0;32;32m[DEBUG:0][${fileFlag}][`date +%Y-%m-%d_%H:%M:%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}]\033[0m ";echo "$1"
        else
            echo -e -n "\033[0;32;32m[DEBUG]\033[0m ";echo "$1"
        fi
    fi
    [ -f "${g10s_logFile}" ] && [ -w "${g10s_logFile}" ] && echo "[DEBUG:0][${fileFlag}][`date +%Y-%m-%d_%H:%M:%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] $1">>"${g10s_logFile}"
    return 0
}


function print_file()
{
    local fileFlag=""

    if [ "$2" != "" ]
    then
        fileFlag=$2
    else
        fileFlag=${0##*/}
        fileFlag=${fileFlag%%_*}
    fi
    [ -f "${g10s_logFile}" ] && [ -w "${g10s_logFile}" ] && echo  "[FILE:1][${fileFlag}][`id -u`][`date +%Y-%m-%d_%H:%M:%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] $1">>"${g10s_logFile}"

    return 0
}

function print_oper()
{
    local fileFlag=""
    local index=0
    if [ -f "$g10s_seriFile" ]
    then 
        index=`cat "$g10s_seriFile"`
    fi

    if [ "$2" != "" ]
    then
        fileFlag=$2
    else
        fileFlag=${0##*/}
        fileFlag=${fileFlag%%_*}
    fi
    [ -f "${g10s_operFile}" ] && [ -w "${g10s_operFile}" ] && echo  "[$index] [`date +%Y-%m-%d_%H:%M:%S`] [`id -u`] [CLI] [$3] [$t100s_OPER_TYPE] $1 [Execute scan disk] [ִɨ] [$2]">>"${g10s_operFile}"
    let index=$index+1
    echo "$index"> "$g10s_seriFile"

    return 0
}


function print_stack()
{
    [ "${g10s_debugon}" = "1" ] && echo "-----------------------------stack begin---------------------------------"
    [ -f "${g10s_logFile}" ] && [ -w "${g10s_logFile}" ] && echo "-----------------------------stack begin---------------------------------" >> "${g10s_logFile}"
    local i=0
    local j=0
    local tmpFun=""

    [ "${g10s_debugon}" = "1" ] && echo "-----FILENAME , FUNCTIONNAME , LINENO-----"
    [ -f "${g10s_logFile}" ] && [ -w "${g10s_logFile}" ] && echo "-----FILENAME , FUNCTIONNAME , LINENO-----" >> "${g10s_logFile}"
    for tmpFun in ${FUNCNAME[@]}
    do
        if [ ${i} -eq 0 ]
        then
            let "i=i+1"
            continue
        fi
        [ "${g10s_debugon}" = "1" ] && echo "-----${BASH_SOURCE[${i}]} , ${FUNCNAME[${i}]} , ${BASH_LINENO[${j}]}-----"
        [ -f "${g10s_logFile}" ] &&  [ -w "${g10s_logFile}" ] && echo "-----${BASH_SOURCE[${i}]} , ${FUNCNAME[${i}]} , ${BASH_LINENO[${j}]}-----" >> "${g10s_logFile}"
        j=${i}
        let "i=i+1"
    done
    [ "${g10s_debugon}" = "1" ] && echo "-----------------------------stack end---------------------------------"
    [ -f "${g10s_logFile}" ] && [ -w "${g10s_logFile}" ] && echo "-----------------------------stack end---------------------------------" >> "${g10s_logFile}"
    return 0
}


function print_error()
{
    local fileFlag=""
    local tmpDebugon=""

    if [ "$2" = "" ] || [ `echo "$1"|grep -E -c "[0-9]+"` -eq 0 ]
    then
        echo -e -n "\033[31m[ERROR:2][g10s][`date +%Y-%m-%d_%H:%M:%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}]\033[0m ";echo "param error[$1,$2]"
        [ -f "${g10s_logFile}" ] && [ -w "${g10s_logFile}" ] && echo "[ERROR:2][g10s][`date +%Y-%m-%d_%H:%M:%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] param error[$1,$2]" >> "${g10s_logFile}"
        print_stack
        return 1
    fi
    if [ "$3" != "" ]
    then
        fileFlag=$3
    else
        fileFlag=${0##*/}
        fileFlag=${fileFlag%%_*}
    fi
    if [ "${g10s_closeMsgHead}" = "0" ]
    then
        echo -e -n "\033[31m[ERROR:$1][${fileFlag}][`date +%Y-%m-%d_%H:%M:%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}]\033[0m ";echo "$2"
    else
        echo -e -n "\033[31m[ERROR]\033[0m ";echo "$2"
    fi
    [ -f "${g10s_logFile}" ] && [ -w "${g10s_logFile}" ] && echo  "[ERROR:$1][${fileFlag}][`date +%Y-%m-%d_%H:%M:%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] $2">>"${g10s_logFile}"

    if [ "${g10s_closeMsgHead}" = "0" ]
    then
        tmpDebugon="${g10s_debugon}"
        g10s_debugon="1"
        print_stack
        g10s_debugon="${tmpDebugon}"
    else
        print_stack
    fi
    return 0
}


function t100s_lockShell()
{
    local thepid=""
    local ret=""

    if [ "$1" = "" ]
    then
        print_error "2" "need a param"
        exit 1
    fi
    print_debug "lock shell[${1}]"
    if [ -f "${1}" ]
    then
        thepid=`cat "${1}"`
        if [ "${thepid}" != "" ]
        then
            ret=`ps ax |awk '{print $1}'| grep -c "^[[:blank:]]*${thepid}[[:blank:]]*$"`
            if [ ${ret} -eq 1 ]
            then
                print_error "18" "this shell is already running, exit." "g10s"
                [ -f "${g10s_logFile}" ] && ps aux>>"${g10s_logFile}" 2>&1
                exit 1
            fi
        fi
    fi
    [ -w /var/lock ] && echo $$ > "${1}"
    return 0
}


function t100s_unlockShell()
{
    if [ "$1" = "" ]
    then
        print_error "2" "need a param"
        exit 1
    fi
    if [ -f "${1}" ]
    then
        print_debug "unlock shell[${1}]"
        rm -f "${1}"
    fi
    return 0
}

function t100s_exit()
{
    if [ "${t100s_VPWhiteList}" != "/etc/up.conf" ] && [ "${t100s_VPWhiteList}" != "/etc/up.conf.bak" ]
    then
        rm -rf "${t100s_VPWhiteList}"
    fi
    
    t100s_unlockShell "${t100s_lockFile}"
        
    if [ 0 -eq $1 ]
    then
        print_oper "[execute $0 ${t100s_param}]" "1" "$t100s_LEVEL_INFO"
    else
        print_oper "[execute $0 ${t100s_param}]" "0" "$t100s_LEVEL_ERROR"
    fi
    
    if [ -d "${t100s_tmpDir}" ]
    then
        chmod -R 600 "${t100s_tmpDir}"
    fi
    
    exit "$1"
}


function print_warning()
{
    local fileFlag=""

    if [ "$2" = "" ] || [ `echo "$1"|grep -E -c "[0-9]+"` -eq 0 ]
    then
        echo -e -n "\033[31m[ERROR:2][g10s][`date +%Y-%m-%d_%H:%M:%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}]\033[0m ";echo "param error[$1,$2]"
        [ -f "${g10s_logFile}" ] && [ -w "${g10s_logFile}" ] && echo  "[ERROR:2][g10s][`date +%Y-%m-%d_%H:%M:%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] param error[$1,$2]" >> "${g10s_logFile}"
        print_stack
        t100s_exit 1
    fi
    if [ "$3" != "" ]
    then
        fileFlag=$3
    else
        fileFlag=${0##*/}
        fileFlag=${fileFlag%%_*}
    fi

    if [ "${g10s_closeMsgHead}" = "0" ]
    then
        echo -e -n "\033[35m[WARNING:$1][${fileFlag}][`date +%Y-%m-%d_%H:%M:%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}]\033[0m ";echo "$2"
    else
        echo -e -n "\033[35m[WARNING]\033[0m ";echo "$2"
    fi
    [ -f "${g10s_logFile}" ] && [ -w "${g10s_logFile}" ] && echo  "[WARNING:$1][${fileFlag}][`date +%Y-%m-%d_%H:%M:%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] $2">>"${g10s_logFile}"
    print_stack
    return 0
}


function print_event()
{
    local fileFlag=""

    if [ `echo "$1"|grep -E -c "[0-9]+"` -eq 0 ] || [ "$2" = "" ]
    then
        echo -e -n "\033[31m[ERROR:2][g10s][`date +%Y-%m-%d_%H:%M:%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}]\033[0m ";echo "param error[$1,$2]"
        [ -f "${g10s_logFile}" ] && [ -w "${g10s_logFile}" ] && echo "[ERROR:2][g10s][`date +%Y-%m-%d_%H:%M:%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] param error[$1,$2]" >> "${g10s_logFile}"
        print_stack
        t100s_exit 1
    fi
    if [ "$3" != "" ]
    then
        fileFlag=$3
    else
        fileFlag=${0##*/}
        fileFlag=${fileFlag%%_*}
    fi
    if [ "${t100s_isQuiet}" = "0" ]
    then
        if [ "${g10s_closeMsgHead}" = "0" ]
        then
            echo -e -n "[EVENT:$1][${fileFlag}][`date +%Y-%m-%d_%H:%M:%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] "
        fi
        echo "$2"
    fi
    [ -f "${g10s_logFile}" ] && [ -w "${g10s_logFile}" ] && echo "[EVENT:$1][${fileFlag}][`date +%Y-%m-%d_%H:%M:%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] $2">>"${g10s_logFile}"
    return 0
}


 function print_yesOrNo()
 {
     local fileFlag=""
     local userInput=""
     local wrongTimes=0

     if [ "$2" = "" ] || [ `echo "$1"|grep -E -c "[0-9]+"` -eq 0 ]
     then
         print_error "2" "param error[$1,$2]"
         t100s_exit 1
     fi
     if [ "$3" != "" ]
     then
         fileFlag=$3
     else
         fileFlag=${0##*/}
         fileFlag=${fileFlag%%_*}
     fi

     while [ 1 -eq 1 ]
     do
         if [ "${g10s_closeMsgHead}" = "0" ]
      then
             echo -e "\033[0;32;32m[YESORNO:$1][${fileFlag}][`date +%Y-%m-%d_%H:%M:%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}]\033[0m "
         fi
         [ "${g10s_closeMsgHead}" = "0" ] && echo "------------begin-----------YES or NO box---------------------------"
         echo "$2"
         [ -f "${g10s_logFile}" ] && echo "[YESORNO:$1][${fileFlag}][`date +%Y-%m-%d_%H:%M:%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] " >> "${g10s_logFile}"
         [ -f "${g10s_logFile}" ] && echo "$2" >> "${g10s_logFile}"

         [ "${g10s_closeMsgHead}" = "0" ] && echo "------------end-------------YES or NO box---------------------------"
         echo -n "<Y|N>:"

         read userInput
         userInput=$(echo "${userInput}"|tr '[A-Z]' '[a-z]')
         if [ "${userInput}" != "yes" ] && \
             [ "${userInput}" != "y" ] && \
             [ "${userInput}" != "no" ] && \
             [ "${userInput}" != "n" ]
         then
             print_warning "31" "you select is incorrect[${userInput}]" "g10s"
             let wrongTimes=wrongTimes+1
             if [ ${wrongTimes} -gt 10 ]
             then
                return 2
             fi
             continue
         fi

         print_debug "your selection is [${userInput}]" "g10s"
         if [ "${userInput}" = "yes" ] || \
                 [ "${userInput}" = "y" ]
         then
             return 0
         else
             return 1
         fi
     done
     return 0
 }


function t100s_rootCheck()
{
    if [ "`id -u`" -ne 0 ]
    then
        print_error "7" "must login as root"
        t100s_exit 1
    fi
    return 0
}



function t100s_checkExist()
{
    if [ -z "$1" ] || [ -z "$2" ] || [ -z "$3" ]
    then
        print_error "1" "param error[$1,$2,$3]"
        t100s_exit 1
    fi

    if [ "$1" "$2" ]
    then

        if [ "$3" = "1" ]
        then
            print_error "4" "file or directory is exist[$2]"
            t100s_exit 1
        fi
    else

        if [ "$3" = "2" ]
        then
            print_error "5" "file or directory is not exist[$2]"
            t100s_exit 1
        fi
    fi
    return 0
}


t100s_virtualHostNum=""
function t100s_getVirtualHostNum()
{
    local ret
    local tmphost

    ret=`ls /sys/class/scsi_host/ 2>/dev/null |grep -c "^host[0-9][0-9]*$"`
    if [ $ret -eq 0 ]
    then
        print_warning "1012" "not found any host in /sys/class/scsi_host/"
        return 1
    fi
    for tmphost in `ls /sys/class/scsi_host/`
    do
        ret=`echo "${tmphost}"|grep -c "^host[0-9][0-9]*$"`
        if [ ${ret} -eq 0 ]
        then
            print_debug "wrong host format [${tmphost}]"
            continue
        fi

        if [ -f "/sys/class/scsi_host/${tmphost}/proc_name" ]
        then
            if [ `cat "/sys/class/scsi_host/${tmphost}/proc_name"|grep -c "^[[:blank:]]*${t100s_virtualHostName}[[:blank:]]*$"` -ne 0 ]
            then
                t100s_virtualHostNum=${tmphost#host}
                print_debug "found virtual host${t100s_virtualHostNum}"
                return 0
            fi
        fi
    done
    print_debug "not found virtual host."
    return 1
}

function t100s_getOSType()
{
    local t100s_confValue
    if [ ! -f /etc/UltraPath.packages.info ]
    then
        print_error "1001" "file /etc/UltraPath.packages.info not found."
        t100s_exit 1
    fi
    t100s_confValue=`cat /etc/UltraPath.packages.info 2>/dev/null|grep "^[[:blank:]]*g10s_OSVersion[[:blank:]]*=[^=]*"`
    t100s_confValue=${t100s_confValue%%#*}
    t100s_confValue=${t100s_confValue##*=}
    t100s_confValue=${t100s_confValue#*\"}
    t100s_confValue=${t100s_confValue#*\'}
    t100s_confValue=${t100s_confValue%\"*}
    t100s_confValue=${t100s_confValue%\'*}

    t100s_OSVersion=$(echo ${t100s_confValue})
    print_debug "the current os is ${t100s_OSVersion}."
    return 0
}

function t100s_eval()
{
    if [ "$1" = "" ]
    then
        print_error "2" "param error"
        t100s_exit 1
    fi
    eval "${1}"
    if [ $? -ne 0 ]
    then
        print_error "34" "eval failed.[${1}]"
        t100s_exit 1
    fi
    return 0
}


function t100s_labelToIDS()
{
    local label=`echo "$1"`
    if [ `echo "${label}"|grep -E -c "^(sd[a-z]+|-[0-9]+)$"` -ne 0 ]
    then
        t100s_LUNIDS=`ls -l /sys/block/${label}/device|sed  "s/^.*\/\([0-9][0-9]*:[0-9][0-9]*:[0-9][0-9]*:[0-9][0-9]*\).*$/\1/"`
    else
        t100s_LUNIDS="$1"
    fi


    print_debug "t100s_LUNIDS=${t100s_LUNIDS}"
    if [ `echo "${t100s_LUNIDS}"|grep -E -c "^[0-9]+:[0-9]+:[0-9]+:[0-9]+$"` -ne 1 ]
    then
        print_error "1001" "LUNIDS must be H:C:T:L or sd*."
        t100s_exit 1
    fi
    return 0
}


function t100s_testExist()
{
    local host=""
    local channel=""
    local id=""
    local lun=""
    local grepstr=""

    t100s_labelToIDS "$1"
    if [ -d /sys/class/scsi_device/${t100s_LUNIDS} ]
    then
        return 1
    else
        return 0
    fi
    return 0
}


function t100s_removeTmpDevByLssci()
{
    which lsscsi > /dev/null 2>&1
    ret="$?"
    if [ "${ret}" -ne "0" ]
    then
        print_debug "command \"lsscsi\" not exist."
        return 1
    fi

    for tmpLine in $(lsscsi | grep "updisk" | grep "tmp" | sed -e 's/\].*$//' -e 's/^.*\[//')
    do
        ret=$(echo "${tmpLine}" | grep "[[:digit:]][[:digit:]]*:[[:digit:]][[:digit:]]*:[[:digit:]][[:digit:]]*:[[:digit:]][[:digit:]]*")
        if [ "${ret}" = "" ]
        then
            print_warning "1004" "${tmpLine} is error."
            continue
        fi
        host=${tmpLine%%:*}
        channel=${tmpLine#*:}
        channel=${channel%%:*}
        tgt=${tmpLine%:*}
        tgt=${tgt##*:}
        lun=${tmpLine##*:}

        t100s_testExist "${tmpLine}"
        ret=$?
        if [ ${ret} -ne 0 ]
        then
            if [ -f /sys/class/scsi_device/${tmpLine}/device/delete ]
            then
                echo 1 > /sys/class/scsi_device/${tmpLine}/device/delete
            else
                print_debug "not found /sys/class/scsi_device/${tmpLine}/device/delete"
            fi
            t100s_testExist "${tmpLine}"
            ret=$?
        fi

        if [ ${ret} -ne 0 ]
        then
            print_warning "1004" "delete lun failed.[$tmpLine]"
        else
            print_event "1005" "delete lun successfully.[$tmpLine]"
            if [ -f /sys/class/scsi_host/host${host}/scan ]
            then
                echo "${channel} ${tgt} ${lun}" > /sys/class/scsi_host/host${host}/scan
            else
                print_debug "not found /sys/class/scsi_host/host${host}/scan"
            fi
        fi
    done
}

function t100s_do_removeLUN()
{
   t100s_testExist "$1"
   ret=$?
   if [ ${ret} -ne 0 ]
   then
       if [ -f /sys/class/scsi_device/${t100s_LUNIDS}/device/delete ]
       then
           echo 1 > /sys/class/scsi_device/${t100s_LUNIDS}/device/delete
       else
           print_debug "not found /sys/class/scsi_device/${t100s_LUNIDS}/device/delete"
       fi
    fi
    t100s_testExist "${t100s_LUNIDS}"
    ret=$?
    if [ ${ret} -ne 0 ]
    then
        print_warning "1004" "delete lun failed.[$1]"
    else
        print_event "1005" "delete lun successfully.[$1]"
    fi
    return ${ret}
}

function t100s_removePLUN()
{
    local devnr=""
    local ret=0
    
    t100s_testExist "$1"
    if [ $? -eq 0 ]
    then
        print_debug "the lun does not exist.[$1]"
        return 0
    fi
    
    devnr=`echo "${t100s_LUNIDS}"|sed 's/:/ /g'`
    print_debug "delete lun ${devnr}"
    if [ -f /proc/scsi/scsi ]
    then
        echo "scsi remove-single-device ${devnr}" >/proc/scsi/scsi 2>&1
    fi
    
    t100s_do_removeLUN "$1"
    return 0
}

function t100s_getProcInfo()
{
    t100s_tmpLunMap="${t100s_tmpDir}t100s_nxup_lun_map"
    
    if [ ! -f ${t100s_nxupLunMap_a} ]
    then
        t100s_nxupLunMap=${t100s_nxupLunMap_b}
    else
        t100s_nxupLunMap=${t100s_nxupLunMap_a}
    fi

    print_debug "t100s_nxupLunMap=${t100s_nxupLunMap}"

    if [ ! -f ${t100s_nxupLunMap} ]
    then
        print_warning "1004" "not found [${t100s_nxupLunMap}]"
        return 1
    fi

    cat "${t100s_nxupLunMap}" > "${t100s_tmpLunMap}"

    if [ ! -f ${t100s_tmpLunMap} ]
    then
        t100s_tmpLunMap=${t100s_nxupLunMap}
    fi

    return 0
}

function t100s_modifyVLUNByMap()
{
    local pLunID=""
    local DiskId=""

    pLunID="$1"
    mode="$2"

    if [ `cat "${t100s_tmpLunMap}" | grep -c "=${pLunID}="` -eq 0 ]
    then
        return 0
    fi

    DiskId_SdId=`cat "${t100s_tmpLunMap}" | grep "=${pLunID}=" | grep "online" | awk -F "=" '{print $7"_"$3}'`
    
    if [ "${mode}" = "removing" ]
    then
        if [ `cat ${t100s_vlunRemoving} | grep "^${DiskId_SdId}$" -c` -eq 0 ]
        then
            echo "${DiskId_SdId}" >> ${t100s_vlunRemoving}
            print_debug "add Removing lun [${DiskId_SdId}]"
        fi
    fi

    if [ "${mode}" = "active" ]
    then
        if [ `cat ${t100s_vlunActive} | grep "^${DiskId_SdId}$" -c` -eq 0 ]
        then
            echo "${DiskId_SdId}" >> ${t100s_vlunActive}
            print_debug "add active lun [${DiskId_SdId}]"
        fi
    fi

     return 0
}


function t100s_removeLUN()
{
    if [ "${t100s_VlunCheck}" = "1" ]
    then
        t100s_modifyVLUNByMap "$1" "removing"
    fi
    t100s_removePLUN "$1"
    return 0
}


function t100s_IsWWNChanged()
{
    if [ "$1" = "" ] || [ "$2" = "" ]
    then
        print_error "2" "param error."
        t100s_exit 1
    fi
    local sgResult
    local sgWWN
    local sgDev

    if test -e /sys/class/scsi_device/${1}/device/generic; then    
        sgDev=`readlink /sys/class/scsi_device/${1}/device/generic`
        sgDev=`basename $sgDev`
    else
        sgDev=""
    fi
    if [ "${sgDev}" = "" ]
    then
        print_debug "not found sg device for the lun[$1]"
        return 2
    fi

    t100s_checkExist -f "/sbin/slTools" 2
    sgResult="`/sbin/slTools GETWWN /dev/${sgDev} 2>&1`"
    print_debug "sgResult=${sgResult}"
    if [ `echo "${sgResult}"|grep -E -c "Designation descriptor number"` -eq 0 ]
    then
       print_debug "inquiry WNN failed! [$1, dev/${sgDev}]"
       return 2
    fi
    sgWWN=`echo "${sgResult}"|sed -n "s/^.*\[0[x|X]\(.*\)\].*$/\1/p"`
    sgWWN=$(echo ${sgWWN})
    if [ "${sgWWN}" = "" ]
    then
        print_debug "WNN invalid! [$1, dev/${sgDev}]"
        return 2
    fi
    if [ "${sgWWN}" = "$2" ]
    then
       return 0
    fi
    return 1
}

function t100s_listPhyPathByHost()
{
    local hostID="="
    local hostTmp
    local tmpLine
    local arrTmp
    local k
    local j

    for tmpLine in ${t100s_pPathInfo}
    do
        hostTmp=`echo ${tmpLine} | awk -F ":" '{print $1}'`
        if [ `echo ${hostTmp} | grep -E "$hostID" -c` -eq 0 ]
        then
            hostID="${hostID}|^${hostTmp}$"
        fi
    done

    k=0
    for tmpLine in `echo $hostID | awk -F "|" '{for(j=2;j<=NF;j++)printf $j" "}'`
    do
        tmpLine=`echo ${tmpLine#*^}`
        tmpLine=`echo ${tmpLine%*$}`
        t100s_arr[$k]="$tmpLine"
        let k++
    done

    len=${#t100s_arr[@]}
    for tmpLine in ${t100s_pPathInfo}
    do
        hostTmp=`echo $tmpLine | awk -F ":" '{print $1}'`
        k=0
        while [ $k -lt $len ]
        do
            hostID=`echo ${t100s_arr[k]} | awk '{print $1}'`
            if [ "${hostID}" = "${hostTmp}" ]
            then
                arrTmp=${t100s_arr[k]}
                t100s_arr[k]="$arrTmp $tmpLine"
                break
            fi
            let k++
        done
    done
}

function t100s_checkVlunType()
{
    local vlunInfo
    local i
    local j
    local index
    local vlunID
    local vlunIDTmp
    local vlunWWN
    local vlunType
    local tmpIndex
    
    oldIFS="${IFS}"
    IFS=","
    vlunInfo=(${t100s_vlunInfo})
    IFS="${oldIFS}"

    let index=0
    for (( i=0; i<${#vlunInfo[@]}; i=i+1 ))
    do
    	vlunID="${vlunInfo[${i}]}"
        let "i=i+1"
        vlunWWN="${vlunInfo[${i}]}"
        vlunType="normal"

        if [ ${index} -ge 3 ]
        then
            let tmpIndex=${index}-3
            lastVlunID=${t100s_vlunInfoResult[${tmpIndex}]}
            if [ ${vlunID} == ${lastVlunID} ]
            then
                vlunType="aggre"
                print_debug "${vlunID} is aggre"
            fi
        fi
        
        if [ ${vlunType} = "aggre" ]
        then
            let "tmpIndex=tmpIndex+2"
            t100s_vlunInfoResult[${tmpIndex}]="aggre"
        else
            t100s_vlunInfoResult[${index}]="${vlunID}"
            let "index=index+1"
            t100s_vlunInfoResult[${index}]="${vlunWWN}"
            let "index=index+1"
            t100s_vlunInfoResult[${index}]="normal"
            let "index=index+1"
        fi
    done
}

function t100s_delLunMapChanged()
{
    local vlunID
    local vlunWWN
    local vlunType
    local oldIFS
    local hostNum
	local channel
	local tgt
	local lunID
    local ret
    local allPhyPath
    local tmpPhyPath
    local tmpLine
    local len
    local arrTmp
    local pLunIDs=""
    local pLunID=""
    local i
    local j
	local VlunNum

    print_event "1006" "Begin to delete LUNs whose mappings are changed."

    echo "" >  ${t100s_vlunRemoving}
    echo "" >  ${t100s_vlunActive}

    t100s_getVirtualHostNum
    hostNum=$(echo ${t100s_virtualHostNum})
    if [ `echo "${hostNum}"|grep -E -c "^[0-9]+$"`  -eq 0 ]
    then
        print_debug "virtual host not find."
        return 0
    fi

    t100s_vlunInfo=`upadmin show vlun type=all |grep "^[[:blank:]]*[0-9]"|awk '{printf("%s,%s,",$1,$4)}'`
    if [ "${t100s_vlunInfo}" = "" ]
    then
       print_event "1007" "no vitural lun found"
       return 0
    fi

    t100s_getProcInfo
    ret=$?
    if [ "${ret}" = "1" ]
    then
        print_debug "delLunMapChanged get proc info failed."
        return 0
    fi
    
    unset t100s_vlunInfoResult
    t100s_checkVlunType 

    for (( i=0; i<${#t100s_vlunInfoResult[@]}; i=i+1 ))
    do
        vlunID="${t100s_vlunInfoResult[${i}]}"
        let "i=i+1"
        vlunWWN="${t100s_vlunInfoResult[${i}]}"
        let "i=i+1"
        vlunType="${t100s_vlunInfoResult[${i}]}"

        print_debug "t100s_vlunInfoResult [${vlunID},${vlunWWN},${vlunType}]"

        unset t100s_arr
        t100s_pPathInfo=`upadmin show vlun id=${vlunID} type=all 2>/dev/null |sed -n "/^\s*Path/s/^.*\[\(.*\)\].*$/\1/p"`
        t100s_listPhyPathByHost

    print_debug "t100s_pPathInfo:[${t100s_pPathInfo}]"

        len=${#t100s_arr[@]}
        j=0
        while [ $j -lt $len ]
        do
            arrTmp=${t100s_arr[j]}
            print_debug "t100s_arr[$j]:[${arrTmp}]"
            allPhyPath=`echo ${arrTmp} | awk '{for(k=2;k<=NF;k++)printf $k" "}'`
            for tmpPhyPath in ${allPhyPath}
            do
                [ "${tmpPhyPath}" = "" ] && continue
                t100s_IsWWNChanged "${tmpPhyPath}" "${vlunWWN}"
                ret=$?

                if [ $ret -eq 1 ]
                then
                    t100s_modifyVLUNByMap "${tmpPhyPath}" "removing"
                    if [ ${vlunType} = "normal" ]
                    then
                        for tmpLine in ${allPhyPath}
                        do
                            t100s_removePLUN "${tmpLine}"
                        done
                        break
                    else
                        t100s_removePLUN "${tmpPhyPath}"
                        continue
                    fi
                fi

                if [ $ret -eq 2 ]
                then
                   continue
                fi

                if [ $ret -eq 0 ]
                then
                    t100s_modifyVLUNByMap "${tmpPhyPath}" "active"
                    if [ ${vlunType} = "normal" ]
                    then
                        break
                    else
                        continue
                    fi
                fi

            done

            let j++
        done

    done
    for tmpLine in `cat ${t100s_vlunRemoving} | awk '{printf $1" "}'`
    do
        if [ `cat ${t100s_vlunActive} | grep "^${tmpLine}$" -c` -eq 0 ]
        then
            if [ `cat ${t100s_vlunDeleted} | grep "^${tmpLine}$" -c` -eq 0 ]
            then
                echo "${tmpLine}" >> "${t100s_vlunDeleted}"
            fi
            tmpLine=${tmpLine%%_*}
            pLunIDs=`cat "${t100s_tmpLunMap}" | grep "online" | grep "=${tmpLine}$" | awk -F "=" '{print $4}'`
            for pLunID in ${pLunIDs}
            do
                pLunID=$(echo ${pLunID})
                t100s_testExist "${pLunID}"
                ret=$?
                if [ ${ret} -eq 0 ]
                then
                    continue
                fi
                t100s_removePLUN "${pLunID}"
            done
            t100s_vlunNeeDWaitToDeleted="${t100s_vlunNeeDWaitToDeleted}|=${tmpLine}$"
            print_debug "delete VLUN [${tmpLine}]"

        upadmin set stop_iosuspension ${tmpLine}								

        fi
    done

    [ -f ${t100s_vlunRemoving} ] && rm -rf ${t100s_vlunRemoving}
    [ -f ${t100s_vlunActive} ] && rm -rf ${t100s_vlunActive}

    return 0
}


t100s_sgDevs=","
function t100s_getSgDevByTgt()
{
    if [ `echo "$1"|grep -E -c "^[0-9]+$"` -eq 0 ] ||
        [ `echo "$2"|grep -E -c "^[0-9]+$"` -eq 0 ] ||
        [ `echo "$3"|grep -E -c "^[0-9]+$"` -eq 0 ]
    then
        print_error "2" "param error."
        t100s_exit 1
    fi
    local hostNum="${1}"
    local chanNum="${2}"
    local tgtNum="${3}"
    local tmpLine=""
    local tmpSG=""

    t100s_sgDevs=","
    for tmpLine in `ls -l /sys/class/scsi_generic/sg*/device 2>/dev/null|grep -E "/${hostNum}:${chanNum}:${tgtNum}:[0-9]+"|sed 's/^.*\(\/sys.*\/device\).*>.*$/\1/'`
    do
        tmpSG=`echo "${tmpLine}"|awk -F "/" '{print $5}'`
        tmpSG=$(echo ${tmpSG})
        t100s_sgDevs="${t100s_sgDevs}${tmpSG},"
    done
    print_debug "host=${hostNum} chan=${chanNum} tgt=${tgtNum} t100s_sgDevs=${t100s_sgDevs}"
    return 0
}


function t100s_reportLunBySg()
{
    local sgResult=""
    local tmpLun=""

    t100s_reportLunResult=","
    if [ `echo "$1"|grep -E -c "^sg[0-9]+$"` -eq 0 ]
    then
        print_error "2" "param error."
        t100s_exit 1
    fi
    t100s_checkExist -f "/sbin/slTools" 2
    sgResult=`/sbin/slTools REPORTLUN /dev/${1} 2>&1`
    print_debug "report lun by $1,sgResult=${sgResult}"
    if [ `echo "${sgResult}"|grep -E -c "Lun list length = [0-9]+ which imples [0-9]+ lun entr"` -ne 0 ]
    then

        for tmpLun in `echo "${sgResult}"|sed -n 's/.*lun=\(.*\)/\1/p'`
        do
            t100s_reportLunResult="${t100s_reportLunResult}${tmpLun},"
        done
    elif [ `echo "${sgResult}"|grep -E -c "report lun failed.status:0,host_status:(1|15)"` -ne 0 ]
    then
        return 2
    else
        return 1
    fi
    return 0
}


function t100s_checkSGExist()
{
    local tmpSG=""
    local ret=""
    local sgNum=""
    tmpSG=$1

    t100s_getOSType
    if [ "${t100s_OSVersion}" != "SLES9SP3" ]
    then
        return 0
    fi

    ret=`echo "${tmpSG}"|grep -c "^sg[0-9][0-9]*$"`
    if [ ${ret} -eq 0 ]
    then
        print_debug "wrong sg format [${tmpSG}]"
        return 1
    fi
    sgNum=${tmpSG#sg}
    ls /dev/${tmpSG} 1>/dev/null 2>&1
    ret=$?
    if [ ${ret} -ne 0 ]
    then
        mknod /dev/sg${sgNum} c 21 ${sgNum}
        if [ $? -ne 0 ]
        then
            print_debug "mknod [/dev/sg${sgNum}] failed."
            return 1
        fi
    fi

    return 0
}


function t100s_reportLuns()
{
    t100s_reportLunResult=""
    local sgDevs=""
    local tmpSG=""
    local ret=""

    sgDevs="${t100s_sgDevs}"
    if [ "${sgDevs}" = "" ] || [ "${sgDevs}" = "," ]
    then
        print_debug "not found sg devices for the host,report lun failed.[${sgDevs}]"
        return 3
    fi

    local oldIFS="${IFS}"
    IFS=","
    sgDevs=(${sgDevs})
    IFS="${oldIFS}"
    for tmpSG in ${sgDevs[@]}
    do
        if [ "${tmpSG}" = "" ]
        then
            continue
        fi

        t100s_checkSGExist "${tmpSG}"
        t100s_reportLunBySg "${tmpSG}"
        ret=$?
        if [ ${ret} -eq 0 ]
        then
            ret=${t100s_reportLunResult}
            print_debug "report lun:[${tmpSG}--${ret}]"
            return 0
        else
            print_debug "report lun from ${tmpSG} failed."
            continue
        fi
    done
    return ${ret}
}


function t100s_lun0IsNotConfigured()
{
    if [ `echo "$1"|grep -E -c "^[0-9]+$"` -eq 0 ] ||
        [ `echo "$2"|grep -E -c "^[0-9]+$"` -eq 0 ] ||
        [ `echo "$3"|grep -E -c "^[0-9]+$"` -eq 0 ]
    then
        print_error "2" "param error."
        t100s_exit 1
    fi

    t100s_checkExist -d "${t100s_ScsiSdDir}" 2
    t100s_checkExist -d "${t100s_nxupScsiDir}" 2
    if [ `ls ${t100s_nxupScsiDir}/${1}:${2}:${3}:0 2>/dev/null|wc -l` -ne 0 ]
    then
        return 0
    elif [ `ls ${t100s_ScsiSdDir}/${1}:${2}:${3}:0 2>/dev/null|wc -l` -ne 0 ]
    then
        return 0
    else
        print_debug "the lun is unconfigured.[${1}:${2}:${3}:0]"
        return 1
    fi
    return 0
}

function t100s_delLun0NotExistInReport()
{
    local lunNum=0
    if [ -f /sys/class/scsi_device/${hostNum}:${chanNum}:${tgtNum}:${lunNum}/device/delete ]
    then
        thisVendor=`cat /sys/class/scsi_device/${hostNum}:${chanNum}:${tgtNum}:${lunNum}/device/vendor 2>/dev/null`
        thisVendor=$(echo ${thisVendor})
        thisProduct=`cat /sys/class/scsi_device/${hostNum}:${chanNum}:${tgtNum}:${lunNum}/device/model 2>/dev/null`
        thisProduct=$(echo ${thisProduct})
        print_debug "${hostNum}:${chanNum}:${tgtNum}:${lunNum},thisVendor=${thisVendor},thisProduct=${thisProduct}"
        if [ `cat "${t100s_VPWhiteList}" | grep -c "vendor_id *= *\"${thisVendor}\"[, ]*product_id *= *\"${thisProduct}\""` -eq 0 ]
        then
            return 0
        fi

        if [ `echo "${t100s_reportLunResult}"|grep -c ",${lunNum},"` -eq 0 ]
        then
            t100s_lun0IsNotConfigured "${hostNum}" "${chanNum}" "${tgtNum}"
            ret=$?
            if [ ${ret} -ne 0 ]
            then
                return
            fi
            t100s_removeLUN "${hostNum}:${chanNum}:${tgtNum}:${lunNum}"
        else
            t100s_lun0IsNotConfigured "${hostNum}" "${chanNum}" "${tgtNum}"
            ret=$?
            if [ ${ret} -ne 0 ]
            then
                t100s_removeLUN "${hostNum}:${chanNum}:${tgtNum}:${lunNum}"
                return
            fi
            t100s_modifyVLUNByMap "${hostNum}:${chanNum}:${tgtNum}:${lunNum}" "active"
        fi
    fi
}

function t100s_delLunsNotExistInReport()
{
    if [ `echo "$1"|grep -E -c "^[0-9]+$"` -eq 0 ] || [ "$2" = "" ] || [ "$3" = "" ]
    then
        print_error "2" "param error."
        t100s_exit 1
    fi

    if [ "${t100s_reportLunResult}" = "" ]
    then
        return 0
    fi

    local hostNum="${1}"
    local chanNum="${2}"
    local tgtNum="${3}"
    local thisVendor
    local thisProduct
    local VlunNum
    
    if [ ${t100s_handleLun0only} -eq 1 ]
    then
        t100s_delLun0NotExistInReport
        return 0
    fi

    if [ `ls  /sys/class/scsi_device/${hostNum}:${chanNum}:${tgtNum}:*/device/delete 2>/dev/null|wc -l` -ne 0 ]
    then
        for tmpDev in `ls  /sys/class/scsi_device/${hostNum}:${chanNum}:${tgtNum}:*/device/delete`
        do
            lunID=${tmpDev#*/sys/class/scsi_device/}
            lunID=${lunID%/device/delete*}
            lunNum=`echo "${lunID}"|awk -F ":" '{print $4}'`

            thisVendor=`cat /sys/class/scsi_device/${hostNum}:${chanNum}:${tgtNum}:${lunNum}/device/vendor 2>/dev/null`
            thisVendor=$(echo ${thisVendor})
            thisProduct=`cat /sys/class/scsi_device/${hostNum}:${chanNum}:${tgtNum}:${lunNum}/device/model 2>/dev/null`
            thisProduct=$(echo ${thisProduct})
            print_debug "${hostNum}:${chanNum}:${tgtNum}:${lunNum},thisVendor=${thisVendor},thisProduct=${thisProduct}"
            if [ `cat "${t100s_VPWhiteList}" | grep -c "vendor_id *= *\"${thisVendor}\"[, ]*product_id *= *\"${thisProduct}\""` -eq 0 ]
            then

                continue
            fi

            if [ `echo "${t100s_reportLunResult}"|grep -c ",${lunNum},"` -eq 0 ]
            then

                if [ ${lunNum} -eq 0 ]
                then
                    t100s_lun0IsNotConfigured "${hostNum}" "${chanNum}" "${tgtNum}"
                    ret=$?
                    if [ ${ret} -ne 0 ]
                    then
                        continue
                    fi
                fi
                t100s_removeLUN "${hostNum}:${chanNum}:${tgtNum}:${lunNum}"
            else
                if [ ${lunNum} -eq 0 ]
                then
                    t100s_lun0IsNotConfigured "${hostNum}" "${chanNum}" "${tgtNum}"
                    ret=$?
                    if [ ${ret} -ne 0 ]
                    then
                        t100s_removeLUN "${hostNum}:${chanNum}:${tgtNum}:${lunNum}"
                        continue
                    fi
                fi

                t100s_modifyVLUNByMap "${hostNum}:${chanNum}:${tgtNum}:${lunNum}" "active"

            fi
        done
    fi
    return 0
}


t100s_thisProcName="unknown"
function t100s_getHostName()
{
    local hostdir=""

    t100s_thisProcName="unknown"
    if [ `echo "$1"|grep -E -c "^host[0-9]+"` -eq 0 ]
    then
        print_error "2" "param error."
        t100s_exit 1
    fi

    hostdir="/sys/class/scsi_host/${1}"
    if [ -f "${hostdir}/isp_name" ]
    then
        t100s_thisProcName="qla2xxx"
    elif [ -f "${hostdir}/lpfc_drvr_version" ]
    then
        t100s_thisProcName="lpfc"
    elif [ -d "${hostdir}" ]
    then

        t100s_thisProcName=`cat "${hostdir}/proc_name" 2>/dev/null|head -n 1`
    else
        t100s_thisProcName="linuxIscsi"
    fi

    t100s_thisProcName=$(echo ${t100s_thisProcName})
    print_debug "hostname:$1==${t100s_thisProcName}"
    if [ `echo "${t100s_thisProcName}"|grep -c "^[[:blank:]]*$"` -ne 0 ]
    then
        print_debug "the host proc_name is NULL,used \"unknown\" now.[$1]"
        t100s_thisProcName="unknown"
    fi
    return 0
}

function t100s_isPhyPathFault()
{
    if [ `echo "$1"|grep -E -c "^[0-9]+$"` -eq 0 ] || [ "$2" = "" ] || [ "$3" = "" ]
    then
        print_error "2" "param error."
        t100s_exit 1
    fi

    local hostNum="${1}"
    local chanNum="${2}"
    local tgtNum="${3}"
    local vlunId=""
    local vlunIds
    local phyPathId
    local phyPathState

    vlunIds=`cat ${t100s_tmpLunMap} | grep "=${hostNum}:${chanNum}:${tgtNum}:" | awk -F "=" '{printf $2" "}'`

    if [ "${vlunIds}" = "" ]
    then
        return 1
    fi

    for vlunId in ${vlunIds}
    do
        if [ "${vlunId}" = "" ]
        then
            return 1
        else
            phyPathId=`upadmin show vlun id=${vlunId} type=all | grep "\[${hostNum}:${chanNum}:${tgtNum}:" | awk '{print $2}'`
            if [ "${phyPathId}" = "" ]
            then
                continue
            fi

            
            phyPathState=$(echo $(upadmin show path id=${phyPathId} | grep "^[[:blank:]]*Path State" | awk -F ":" '{print $2}'))
            if [ "${phyPathState}" != "Fault" ]
            then
                return 1
            fi

            return 0
        fi
    done


}

function t100s_delNotExistLunByTgt()
{
    local hostNum=""
    local channel=""
    local tgt=""
    local qlaProcDir=""
    local ret
    local isPhyPathFault

    if [ "$1" = "" ] || [ "$2" = "" ] || [ "$3" = "" ]
    then
        print_error "2" "param error."
        t100s_exit 1
    fi
    hostNum="$1"
    channel="$2"
    tgt="$3"

    print_debug "delete LUNs not exist in report.[${hostNum},${channel},${tgt}]"
    t100s_getSgDevByTgt "${hostNum}" "${channel}" "${tgt}"
    t100s_reportLuns
    ret=$?
    if [ ${ret} -eq 0 ]
    then
        t100s_VlunCheck="1"
        t100s_delLunsNotExistInReport "${hostNum}" "${channel}" "${tgt}"
        t100s_VlunCheck="0"
    elif [ ${ret} -eq 2 ]
    then

        print_debug "the connect is faild,delete all luns.[H:${hostNum}C:${channel}T:${tgt}]"
        t100s_reportLunResult=","
        t100s_delLunsNotExistInReport "${hostNum}" "${channel}" "${tgt}"
        t100s_needScanVirtualHost="0"

    elif [ ${ret} -eq 3 ]
    then
        return 0
    else

        print_debug "report lun failed.[H:${hostNum}C${channel}:T${tgt},proc_name:${2}]"
        t100s_isPhyPathFault "${hostNum}" "${channel}" "${tgt}"
        isPhyPathFault=$?
        if [ ${isPhyPathFault} -eq 0 ]
        then
            print_debug "the PhyPath State is Fault, delete all luns.[H:${hostNum}C:${channel}T:${tgt}]"
            t100s_reportLunResult=","
            t100s_delLunsNotExistInReport "${hostNum}" "${channel}" "${tgt}"
            t100s_needScanVirtualHost="0"
        fi

    fi
    return 0
}


function t100s_qla_scan_done()
{
    local procFile="$1"
    local delay=1
    local interation=10

    print_file "wait for scsi-qlascan"
    echo -n "wait for scsi-qlascan"
    while [ ${interation} -ge 1 ]
    do
        echo -n "."
        sleep ${delay}
        cat "${procFile}" | grep -v "\(.*:[[:space:]]\+0\)" | grep "\(.*\): Total.*flags.*\*.*" >/dev/null
        if [ $? -eq 0 ]
        then
            sleep 2
            break;
        fi
        let "interation=interation-1"
    done
    echo ""
    return 0
}

function t100s_scanSpecPhyScsihost()
{
    local tmpHost=""
    local hostNum=""
    local ret=""
    local tmpDevice=""
    local isNeedScan=""

    t100s_getOSType
    if [ "${t100s_OSVersion}" = "SLES10SP2" ] && [ -d /sys/class/fc_host ]
    then
        for tmpHost in `ls /sys/class/fc_host/`
        do
            isNeedScan="1"
            ret=`echo "${tmpHost}"|grep -c "^host[0-9][0-9]*$"`
            if [ ${ret} -eq 0 ]
            then
            print_debug "wrong host format [${tmpHost}]"
            continue
            fi

            hostNum=${tmpHost#host}
            for tmpDevice in `ls /sys/class/scsi_device/`
            do
                ret=`echo "${tmpDevice}" | grep -c "^${hostNum}:.*"`
                if [ ${ret} -ne 0 ]
                then
                    isNeedScan="0"
                    break
                fi
            done

            if [ "${isNeedScan}" = "1" ]
            then
                print_debug "re-init [host${hostNum}] issue_lip."
                echo "1" > /sys/class/fc_host/host${hostNum}/issue_lip
            fi
        done
    fi

    return 0
}

function t100s_scanPhyScsiHost_normal()
{
    local tmpHost
    local hostNum

    tmpHost="$1"
    hostNum=${tmpHost#host}

    if [ ! -f /sys/class/scsi_host/${tmpHost}/scan ]
    then
        print_debug "not found /sys/class/scsi_host/${tmpHost}/scan"
        return 0
    fi
    print_event "1008" "begin scan ${tmpHost}"

    for qlaProcDir in /proc/scsi/qla2*
    do
        if [ -e "${qlaProcDir}/${hostNum}" ]
        then
            echo "scsi-qlascan" > ${qlaProcDir}/${hostNum}
            t100s_qla_scan_done "${qlaProcDir}/${hostNum}"
        fi
    done

    echo "- - -" > /sys/class/scsi_host/${tmpHost}/scan
    return 0
}


t100s_phyScsiHosts=","
function t100s_getScsiHosts()
{
    local thisHostType=""
    local tmpHost=""
    local ret=""
    local oldIFS

    t100s_phyScsiHosts=","
    t100s_checkExist -d "/sys/class/scsi_host/" 2

    for tmpHost in `ls /sys/class/scsi_host/`
    do
        thisHostType="normal"
        ret=`echo "${tmpHost}"|grep "^host[0-9][0-9]*$"|wc -l`
        if [ ${ret} -eq 0 ]
        then
            print_debug "wrong host format [${tmpHost}]"
            continue
        fi
        if [ ! -f "/sys/class/scsi_host/${tmpHost}/scan" ]
        then
            print_warning "1009" "the file does not exist[/sys/class/scsi_host/${tmpHost}/scan]"
            continue
        fi


        t100s_getHostName "${tmpHost}"


        if [ `echo "${t100s_PROC_NAME_FILTER}"|grep -c ",${t100s_thisProcName},"` -ne 0 ]
        then
              print_debug "filter host[host:${tmpHost},proc_name:${t100s_thisProcName}]"
            continue
        fi

        t100s_phyScsiHosts="${t100s_phyScsiHosts}${tmpHost},${t100s_thisProcName},${thisHostType},"
    done
    print_debug "t100s_phyScsiHosts=${t100s_phyScsiHosts}"
    oldIFS="${IFS}"
    IFS=","
    t100s_phyScsiHosts=(${t100s_phyScsiHosts})
    IFS="${oldIFS}"
    return 0
}

function t100s_delAllNotExistLun()
{
    local tgt
    local hostNum
    local channel
    local tmpLine
    local thisTgt=""
    local pLunIDs=""
    local pLunID=""
    if [ ${t100s_handleLun0only} -ne 1 ]
    then
    	print_event "1010" "Begin to delete LUNs whose mappings do not exist"
    fi

    echo "" >  ${t100s_vlunRemoving}
    echo "" >  ${t100s_vlunActive}
    echo "" >  ${t100s_vlunDeleted}

    t100s_getProcInfo
    ret=$?
    if [ "${ret}" = "1" ]
    then
        print_debug "delAllNotExistLun get proc info failed."
        return 0
    fi

    for tmpLine in `ls -l /sys/class/scsi_generic/sg*/device 2>/dev/null|sed 's/^.*\///'|grep -E "^[0-9]+:[0-9]+:[0-9]+:[0-9]+$"|sort -t ':' -k 1n,1 -k 2n,2 -k 3n,3 -k 4n,4`
    do
        if [ "${thisTgt}" = "${tmpLine%:*}" ]
        then
            continue
        else
            thisTgt="${tmpLine%:*}"
        fi

        t100s_testExist "${tmpLine}"
        ret=$?
        if [ ${ret} -eq 0 ]
        then
            continue
        fi

        print_debug "analyse ${tmpLine}"
        hostNum=${tmpLine%%:*}
        channel=${tmpLine#*:}
        channel=${channel%%:*}
        tgt=${tmpLine%:*}
        tgt=${tgt##*:}

        t100s_getHostName "host${hostNum}"
        if [ `echo "${t100s_PROC_NAME_FILTER}"|grep -c ",${t100s_thisProcName},"` -ne 0 ]
        then
            print_debug "host${hostNum} is in filter.[procName:${t100s_thisProcName}]"
            continue
        fi
        t100s_delNotExistLunByTgt "${hostNum}" "${channel}" "${tgt}"
    done

    for tmpLine in `cat ${t100s_vlunRemoving} | awk '{printf $1" "}'`
    do
        if [ `cat ${t100s_vlunActive} | grep "^${tmpLine}$" -c` -eq 0 ]
        then
            if [ `cat ${t100s_vlunDeleted} | grep "^${tmpLine}$" -c` -eq 0 ]
            then
                echo "${tmpLine}" >> "${t100s_vlunDeleted}"
            fi
            tmpLine=${tmpLine%%_*}
            pLunIDs=`cat "${t100s_tmpLunMap}" | grep "online" | grep "=${tmpLine}$" | awk -F "=" '{print $4}'`
            for pLunID in ${pLunIDs}
            do
                pLunID=$(echo ${pLunID})
                t100s_testExist "${pLunID}"
                ret=$?
                if [ ${ret} -eq 0 ]
                then
                    continue
                fi
                t100s_removePLUN "${pLunID}"
            done
            t100s_vlunNeeDWaitToDeleted="${t100s_vlunNeeDWaitToDeleted}|=${tmpLine}$"
            print_debug "delete VLUN [${tmpLine}]"

        upadmin set stop_iosuspension ${tmpLine}								
        fi
    done

    [ -f ${t100s_vlunRemoving} ] && rm -rf ${t100s_vlunRemoving}
    [ -f ${t100s_vlunActive} ] && rm -rf ${t100s_vlunActive}
    return 0
}

function t100s_scanLinuxIscsi()
{
    if [ `echo "${t100s_PROC_NAME_FILTER}"|grep -c ",linuxIscsi,"` -ne 0 ]
    then
        print_debug "skip linux iscsi scan."
        return 0
    fi

    if [ -f /sys/class/scsi_host/iscsi/scan ]
    then
        print_event "1011" "begin scan linux-iscsi host."
        t100s_getOSType
        if [ "${t100s_OSVersion}" != "SLES9SP3" ] && [ "${t100s_OSVersion}" != "SLES9SP2_201" ]
        then

            echo "- - -" > /sys/class/scsi_host/iscsi/scan
        else
            print_debug "used rciscsi to scan linux-iscsi hosts"
            if [ -f "${g10s_logFile}" ]
            then
                rciscsi reload 1>>"${g10s_logFile}" 2>&1
            else
                rciscsi reload
            fi
        fi
    fi
    return 0
}

function t100s_scanAllPhyScsiHosts()
{
    local i=0
    local tmpHost=""
    local thisProcName=""
    local thisHostType=""

    t100s_getScsiHosts
    for (( i=1; i<${#t100s_phyScsiHosts[@]}; i=i+1 ))
    do
        tmpHost="${t100s_phyScsiHosts[${i}]}"

        if [ `echo "${tmpHost}"|grep -E -c "^host[0-9]+$"`  -eq 0 ]
        then
            print_error "1013" "host format error.[${tmpHost}]"
            t100s_exit 1
        fi
        let "i=i+1"
        thisProcName="${t100s_phyScsiHosts[${i}]}"
        let "i=i+1"
        thisHostType="${t100s_phyScsiHosts[${i}]}"
        if [ "${thisProcName}" = "" ] || [ "${thisHostType}" = "" ]
        then
            print_error "1014" "proc name or host type is NULL.[proc_name:${thisProcName},hostType:${thisHostType}]"
            t100s_exit 1
        fi

        print_debug "analyse [${tmpHost}]"
        t100s_eval "t100s_scanPhyScsiHost_${thisHostType} \"${tmpHost}\" \"${thisProcName}\""

    done

    t100s_scanSpecPhyScsihost
    t100s_scanLinuxIscsi

    return 0
}

function t100s_delLunsNotManageInCheck()
{
    local delLunSets
    local tmpLun
    local iCount
    local i
    local oldIFS
    local lunLabel=""
    local thisVendor=""
    local thisProduct=""
    local deleteSelect=""
    local SCSI_ID=""
    local same=0
    local ret

    delLunSets=("")
    wwn_ids=("")

    i=2
    t100s_checkExist -d "/sys/bus/scsi/drivers/sd/" 2
    t100s_checkExist -d "/sys/bus/scsi/drivers/sd/" 2

    t100s_getVirtualHostNum
    [ "${t100s_virtualHostNum}" = "" ] && t100s_virtualHostNum="-1"
    for tmpLun in `ls /sys/bus/scsi/drivers/sd/|grep -E "[0-9]+:"|grep -v "^[[:blank:]]*${t100s_virtualHostNum}:"`
    do
        thisVendor=`cat /sys/bus/scsi/drivers/sd/${tmpLun}/vendor 2>/dev/null`
        thisVendor=$(echo ${thisVendor})
        thisProduct=`cat /sys/bus/scsi/drivers/sd/${tmpLun}/model 2>/dev/null`
        thisProduct=$(echo ${thisProduct})
        print_debug "tmpLun=${tmpLun},thisVendor=${thisVendor}"
        if [ `cat "${t100s_VPWhiteList}" | grep -c "vendor_id *= *\"${thisVendor}\"[, ]*product_id *= *\"${thisProduct}\""` -gt 0 ]
        then
            if [ -d /sys/class/scsi_disk/${tmpLun}/device/block/ ]
            then
                lunLabel=`ls /sys/class/scsi_disk/${tmpLun}/device/block/`
            else
                lunLabel=`ls -l /sys/block/sd*/device|grep -E "/${tmpLun}$"`
                lunLabel=${lunLabel##*/sys/block/}
                lunLabel=${lunLabel%%/device*}
            fi
            let "i=i+1"
            delLunSets=("${delLunSets[@]}" "${i}|${tmpLun}")
            if [ "${t100s_isQuiet}" = "1" ]
            then
                if [ -f /sbin/scsi_id ]
                then
                    SCSI_ID="/sbin/scsi_id"
                else
                    SCSI_ID="/lib/udev/scsi_id"
                fi
                wwn_id=`${SCSI_ID} --page=0x83 --whitelisted --device=/dev/${lunLabel} 2>/dev/null`
                if [ "${wwn_id}" = "" ]
                then
                    wwn_id=`${SCSI_ID} -g -p 0x83 -s /block/${lunLabel} 2>/dev/null`
                fi
                if [ "${wwn_id}" = "" ]
                then
                    wwn_id=`${SCSI_ID} --page=0x83 --whitelisted --devpath=/block/${lunLabel} 2>/dev/null`
                fi
                for tmpWwn in "${wwn_ids[@]}"
                do
                    same=0
                    if [ "${tmpWwn}" = "${wwn_id}" ]
                    then
                        same=1
                        break
                    fi
                done
                if [ ${same} -ne 1 ]
                then
                    print_event "1034" "${wwn_id} will be delete"
                    wwn_ids=("${wwn_ids[@]}" "${wwn_id}")
                    let "t100s_NotManagedLuns=${t100s_NotManagedLuns}+1"
                fi
            else
                echo "${tmpLun}---${lunLabel}"
            fi
        fi
    done

    if [ "${i}" = "2" ]
    then
        print_debug "no lun need to delete."
        return 0
    fi


    t100s_getOSType
    if [ `echo "${t100s_OSVersion}" | grep -c -E "(UVPV1R2C00|UVPV1R1C03)"` -eq 0 ] && [ "${t100s_isQuiet}" = "0" ]
    then
        print_yesOrNo "1015" "These scsi devices below, are not managed but can be supported by UltraPath.Please make sure no IO on the devices.Do you want to remove them now?"
        ret=$?
        if [ ${ret} -eq 1 ]
        then
            print_warning "1016" "those lun can't be managed by UltraPath, insmod nxup modules failed."
            t100s_exit 2
        elif [ ${ret} -eq 2 ]
        then
            print_error "1019" "Invalid input too many times, those lun can't be managed by UltraPath, insmod nxup modules failed."
            t100s_exit 2
        fi
    fi

    if [ "${t100s_isQuiet}" = "1" ]
    then
        print_event "1031" "the number of luns to be managed is ${t100s_NotManagedLuns}."
    fi

    for tmpLun in "${delLunSets[@]}"
    do
        if [ "${tmpLun}" = "" ]
        then

            continue
        fi
        i=${tmpLun%%|*}
        tmpLun=${tmpLun##*|}
        if [ "${deleteSelect}" != "" ]
        then

            if [ `echo "${deleteSelect}"|grep -c ",${i},"` -eq 0 ]
            then
                continue
            fi
        fi

        t100s_removeLUN "${tmpLun}" >/dev/null
        if [ $? -ne 0 ]
        then
            print_error "1019" "Delete lun failed! some scsi devices can not be mananged by UltraPath. We suggest reboot your systerm.[${tmpLun}]"
            t100s_exit 1
        fi

    done
    return 0
}

function t100s_delLunsNotManage()
{
    local delLunSets
    local tmpLun
    local iCount
    local i
    local oldIFS
    local lunLabel=""
    local thisVendor=""
    local thisProduct=""
    local deleteSelect=""
    local SCSI_ID=""
    local thisTgt=""
    local disksOnOnePhypath
    local diskOnOnePhypath

    delLunSets=("")
    t100s_NotManagedLuns=0
    
    t100s_checkExist -d "/sys/bus/scsi/drivers/sd/" 2
    t100s_checkExist -d "/sys/bus/scsi/drivers/sd/" 2

    t100s_getVirtualHostNum
    for tmpLun in `ls /sys/bus/scsi/drivers/sd/ |grep -E "[0-9]+:"|grep -v "^[[:blank:]]*${t100s_virtualHostNum}:" | sort -t ':' -k 1n,1 -k 2n,2 -k 3n,3 -k 4n,4`
    do
        if [ "${thisTgt}" = "${tmpLun%:*}" ]
        then
            continue
        else
            thisTgt="${tmpLun%:*}"
        fi

        thisVendor=`cat /sys/bus/scsi/drivers/sd/${tmpLun}/vendor 2>/dev/null`
        thisVendor=$(echo ${thisVendor})
        thisProduct=`cat /sys/bus/scsi/drivers/sd/${tmpLun}/model 2>/dev/null`
        thisProduct=$(echo ${thisProduct})
        print_debug "tmpLun=${tmpLun},thisVendor=${thisVendor}"

        if [ `cat "${t100s_VPWhiteList}" | grep -c "vendor_id *= *\"${thisVendor}\"[, ]*product_id *= *\"${thisProduct}\""` -eq 0 ]
        then
            print_debug "phypath ${thisTgt} is not connect to array belongs to white array list"
            continue
        fi

        disksOnOnePhypath=(`ls /sys/bus/scsi/drivers/sd/ |grep -E "^${thisTgt}:[0-9]+$"| sort -t ':' -k 1n,1 -k 2n,2 -k 3n,3 -k 4n,4`)
        diskNumOnOnePhypath=${#disksOnOnePhypath[@]}
        if [ ${diskNumOnOnePhypath} -ne 0 ]
        then
            delLunSets=("${delLunSets[@]}" "${disksOnOnePhypath[@]}")
            let "t100s_NotManagedLuns=${t100s_NotManagedLuns}+${diskNumOnOnePhypath}"
            let "i=${diskNumOnOnePhypath}-1"
            t100s_notManageLunNumPerPath="${t100s_notManageLunNumPerPath}${thisTgt},${diskNumOnOnePhypath},"
            t100s_lastDisksPerPhyPath="${t100s_lastDisksPerPhyPath}${disksOnOnePhypath[${i}]},"	
        
            print_debug "t100s_notManageLunNumPerPath is: ${t100s_notManageLunNumPerPath}"
            print_debug "t100s_lastDisksPerPhyPath is: ${t100s_lastDisksPerPhyPath}"    
            
            if [ "${t100s_isQuiet}" = "0" ]
            then
                for tmpLun in ${disksOnOnePhypath[@]}
                do
                    if [ -d /sys/class/scsi_disk/${tmpLun}/device/block/ ]
                    then
                        lunLabel=`ls /sys/class/scsi_disk/${tmpLun}/device/block/`
                    else
                        lunLabel=`ls -l /sys/block/sd*/device|grep -E "/${tmpLun}$"`
                        lunLabel=${lunLabel##*/sys/block/}
                        lunLabel=${lunLabel%%/device*}
                    fi
                    echo "${tmpLun}---${lunLabel}"
                done
            fi
        fi
    done
    
    if [ ${t100s_NotManagedLuns} -eq 0 ]
    then
        print_debug "no lun need to delete."
        return 0
    fi

    t100s_getOSType
    if [ `echo "${t100s_OSVersion}" | grep -c -E "(UVPV1R2|UVPV1R3|UVPSP3|UVPV1R1C03|UVPV2R3)"` -eq 0 ] && [ "${t100s_isQuiet}" = "0" ]
    then
        print_yesOrNo "1015" "These scsi devices below, are not managed but can be supported by UltraPath.Please make sure no IO on the devices.Do you want to remove them now?"
        ret=$?
        if [ ${ret} -eq 1 ]
        then
            print_warning "1016" "those lun can't be managed by UltraPath, insmod nxup modules failed."
            t100s_exit 2
        elif [ ${ret} -eq 2 ]
        then
            print_error "1019" "Invalid input too many times, those lun can't be managed by UltraPath, insmod nxup modules failed."
            t100s_exit 2
        fi
    fi

    for((i=0;i<${#delLunSets[@]};i++))
    do
        let "k=${i}+10"
        if [ ${k} -ge ${#delLunSets[@]} ]
        then
            let "k=${#delLunSets[@]}-1"
        fi
        for((j=${i};j<=${k};j++))
        do
        {
            tmpLun=$(echo ${delLunSets[${j}]})
            if [ "${tmpLun}" = "" ]
            then
                continue
            fi
            t100s_removeLunByScsiID ${tmpLun}	
        }&        
        done
        wait
        let i=${k}
    done

    return 0
}

function t100s_removeLunByScsiID()
{
    local tmpLunID
    local devnr
    
    tmpLunID=$1
    if [ ! -d /sys/class/scsi_device/${tmpLunID} ]
    then
        return 0
    fi

    devnr=`echo "${tmpLunID}"|sed 's/:/ /g'`
    print_debug "delete lun ${devnr}"
    if [ -f /proc/scsi/scsi ]
    then
        echo "scsi remove-single-device ${devnr}" >/proc/scsi/scsi 2>&1
    fi
   
    if [ ! -d /sys/class/scsi_device/${tmpLunID} ]
    then
        return 0
    fi

    if [ -f /sys/class/scsi_device/${tmpLunID}/device/delete ]
    then
        echo 1 > /sys/class/scsi_device/${tmpLunID}/device/delete
    else
        print_debug "not found /sys/class/scsi_device/${tmpLunID}/device/delete"
    fi

    if [ ! -d /sys/class/scsi_device/${tmpLunID} ]
    then
        print_event "1005" "delete lun successfully.[$1]"
    else
        print_warning "1004" "delete lun failed.[$1]"
    fi
    
    return 0
}


function t100s_resumeDev()
{
    local sdDev=""
    sdDev="$1"

    if [ -f ${t100s_tmpDiskByIDLink} ]
    then
    	ret=`cat ${t100s_tmpDiskByIDLink} | grep -c "scsi-.*${sdDev}$"`
        if [ ${ret} -eq 0 ]
        then
            if [ -f "/sys/block/${sdDev}/uevent" ]
            then
                print_debug "resume disk by-id [${sdDev}]."
                echo add > /sys/block/${sdDev}/uevent
            fi
        fi
    else
        if [ -f "/sys/block/${sdDev}/uevent" ]
        then
            print_debug "resume disk by-id [${sdDev}]."
            echo add > /sys/block/${sdDev}/uevent
        fi
    fi

    return 0
}

function t100s_resumeDevPart()
{
    local sdDev=""
    local sdDevPart=""

    sdDev="$1"
    sdDevPart="$2"

    if [ -f ${t100s_tmpDiskByIDLink} ]
    then
        ret=`cat ${t100s_tmpDiskByIDLink} | grep -c "scsi-.*${sdDevPart}$"`
        if [ ${ret} -eq 0 ]
        then
            if [ -f "/sys/block/${sdDev}/${sdDevPart}/uevent" ]
            then
                print_debug "resume disk part by-id [${sdDevPart}]."
                echo add > /sys/block/${sdDev}/${sdDevPart}/uevent
            fi
        fi
    else
        if [ -f "/sys/block/${sdDev}/${sdDevPart}/uevent" ]
        then
            print_debug "resume disk part by-id [${sdDevPart}]."
            echo add > /sys/block/${sdDev}/${sdDevPart}/uevent
        fi
    fi

    return 0
}

function t100s_resumeDiskById()
{
    local ret=""
    local sdDev=""
    local sdDevPart=""
    local thisVendor

    [ -f ${t100s_tmpDiskByIDLink} ] && rm -rf ${t100s_tmpDiskByIDLink}
    if [ -d /dev/disk/by-id ]
    then
        ls -l /dev/disk/by-id/ > ${t100s_tmpDiskByIDLink}
    fi

    for sdDev in `ls /sys/block/ | grep "sd[a-z][a-z]*"`
    do
        if [ ! -f /sys/block/${sdDev}/device/vendor ]
        then
            continue
        fi

        thisVendor=`cat /sys/block/${sdDev}/device/vendor`
        thisVendor=$(echo ${thisVendor})
        if [ "${thisVendor}" != "up" ]
        then

            continue
        fi

        t100s_resumeDev "${sdDev}"

        if [ ! -f /sys/block/${sdDev} ]
        then
            continue
        fi

        for sdDevPart in `ls /sys/block/${sdDev}/ | grep "${sdDev}[0-9][0-9]*"`
        do
            t100s_resumeDevPart "${sdDev}" "${sdDevPart}"
        done
    done
    [ -f ${t100s_tmpDiskByIDLink} ] && rm -rf ${t100s_tmpDiskByIDLink}
    return 0
}

function t100s_scanVirtualHost()
{
    local allVlun
    local vLunId
    local oldvLunId
    local vlun_h
    local vlun_c
    local vlun_t
    local vlun_l
    local ret

    if [ "${t100s_needScanVirtualHost}" = "0" ]
    then
        return 0
    fi

    t100s_getProcInfo
    ret=$?
    if [ "${ret}" = "1" ]
    then
        print_debug "scanVirtualHost get proc info failed."
        return 0
    fi

    which uniq > /dev/null 2>&1
    ret="$?"
    if [ "${ret}" -ne "0" ]
    then
        print_debug "command \"uniq\" not exist."
        allVlun="$(cat "${t100s_tmpLunMap}" | grep "online" | awk -F "=" '{print $3}')"
    else
    	allVlun="$(cat "${t100s_tmpLunMap}" | grep "online" | awk -F "=" '{print $3}') | sort | uniq"
    fi
	
    for vLunId in ${allVlun}
    do
        vLunId="$(echo ${vLunId})"
        if [ "${oldvLunId}" = "${vLunId}" ]
        then
            continue
        fi
        oldvLunId="${vLunId}"
		
        if [ `echo "${vLunId}"|grep -E -c "^[0-9]+:[0-9]+:[0-9]+:[0-9]+$"` -ne 1 ]
        then
            print_debug "LUNIDS must be H:C:T:L.vLunId=${vLunId}"
            break
        fi
        t100s_testExist "${vLunId}"
        ret=$?
        if [ ${ret} -eq 1 ]
        then
            continue
        fi
        vlun_h="$(echo "${vLunId}" | awk -F ":" '{print $1}')"
        vlun_c="$(echo "${vLunId}" | awk -F ":" '{print $2}')"
        vlun_t="$(echo "${vLunId}" | awk -F ":" '{print $3}')"
        vlun_l="$(echo "${vLunId}" | awk -F ":" '{print $4}')"
        print_debug "vlun ${vLunId} is missing,rescan it"
        if [ ! -f /sys/class/scsi_host/host${vlun_h}/scan ]
        then
            print_debug "not found /sys/class/scsi_host/host${vlun_h}/scan"
            continue
        fi
        echo "${vlun_c} ${vlun_t} ${vlun_l}" > /sys/class/scsi_host/host${vlun_h}/scan
    done

    return 0
}


function t100s_deleteVritualLun()
{
   local vLunId
   
   t100s_getProcInfo
   ret=$?
   if [ "${ret}" = "1" ]
   then
       print_debug "deleteHoldonLun get proc info failed."
       return 0
   fi
   
   t100s_checkExist -d "/sys/bus/scsi/drivers/sd/" 2
   
   for vLunId in `ls /sys/bus/scsi/drivers/sd/|grep -E "[0-9]+:"`
   do
      thisVendor=`cat /sys/bus/scsi/drivers/sd/"${vLunId}"/vendor 2>/dev/null`
      thisVendor=$(echo ${thisVendor})
      print_debug "vLunId=${vLunId},thisVendor=${thisVendor}"
         
      if [ `echo "${thisVendor}" | grep -c "^up$" ` -ne 0  ]
      then
            if [ `cat "${t100s_tmpLunMap}" | grep -E -c "=${vLunId}="` -eq 0 ]
            then
                if [ -f /sys/class/scsi_device/${vLunId}/device/delete ]
                then
                    echo 1 > /sys/class/scsi_device/${vLunId}/device/delete
                    print_debug "Delete holdon lun ${vLunId} successful."
                else
                    print_debug "Not found holdon lun ${vLunId} , delete fault."
                fi
            fi           
       fi    
    done
}


function t100s_deleteHoldonLun()
{
    local allVlun
    local vLunId

    t100s_deleteVritualLun   
    return 0
}

function t100s_deleteNoPathLun()
{
    local allVlun
    local vLunId
    local ret
    local noPathLine
    local noPathId
    local bNoPathLun
    local sdName

    t100s_getProcInfo
    ret=$?
    if [ "${ret}" = "1" ]
    then
        print_debug "deleteNoPathLun get proc info failed."
        return 0
    fi

    allVlun="$(cat "${t100s_tmpLunMap}" | grep "^nopath")"
    for noPathLine in ${allVlun}
    do
        vLunId="$(echo "${noPathLine}"| awk -F "=" '{print $3}')"
        if [ `echo "${vLunId}"|grep -E -c "^[0-9]+:[0-9]+:[0-9]+:[0-9]+$"` -ne 1 ]
        then
            print_debug "LUNIDS must be H:C:T:L.vLunId=${vLunId}"
            break
        fi
        noPathId="$(echo "${noPathLine}"| awk -F "=" '{print $2}')"
        noPathId="$(echo ${noPathId})"

        t100s_testExist "${vLunId}"
        ret=$?
        if [ ${ret} -ne 1 ]
        then
            continue
        fi
        if [ "$1" = "0" ]
        then
            sdName="$(echo "${noPathLine}"| awk -F "=" '{print $4}')"
            sdName="$(echo ${sdName})"
            print_event "1004" "disk ${sdName} has no paths,if you want to delete the disk immediately,please execute the \"$0 -force\" command."
            continue
        fi
        bNoPathLun=1
        t100s_nopathVlunId="${t100s_nopathVlunId}${noPathId},"
        if [ "${t100s_ForceDelNoPathVLun}" = "1" ]
        then
            if [ -f /sys/class/scsi_device/${vLunId}/device/delete ]
            then
                echo 1 > /sys/class/scsi_device/${vLunId}/device/delete
                print_debug "Delete nopath lun ${vLunId} successful."
            else
                print_debug "Not found nopath lun ${vLunId} , delete fault."
            fi
        fi

    done

    return 0
}


function t100s_deleteMultiPlunIdLun()
{
    local i=0
    local allVlun
    local pLunId
    local ret
    local bHavePlunIdLun
    local lastVlunId
    t100s_conflictVlunId=","
    local tmpVlunId=""
    local sdName=""

    t100s_getProcInfo
    ret=$?
    if [ "${ret}" = "1" ]
    then
        print_debug "deleteMultiPlunIdLun get proc info failed."
        return 0
    fi

    bHavePlunIdLun=0
    allVlun="$(cat "${t100s_tmpLunMap}" | grep "^unused")"
    for unusedLine in ${allVlun}
    do

        pLunId="$(echo "${unusedLine}"| awk -F "=" '{print $3}')"
        oldVlunId="$(echo "${unusedLine}"| awk -F "=" '{print $4}')"
        newVlunId="$(echo "${unusedLine}"| awk -F "=" '{print $2}')"
        bHavePlunIdLun=1
        if [ `echo "${pLunId}"|grep -E -c "^[0-9]+:[0-9]+:[0-9]+:[0-9]+$"` -ne 1 ]
        then
            print_debug "LUNIDS must be H:C:T:L.pLunId=${pLunId}"
            break
        fi
        t100s_testExist "${pLunId}"
        ret=$?
        if [ ${ret} -ne 1 ]
        then
            continue
        fi

        oldVlunId="$(echo ${oldVlunId})"
        if [ "${lastVlunId}" != "${oldVlunId}" ]
        then
            lastVlunId="${oldVlunId}"
            t100s_conflictVlunId="${t100s_conflictVlunId}${oldVlunId},"
        fi

        if [ -f /sys/class/scsi_device/${pLunId}/device/delete ]
        then
            echo 1 > /sys/class/scsi_device/${pLunId}/device/delete
            print_debug "Delete MultiPlunId lun ${pLunId} for old vlunid {${oldVlunId}} ,new vlunid {${newVlunId}} successful."
        else
            print_debug "Not found MultiPlunId lun ${pLunId} , delete fault."
        fi
    done

    if [ ${bHavePlunIdLun} -eq 1 ] && [ $1 -eq 1 ]
    then
        oldIFS="${IFS}"
        IFS=","
        t100s_conflictVlunId=(${t100s_conflictVlunId})
        IFS="${oldIFS}"

        for (( i=1; i<${#t100s_conflictVlunId[@]}; i=i+1 ))
        do
            tmpVlunId=${t100s_conflictVlunId[${i}]}
            sdName=`cat "${t100s_tmpLunMap}" | grep "^online=${tmpVlunId}=" | awk -F "=" '{if(NR < 2){print $5}}'`
            if [ "${sdName}" = "" ]
            then
                sdName=`cat "${t100s_tmpLunMap}" | grep "^nopath=${tmpVlunId}=" | awk -F "=" '{print $4}'`
            fi
            if [ "${sdName}" = "" ]
            then
                continue
            fi
            print_warning "1004" "Disk ${sdName} has a different host LUN ID. The possible cause is that the array LUN has a different LUN ID or the LUN mapping changes. Log in to the storage array and change the host LUN ID to a correct one."
        done
        t100s_deleteNoPathLun 0
    fi

    return 0
}

function t100s_waitAndDeleteVirtualLun()
{
    local wait_sec=1
    local wait_all_sec=40
    local waited=0
    local deleteLun=","

    if [ ! -f ${t100s_nxupLunMap_a} ]
    then
        t100s_nxupLunMap=${t100s_nxupLunMap_b}
    else
        t100s_nxupLunMap=${t100s_nxupLunMap_a}
    fi

    if [ ! -f ${t100s_nxupLunMap} ]
    then
        print_warning "1004" "not found [${t100s_nxupLunMap}]"
        return 0
    fi
    t100s_getOSType
    
    while [ 1 -eq 1 ]
    do
        AllDeleteflag=1
        for tmpLine in `cat ${t100s_vlunDeleted} | awk '{printf $1" "}'`
        do
            diskId=${tmpLine%%_*}
            if [ `cat "${t100s_nxupLunMap}" | grep "online" | grep -E -c "=${diskId}$" ` -ne 0 ] || [ `cat "${t100s_nxupLunMap}"  | grep "nopath" | grep -E -c "=${diskId}$" ` -ne 0 ]
            then
                AllDeleteflag=0
            else
                if [ `echo "${t100s_OSVersion}" | grep -c -E "(UVPV1R2|UVPV1R3|UVPSP3|UVPV1R1C03|UVPV2R3)"` -ne 0 ]
                then
                    sdId=${tmpLine##*_}
                    if [ `cat "${t100s_nxupLunMap}"  | grep -E -c "=${sdId}=" ` -eq 0 ]
                    then
                        if [ `echo "=${sdId}=" | grep -E -c "${deleteLun}"` -eq 0  ]
                        then
                            t100s_do_removeLUN "${sdId}"
                            deleteLun="${deleteLun}|=${sdId}=$"
                        fi
                    fi   
                fi
            fi 
        done
        
        if [ ${AllDeleteflag} -eq 1 ] || [ ${waited} -ge ${wait_all_sec} ]
        then            
            break
        fi

        sleep ${wait_sec}
        let "waited=${waited} + ${wait_sec}"
    done
    
    [ -f ${t100s_vlunDeleted} ] && rm -rf ${t100s_vlunDeleted}
    print_debug "wait ${waited}s for deleted disk report done "
}

function t100s_waitAllVirtualLunDeleted()
{
    local wait_sec=1
    local wait_all_sec=100
    local waited=0

    if [ ! -f ${t100s_nxupLunMap_a} ]
    then
        t100s_nxupLunMap=${t100s_nxupLunMap_b}
    else
        t100s_nxupLunMap=${t100s_nxupLunMap_a}
    fi

    if [ ! -f ${t100s_nxupLunMap} ]
    then
        print_warning "1004" "not found [${t100s_nxupLunMap}]"
        return 0
    fi

    while [ 1 -eq 1 ]
    do
        if [ `cat "${t100s_nxupLunMap}" | grep -E -c "${t100s_vlunNeeDWaitToDeleted}"` -eq 0 ] || [ ${waited} -ge ${wait_all_sec} ]
        then
            break
        fi
        sleep ${wait_sec}
        let "waited=${waited} + ${wait_sec}"
    done
    print_debug "wait ${waited}s for virtual lun report done "

}

function t100s_waitAllLunReport()
{
    local wait_sec=2
    local wait_all_sec=100
    local waited=0
    local PhyLunNum=`ls ${t100s_nxupScsiDir} | grep -E -c "^[0-9]+:[0-9]+:[0-9]+:[0-9]+$"`
    local ReprotLunNum
    print_debug "start ${PhyLunNum}"
    sleep ${wait_sec}
    print_debug "wait for lun report."
    if [ ! -f ${t100s_nxupLunMap_a} ]
    then
        t100s_nxupLunMap=${t100s_nxupLunMap_b}
    else
        t100s_nxupLunMap=${t100s_nxupLunMap_a}
    fi

    if [ ! -f ${t100s_nxupLunMap} ]
    then
        print_warning "1004" "not found [${t100s_nxupLunMap}]"
        return 0
    fi

    PhyLunNum=`ls ${t100s_nxupScsiDir} | grep -E -c "^[0-9]+:[0-9]+:[0-9]+:[0-9]+$"`
    while [ 1 -eq 1 ]
    do
       PhyLunNum=`ls ${t100s_nxupScsiDir} | grep -E -c "^[0-9]+:[0-9]+:[0-9]+:[0-9]+$"`
       ReprotLunNum=`cat "${t100s_nxupLunMap}" | grep -E -c "^[unused|online]"`
        if [ ${ReprotLunNum} -ge ${PhyLunNum} ] || [ ${waited} -ge ${wait_all_sec} ]
        then
            print_debug "ReprotLunNum=${ReprotLunNum},PhyLunNum=${PhyLunNum} "
            break
        fi
        sleep ${wait_sec}
        let "waited=${waited} + ${wait_sec}"
        print_debug "."
    done
    print_debug "done"

}


function t100s_deleteObsoleteLun()
{
   local vLunId
   local vlunIdSet
   local tmpFilterLunMap="${t100s_tmpDir}/tmpFilterLunMap"
   
   t100s_checkExist -d "/sys/bus/scsi/drivers/NXUP/" 2
   vlunIdSet=`ls /sys/bus/scsi/drivers/NXUP/|grep -E "[0-9]+:"`
   t100s_getProcInfo
   ret=$?
   if [ "${ret}" = "1" ]
   then
       print_debug "get proc info failed."
       return 0
   fi

   cat "${t100s_tmpLunMap}" | awk -F "=" '{print $4}' > ${tmpFilterLunMap}
   
   for vLunId in ${vlunIdSet}
   do
       if [ `cat "${tmpFilterLunMap}" | grep -E -c "^${vLunId}$"` -eq 0 ]
       then
           if [ -f /sys/class/scsi_device/${vLunId}/device/delete ]
           then
               echo 1 > /sys/class/scsi_device/${vLunId}/device/delete
               print_debug "Delete obsolete lun ${vLunId} successful."
            fi
       fi           
    done
    [ -f "${tmpFilterLunMap}" ] && rm -rf "${tmpFilterLunMap}"
}


function t100s_main()
{
    t100s_removeTmpDevByLssci
    t100s_deleteObsoleteLun
    t100s_delAllNotExistLun
    t100s_delLunMapChanged   
    t100s_deleteMultiPlunIdLun 0
    if [ "${t100s_ForceDelNoPathVLun}" = "1" ]
    then
        t100s_deleteNoPathLun 1
		t100s_waitAllVirtualLunDeleted
    fi
    t100s_waitAndDeleteVirtualLun
    
    t100s_scanAllPhyScsiHosts
    t100s_scanVirtualHost
    t100s_resumeDiskById
    if [ "${t100s_ForceDelVLun}" = "1" ]
    then
        t100s_deleteHoldonLun
    fi

    t100s_waitAllLunReport
    t100s_deleteMultiPlunIdLun 1

    if [ -f ${t100s_nxupDismtachMap} ]
    then
        if [ "`cat ${t100s_nxupDismtachMap} | grep "NoDismatch"`" = "" ]
        then
            print_warning "1025" "Detect more than one type of HBA connecting to storage on your systerm at the same time or in tandem, please make sure only one type of connection, and reboot your systerm."
        fi
    fi

    [ -f "${t100s_tmpDir}t100s_nxup_lun_map" ] && rm -rf "${t100s_tmpDir}t100s_nxup_lun_map"
    print_event "1021" "The device scanning is complete."
    return 0
}

function t100s_forStartNxup()
{
    t100s_removeTmpDevByLssci
    t100s_handleLun0only=1
    t100s_delAllNotExistLun
    t100s_handleLun0only=0
    t100s_deleteMultiPlunIdLun 0

    t100s_waitAndDeleteVirtualLun
    
    t100s_scanAllPhyScsiHosts
    t100s_scanVirtualHost
    t100s_resumeDiskById

    t100s_waitAllLunReport
    t100s_deleteMultiPlunIdLun 1

    if [ -f ${t100s_nxupDismtachMap} ]
    then
        if [ "`cat ${t100s_nxupDismtachMap} | grep "NoDismatch"`" = "" ]
        then
            print_warning "1025" "Detect more than one type of HBA connecting to storage on your systerm at the same time or in tandem, please make sure only one type of connection, and reboot your systerm."
        fi
    fi

    [ -f "${t100s_tmpDir}t100s_nxup_lun_map" ] && rm -rf "${t100s_tmpDir}t100s_nxup_lun_map"
    print_event "1021" "The device scanning is complete."
    return 0
}


function t100s_nxupCheck()
{
    if [ `lsmod | grep -c "^[[:blank:]]*nxup"` -eq 0 ]
    then
        print_warning "1025" "Start the UltraPath to enable it to take effect."
        t100s_exit 1
    fi
    return 0
}

function t100s_createWhiteList()
{
    local line
    local upconf
    
    if test -s /etc/up.conf
    then
        upconf="/etc/up.conf"
    elif test -s /etc/up.conf.bak
    then
        upconf="/etc/up.conf.bak"
    else
        return
    fi

    echo "" > ${t100s_VPWhiteList}
    
    if [ ! -f ${t100s_VPWhiteList} ]
    then
        t100s_VPWhiteList="${upconf}"
        return
    fi
    
    while read line
    do
        line=$(echo ${line})
        if [ -z "${line}" ]
        then
            continue
        fi
        echo "${line}" >> ${t100s_VPWhiteList}
    done < "${upconf}"
}

function t100s_serviceNxupCheck()
{
    local thepid=""
    local ret=""

    if [ -f "${t300s_lockFile}" ]
    then
        thepid=`cat "${t300s_lockFile}" 2>/dev/null`
        if [ "${thepid}" != "" ]
        then
            ret=`ps ax |awk '{print $1}'| grep -c "^[[:blank:]]*${thepid}[[:blank:]]*$"`
            if [ ${ret} -eq 1 ]
            then
                print_warning "1026" "service nxup stop is running, exit."
                t100s_exit 1
            fi
        fi
        rm -f "${t300s_lockFile}"
    fi
}

function t100s_InstallCheck()
{
    local thepid=""
    local ret=""

    if [ -f "${i10s_lockFile}" ]
    then
        thepid=`cat "${i10s_lockFile}" 2>/dev/null`
        if [ "${thepid}" != "" ]
        then
            ret=`ps ax |awk '{print $1}'| grep -c "^[[:blank:]]*${thepid}[[:blank:]]*$"`
            if [ ${ret} -eq 1 ]
            then
                print_warning "1026" "install is running, exit."
                t100s_exit 1
            fi
        fi
        rm -f "${i10s_lockFile}"
    fi
}

function t100s_countVlunNum()
{
    local ret=""
    local sdDev=""
    local sdDevPart=""
    local thisVendor
    t100s_VLunNum=0

    for sdDev in `ls /sys/block/`
    do
    ret=`echo ${sdDev} | grep -c "sd[a-z][a-z]*"`
    if [ ${ret} -eq 0 ]
    then
        continue
    fi

        if [ ! -f /sys/block/${sdDev}/device/vendor ]
        then
            continue
        fi

        thisVendor=`cat /sys/block/${sdDev}/device/vendor`
        thisVendor=$(echo ${thisVendor})
        if [ "${thisVendor}" = "up" ]
        then
            let "t100s_VLunNum=${t100s_VLunNum}+1"
        fi
    done
}

function t100s_getProcInfoOnBoot()
{
    t100s_tmpLunMap="${t100s_tmpDir}t100s_nxup_lun_map"
    
    if [ ! -f ${t100s_nxupLunMap_a} ]
    then
        t100s_nxupLunMap=${t100s_nxupLunMap_b}
    else
        t100s_nxupLunMap=${t100s_nxupLunMap_a}
    fi

    print_debug "t100s_nxupLunMap=${t100s_nxupLunMap}"

    if [ ! -f ${t100s_nxupLunMap} ]
    then
        print_warning "1004" "not found [${t100s_nxupLunMap}]"
        return 1
    fi

    if [ -f ${t100s_tmpLunMap} ]
    then
        rm -rf ${t100s_tmpLunMap} 1>/dev/null 2>&1
    fi
    
    if [ ! -f ${t100s_tmpLunMap} ]
    then
        touch "${t100s_tmpLunMap}" 1>/dev/null 2>&1
        if [ -f ${t100s_tmpLunMap} ]
        then
            cat "${t100s_nxupLunMap}" > "${t100s_tmpLunMap}" 2>/dev/null
        else
            t100s_tmpLunMap=${t100s_nxupLunMap}
        fi
    else
        t100s_tmpLunMap=${t100s_nxupLunMap}
    fi

    return 0
}

function t100s_waitforLunReport()
{
    local wait_sec=10
    local wait_all_sec=300
    local waited=0
    local oldIFS
    local flag
    local tgtpath
    local lunNum
    
    
    oldIFS="${IFS}"
    IFS=","
    t100s_notManageLunNumPerPath=(${t100s_notManageLunNumPerPath})
    t100s_lastDisksPerPhyPath=(${t100s_lastDisksPerPhyPath})
    IFS="${oldIFS}"
    
    sleep ${wait_sec}
    echo -n "wait for lun report."
    
    flag=0

    while [ ${waited} -lt ${wait_all_sec} ]
    do
        t100s_getProcInfoOnBoot
        ret=$?
        if [ "${ret}" = "1" ]
        then
            print_debug "t100s_waitforLunReport get proc info failed."
            sleep ${wait_sec}
            let "waited=${waited} + ${wait_sec}"
            echo -n "."
            continue
        fi        
    
        for tmppath in ${t100s_lastDisksPerPhyPath[@]}
        do
            if [ `cat "${t100s_tmpLunMap}" | awk -F "=" '/online/ {print $4}' | grep -E -c "^${tmppath}$"` -eq 0 ]
            then
                flag=1
                break
            fi
        done
        
        if [ ${flag} -eq 1 ]
        then
            sleep ${wait_sec}
            let "waited=${waited} + ${wait_sec}"
            echo -n "."
            flag=0
            continue
        fi

        for (( i=0; i<${#t100s_notManageLunNumPerPath[@]}; i=i+1 ))
        do
            tgtpath="${t100s_notManageLunNumPerPath[${i}]}"
            let "i=i+1"
            lunNum="${t100s_notManageLunNumPerPath[${i}]}"
            if [ `cat "${t100s_tmpLunMap}" | awk -F "=" '/online/ {print $4}' | grep -E -c "^${tgtpath}:[0-9]+$"` -lt ${lunNum} ]
            then
                flag=1
                break
            fi
        done

        if [ ${flag} -eq 1 ]
        then
            sleep ${wait_sec}
            let "waited=${waited} + ${wait_sec}"
            echo -n "."
            flag=0
            continue
        fi

        vlunNum=`cat "${t100s_tmpLunMap}" | awk -F "=" '/online/ {print $3}' | sort | uniq | wc -l`
        sysBlockNum=`cat /sys/block/sd*/device/vendor 2>/dev/null| grep -E -c "^[[:blank:]]*up[[:blank:]]*$"`

        if [ ${vlunNum} -gt ${sysBlockNum} ]
        then
            sleep ${wait_sec}
            let "waited=${waited} + ${wait_sec}"
            echo -n "."
            flag=0
            continue
        fi 

        break
    done
    if [ ${waited} -ge ${wait_all_sec} ]
    then
        print_debug "wait for sync scan timeout"
    fi
    
    echo "done"
}

function t100s_scanAllPhyScsiHosts_sync()
{
    t100s_scanAllPhyScsiHosts
    print_event "1032" "begin wait for lun report."
    t100s_waitforLunReport
    print_event "1033" "end wait for lun report."
    if [ ${t100s_NotManagedLuns} -gt 0 ]
    then
        t100s_getOSType
        if [ `echo "${t100s_OSVersion}"|grep -c "^[[:blank:]]*ROCKY.*$"` -ne 0 ]
        then
            print_event "1035" "refresh lvm."
            vgchange -an --ignorelockingfailure 2>/dev/null
            vgchange -ay --ignorelockingfailure 2>/dev/null
        fi
    fi
    return 0
}

print_debug "upRescan begin..."
t100s_lockShell "${t100s_lockFile}"

t100s_serviceNxupCheck
t100s_InstallCheck
t100s_createWhiteList

while [ $# -gt 0 ]
do
    t100s_param="$1"
    if [ $(echo "${t100s_param}"|grep -c "^-[^-]") -eq 1 ]
    then
        t100s_tmpParam=${t100s_param#-}
        t100s_paramLen=${#t100s_tmpParam}
        t100s_i=1
        case "${t100s_tmpParam}" in
            "m")
                t100s_nxupCheck
                UP_CMD_PATH=/UltraPath/bin
                export PATH=$PATH:$UP_CMD_PATH
                t100s_isQuiet="1"
                t100s_delLunsNotManage
                t100s_scanAllPhyScsiHosts_sync
                t100s_exit $?
                ;;
            "q")
                t100s_nxupCheck
                t100s_isQuiet="0"
                t100s_delLunsNotManage
                t100s_exit $?
                ;;
            "s")
                t100s_delLunsNotManageInCheck
                t100s_exit $?
                ;;
            "f")
                t100s_ForceDelVLun=1
                ;;
            "force")
                t100s_ForceDelNoPathVLun=1
                ;;
            "su")
                t100s_forStartNxup
                t100s_exit 0
                ;;
            *)
                print_warning "1022" "Usage Error.[${t100s_param}]"
                t100s_useage
                ;;
        esac
    else
        print_warning "1024" "Usage Error.[${t100s_param}]"
        t100s_useage
    fi
    shift
done

t100s_nxupCheck
t100s_rootCheck
t100s_main

print_debug "upRescan end..."
t100s_exit 0


