#!/bin/bash

g30s_LIBFILE=${g30s_LIBFILE="../globalLib/g30s_systemEnvironment"}
if [ -f "${g30s_LIBFILE}" ]
then
    . "${g30s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
    echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g30s_LIBFILE}]\033[0m"
    exit 1
fi
g20s_LIBFILE=${g20s_LIBFILE="../globalLib/g20s_install"}
if [ -f "${g20s_LIBFILE}" ]
then
    . "${g20s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
    echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g20s_LIBFILE}]\033[0m"
    exit 1
fi


t110s_action=""
t110s_bootLoaderType="auto"

readonly t110s_solutionFile="${g10s_bakPath}/t110s_bootLoader.solution"
if [ ! -f "${t110s_solutionFile}" ]
then
    touch "${t110s_solutionFile}"
fi

function t110s_useage()
{
    echo "$0 action=append|remove [type=XXX]"
    exit 1
}



function t110s_appendRemoveAction()
{
    local oldUninstallSolution

    if [ "$1" = "" ]
    then
        print_error "2" "param error"
        exit 1
    fi

    g20s_appendUninstallAction "50_modifyBootLoader" "shell" "./" "bash t110s_modifyBootLoader.sh action=remove"


    oldUninstallSolution="${g20s_unistallSolution}"
    g20s_unistallSolution="${t110s_solutionFile}"
    g20s_appendUninstallAction "999_modifyBootLoader_${t110s_bootLoaderType}" "shell" "`pwd`" "$1"
    g20s_unistallSolution="${oldUninstallSolution}"
    return 0
}

function t110s_excuteAction()
{
    if [ "$1" = "" ] || [ "$2" = "" ]
    then
        print_error "2" "param error"
        exit 1
    fi
    t110s_appendRemoveAction "$2"

    echo "${g10s_isspinitrdrecordname}" >> "${g10s_isspinitrdrecordfile}"

    g10s_getOtherIsspInstalled
    if [ "${g10s_OtherIsspInstalled}" != "" ]
    then
        print_debug "you have installed ${g10s_OtherIsspInstalled}, no need to append boot loader"
        return 0
    fi

    g10s_eval "$1"
    return 0
}


function t110s_modifyBootLoader_grub()
{
    t110s_excuteAction "bash t20s_menulstModify.sh action=append" "bash t20s_menulstModify.sh action=remove"
    return 0
}

function t110s_modifyBootLoader_lilo()
{
    t110s_excuteAction "bash t30s_liloModify.sh file=/etc/lilo.conf action=append type=lilo" "bash t30s_liloModify.sh file=/etc/lilo.conf action=remove type=lilo"
}

function t110s_modifyBootLoader_elilo()
{
    t110s_excuteAction "bash t30s_liloModify.sh file=/etc/elilo.conf action=append type=elilo" "bash t30s_liloModify.sh file=/etc/elilo.conf action=remove type=elilo"
    return 0
}

function t110s_modifyBootLoader_egrub()
{
    t110s_excuteAction "bash t20s_menulstModify.sh file=/etc/grub.conf action=append" "bash t20s_menulstModify.sh file=/etc/grub.conf action=remove"
    return 0
}

function t110s_modifyBootLoader_grub2()
{
    t110s_excuteAction "bash t50s_grubcfgModify.sh action=append" "bash t50s_grubcfgModify.sh action=remove"
    return 0
}

function t110s_modifyBootLoaderForRedhat7_grub2()
{
    t110s_excuteAction "bash t51s_grubcfgModifyForRH7.sh action=append" "bash t51s_grubcfgModifyForRH7.sh action=remove"
    return 0
}

function t110s_modifyBootLoaderForSLES12_grub2()
{
	t110s_excuteAction "bash t51s_grubcfgModifyForSLES12.sh action=append" "bash t51s_grubcfgModifyForSLES12.sh action=remove"
	return 0
}
function t110s_append()
{
    if [ "${t110s_bootLoaderType}" = "auto" ]
    then
        g30s_getBootLoaderType
        if [ $? -ne 0 ]
        then
            print_error "1001" "get boot loader type failed."
            exit 1
        fi
        t110s_bootLoaderType=$(echo ${g30s_bootLoaderType})
    fi
    print_debug "modify boot loader by type[${t110s_bootLoaderType}]"

    g10s_getOSType
    if [ `echo "${g10s_OSVersion}"|grep -c -E "(RHEL7|OL7|KYLIN7SP2)"` -eq 1 ]
    then
        g10s_eval "t110s_modifyBootLoaderForRedhat7_${t110s_bootLoaderType}"
    elif [ `echo "${g10s_OSVersion}"|grep -c "SLES12"` -eq 1 ]
    then
    	g10s_eval "t110s_modifyBootLoaderForSLES12_${t110s_bootLoaderType}"
    else
        g10s_eval "t110s_modifyBootLoader_${t110s_bootLoaderType}"
    fi

    return 0
}

function t110s_remove()
{
    local oldUninstallSolution

    g10s_getOtherIsspInstalled
    if [ "${g10s_OtherIsspInstalled}" != "" ]
    then
        print_debug "you have installed ${g10s_OtherIsspInstalled}, will not remove boot loader configure"
        return 0
    fi


    g20s_doActionIDs=("999_.*")
    oldUninstallSolution="${g20s_unistallSolution}"
    g20s_unistallSolution="${t110s_solutionFile}"
    g20s_uninstallBySolution
    g20s_unistallSolution="${oldUninstallSolution}"
    return 0
}

function t110s_main()
{
    print_debug "begin modify boot loader."
    print_debug "type=${t110s_bootLoaderType},action=${t110s_action}"
    g10s_eval "t110s_${t110s_action}"
    return 0
}

g10s_rootCheck
while [ $# -gt 0 ]
do
    t110s_param="$1"
    t110s_param1=${t110s_param%%=*}
    t110s_param2=${t110s_param##*=}
    print_debug "t110s_param=${t110s_param},t110s_param1=${t110s_param1},t110s_param2=${t110s_param2}"
    case "${t110s_param1}" in
        "type")
            t110s_bootLoaderType="${t110s_param2}"
            ;;
        "action")
            t110s_action="${t110s_param2}"
            ;;
        *)
            print_error "1003" "unknown parameter[${t110s_param}]"
            t110s_useage
            exit 1
            ;;
    esac
    shift
done

if [ "${t110s_action}" != "append" ] && [ "${t110s_action}" != "remove" ]
then
    print_error "1004" "parameter error.[action=${t110s_action}]"
    t110s_useage
fi

if [ "${t110s_bootLoaderType}" = "" ]
then
    t110s_bootLoaderType="auto"
fi

t110s_main


