#!/bin/bash
t130s_FC_PORT_WWN=""
t130s_FC_MODEL=""
t130s_FC_FIRMWARE_VERSION=""
t130s_FC_DRIVER_VERSION=""
t130s_FC_HOST_DIR="/sys/class/fc_host/"


g10s_LIBFILE=${g10s_LIBFILE="../globalLib/g10s_pub"}
if [ -f "${g10s_LIBFILE}" ]
then
	. "${g10s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g10s_LIBFILE}]\033[0m"
	exit 1
fi


t130s_hostname=""
t130s_IPTYPE=""
t130s_IPADDR=""

function t130s_getIPInfo()
{
    local ipv4Flag=""
    local ipv6Flag=""
    local lineCount=0
    local linemsg=""
    local tmpmsg=""
    t130s_IPADDR=""
    
    if [ `echo "${g10s_OSVersion}"|grep -c -E "(RHEL7|OL7|UVPV2R3|KYLIN7SP2)"` -eq 1 ]
    then
        ipv4Flag="inet "
        ipv4CutFlag="netmask"
        ipv6Flag="inet6 "
        ipv6CutFlag="prefixlen"
    else
        ipv4Flag="inet addr:"
        ipv4CutFlag="Bcast"
        ipv6Flag="inet6 addr:"
        ipv6CutFlag="/"
    fi
    
    lineCount=`ifconfig 2>/dev/null |wc -l`
    for ((i=1; i<=${lineCount}; i++))
    do
       let "linenum=${i}"
       linemsg=`ifconfig 2>/dev/null | sed -n "${linenum}p"`
       if [ `echo ${linemsg} |grep -c "${ipv4Flag}"` -ne 0 ]
       then
            tmpmsg=${linemsg##*${ipv4Flag}}
            t130s_IPADDR=${tmpmsg%%${ipv4CutFlag}*}
            t130s_IPADDR=$(echo ${t130s_IPADDR})
            if [ "${t130s_IPADDR}" != "" ]
            then
                t130s_IPTYPE="IPV4"
                break
            fi
       elif [ `echo ${linemsg} |grep -c "${ipv6Flag}"` -ne 0 ]
       then
   	       tmpmsg=${linemsg##*${ipv6Flag}}
	       t130s_IPADDR=${tmpmsg%%${ipv6CutFlag}*}
           t130s_IPADDR=$(echo ${t130s_IPADDR})

 	       if [ "${t130s_IPADDR}" != "" ]
 	       then
               t130s_IPTYPE="IPV6"
     		   break
   	       fi
       fi    
     done
     
     
     if [ "${t130s_IPADDR}" = "" ]
     then
         print_warning "1001" "get IP address failed."
         return 1
     fi
     return 0
}


t130s_hostname=`hostname`
g10s_getOSType
t130s_getIPInfo
g10s_getSysBit

function t130s_getHBAPortInfo()
{
    if [ ! -d "$t130s_FC_HOST_DIR" ]
    then
        return 0
    fi

    cd $t130s_FC_HOST_DIR
    t130s_ALL_FC_HOST=`ls`
    for t130s_per_fc_host in $t130s_ALL_FC_HOST
    do
        t130s_FC_PORT_WWN=""
        t130s_FC_MODEL=""
        t130s_FC_FIRMWARE_VERSION=""
        t130s_FC_DRIVER_VERSION=""
        if [ -f /sys/class/fc_host/$t130s_per_fc_host/port_name ]
        then
            t130s_temp_fcport_wwn=`cat /sys/class/fc_host/$t130s_per_fc_host/port_name`
            t130s_FC_PORT_WWN=`echo ${t130s_temp_fcport_wwn:0-16}` 
            t130s_temp_fcport_wwn=""
        else
            t130s_FC_PORT_WWN=""
        fi
        
        if [ ! -d "/sys/class/scsi_host/$t130s_per_fc_host/" ]
        then
            continue
        fi
        
        t130s_temp_model_name=`ls /sys/class/scsi_host/$t130s_per_fc_host/ |grep  -E "model.*name" | head -n 1`
        if [ "$t130s_temp_model_name" != "" ]
        then
            if [ -f /sys/class/scsi_host/$t130s_per_fc_host/$t130s_temp_model_name ]
            then
                t130s_FC_MODEL=`cat  /sys/class/scsi_host/$t130s_per_fc_host/$t130s_temp_model_name`
            else
                t130s_FC_MODEL=""
            fi
        fi
        
        if [ -f /sys/class/scsi_host/$t130s_per_fc_host/fw_version ]
        then
            t130s_FC_FIRMWARE_VERSION=`cat /sys/class/scsi_host/$t130s_per_fc_host/fw_version`
        elif [ -f /sys/class/scsi_host/$t130s_per_fc_host/fwrev ]
        then
            t130s_FC_FIRMWARE_VERSION=`cat /sys/class/scsi_host/$t130s_per_fc_host/fwrev`
        elif [ -f /sys/class/scsi_host/$t130s_per_fc_host/firmware_version ]
        then
            t130s_FC_FIRMWARE_VERSION=`cat /sys/class/scsi_host/$t130s_per_fc_host/firmware_version`
        else
           t130s_FC_FIRMWARE_VERSION=""
        fi

        if [ -f /sys/class/scsi_host/$t130s_per_fc_host/lpfc_drvr_version ]
        then
            t130s_FC_DRIVER_VERSION=`cat /sys/class/scsi_host/$t130s_per_fc_host/lpfc_drvr_version`
        elif [ -f /sys/class/scsi_host/$t130s_per_fc_host/driver_version ]
        then
            t130s_FC_DRIVER_VERSION=`cat /sys/class/scsi_host/$t130s_per_fc_host/driver_version`
        else
            t130s_FC_DRIVER_VERSION="" 
        fi

        if [ "$t130s_FC_PORT_WWN" != "" ]
        then
            echo ""FC_Port_WWN"="\"$t130s_FC_PORT_WWN\"","FC_Model"="\"$t130s_FC_MODEL\"","FC_Driver_Version"="\"$t130s_FC_DRIVER_VERSION\"","FC_Firmware_Version"="\"$t130s_FC_FIRMWARE_VERSION\""; " >> "${1}.tmp"
        fi
    done
    return 0
}

if [ "$1" != "" ] && [ `echo "$1"|grep -c ".*/$"` -eq 0 ]
then
    rm -f "$1"
   
    set -e
    echo "HostName=${t130s_hostname};" > "${1}.tmp"
    echo "IpType=${t130s_IPTYPE};" >> "${1}.tmp"
    echo "IpAddress=${t130s_IPADDR};" >> "${1}.tmp"
    echo "OsVersion=${g10s_OSVersion}_${g10s_kernelType}_${g10s_sysbit};" >> "${1}.tmp"
    set +e
    t130s_getHBAPortInfo ${1}
	IS_UTF8=`file ${1}.tmp | grep "[Uu][Tt][Ff]" | grep 8 -c`
	if [ "${IS_UTF8}" == "0" ]
	then
		IS_UTF16=`file ${1}.tmp | grep "[Uu][Tt][Ff]" | grep 16 -c`
		if [ "${IS_UTF16}" == "1" ] 
		then
			if [ `file ${1}.tmp | grep "[Uu][Tt][Ff]" | grep 16 | grep "[Ll]ittle-[Ee]ndian" -c` -eq 1 ]
			then
				iconv -f UTF16LE -t UTF8 "${1}.tmp" 1>/dev/null 2>&1
				if [ $? -eq 0 ]
				then
					iconv -f UTF16LE -t UTF8 "${1}.tmp" > "${1}"
				fi
			else
				iconv -f UTF16BE -t UTF8 "${1}.tmp" 1>/dev/null 2>&1
				if [ $? -eq 0 ]
				then
					iconv -f UTF16BE -t UTF8 "${1}.tmp" > "${1}"
				fi
			fi
		fi
		
		IS_UTF32=`file ${1}.tmp | grep "[Uu][Tt][Ff]" | grep 32 -c`
		if [ "${IS_UTF32}" == "1" ]
		then
			if [ `file ${1}.tmp | grep "[Uu][Tt][Ff]" | grep 32 | grep "[Ll]ittle-[Ee]ndian" -c` -eq 1 ]
			then
				iconv -f UTF32LE -t UTF8 "${1}.tmp" 1>/dev/null 2>&1
				if [ $? -eq 0 ]
				then
					iconv -f UTF32LE -t UTF8 "${1}.tmp" > "${1}"
				fi
			else
				iconv -f UTF32BE -t UTF8 "${1}.tmp" 1>/dev/null 2>&1
				if [ $? -eq 0 ]
				then
					iconv -f UTF32BE -t UTF8 "${1}.tmp" > "${1}"
				fi
			fi
		fi
		
		IS_UTF=`file ${1}.tmp | grep "[Uu][Tt][Ff]" -c`
		if [ "${IS_UTF}" == "0" ]
		then
			iconv -f GB18030 -t UTF8 "${1}.tmp" 1>/dev/null 2>&1
			if [ $? -eq 0 ]
			then
				iconv -f GB18030 -t UTF8 "${1}.tmp" > "${1}"
			fi
		fi
	fi
	
	if [ ! -f "${1}" ]
	then
		/bin/cat "${1}.tmp" > "${1}"
	fi
	
	rm -f "${1}.tmp"
	g10s_chmodExec 600 "$1"
	
fi
    	




