#!/bin/bash

g20s_LIBFILE=${g20s_LIBFILE="../globalLib/g20s_install"}
if [ -f "${g20s_LIBFILE}" ]
then
	. "${g20s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g20s_LIBFILE}]\033[0m"
	exit 1
fi

t140s_solutionFile="$1"
t140s_solutionType="$2"


t140s_serviceActionIDs=("10_preInstall" "20_installFiles" "60_mkBootService" "70_runServiceForService")
t140s_mkinitrdActionIDs=("10_preInstall" "20_installFiles" "30_modifyMkinitrd" \
                        "40_mkImage" "50_modifyBootLoader"  "80_runServiceForMkinitrd")

function t140s_useage()
{
    echo "$0 [filename] [service|mkinitrd]
    filename    the solution filename for output.
    [service|mkinitrd] the solution type."
    exit 1
}

function t140s_appendActionToSolutionFile()
{
    local solutionFile
    local ret

    if [ "$1" = "" ]
    then
        print_error "2" "param error."
        exit 1
    fi
    solutionFile=`cat "${g20s_actionList}"|grep "^[[:blank:]]*${1}=="`
    if [ "${solutionFile}" = "" ]
    then

        print_warning "1002" "not found the action.[${1},${g20s_actionList}]"
    fi

    if [ -f "${t140s_solutionFile}" ]
    then
        cat "${g20s_actionList}"|grep "^[[:blank:]]*${1}" >> "${t140s_solutionFile}"
        ret=$?
    else
        cat "${g20s_actionList}"|grep "^[[:blank:]]*${1}" > "${t140s_solutionFile}"
        ret=$?
    fi
    if [ ${ret} -ne 0 ]
    then
        print_error "1003" "append action to solution file failed.[${t140s_solutionFile}]"
        exit 1
    fi
    return 0
}


function t140s_chooseSolution()
{
    local tmpActionID
    g20s_getActionListBySystem
    rm -f "${t140s_solutionFile}"
    print_debug "create solution file.[type=${t140s_solutionType},file=${t140s_solutionFile}]"
    if [ "${t140s_solutionType}" = "service" ]
    then
        for tmpActionID in "${t140s_serviceActionIDs[@]}"
        do
            t140s_appendActionToSolutionFile "${tmpActionID}"
        done
    else
        for tmpActionID in "${t140s_mkinitrdActionIDs[@]}"
        do
            t140s_appendActionToSolutionFile "${tmpActionID}"
        done
    fi
    return 0
}


if [ "${t140s_solutionFile}" = "" ] || [ "${t140s_solutionType}" = "" ]
then
    print_error "1001" "$0 need 2 parameter."
    t140s_useage
fi
g10s_curDirCheck
t140s_chooseSolution
g10s_saveConf "${g10s_ultraPathPackagesInfo}" "t140s_solutionType" "${t140s_solutionType}"


