#!/bin/bash

g20s_LIBFILE=${g20s_LIBFILE="../globalLib/g20s_install"}
if [ -f "${g20s_LIBFILE}" ]
then
    . "${g20s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
    echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib\033[0m"
    exit 1
fi

t150s_type=""

readonly t150s_allTypes=",SUSE11,ROCKY6,SUSE10,REDHAT5,REDHAT6,REDHAT7,ROCKY4,SUSE12,DEEPIN15,"


export UP_makeUltraPathImage=${UP_makeUltraPathImage:="1"}

function t150s_useage()
{
    local allTypes
    allTypes=${t150s_allTypes//,/|}
    allTypes=${allTypes#|}
    allTypes=${allTypes%|}
    echo "$0 [action=append|remove] [kernel=XXX] [type=XXX]
    action  you want to append or remove image. default value is append.
    kernel  which kernel the image build for. default value is the kernel which you installed UltraPath at.
    type    ${allTypes}"
    exit 1
}

function t150s_chmodExec()
{
    if [ "$1" = "" ]
    then
        print_error "2" "param error."
        exit 1
    fi
    local filePath=""
    filePath=`which $1 2>/dev/null|awk '{print $1}'`
    if [ "${filePath}" != "" ]
    then
        [ -f "${filePath}" ] && chmod u+x "${filePath}"
    fi
    return 0
}



function t150s_updateImageForROCKY6()
{
    local ret
    t150s_chmodExec "mkinitramfs"
    if [ "${g10s_logFile}" != "" ]
    then
        mkinitramfs "${t50s_kernel}" -o "${t150s_image}" >> "${g10s_logFile}" 2>&1
        ret=$?
    else
        mkinitramfs "${t50s_kernel}" -o "${t150s_image}"
        ret=$?
    fi

    if [ ${ret} -ne 0 ]
    then
        print_error "1002" "mkinitramfs failed."
        exit 1
    fi
    return 0
}

function t150s_updateImageForDEEPIN15()
{
    local ret
    t150s_chmodExec "mkinitramfs"
    if [ "${g10s_logFile}" != "" ]
    then
        mkinitramfs "${t50s_kernel}" -o "${t150s_image}" >> "${g10s_logFile}" 2>&1
        ret=$?
    else
        mkinitramfs "${t50s_kernel}" -o "${t150s_image}"
        ret=$?
    fi

    if [ ${ret} -ne 0 ]
    then
        print_error "1002" "mkinitramfs failed."
        exit 1
    fi
    return 0
}

function t150s_getImageNameForROCKY6()
{
    t150s_image="/boot/${g10s_isspImageName}-${t50s_kernel}.img"
    t150s_nxup_link="/boot/nxup-${t50s_kernel}.img"
    return 0
}

function t150s_getImageNameForDEEPIN15()
{
    t150s_image="/boot/${g10s_isspImageName}-${t50s_kernel}.img"
    t150s_nxup_link="/boot/nxup-${t50s_kernel}.img"
    return 0
}

function t150s_updateImageForSUSE12()
{
    t150s_chmodExec "dracut"
	if [ -f "/lib/modules/${t50s_kernel}/kernel/drivers/scsi/nxup.ko" ] && [ -f "/lib/modules/${t50s_kernel}/kernel/drivers/scsi/nxupext_a.ko" ]
	then
		dracut -f --add-drivers "sd_mod sg nxup nxupext_a" "${t150s_image}" "${t50s_kernel}" > /dev/null 2>&1
	else
		dracut -f "${t150s_image}" "${t50s_kernel}" > /dev/null 2>&1
	fi
    if [ $? -ne 0 ]
    then
        print_error "1005" "dracut failed.[${t150s_type}]"
        exit 1
    fi
    return 0
}
function t150s_getImageNameForSUSE12()
{
    t150s_image="/boot/${g10s_isspImageName}-${t50s_kernel}.img"
    t150s_nxup_link="/boot/nxup-${t50s_kernel}.img"
    return 0
}

function t150s_updateImageForSUSE11()
{
    local ret

    ret=1
    t150s_chmodExec "mkinitrd"
    if [ -f "/boot/vmlinuz-${t50s_kernel}" ]
    then
        if [ "${g10s_logFile}" != "" ]
        then
            mkinitrd -i "${t150s_image}" -k "vmlinuz-${t50s_kernel}" >> "${g10s_logFile}" 2>&1
            ret=$?
        else
            mkinitrd -i "${t150s_image}" -k "vmlinuz-${t50s_kernel}"
            ret=$?
        fi
    else
        print_error "1004" "not found vmlinuz for this kernel.[/boot/vmlinuz-${t50s_kernel}]"
        exit 1
    fi

    if [ ${ret} -ne 0 ]
    then
        print_error "1005" "mkinitrd failed.[${t150s_type}]"
        exit 1
    fi
    return 0
}

function t150s_getImageNameForSUSE11()
{
    t150s_image="/boot/${g10s_isspImageName}-${t50s_kernel}.img"
    t150s_nxup_link="/boot/nxup-${t50s_kernel}.img"
    t150s_image_kdump="${t150s_image}-kdump"
    return 0
}

function t150s_updateImageForSUSE10()
{
    local ret

    ret=1
    t150s_chmodExec "mkinitrd"
    if [ -f "/boot/vmlinuz-${t50s_kernel}" ]
    then
        if [ "${g10s_logFile}" != "" ]
        then
            mkinitrd -i "${t150s_image}" -k "vmlinuz-${t50s_kernel}" >> "${g10s_logFile}" 2>&1
            ret=$?
        else
            mkinitrd -i "${t150s_image}" -k "vmlinuz-${t50s_kernel}"
            ret=$?
        fi
    else
        print_error "1007" "not found vmlinuz for this kernel.[/boot/vmlinuz-${t50s_kernel}]"
        exit 1
    fi

    if [ ${ret} -ne 0 ]
    then
        print_error "1008" "mkinitrd failed.[${t150s_type}]"
        exit 1
    fi
    return 0
}

function t150s_getImageNameForSUSE10()
{
    t150s_image="/boot/${g10s_isspImageName}-${t50s_kernel}.img"
    t150s_nxup_link="/boot/nxup-${t50s_kernel}.img"
    return 0
}

function t150s_updateImageForREDHAT5()
{
    local ret

    ret=1
    t150s_chmodExec "mkinitrd"
    

    g10s_getOSType
    if [ `echo "${g10s_OSVersion}" | grep -c "OL5U7"` -ne 0 ] && [ "${g10s_kernelType}" = "uek" ]
    then

        mkinitrd  -f  "${t150s_image}" "${t50s_kernel}" --builtin=ehci-hcd --builtin=ohci-hcd --builtin=uhci-hcd
    elif [ `modprobe -l sd_mod | wc -l` -eq 0 ] && [ -f "/lib/modules/${t50s_kernel}/kernel/drivers/scsi/sd_mod.ko" ]
    then
        mkinitrd -f --preload=sd_mod "${t150s_image}" "${t50s_kernel}"
    else
        mkinitrd  -f "${t150s_image}" "${t50s_kernel}"
    fi

    ret=$?
    if [ ${ret} -ne 0 ]
    then
        print_error "1010" "mkinitrd failed.[${t150s_type}]"
        exit 1
    fi
    return 0
}

function t150s_getImageNameForREDHAT5()
{
    t150s_image="/boot/${g10s_isspImageName}-${t50s_kernel}.img"
    t150s_nxup_link="/boot/nxup-${t50s_kernel}.img"
    g10s_getSysBit
    if [ "${g10s_sysbit}" = "ia64" ]
    then
        t150s_image="/boot/efi/efi/redhat/${g10s_isspImageName}-${t50s_kernel}.img"
        t150s_nxup_link="/boot/efi/efi/redhat/nxup-${t50s_kernel}.img"
    fi
    return 0
}

function t150s_updateImageForREDHAT6()
{

    t150s_chmodExec "dracut"
    dracut -f "${t150s_image}" "${t50s_kernel}"
    if [ $? -ne 0 ]
    then
        print_error "1012" "mkinitrd failed.[${t150s_type}]"
        exit 1
    fi
    return 0
}

function t150s_getImageNameForREDHAT6()
{
    t150s_image="/boot/${g10s_isspImageName}-${t50s_kernel}.img"
    t150s_nxup_link="/boot/nxup-${t50s_kernel}.img"
    return 0
}

function t150s_updateImageForREDHAT7()
{
    t150s_chmodExec "dracut"
    
    
    if [ -f "/lib/modules/${t50s_kernel}/kernel/drivers/scsi/nxup.ko" ] && [ -f "/lib/modules/${t50s_kernel}/kernel/drivers/scsi/nxupext_a.ko" ]
    then
    	dracut -f --add-drivers "sd_mod sg nxup nxupext_a" "${t150s_image}" "${t50s_kernel}"
    else
    	dracut -f "${t150s_image}" "${t50s_kernel}"
    fi
    
    if [ $? -ne 0 ]
    then
        print_error "1012" "mkinitrd failed.[${t150s_type}]"
        exit 1
    fi
    return 0
}

function t150s_getImageNameForREDHAT7()
{
    t150s_image="/boot/${g10s_isspImageName}-${t50s_kernel}.img"
    t150s_nxup_link="/boot/nxup-${t50s_kernel}.img"
    
    return 0
}

function t150s_getImageNameForROCKY4()
{
    t150s_image="/boot/${g10s_isspImageName}-${t50s_kernel}.img"
    t150s_nxup_link="/boot/nxup-${t50s_kernel}.img"
    return 0
}

function t150s_updateImageForROCKY4()
{
    local ret=""

    g10s_checkExist -f "t360s_mkinitrdRocky4.sh" 2
    bash t360s_mkinitrdRocky4.sh ${t150s_image}
    ret=$?

    return ${ret}
}


function t150s_getImageDirAndName()
{

   if [ "${t150s_image}" != "" ]
   then
       return 0
   fi
   g10s_eval "t150s_getImageNameFor${t150s_type}"
   return 0
}

function t150s_selectType()
{
    local action40
    local action30
    print_debug "choose type..."
    g20s_getActionListBySystem
    action40=`cat "${g20s_actionList}"|grep "^[[:blank:]]*40_mkImage=="`
    if [ "${action40}" = "" ]
    then
        print_event "1013" "this enviroment is not support update image.[${t50s_kernel},${g10s_confValue}]"
        exit 0
    fi
    action30="`cat "${g20s_actionList}"|grep "^[[:blank:]]*30_modifyMkinitrd=="`"
    if [ "${action30}" != "" ]
    then
        print_debug "append action"
        g20s_appendUninstallByLine "${action30}"
    fi
    g20s_appendUninstallByLine "${action40}"
    g20s_excuteInstallAction "${action40}"
    return 0
}

function t150s_updateImage()
{
    if [ "${t150s_type}" = "" ]
    then
        t150s_selectType
    else
        if [ `echo "${t150s_allTypes}"|grep -c ",${t150s_type},"` -ne 1 ]
        then
            print_error "1018" "unsupport type[type=${t150s_type}]"
            exit 1
        fi
        t150s_getImageDirAndName

        if [ "${UP_makeUltraPathImage}" = "1" ]
        then
            print_debug "begin depmod nxup for kernel ${t50s_kernel}"
            g10s_checkExist -f "/lib/modules/${t50s_kernel}/kernel/drivers/scsi/nxup.ko" 2
            g10s_checkExist -f "/lib/modules/${t50s_kernel}/kernel/drivers/scsi/nxupext_a.ko" 2
            depmod "${t50s_kernel}"
            if [ $? -ne 0 ]
            then
                print_error "1017" "depmod failed"
                exit 1
            fi
        fi

        print_debug "update image[kernel=${t50s_kernel},image=${t150s_image}]"
        g10s_eval "t150s_updateImageFor${t150s_type}"
        print_event "1020" "The mirror is updated successfully."
        g10s_chmodExec 500 "${t150s_image}"
	if [ "${UP_makeUltraPathImage}" = "1" ] && [ -f "${t150s_image}" ] && [ ! -f "${t150s_nxup_link}" ]
        then
            ln -s "${t150s_image}" "${t150s_nxup_link}"
        fi
    fi
    return 0
}

function t150s_updatehostinfo()
{

    g10s_checkExist -f "t130s_getOSInfo.sh" 2
    bash t130s_getOSInfo.sh "/etc/up.hostinfo" >/dev/null
    if [ $? -ne 0 ]
    then
        print_warning "1006" "update hostinfo failed."
        return 1
    fi
    return 0
}


while [ $# -gt 0 ]
do
    t150s_param="$1"
    t150s_param1=${t150s_param%%=*}
    t150s_param2=${t150s_param##*=}
    print_debug "t150s_param=${t150s_param},t150s_param1=${t150s_param1},t150s_param2=${t150s_param2}"
    case "${t150s_param1}" in
        "action")
            t150s_action="${t150s_param2}"
            ;;
        "kernel")
            t50s_kernel="${t150s_param2}"
            ;;
        "type")
            t150s_type="${t150s_param2}"
            ;;
        *)
            print_error "1021" "unknown parameter[${t150s_param}]"
            exit 1
            ;;
    esac
    shift
done

if [ "${t50s_kernel}" = "" ]
then
    t50s_kernel="`g10s_curUnamer`"
else
    print_event "1023" "Update the kernel mirror."
fi
export t50s_kernel


if [ "${t150s_action}" = "remove" ]
then
    if [ `echo "${t150s_allTypes}"|grep -c ",${t150s_type},"` -ne 1 ]
    then
        print_error "1024" "unsupport type[type=${t150s_type}]"
        exit 1
    fi
    t150s_getImageDirAndName
    g10s_getOSKernelsOnline
    OLD_IFS="${IFS}"
    IFS="|"
    g10s_allKernels=(${g10s_allKernels})
    IFS="${OLD_IFS}"
    [ -f /etc/modprobe.d/nxupmodules.conf ] && rm -f /etc/modprobe.d/nxupmodules.conf
    for tmpKernel in "${g10s_allKernels[@]}"
    do
        if [ "${tmpKernel}" != "" ]
        then
            if [ -f "/lib/modules/${tmpKernel}/kernel/drivers/scsi/nxup.ko" ]
            then
                print_debug "rm -f /lib/modules/${tmpKernel}/kernel/drivers/scsi/nxup.ko"
                rm -f "/lib/modules/${tmpKernel}/kernel/drivers/scsi/nxup.ko"
                depmod "${tmpKernel}"
            fi
            if [ -f "/lib/modules/${tmpKernel}/kernel/drivers/scsi/nxupext_a.ko" ]
            then
                print_debug "rm -f /lib/modules/${tmpKernel}/kernel/drivers/scsi/nxupext_a.ko"
                rm -f "/lib/modules/${tmpKernel}/kernel/drivers/scsi/nxupext_a.ko"
                depmod "${tmpKernel}"
            fi
        fi
    done

    if [ -f ${g10s_isspinitrdrecordfile} ]
    then
        sed -i "/^${g10s_isspinitrdrecordname}$/d" "${g10s_isspinitrdrecordfile}"
    fi

    if [ -f "${t150s_nxup_link}" ]
    then
        rm -f "${t150s_nxup_link}"
        print_debug "remove nxup link successfully"
    else
        print_debug "the link image is not exist.[${t150s_nxup_link}]"
    fi

    g10s_getOtherIsspInstalled
    if [ "${g10s_OtherIsspInstalled}" = "" ]
    then
        if [ -f ${g10s_isspinitrdrecordfile} ]
        then
            rm -f ${g10s_isspinitrdrecordfile}
        fi
        print_debug "begin removing image[${t150s_image}]"
        if [ -f "${t150s_image}" ]
        then
            cp -f "${t150s_image}" "${g10s_bakPath}/t150s_issp.img"
            rm -f "${t150s_image}"
            print_debug "remove image successfully"
        else
            print_debug "the image is not exist.[${t150s_image}]"
        fi

        if [ "${t150s_image_kdump}" != "" ] && [ -f "${t150s_image_kdump}" ]
        then
            cp -f "${t150s_image_kdump}" "${g10s_bakPath}/t150s_issp.img-kdump"
            rm -f "${t150s_image_kdump}"
            print_debug "remove image-kdump [${t150s_image_kdump}] successfully"
        fi

    else
        print_debug "begin update image[${t150s_image}]"
        cp -f "${t150s_image}" "${g10s_bakPath}/t150s_issp.img"
        export UP_makeUltraPathImage="0"
        t150s_updateImage
    fi
else
    t150s_updatehostinfo
    t150s_updateImage
fi



