#!/bin/bash

g10s_LIBFILE=${g10s_LIBFILE="../globalLib/g10s_pub"}
if [ -f "${g10s_LIBFILE}" ]
then
	. "${g10s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g10s_LIBFILE}]\033[0m"
	exit 1
fi

t170s_servPathName=""
t170s_type="normal"
t170s_action="insert"


t170s_targetServ="/etc/init.d/boot.010LoadNxup"
t170s_targetName="boot.010LoadNxup"
t170s_udev="/etc/init.d/boot.udev"


readonly t170s_tmpFile="${g10s_tmpPath}/t170s_010LoadNxup.tmp"

readonly t170s_allTypes=",normal,rocky6,rocky4,"

function t170s_useage()
{
    local allTypes
    allTypes=${t170s_allTypes//,/|}
    allTypes=${allTypes#|}
    allTypes=${allTypes%|}
    echo "usage:$0 servPathName=XXX type=XXX action=[insert|remove]
    servPathName=the service path name which want to insert.
    action=[insert|remove] insert or remove the service
    type=${allTypes}"
    exit 1
}


function t170s_paramCheck()
{
    if [ "${t170s_action}" = "insert" ]
    then
        if  [ "${t170s_servPathName}" = "" ]
        then
            print_error "1001" "please set servPathName"
            t170s_useage
        fi
        g10s_checkExist "-f" "${t170s_servPathName}" 2
    fi


    if [ "${t170s_action}" != "insert" ] && [ "${t170s_action}" != "remove" ] && [ "${t170s_action}" != "repair" ]
    then
        print_error "1002" "unknown action[action=${t170s_action}]"
        t170s_useage
    fi

    if [ `echo "${t170s_allTypes}"|grep -c ",${t170s_type},"` -eq 0 ]
    then
        print_error "1003" "unknown type[type=${t170s_type}]"
        t170s_useage
    fi
    return 0
}


function t170s_normal_insert()
{

    g10s_cp "${t170s_servPathName}" "${t170s_tmpFile}"
    install -o root -g root -m 0500 -D "${t170s_tmpFile}" "${t170s_targetServ}"
    if [ $? -ne 0 ]
    then
        if [ "$1" = "" ]
        then
            print_error "1004" "install service failed.[${t170s_servPathName}  to  ${t170s_targetServ}]"
        else
            print_error "1004" "repair service failed.[${t170s_servPathName}  to  ${t170s_targetServ}]"
        fi
        exit 1
    fi
    rm -f "${t170s_tmpFile}"


    if [ -f "${t170s_udev}" ] && [ ! -L "${t170s_udev}" ]
    then
        print_debug "append Required-Start to udev.[${t170s_targetName},"${t170s_udev}"]"
        sed -i "s/^[[:blank:]]*#[[:blank:]]*Required-Start[[:blank:]]*:\(.*\)$/# Required-Start: ${t170s_targetName} \1/" "${t170s_udev}"
        if [ $? -ne 0 ]
        then
            if [ "$1" = "" ]
            then
                print_debug "append Required-Start failed.[${t170s_udev}]"
            else
                print_debug "repair Required-Start failed.[${t170s_udev}]"
            fi
            exit 1
        fi
    else
        if [ "$1" = "" ]
        then
            print_error "1005" "the udev is not exist or is a link file in installing.[${t170s_udev}]"
        else
            print_error "1005" "the udev is not exist or is a link file in repairing.[${t170s_udev}]"
        fi
        exit 1
    fi

    g10s_checkExist -f "t330s_serviceMgr.sh" 2
    bash t330s_serviceMgr.sh action=add "serviceFile=${t170s_targetName}"
    if [ $? -ne 0 ]
    then
        if [ "$1" = "" ]
        then
            print_error "1006" "install run-service failed.[${t170s_targetServ}]"
        else    
            print_error "1006" "repair run-service failed.[${t170s_targetServ}]"
        fi
        exit 1
    fi
    return 0
}

function t170s_normal_repair()
{
    if [ ! -f "${t170s_targetServ}" ] 
    then
        print_debug "The boot servive need to repair. not exist [${t170s_targetServ}]"
        t170s_normal_insert "repair"
    fi
    
    if [ -f "${t170s_udev}" ] && [ ! -L "${t170s_udev}" ]
    then
        print_debug "The boot servive need to repair. not exist or link file [${t170s_udev}]"
        ret=0
        if [ `grep -c "^[[:blank:]]*#[[:blank:]]*Required-Start[[:blank:]]*:.* ${t170s_targetName} .*" "${t170s_udev}"` -ne 0 ]
        then
            print_debug "repair for clear ${t170s_targetName} from udev by type1"
            sed -i "/^[[:blank:]]*#[[:blank:]]*Required-Start[[:blank:]]*:/s/ ${t170s_targetName} //" "${t170s_udev}"
            ret=$?
        else
            print_debug "repair for clear ${t170s_targetName} from udev by type2"
            sed -i "/^[[:blank:]]*#[[:blank:]]*Required-Start[[:blank:]]*:/s/${t170s_targetName}//" "${t170s_udev}"
            ret=$?
        fi
        if [ ${ret} -ne 0 ]
        then
            print_error "1007" "repair for clear Required-Start failed.[${t170s_udev}]"
            exit 1
        fi
        t170s_normal_insert "repair"
    fi
    
    return 0
}

function t170s_normal_remove()
{
    local hasError="0"

    if [ -f "${t170s_udev}" ] && [ ! -L "${t170s_udev}" ]
    then
        ret=0
        if [ `grep -c "^[[:blank:]]*#[[:blank:]]*Required-Start[[:blank:]]*:.* ${t170s_targetName} .*" "${t170s_udev}"` -ne 0 ]
        then
            print_debug "delete ${t170s_targetName} from udev by type1"
            sed -i "/^[[:blank:]]*#[[:blank:]]*Required-Start[[:blank:]]*:/s/ ${t170s_targetName} //" "${t170s_udev}"
            ret=$?
        else
            print_debug "delete ${t170s_targetName} from udev by type2"
            sed -i "/^[[:blank:]]*#[[:blank:]]*Required-Start[[:blank:]]*:/s/${t170s_targetName}//" "${t170s_udev}"
            ret=$?
        fi
        if [ ${ret} -ne 0 ]
        then
            print_error "1007" "remove Required-Start failed.[${t170s_udev}]"
            hasError="1"
        fi
    fi

    g10s_checkExist -f "t330s_serviceMgr.sh" 2
    bash t330s_serviceMgr.sh action=del "serviceFile=${t170s_targetName}"
    if [ $? -ne 0 ]
    then
        print_error "1008" "remove service failed.[${t170s_targetServ}]"
        hasError="1"
    else
        rm -f "${t170s_targetServ}"
    fi

    if [ "${hasError}" = "1" ]
    then
        exit 1
    fi
    return 0
}

function t170s_rocky6_insert()
{

    g10s_cp "${t170s_servPathName}" "${t170s_tmpFile}"
    install -o root -g root -m 0500 -D "${t170s_tmpFile}" "${t170s_targetServ}"
    if [ $? -ne 0 ]
    then
        print_error "1004" "install service failed.[${t170s_servPathName}  to  ${t170s_targetServ}]"
        exit 1
    fi
    rm -f "${t170s_tmpFile}"


    print_debug "append Required-Start to boot.010LoadNxup"
    sed -i "s/^[[:blank:]]*#[[:blank:]]*Required-Start[[:blank:]]*:\(.*\)$/# Required-Start: mountdevsubfs \1/" "${t170s_targetServ}"
    if [ $? -ne 0 ]
    then
        print_debug "append Required-Start failed."
        exit 1
    fi
	print_debug "Change Default-Start to S"
    sed -i "s/^[[:blank:]]*#[[:blank:]]*Default-Start[[:blank:]]*:[[:blank:]]*B\(.*\)$/# Default-Start: S \1/" "${t170s_targetServ}"
    if [ $? -ne 0 ]
    then
        print_debug "Change Default-Start to S failed."
        exit 1
    fi

    g10s_checkExist -f "t330s_serviceMgr.sh" 2
    bash t330s_serviceMgr.sh action=add "serviceFile=${t170s_targetName}"
    if [ $? -ne 0 ]
    then
        print_error "1006" "install run-service failed.[${t170s_targetServ}]"
        exit 1
    fi
    return 0
}

function t170s_rocky6_remove()
{
    g10s_checkExist -f "t330s_serviceMgr.sh" 2
    bash t330s_serviceMgr.sh action=del "serviceFile=${t170s_targetName}"
    if [ $? -ne 0 ]
    then
        print_error "1008" "remove service failed.[${t170s_targetServ}]"
        hasError="1"
    else
        rm -f "${t170s_targetServ}"
    fi

    if [ "${hasError}" = "1" ]
    then
        exit 1
    fi
    return 0
}

function t170s_rocky4_insert()
{

    g10s_cp "${t170s_servPathName}" "${t170s_tmpFile}"
    install -o root -g root -m 0500 -D "${t170s_tmpFile}" "${t170s_targetServ}"
    if [ $? -ne 0 ]
    then
        print_error "1004" "install service failed.[${t170s_servPathName}  to  ${t170s_targetServ}]"
        exit 1
    fi
    rm -f "${t170s_tmpFile}"


	if [ -d /etc/rc.d/rcsysinit.d ]
    then
        [ -f /etc/rc.d/rcsysinit.d/S040${t170s_targetName} ] && rm -rf /etc/rc.d/rcsysinit.d/S040${t170s_targetName}
        ln -s  ${t170s_targetServ} /etc/rc.d/rcsysinit.d/S040${t170s_targetName}
        ret=$?
    fi

    return 0
}

function t170s_rocky4_remove()
{
    local ret
    if [ -f /etc/rc.d/rcsysinit.d/S040${t170s_targetName} ]
    then
        rm -rf /etc/rc.d/rcsysinit.d/S040${t170s_targetName}
        ret=$?
    fi

    if [ -f ${t170s_targetServ} ]
    then
        rm -rf ${t170s_targetServ}
        ret=$?
    fi
    return ${ret}
}

function t170s_main()
{
    t170s_paramCheck
    g10s_eval "t170s_${t170s_type}_${t170s_action}"
    if [ "${t170s_action}" = "insert" ]
    then
        print_event "1009" "The boot service is installed successfully."
    elif [ "${t170s_action}" = "remove" ]
    then
        print_event "1010" "The boot service is removed successfully."
    else
        print_debug "The boot servive is repaied successfully."
    fi
    return 0
}


g10s_rootCheck
while [ $# -gt 0 ]
do
    t170s_param="$1"
    t170s_param1=${t170s_param%%=*}
    t170s_param2=${t170s_param##*=}
    print_debug "t170s_param=${t170s_param},t170s_param1=${t170s_param1},t170s_param2=${t170s_param2}"
    case "${t170s_param1}" in
        "servPathName")
            t170s_servPathName="${t170s_param2}"
            t170s_servPathName=$(echo ${t170s_servPathName})
            ;;
        "type")
            t170s_type="${t170s_param2}"
            ;;
        "action")
            t170s_action="${t170s_param2}"
            ;;
        *)
            print_error "1011" "unknown parameter[${t170s_param}]"
            t170s_useage
            ;;
    esac
    shift
done

t170s_main
