#!/bin/bash

t190s_toolsDir="../toolLib"
g20s_LIBFILE=${g20s_LIBFILE="../globalLib/g20s_install"}
g30s_LIBFILE=${g30s_LIBFILE="../globalLib/g30s_systemEnvironment"}
t190s_timeoutstr="scsi_timeout"
t190s_timeoutinputstr="scsi_timeout=|linkdown_tmo="
t190s_ibconfigstr="ib"
t190s_ibshowconfigstr="ib_config"
t190s_linkdownTmoShowStr="linkdown_tmo"
t190s_updatekernel="updatekernel"
t190s_udevRuleName="99-ultrapath.rules"
if [ -f "${g20s_LIBFILE}" ] && [ -f "${g30s_LIBFILE}" ]
then
	. "${g20s_LIBFILE}"

	. "${g30s_LIBFILE}"

elif [ -f "/UltraPath/install/linuxTools/globalLib/g20s_install" ] && [ -f "/UltraPath/install/linuxTools/globalLib/g30s_systemEnvironment" ]
then
    pwdPath=`pwd`
    t190s_toolsDir="/UltraPath/install/linuxTools/toolLib"
    cd "${t190s_toolsDir}"

    . ../globalLib/g20s_install

    . ../globalLib/g30s_systemEnvironment

    cd ${pwdPath}
    unset pwdPath
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib file.\033[0m"
	exit 1
fi

t190s_action=""

t190s_OPER_TYPE=1396789576
readonly t190s_allActions=",updateImage,set,show,"
t190s_lockFile="/var/lock/t190s_shell.lock"
g10s_exitFun="t190s_exit"

function t190s_exit()
{
    g10s_unlockShell "${t190s_lockFile}"
    
    if [ 0 -eq $1 ]
    then
    	print_oper "[execute upLinux ${t190s_action}]" "[1]" "$g10s_LEVEL_INFO" "[execute update initrd information] [ִи¾]" "[$t190s_OPER_TYPE]"
    else
    	print_oper "[execute upLinux ${t190s_action}]" "[0]" "$g10s_LEVEL_ERROR" "[execute update initrd information] [ִи¾]" "[$t190s_OPER_TYPE]"
    fi
    
    exit "$1"
}

function t190s_unlock_exit()
{
    g10s_unlockShell "${t190s_lockFile}"
    exit "$1"
}

function t190s_scsi_oper_exit() 
{ 
    g10s_unlockShell "${t190s_lockFile}" 
     
    if [ 0 -eq $1 ] 
    then 
        print_oper "[execute upLinux ${t190s_action} scsi_timeout ${2}]" "[1]" "$g10s_LEVEL_INFO" "[execute set_scsi_timeout] [ִ޸scsiʱʱ]" "[$t190s_OPER_TYPE]" 
    else 
        print_oper "[execute upLinux ${t190s_action} scsi_timeout ${2}]" "[0]" "$g10s_LEVEL_ERROR" "[execute set_scsi_timeout] [ִ޸scsiʱʱ]" "[$t190s_OPER_TYPE]" 
    fi 
   
    exit "$1" 
} 

function t190s_linkdown_oper_exit() 
{ 
    g10s_unlockShell "${t190s_lockFile}" 
     
    if [ "${t190s_action}" = "set" ];then
        printalph=""
    elif [ "${t190s_action}" = "unset" ]; then
        printalph=""
    fi
     
    if [ 0 -eq $1 ] 
    then 
        print_oper "[execute upLinux ${t190s_action} linkdown_tmo ${2}]" "[1]" "$g10s_LEVEL_INFO" "[execute ${t190s_action} linkdown_tmo] [${printalph}linkdown_tmo]" "[$t190s_OPER_TYPE]" 
    else 
        print_oper "[execute upLinux ${t190s_action} linkdown_tmo ${2}]" "[0]" "$g10s_LEVEL_ERROR" "[execute ${t190s_action} linkdown_tmo] [${printalph}linkdown_tmo]" "[$t190s_OPER_TYPE]" 
    fi 
   
    exit "$1" 
} 

function t190s_updatekernel_oper_exit()
{
    g10s_unlockShell "${t190s_lockFile}" 
     
    if [ 0 -eq $1 ] 
    then 
        print_oper "[execute upLinux ${t190s_action}]" "[1]" "$g10s_LEVEL_INFO" "[execute update new kernel] [ִиں˲װ·]" "[$t190s_OPER_TYPE]" 
    else 
        print_oper "[execute upLinux ${t190s_action}]" "[0]" "$g10s_LEVEL_ERROR" "[execute update new kernel] [ִиں˲װ·]" "[$t190s_OPER_TYPE]" 
    fi 
   
    exit "$1" 
}

function t190s_config_ibsrp_exit() 
{ 
    g10s_unlockShell "${t190s_lockFile}" 
    IBcall=$1
    if [ ${t190s_type} = ${t190s_ibshowconfigstr} ]
    then
        IBaction=${t190s_ibshowconfigstr}
    else
        IBaction=${t190s_ibconfigstr}
    fi
    
    if [ ${t190s_action} == "show" ]; then
        printalph="ʾ"
    elif [ ${t190s_action} == "config" ]; then
        printalph=""
    elif [ ${t190s_action} == "unconfig" ]; then
        printalph=""
    else
        exit "$1"
    fi
    
    if [ 0 -eq ${IBcall} ] 
    then 
        print_oper "[execute upLinux ${t190s_action} ${IBaction}]" "[1]" "$g10s_LEVEL_INFO" "[execute ${t190s_action}_${IBaction}] [${printalph}IB]" "[$t190s_OPER_TYPE]" 
    else 
        print_oper "[execute upLinux ${t190s_action} ${IBaction}]" "[0]" "$g10s_LEVEL_ERROR" "[execute ${t190s_action}_${IBaction}] [${printalph}IB]" "[$t190s_OPER_TYPE]" 
    fi 
    exit "$1" 
}

function t190s_useage()
{
    cat<<EOF
    Parameters:
    1. updateImage: Update initrd information about the UltraPath.
    2. updateForNewKernel: UltraPath is updated to match the new kernel.
    3. set scsi_timeout=<num>: Modify the timeout parameter of a SCSI device
       that is taken over by UltraPath.
    4. show scsi_timeout: Query the timeout parameter of a SCSI device that 
       is taken over by UltraPath.
    5. show ib_config: Query status of ib configuration.
    6. config ib: Setup the ib configuration.
    7. unconfig ib: Clear up the ib configuration.
    8. set linkdown_tmo=<num>: Modify the timeout parameter of HBA linkdown tmo.
    9. unset linkdown_tmo: Unset the timeout parameter of HBA linkdown tmo.
    10. show linkdown_tmo: Show the timeout parameter of HBA linkdown tmo.
EOF
    exit 1
}

function t190s_set_useage()
{
    cat<<EOF
    Parameters:
    1. scsi_timeout=<num>: Modify the timeout parameter of a SCSI device
       that is taken over by UltraPath.
    2. linkdown_tmo=<num>: Modify the timeout parameter of HBA linkdown tmo.
EOF
    exit 1
}

function t190s_unset_useage()
{
    cat<<EOF
    Parameters:
    1. linkdown_tmo: Unset the timeout parameter of HBA linkdown tmo.
EOF
    exit 1
}

function t190s_updateForNewKernel()
{
    g10s_getOSType
    if [ `echo "${g10s_OSVersion}"|grep -c -E "(RHEL7|RHEL6|RHEL5|OL7|OL6|OL5|SLES10|SLES11|SLES12)"` -eq 0 ]
    then
        print_error "1040" "The system currently does not support this command."
        t190s_updatekernel_oper_exit 1
    fi    
    cd "${t190s_toolsDir}"
    g10s_getConf "${g10s_ultraPathPackagesInfo}" "t140s_solutionType"
    if [ "${g10s_confValue}" = "mkinitrd" ]
    then
        bash t400s_updatekernel.sh 
        if [ $? -eq 0 ]
        then
            print_event "1030" "The system kernel is updated successfully in UltraPath. Please reboot."
            t190s_updatekernel_oper_exit 0
        else
            print_event "1005" "The system kernel fails to be updated."
            t190s_updatekernel_oper_exit 1
        fi
    else
        print_error "1040" "This command is not needed when the ultrapath software is installed using the boot-from-local type."
        t190s_updatekernel_oper_exit 1
    fi
}

function t190s_updateImage()
{
    local imageName=""

    cd "${t190s_toolsDir}"
    g10s_getConf "${g10s_ultraPathPackagesInfo}" "t140s_solutionType"
    if [ "${g10s_confValue}" != "mkinitrd" ]
    then

        g10s_checkExist -f "t340s_getsysimage.sh" 2
        g10s_checkExist -f "t150s_updateImage.sh" 2


        g30s_getBootLoaderType

    g10s_getOSType
    if [ `echo "${g10s_OSVersion}" | grep -c "SLES12"` -ne 0 ]
    then
  	    . t340s_getsysimage.sh	
	    bash t340s_getsysimage.sh
	    imageName=`echo ${t340s_defaultSysImage}`
	else
        imageName=`bash t340s_getsysimage.sh | grep "^[[:blank:]]*t340s_defaultSysImage[[:blank:]]*=[[:blank:]]*" | awk -F "=" '{print $2}'`
        fi
        if [ -z "${imageName}" ]
        then
            print_error "1000" "get system image failed."
            t190s_exit 1
        fi
        
        print_yesOrNo "1024" "You will update the system mirrors. Do you want to continue?"
        if [ $? -ne 0 ]
        then
            return 0
        fi


        export UP_makeUltraPathImage="0"
        export t150s_image="${imageName}"
        bash t150s_updateImage.sh action=append
        export t150s_image=""
        export UP_makeUltraPathImage="1"
        t190s_exit $?
    else
        g10s_checkExist -f "t150s_updateImage.sh" 2
        bash t150s_updateImage.sh action=append
        t190s_exit $?
    fi
    return 0
}

function t190s_status()
{
    cd "${t190s_toolsDir}"
    g10s_checkExist -f "t220s_statusCheck.sh" 2
    bash t220s_statusCheck.sh
    t190s_unlock_exit $?
}

function t190s_set_linkdowntmo_usage()
{
    echo "Usage:"
    echo "upLinux set linkdown_tmo=<num>"
    echo "   --num is in the range 1-60"
}
function t190s_settimeoutusage()
{
    echo "Usage:"
    echo "upLinux set scsi_timeout=<num>"
    echo "   --num is in the range 1-600"
}
function t190s_clearibconfigusage()
{
    echo "Usage:"
    echo "upLinux unconfig ib"
    t190s_unlock_exit 1
}
function t190s_configibusage()
{
    echo "Usage:"
    echo "upLinux config ib"
    t190s_unlock_exit 1
}
function t190s_showinfousage()
{
    echo "Usage:"
    echo "upLinux show scsi_timeout"
    echo "upLinux show ib_config"
    echo "upLinux show linkdown_tmo"
    t190s_unlock_exit 1
}

function t190s_checkscsiTimeoutWithRules()
{
    local timeoutValue=${1}
    local flag=0
    for LunId in `ls /sys/bus/scsi/drivers/NXUP/|grep -E "[0-9]+:"`
    do
       if [ -f /sys/class/scsi_device/${LunId}/device/timeout ]
       then
           tmpTimeoutVlue=`cat /sys/class/scsi_device/${LunId}/device/timeout`
           if [ ${timeoutValue} -ne  ${tmpTimeoutVlue} ]
           then
                flag=1
                print_event "1040" "The timeout parameter's value of the SCSI device ${LunId} is ${tmpTimeoutVlue} that is different from ${t190s_udevRuleName}." 
           fi
       fi
    done
    return ${flag} 
    
}
function t190s_showscsiTimeout() 
{
   g10s_getOSType
   local timeoutValue
   local checkstr="/sys/class/scsi_device/%k/device/timeout"
   if  [ "${g10s_OSVersion}" = "RHEL4" ] ||  [ `echo "${g10s_OSVersion}" | grep -c "SLES9"` -eq 1 ]
   then    
       print_event "1040" "This system does not support this operation."
       t190s_unlock_exit 1
   fi
    if [ ! -f  /etc/udev/rules.d/99-ultrapath.rules ]
    then
         print_error  "1040" "The ${t190s_udevRuleName} does not exist."
         t190s_unlock_exit 1
    else
        timeoutValue=`cat  /etc/udev/rules.d/99-ultrapath.rules | grep ${checkstr} |awk -F "echo " '{print $2}' |awk -F " >" '{print $1}'`
        print_event "1040" "scsi timeout:${timeoutValue}" 
    fi
    t190s_checkscsiTimeoutWithRules ${timeoutValue}
    t190s_unlock_exit $?
}

function t190s_checkParameter()
{
    if [ ! -z $3 ]
    then
       if [ $2 -lt $3 ]
       then
            print_event "1040" "Invalid input!"
            t190s_settimeoutusage
            t190s_scsi_oper_exit 1 ${2}
       fi
    fi

    if [ ! -z $4 ]
    then
       if [ $2 -gt $4 ]
       then
            print_event "1040" "Invalid input!"
            t190s_settimeoutusage
            t190s_scsi_oper_exit 1 ${2}
       fi
    fi
}

function t190s_modifytimeoutValue()
{
   t190_timeout=${1}
   t190_path=${2}
   sed -i -r 's/(echo )[^<]*( > \/sys\/class\/scsi_device\/%k\/device\/timeout)/\1'"${t190_timeout}"'\2/' /etc/udev/rules.d/99-ultrapath.rules > /dev/null
}


function t190s_modifyNXUPTimeoutValue()
{
   local t190_timeout=${1}
   for LunId in `ls /sys/bus/scsi/drivers/NXUP/|grep -E "[0-9]+:"`
   do
    if [ -f /sys/class/scsi_device/${LunId}/device/timeout ]
    then
        echo ${t190_timeout} > /sys/class/scsi_device/${LunId}/device/timeout
    fi
   done
}

function t190s_modifyudevTimeoutRule()
{
   local t190_timeout=${1}
   local t190s_rule_path="/etc/udev/rules.d/99-ultrapath.rules"
   g10s_getOSType
  
   if  [ "${g10s_OSVersion}" = "RHEL4" ] ||  [ `echo "${g10s_OSVersion}" | grep -c "SLES9"` -eq 1 ]
   then    
       print_event "1040" "This system does not support this operation."
       t190s_scsi_oper_exit 1 ${t190_timeout}
   fi
   
   if [ ! -f  ${t190s_rule_path} ]
   then
        print_error  "1040" "The ${t190s_udevRuleName} does not exist."
        t190s_scsi_oper_exit 1 ${t190_timeout}
   fi 
   t190s_modifytimeoutValue  ${t190_timeout} ${t190s_rule_path}
   t190s_modifyNXUPTimeoutValue ${t190_timeout}
}

function t190s_checkudevTimeout()
{  
    local t190_timeout=${1}
    local t190_sucessflag=1
    local t190s_rule_path="/etc/udev/rules.d/99-ultrapath.rules"
    local checkstr="/sys/class/scsi_device/%k/device/timeout"
    if [ ! -f  ${t190s_rule_path} ]
    then
        print_error  "1040" "The ${t190s_udevRuleName} does not exist."
        t190s_scsi_oper_exit 1 ${t190_timeout}
   fi 
    num=`cat ${t190s_rule_path} |grep ${checkstr} |awk -F "echo " '{print $2}' |awk -F " >" '{print $1}'`
    if [ "${num}" !=  "${t190_timeout}" ]
    then    
        print_error "1040" "Failed to modify the ${t190s_udevRuleName}."
        t190s_scsi_oper_exit 1 ${t190_timeout}
    fi
   print_event "1040" "Succeeded in changing the value of the timeout parameter in the ${t190s_udevRuleName} to ${t190_timeout}."
    for LunId in `ls /sys/bus/scsi/drivers/NXUP/|grep -E "[0-9]+:"`
    do
        if [ -f /sys/class/scsi_device/${LunId}/device/timeout ]
        then
            num=`cat /sys/class/scsi_device/${LunId}/device/timeout`
            if [ "${num}" !=  "${t190_timeout}" ] 
            then
                print_error  "1040" "Failed to modify the timeout parameter of the SCSI device ${LunId}."
                t190_sucessflag=0
            fi
        fi
    done
    if [ ${t190_sucessflag} -eq 1 ]
    then
        print_event "1040" "Succeeded in modifying the timeout parameter of the SCSI devices managed by UltraPath."  
        return 0
    else
        return 1
    fi
}

function t190s_checkPramterLen() 
{
   local len
   local t190_timeout=${1}
   if [ `echo "$t190_timeout"|grep -E -c  "^[0-9]+$"` -ne 1 ]
    then 
        print_event "1040" "Invalid input!"
        t190s_settimeoutusage
        t190s_scsi_oper_exit 1 ${t190_timeout}
   fi
   len=`echo "$t190_timeout"|awk '{print length($0)}'`
   if [ ${len} -lt 1  ]
   then 
       print_event "1040" "Invalid input!"
       t190s_settimeoutusage
       t190s_scsi_oper_exit 1 ${t190_timeout}
   elif [ ${len} -gt 6 ]
   then
       print_event "1040" "Invalid input!"
       t190s_settimeoutusage
       t190s_scsi_oper_exit 1 ${t190_timeout}
   fi
}

function t190s_create_udevrule_for_linkdowntmo()
{
    local udev_rule_string="KERNEL==\"*\", SUBSYSTEM==\"fc_transport\", ACTION==\"add\", RUN+=\"/bin/sh /UltraPath/install/linuxTools/toolLib/t370s_udevnxup.sh %p $1\""
    
    sed -i -e '/fc_transport/d' /etc/udev/rules.d/99-ultrapath.rules
    echo ${udev_rule_string} >> /etc/udev/rules.d/99-ultrapath.rules
}

function t190s_check_linkdown_tmo()
{
    local tmo=$1
    local curTmo=""
    local flag=0
    local rportAddr="/sys/class/fc_remote_ports"
    local port_name=""
    
    upadmin show path | awk '{print $5}' > /tmp/showpath.up
    
    for rport in `ls ${rportAddr}`
    do
        if [ -f ${rportAddr}/${rport}/dev_loss_tmo ]
        then
            curTmo=`cat ${rportAddr}/${rport}/dev_loss_tmo`
            if [ ${curTmo} -ne ${tmo} ]
            then
                port_name=`cat ${rportAddr}/${rport}/port_name`
                port_name=${port_name#0x}
                if [ `cat /tmp/showpath.up | grep -c ${port_name}` -ne 0 ]
                then
                    flag=1
                    print_event "1040" "The linkdown_tmo value of the rport ${rport} is ${curTmo}, which is different from the set value ${tmo}."
                fi
            fi
        fi
    done
    return ${flag}
}

function t190s_show_linkdown_tmo()
{
    g10s_getOSType
    
    if [ `echo ${g10s_OSVersion} | grep -c -E "RHEL6|RHEL7|SLES11|SLES12|OL6|OL7"` -eq 0 ]
    then
        print_error "1040" "This system does not support this operation."
        t190s_unlock_exit 1
    fi
    if [ ! -f  /etc/udev/rules.d/99-ultrapath.rules ]
    then
        print_error  "1040" "The ${t190s_udevRuleName} does not exist."
        t190s_unlock_exit 1
    fi
    
    if [ `cat /etc/udev/rules.d/99-ultrapath.rules | grep -c "fc_transport"` -eq 0 ]
    then
        print_event "1040" "The linkdown_tmo has not been set."
        t190s_unlock_exit 1
    fi
    
    tmo=`cat /etc/udev/rules.d/99-ultrapath.rules | grep "fc_transport" | awk '{print $NF}' | awk -F '"' '{print $1}'`
    if [ -z "${tmo}" ] || [ `echo ${tmo} | grep -c -E '^[0-9]+$'` -eq 0 ]
    then
        print_error "1040" "The linkdown_tmo value is invalid, maybe the 99-ultrapath.rules is broken."
        t190s_unlock_exit 1
    fi
    print_event "1040" "linkdown tmo:${tmo}" 
    t190s_check_linkdown_tmo ${tmo}
    t190s_unlock_exit $?
}

function t190s_unset_linkdown_tmo()
{
    local save_path="${g10s_bakPath}/linkdowntmo.default"
    local setValueBackup="${g10s_bakPath}/linkdowntmo.last"
    local rportAddr="/sys/class/fc_remote_ports"
    local value=""
    local flag=0

    g10s_getOSType
    
    if [ `echo ${g10s_OSVersion} | grep -c -E "RHEL6|RHEL7|SLES11|SLES12|OL6|OL7"` -eq 0 ]
    then
        print_error "1040" "This system does not support this operation."
        t190s_linkdown_oper_exit 1
    fi
    
    if [ ! -f  /etc/udev/rules.d/99-ultrapath.rules ]
    then
        print_error  "1040" "The ${t190s_udevRuleName} does not exist."
        t190s_linkdown_oper_exit 1
    fi
    
    if [ ! -f ${save_path} ] || [ ! -f ${setValueBackup} ]
    then
        print_event "1040" "The linkdown_tmo has not been set."
        t190s_linkdown_oper_exit 1
    fi
    
    for rport in `ls ${rportAddr}`
    do
        if [ ! -f "${rportAddr}/${rport}/dev_loss_tmo" ]
        then
            continue
        fi
        value=`cat ${save_path} | grep -w ${rport} | awk -F "=" '{print $2}'`
        if [ -z "$value" ]
        then
            print_debug "not found ${rport} in ${save_path}."
            continue
        fi
        g10s_checkAndSetValue ${rportAddr}/${rport}/dev_loss_tmo ${value}
        if [ $? -ne 0 ]
        then
            flag=1
            rportState=`cat ${rportAddr}/${rport}/port_state`
            print_warning 1040 "Clearing the dev_loss_tmo configuration of ${rport} fails because this port is in \"${rportState}\" status. Restart the host, or run the \"echo ${value} > ${rportAddr}/${rport}/dev_loss_tmo\" command after the path is recovered to clear it. If this path is not available any more, ignore this alarm."
        fi
    done
    sed -i -e '/fc_transport/d' /etc/udev/rules.d/99-ultrapath.rules

    rm -rf ${save_path}
    rm -rf ${setValueBackup}
    if [ ${flag} -eq 1 ]
    then
        t190s_linkdown_oper_exit 1
    else
        print_event "1040" "Unset the linkdown tmo successfully."
        t190s_linkdown_oper_exit 0
    fi
}

function t190s_save_linkdown_tmo()
{
    local save_path="${g10s_bakPath}/linkdowntmo.default"
    local rportAddr="/sys/class/fc_remote_ports"
    local value=""
    
    for rport in `ls ${rportAddr}`
    do
        if [ ! -f "${rportAddr}/${rport}/dev_loss_tmo" ]
        then
            continue
        fi
        
        value=`cat ${rportAddr}/${rport}/dev_loss_tmo`
        echo "${rport}=${value}" >> ${save_path}
    done
}

function t190s_set_linkdown_tmo()
{
    local tmo="$1"
    local rportAddr="/sys/class/fc_remote_ports"
    local offFlag=0
    local setValueBackup="${g10s_bakPath}/linkdowntmo.last"
    local defaultValueBackup="${g10s_bakPath}/linkdowntmo.default"
    
    g10s_getOSType
    
    if [ `echo ${g10s_OSVersion} | grep -c -E "RHEL6|RHEL7|SLES11|SLES12|OL6|OL7"` -eq 0 ]
    then
        print_error "1040" "This system does not support this operation."
        t190s_linkdown_oper_exit 1 ${tmo}
    fi
    
    if [ ! -f  /etc/udev/rules.d/99-ultrapath.rules ]
    then
        print_error  "1040" "The ${t190s_udevRuleName} does not exist."
        t190s_linkdown_oper_exit 1 ${tmo}
    fi
    
    if [ ! -d ${rportAddr} ]
    then
        print_error "1040" "The ${rportAddr} dir is not exist."
        t190s_linkdown_oper_exit 1 ${tmo}
    fi
    
    if [ `ls ${rportAddr} | grep -c "rport"` -eq 0 ]
    then
        print_error "1040" "There is no rport in ${rportAddr}."
        t190s_linkdown_oper_exit 1 ${tmo}
    fi
    
    if [ -z "${tmo}" ]
    then
        print_event "1040" "Invalid input!"
        t190s_set_linkdowntmo_usage
        t190s_linkdown_oper_exit 1 ${tmo}
    fi
    
    if [ `echo "$tmo"|grep -E -c  "^[0-9]+$"` -ne 1 ]
    then 
        print_event "1040" "Invalid input!"
        t190s_set_linkdowntmo_usage
        t190s_linkdown_oper_exit 1 ${tmo}
    fi
    
    if [ ${tmo} -gt 60 ] || [ ${tmo} -lt 1 ]
    then
        print_event "1040" "Invalid input!"
        t190s_set_linkdowntmo_usage
        t190s_linkdown_oper_exit 1 ${tmo}
    fi
    
    if [ ! -f "${g10s_bakPath}/../linkdowntmo.last.uninst" ]
    then
        print_yesOrNo "1050" "Warning: This command will modify the driver's parameters, which affects the time taken to delete a disk when a link is down. This modification also has an impact on all storage ports that connect to the host. Are you sure you want to modify the parameters?"
        if [ $? -ne 0 ]
        then
            t190s_linkdown_oper_exit 1 ${tmo}
        fi
    fi
    
    if [ ! -f ${defaultValueBackup} ]
    then
        t190s_save_linkdown_tmo
    fi
    
    for rport in `ls ${rportAddr}`
    do
        if [ ! -f "${rportAddr}/${rport}/dev_loss_tmo" ]
        then
            continue
        fi
        
        g10s_checkAndSetValue ${rportAddr}/${rport}/dev_loss_tmo ${tmo}
        
        if [ $? -ne 0 ]
        then
            rportState=`cat ${rportAddr}/${rport}/port_state`
            print_debug "modify ${rport} dev_loss_tmo failed, state=${rportState}"
        fi
    done
    
    t190s_create_udevrule_for_linkdowntmo ${tmo}
    
    echo "${tmo}" > ${setValueBackup}
    
    print_event "1040" "The linkdown tmo is set to ${tmo}."
    t190s_linkdown_oper_exit 0 ${tmo}
}

function t190s_set_scsi_timeout()
{
   local  t190_timeout=${1}
 
    
   t190_timeout=`echo ${t190_timeout} | sed 's/^0*//'`
   t190s_checkPramterLen ${t190_timeout}
   t190s_checkParameter "scsi_timeout" ${t190_timeout} 1 600
   t190s_modifyudevTimeoutRule ${t190_timeout}
   t190s_checkudevTimeout  ${t190_timeout}
   t190s_scsi_oper_exit $? ${t190_timeout}
}

function t190s_uncofigibsrp()
{
    print_yesOrNo "1050" "Warning: The operation will clear up the configuration of ib. Do you want to go on?"
    if [ $? -eq 0 ]
    then
        chkconfig --list | grep -w "nxup_ibsrp" 1>/dev/null 2>&1
        if [ $? -eq 0 ]
        then
            chkconfig --del nxup_ibsrp 1>/dev/null 2>&1
            if [ $? -ne 0 ]
            then
                print_error "1050" "chkconfig delete nxup_ibsrp failed."
                t190s_config_ibsrp_exit 1
            fi
        fi
        if [ -f /etc/init.d/nxup_ibsrp ]
        then
            rm -rf /etc/init.d/nxup_ibsrp
        fi
        print_event "1050" "Unconfig ib successfully."
    else
        t190s_config_ibsrp_exit 1
    fi    
    
    print_yesOrNo "1050" "Warning: The operation will stop opensm and run_srp_daemon of all ib ports. Do you want to go on?"
    if [ $? -ne 0 ]
    then
        t190s_config_ibsrp_exit 1
    fi
    
    ps -ef | grep "opensm" | grep -vq "grep"
    if [ $? -eq 0 ]
    then
        killall opensm >/dev/null 2>/dev/null
        killopensm=$?
    else
        killopensm=0
    fi
    
    sleep 1
    ps -ef | grep "run_srp_daemon" | grep -vq "grep"
    if [ $? -eq 0 ]
    then
        killall run_srp_daemon >/dev/null 2>/dev/null
        killdaemon=$?
    else
        killdaemon=0
    fi
    
    sleep 1
    if [ $killopensm -eq 0 ] && [ $killdaemon -eq 0 ]; then
        print_event "1050" "Stop opensm and run_srp_daemon successfully."
        t190s_config_ibsrp_exit 0
    else
        print_event "1050" "Stop opensm or run_srp_daemon failed."
        t190s_config_ibsrp_exit 1
    fi
}

function t190s_configibsrp()
{
    local judgeIB
    g10s_checkIBSrp
    judgeIB=$?
    if [ $judgeIB -eq 1 ]
    then
        print_event "1002" "Cannot find Mellanox IB card."
        print_event "1050" "Config ib failed."
        t190s_config_ibsrp_exit 1
    elif [ $judgeIB -eq 2 ]
    then
        print_event "1050" "The IB drive is not detected, install one before running this command."
        print_event "1050" "Config ib failed."
        t190s_config_ibsrp_exit 1
    fi
    g10s_createIBConfigFile
    if [ $? -eq 0 ]; then
        chkconfig --add nxup_ibsrp 2>/dev/null 1>&2
        if [ $? -ne 0 ]
        then
            print_error "1050" "chkconfig add nxup_ibsrp failed."
            t190s_config_ibsrp_exit 1
        fi
        
        chkconfig nxup_ibsrp on 2>/dev/null 1>&2
        if [ $? -ne 0 ]
        then
            print_error "1050" "chkconfig on nxup_ibsrp failed."
            t190s_config_ibsrp_exit 1
        fi
        
        /etc/init.d/nxup_ibsrp startcli 2>/dev/null 

        if [ $? -eq 0 ]; then
            print_event "1050"  "Config ib successfully."
            t190s_config_ibsrp_exit 0
        else 
            print_event "1050"  "Config ib failed."
            t190s_config_ibsrp_exit 1
        fi
        
    else
        t190s_config_ibsrp_exit 1
    fi
}

function t190s_showibsrpconfig()
{
    local num
    local device
    local num_port
    local port
    
    num=`chkconfig --list | grep -w "nxup_ibsrp" | grep "3:on" | grep "5:on" | wc -l`
    if [ ! -f /etc/init.d/nxup_ibsrp ] || [ ${num} -eq 0 ]
    then
        print_event "1050" "UltraPath IB service is not configured."
    else
        print_event "1050" "UltraPath IB service is configured."
    fi    
    
    g10s_checkIBSrp
    if [ $? -ne 0 ]; then
        t190s_config_ibsrp_exit 1
    fi
    
    print_event "1050" "IB port information:"
    for device in `ibstat -l`; do
        num_ports=`ibstat ${device} | grep 'Number of ports' | awk -F: '{print $2}'`
        for port in `seq 1 ${num_ports}`; do
            if smpquery -C ${device} -P ${port} -D portinfo 0 | grep -q IsSM; then        
                ps -ef | grep "run_srp_daemon" | grep -v "grep" | grep "${device}" | grep -q "${port}$"
                if [ $? -eq 0 ]; then
                    print_event "1050" "$device:port$port normal"
                else
                    print_event "1050" "$device:port$port abnormal"
                fi
            else
                print_event "1050" "$device:port$port abnormal"
            fi
        done
    done
    t190s_config_ibsrp_exit 0
}

g10s_lockShell "${t190s_lockFile}"
g10s_rootCheck

t190s_paramternum="$#" 
t190s_action="${1}"
t190s_type="${2}"
case $t190s_action in
updateImage)
        t190s_updateImage
        ;;
updateForNewKernel)
        t190s_updateForNewKernel
        ;;
show)
        if [ $t190s_paramternum -ne 2 ]
        then
           t190s_showinfousage
        fi 
        
        if [ ${t190s_type} != ${t190s_timeoutstr} ] && [ ${t190s_type} != ${t190s_ibshowconfigstr} ] && [ ${t190s_type} != ${t190s_linkdownTmoShowStr} ]
        then 
           t190s_showinfousage
        fi 
        if [ ${t190s_type} = ${t190s_timeoutstr} ]
        then
            t190s_showscsiTimeout
        fi
        if [ ${t190s_type} = ${t190s_ibshowconfigstr} ]
        then
            t190s_showibsrpconfig
        fi
        if [ ${t190s_type} = ${t190s_linkdownTmoShowStr} ]
        then
            t190s_show_linkdown_tmo
        fi
        ;;
unconfig)
        if [ $t190s_paramternum -ne 2 ]
        then
            print_event "1040" "Invalid input!"
            t190s_clearibconfigusage
        fi 
        if [ ${t190s_type} != ${t190s_ibconfigstr} ]
        then 
            t190s_clearibconfigusage
        fi 
        t190s_uncofigibsrp
        ;;
config)
        if [ $t190s_paramternum -ne 2 ]
        then
            print_event "1040" "Invalid input!"
            t190s_configibusage
        fi 
        if [ ${t190s_type} != ${t190s_ibconfigstr} ]
        then 
            t190s_configibusage
        fi 
        t190s_configibsrp
        ;;
set)
        if [ $t190s_paramternum -ne 2 ]
        then
            print_event "1040" "Invalid input!"
            t190s_set_useage
        fi 
        num=`echo ${t190s_type} | grep -E "${t190s_timeoutinputstr}" | wc -l`
        if [ ${num} -ne 1 ]
        then 
            print_event "1040" "Invalid input!"
            t190s_set_useage
        fi 
        t190s_setItem=`echo ${2} |awk -F = '{print $1}'`
        t190s_setValue=`echo ${2} |awk -F = '{print $2}'`
        t190s_set_${t190s_setItem} ${t190s_setValue}
        ;;
unset)
        if [ $t190s_paramternum -ne 2 ]
        then
            print_event "1040" "Invalid input!"
            t190s_unset_useage
        fi 
        
        if [ "${t190s_type}" != "linkdown_tmo" ]
        then
            print_event "1040" "Invalid input!"
            t190s_unset_useage
        fi
        t190s_unset_${t190s_type}
        ;;
*)
        t190s_useage
        ;;
esac
g10s_unlockShell "${t190s_lockFile}"










