#!/bin/bash

t200s_toolsDir="../toolLib"
g20s_LIBFILE=${g20s_LIBFILE="../globalLib/g20s_install"}
g30s_LIBFILE=${g30s_LIBFILE="../globalLib/g30s_systemEnvironment"}

if [ -f "${g20s_LIBFILE}" ] && [ -f "${g30s_LIBFILE}" ]
then
	. "${g20s_LIBFILE}"

	. "${g30s_LIBFILE}"

elif [ -f "/UltraPath/install/linuxTools/globalLib/g20s_install" ] \
    && [ -f "/UltraPath/install/linuxTools/globalLib/g30s_systemEnvironment" ]
then
    pwdPath=`pwd`
    t200s_toolsDir="/UltraPath/install/linuxTools/toolLib"
    cd "${t200s_toolsDir}"

    . ../globalLib/g20s_install

    . ../globalLib/g30s_systemEnvironment

    cd "${pwdPath}"
    unset pwdPath
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib file.\033[0m"
	exit 1
fi

t200s_solutionType="service"
t200s_hasWarning="0"

function t200s_getSolutionType()
{
    g10s_getConf "${g10s_ultraPathPackagesInfo}" "t140s_solutionType"
    if [ "${g10s_confValue}" != "" ]
    then
        print_debug "get t140s_solutionType [${g10s_confValue}]"
        return 0
    fi

    t200s_solutionType="service"

    g30s_getIsSanBoot
    if [ "${g30s_isSanBoot}" = "1" ]
    then
        g30s_getBootLoaderType
        if [ $? -ne 0 ]
        then
            exit 2
        else
            t200s_solutionType="mkinitrd"

            g10s_eval "g30s_checkBootFrom_${g30s_bootLoaderType}"
        fi
    fi
    g10s_saveConf "${g10s_ultraPathPackagesInfo}" "t140s_solutionType" "${t200s_solutionType}"
    return 0
}

t200s_checkAttentionValue=0
export t200s_checkAttentionValue
function t200s_checkAttentionType()
{
	if  [ "${g30s_isSanBoot}" = "0" ]
	then
		t200s_checkAttentionValue=1
	fi
}


function t200s_checkEnvironment()
{
    local ret



    bash t240s_checkIscsi.sh action=before
    ret=$?
    if [ ${ret} -ne 0 ]
    then
        t200s_hasWarning="1"
    fi

    bash t250s_checkFC.sh action=before
    ret=$?
    if [ ${ret} -ne 0 ]
    then
        t200s_hasWarning="1"
    fi

   return 0
}

function t200s_checkEnvironment_after()
{
    local ret

    bash t260s_clearEnv.sh


    bash t240s_checkIscsi.sh action=changeConf
    ret=$?
    if [ ${ret} -ne 0 ]
    then
        t200s_hasWarning="1"
    fi

    bash t250s_checkFC.sh action=changeConf
    ret=$?
    if [ ${ret} -ne 0 ]
    then
        t200s_hasWarning="1"
    fi
    
    bash t390s_checkSystemdConf.sh action=changeConf
    ret=$?
    if [ ${ret} -ne 0 ]
    then
        t200s_hasWarning="1"
    fi
   return 0
}


function t200s_checkCompatible()
{
    local hasWarning
    hasWarning="0"
    g30s_checkOtherMutiPath
    if [ $? -ne 0 ]
    then
        t200s_hasWarning="1"
    fi
    g30s_checkRDAC
    if [ $? -ne 0 ]
    then
        t200s_hasWarning="1"
    fi
    return 0
}

function t200s_main_before()
{


    t200s_checkEnvironment

    if [ "${t200s_hasWarning}" = "1" ]
    then
        if [ "${UnattendInstallUltraPathFlag}" = "1" ]
        then 
            return 2
        else
            print_yesOrNo "1005" "Please confirm these warnings, and decide whether to continue?"
            if [ $? -ne 0 ]
            then
                exit 2
            fi
        fi
    fi
}
function t200s_main_after()
{

    t200s_checkEnvironment_after
    if [ "${t200s_hasWarning}" = "1" ]
    then
        if [ "${UnattendInstallUltraPathFlag}" = "1" ]
        then 
            return 2
        else
            print_yesOrNo "1005" "Please confirm these warnings, and decide whether to continue?"
            if [ $? -ne 0 ]
            then
                exit 2
            fi
        fi
    fi

    t200s_getSolutionType
    t200s_checkAttentionType
    return 0

}

if [ "$1" = "before" ]
then
    t200s_main_before
    return $?
elif [ "$1" = "after" ]
then
    t200s_main_after
    return $?
else
    print_error "1005" "unknown parameter[$1]"
fi











