#!/bin/bash

g10s_LIBFILE=${g10s_LIBFILE="../globalLib/g10s_pub"}
if [ -f "${g10s_LIBFILE}" ]
then
    . "${g10s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
    echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g10s_LIBFILE}]\033[0m"
    exit 1
fi


t20s_title=${t20s_title=""}

t20s_initrd=${t20s_initrd:="${g10s_isspImageName}-`g10s_curUnamer`.img"}

t20s_action=""
t20s_file=""
t20s_bakfile="auto"
t20s_defaultTitle=""

t20s_bakFileAppend=${t20s_bakFileAppend="${g10s_isspBakPath}/menu.lst.bak"}
t20s_bakFileRemove=${t20s_bakFileRemove="${g10s_bakPath}/t20s_menu.lst.beforeremove"}
t20s_tmpFile="${g10s_tmpPath}/t20s_tmpMenulst"
t20s_newDefaultNum=""
t20s_newImage=""

function t20s_useage()
{
    echo   "USAGE:$0 [option]
       action=append
              remove
       file=XXXX
       bakfile=XXXX
               auto
               no
       defaultTitle=XXXX"
    exit 1
}

function t20s_checkMultiDefault()
{
    local ret
    if [ "$1" = "" ] || [ ! -f "$1" ]
    then
        print_error "2" "param error[$1]"
        exit 1
    fi
    ret=`grep -E -c "^[[:blank:]]*default[[:blank:]=]+[0-9]+[[:blank:]]*(#.*)?$" "$1"`
    if [ $ret -eq 0 ]
    then
        print_error "1001" "not found \"default\" item in file.[${1}]"
        exit 1
    elif [ $ret -gt 1 ]
    then
        print_error "1002" "there's mutiple \"default\" exist.[$1]"
        exit 1
    fi
    return 0
}

t20s_titleNameResult=""
function t20s_getDefaultTitle()
{
    local defaultNum

    t20s_titleNameResult=""
    if [ "$1" = "" ] || [ ! -f "$1" ]
    then
        print_error "2" "param error[$1]"
        exit 1
    fi

    print_debug "get default title from file[$1]"

    defaultNum=`sed -n "s/^[[:blank:]]*default[[:blank:]=][[:blank:]=]*\([0-9][0-9]*\).*$/\1/p" "$1"|sed "s/[[:blank:]]//g"`
    print_debug "defaultNum=${defaultNum}"
    if ! [ ${defaultNum} -ge 0 ] 2>/dev/null
    then
        print_error "1003" "get default number failed[$1]"
        return 1
    fi

    let "defaultNum=defaultNum+1"

    t20s_titleNameResult=`grep "^[[:blank:]]*title[[:blank:]]" "$1"|sed -n "${defaultNum}p"`
    print_debug "t20s_titleNameResult=${t20s_titleNameResult}"
    t20s_titleNameResult=${t20s_titleNameResult%%#*}

    if [ "${t20s_titleNameResult}" = "" ]
    then
        return 1
    else
        return 0
    fi
}

function t20s_getNewDefaultNum()
{
    local sour
    if [ "$1" = "" ] || [ "$2" = "" ]
    then
        print_error "2" "param error[$1,$2]"
        exit 1
    fi
    t20s_newDefaultNum=""
    t20s_getDefaultTitle "$1"
    if [ "${t20s_titleNameResult}" = "" ]
    then
        return 1
    fi

    sour=`grep "^[[:blank:]]*title[[:blank:]]" "$2"|grep -n "^[[:blank:]]*title[[:blank:]]"`


    t20s_newDefaultNum=`echo "${sour}" | grep "${t20s_titleNameResult}"| awk -F ":" '{print $1}' | head -1`
    if ! [ ${t20s_newDefaultNum} -ge 0 ] 2>/dev/null
    then
        print_debug "get new default number failed."
        return 1
    fi

    let "t20s_newDefaultNum=t20s_newDefaultNum-1"
    print_debug "get new default number[t20s_newDefaultNum=${t20s_newDefaultNum}]"

    return 0
}


function t20s_changeDefault()
{
    local paramItem
    local newdefaultItem
    local defaultValue

    if [ "$1" = "" ] || [ ! -f "$1" ] || [ "$2" = "" ]
    then
        print_error "2" "param error[$1,$2]"
        exit 1
    fi
    paramItem="$2"

    newdefaultItem=`grep -E "^[[:blank:]]*default[[:blank:]=]+.*$" "$1"`
    print_debug "newdefaultItem=${newdefaultItem}"
    if [ `echo  "${newdefaultItem}"|grep -v "^[[:blank:]]*$"|wc -l` -ne 1 ]
    then
        print_error "1004" "get default value failed in file[$1]"
        exit 1
    fi

    defaultValue=${newdefaultItem%%#*}
    if [ `echo "${defaultValue}"|grep -c "="` -eq 0 ]
    then
        paramItem="default ${paramItem}"
    else
        paramItem="default=${paramItem}"
    fi


    if [ `echo "${newdefaultItem}"|grep -c "#"` -ne 0 ]
    then
        newdefaultItem=${newdefaultItem#*#}
        newdefaultItem="${paramItem} #${newdefaultItem}"
    else
        newdefaultItem="${paramItem}"
    fi
    g10s_replaceSingleLine "$1" "^[[:blank:]]*default[[:blank:]=][[:blank:]=]*.*$" "${newdefaultItem}"
    return 0
}


function t20s_recoverDefault()
{
    local bakfile
    if [ "$1" = "" ] || [ ! -f "$1" ]
    then
        print_error "2" "param error[$1]"
        exit 1
    fi
    print_debug "recover default value[$1]"

    bakfile=""
    t20s_newDefaultNum=""

    if [ "${t20s_bakfile}" != "no" ] && [ "${t20s_bakfile}" != "auto" ]
    then
        g10s_checkExist "-f" "${t20s_bakfile}" 2
        t20s_getNewDefaultNum "${t20s_bakfile}" "$1"
        if [ $? -ne 0 ]
        then
            print_error "1005" "get new default value failed[${t20s_bakfile},$1]"
            exit 1
        fi
    elif [ "${t20s_bakfile}" = "auto" ]
    then
        if [ -f "${t20s_bakFileAppend}" ]
        then
            bakfile="${t20s_bakFileAppend}"
            t20s_getNewDefaultNum "${bakfile}" "$1"
            rm -f "${t20s_bakFileAppend}"
        fi

        if [ "${t20s_newDefaultNum}" = "" ] && [ -f "/opt/UltraPath/bak/menu.lst" ]
        then
            bakfile="/opt/UltraPath/bak/menu.lst"
            t20s_getNewDefaultNum "${bakfile}" "$1"
        fi

        if [ "${t20s_newDefaultNum}" = "" ] && [ -f "/opt/UpForLinux/menu.lst" ]
        then
            bakfile="/opt/UpForLinux/menu.lst"
            t20s_getNewDefaultNum "${bakfile}" "$1"
        fi

        if [ "${t20s_newDefaultNum}" = "" ] && [ "${t20s_defaultTitle}" != "" ]
        then
            t20s_newDefaultNum=`grep "^[[:blank:]]*title[[:blank:]]" "$1"|grep -n "${t20s_defaultTitle}"|awk -F ":" '{print $1}'`
            if [ ${t20s_newDefaultNum} -ge 1 ] 2>/dev/null
            then
                let "t20s_newDefaultNum=t20s_newDefaultNum-1"
            else
                print_error "1006" "get default number by \"defaultTitle\" failed."
                exit 1
            fi
        fi
    elif [ "${t20s_bakfile}" = "no" ]
    then
        t20s_newDefaultNum=`grep "^[[:blank:]]*title[[:blank:]]" "$1"|grep -n "${t20s_defaultTitle}"|awk -F ":" '{print $1}'`
        if [ ${t20s_newDefaultNum} -ge 1 ] 2>/dev/null
        then
            let "t20s_newDefaultNum=t20s_newDefaultNum-1"
        else
            print_error "1007" "get default number by \"defaultTitle\" failed."
            exit 1
        fi
    fi

    if ! [ ${t20s_newDefaultNum} -ge 0 ] 2>/dev/null
    then
        print_warning "1008" "It can not get default number,set default to 0"
        t20s_newDefaultNum="0"
    fi
    print_debug "before change:t20s_newDefaultNum=${t20s_newDefaultNum}"

    t20s_changeDefault "$1" "${t20s_newDefaultNum}"
    return 0
}

function t20s_createNewImage()
{
    local defaultNum
    local lineNum
    local nextNum
    local nextLineNum
    local defaultImage
    local hasCommentary
    local initrdLine
    local initrdCommentary
    local newInitrdLine
    local preLineNum
    local lineCount

    if [ "$1" = "" ]
    then
        print_error "2" "param error[$1]"
        exit 1
    fi


    defaultNum=`sed -n "s/^[[:blank:]]*default[[:blank:]=][[:blank:]=]*\([0-9][0-9]*\).*$/\1/p" "$1"|sed "s/[[:blank:]]//g"`
    print_debug "defaultNum=${defaultNum}"
    if ! [ ${defaultNum} -ge 0 ] 2>/dev/null
    then
        print_error "1009" "get default number failed[$1]"
        return 1
    fi

    let "defaultNum=defaultNum+1"
    lineNum=`grep -n "^[[:blank:]]*title[[:blank:]]" "$1"|sed -n "${defaultNum}p"|awk -F ":" '{print $1}'`
    print_debug "lineNum=${lineNum}"
    if ! [ ${lineNum} -ge 0 ] 2>/dev/null
    then
        print_error "1010" "get line number failed[$1]"
        exit 1
    fi
    let "lineNum=lineNum+1"


    nextNum="${defaultNum}"
    let "nextNum=nextNum+1"
    nextLineNum=`grep -n "^[[:blank:]]*title[[:blank:]]" "$1"|sed -n "${nextNum}p"|awk -F ":" '{print $1}'`
    if [ "${nextLineNum}" = "" ]
    then
        nextLineNum=`cat "$1"|wc -l`
    else
        let "nextLineNum=nextLineNum-1"
    fi
    print_debug "nextLineNum=${nextLineNum}"
    if ! [ ${nextLineNum} -ge ${lineNum} ] 2>/dev/null
    then
        print_error "1011" "get next line number failed[$1]"
        exit 1
    fi


    defaultImage=`sed -n "${lineNum},${nextLineNum}p" "$1"`

    defaultImage=`echo "${defaultImage}" | grep -v "^[[:blank:]]*#"`

    if [ `echo   "${defaultImage}"|grep -c -E "^[[:blank:]]*(initrd|module)[[:blank:]][[:blank:]]*"` -eq 0 ] \
        || [ `echo   "${defaultImage}"|grep -c -E "^[[:blank:]]*title[[:blank:]]"` -ne 0 ]
    then
        print_error "1012" "defaultImage format error"
        exit 1
    fi


    hasCommentary="0"
    if [ `echo   "${defaultImage}"|grep -c -E "^[[:blank:]]*(initrd|module)[[:blank:]][[:blank:]]*"` -eq 1 ]
    then
        initrdLine=`echo  "${defaultImage}"|grep -E "^[[:blank:]]*(initrd|module)[[:blank:]][[:blank:]]*"`
        [ `echo "${defaultImage}"|grep -E -c "^[[:blank:]]*(initrd|module)[[:blank:]][[:blank:]]*.*#"` -ne 0 ] && hasCommentary="1"
    elif [ `echo   "${defaultImage}"|grep -c -E "^[[:blank:]]*(initrd|module)[[:blank:]]+.*(initrd|initramfs)"` -eq 1 ]
    then
        initrdLine=`echo "${defaultImage}"|grep -E "^[[:blank:]]*(initrd|module)[[:blank:]]+.*(initrd|initramfs)"` 
        [ `echo "${defaultImage}"|grep -E -c "^[[:blank:]]*(initrd|module)[[:blank:]]+.*(initrd|initramfs).*#"` -ne 0 ] && hasCommentary="1"     
    elif [ `echo   "${defaultImage}"|grep -c -E "^[[:blank:]]*(initrd|module)[[:blank:]][[:blank:]]*"` -gt 1 ]      
    then
        initrdLine=`echo  "${defaultImage}"|grep -E "^[[:blank:]]*(initrd|module)[[:blank:]][[:blank:]]*"|tail -1`
        [ `echo "${initrdLine}"|grep -E -c "^[[:blank:]]*(initrd|module)[[:blank:]][[:blank:]]*.*#"` -ne 0 ] && hasCommentary="1"
    else
        print_error "1013" "not found initrd line"
        exit 1
    fi
    print_debug "initrdLine=${initrdLine}"
    initrdCommentary=${initrdLine#*#}
    print_debug "initrdCommentary=${initrdCommentary}"
    initrdLine=${initrdLine%%#*}
    if [ `echo "${initrdLine}"|grep -c "/"` -eq 0 ]
    then
        newInitrdLine="${initrdLine%%=*}="
    else
        newInitrdLine=${initrdLine%/*}
    fi
    print_debug "newInitrdLine=${newInitrdLine}"
    if [ "${hasCommentary}" = "1" ]
    then
        newInitrdLine="${newInitrdLine}/${t20s_initrd} #${initrdCommentary}"
    else
        newInitrdLine="${newInitrdLine}/${t20s_initrd}"
    fi

    g10s_cp "$1" "${t20s_tmpFile}"
    if [ `cat "${t20s_tmpFile}"|sed -n '$p'|grep -c "^[[:blank:]]*$"` -ne 0 ]
    then
        echo "">>"${t20s_tmpFile}"
    fi
    echo "${t20s_title}" >> "${t20s_tmpFile}"


    if [ `echo "${defaultImage}"|grep -c -E "^[[:blank:]]*(initrd|module)[[:blank:]][[:blank:]]*"` -eq 1 ]
    then
        lineNum=`echo  "${defaultImage}"|grep -E -n "^[[:blank:]]*(initrd|module)[[:blank:]][[:blank:]]*"|awk -F ":" '{print $1}'`
    elif [ `echo   "${defaultImage}"|grep -c -E "^[[:blank:]]*(initrd|module)[[:blank:]]+.*(initrd|initramfs)"` -eq 1 ]
    then
        lineNum=`echo  "${defaultImage}"|grep -E -n "^[[:blank:]]*(initrd|module)[[:blank:]]+.*(initrd|initramfs)"|awk -F ":" '{print $1}'`
    elif [ `echo "${defaultImage}"|grep -c -E "^[[:blank:]]*(initrd|module)[[:blank:]][[:blank:]]*"` -gt 1 ]
    then
        lineNum=`echo  "${defaultImage}"|grep -E -n "^[[:blank:]]*(initrd|module)[[:blank:]][[:blank:]]*"|tail -1|awk -F ":" '{print $1}'`
    else
        print_error "1014" "not found initrd line"
        exit 1
    fi
    print_debug "lineNum=${lineNum}"
    if ! [ ${lineNum} -ge 1 ] 2>/dev/null
    then
        print_error "1015" "get lineNum failed[${lineNum}]"
        exit 1
    fi
    preLineNum=${lineNum}
    nextLineNum=${lineNum}
    let "preLineNum=preLineNum-1"
    let "nextLineNum=nextLineNum+1"
    if [ ${preLineNum} -ge 1 ]
    then
        echo  "${defaultImage}"|sed -n "1,${preLineNum}p">>"${t20s_tmpFile}"
    fi
    sed -i "$ s/$/ UltraPath/g" ${t20s_tmpFile}
    echo "${newInitrdLine}">>"${t20s_tmpFile}"
    lineCount=`echo "${defaultImage}"|wc -l`
    print_debug "lineCount=${lineCount}"
    if [ ${lineCount} -ge ${nextLineNum} ] 2>/dev/null
    then
        echo   "${defaultImage}"|sed -n "${nextLineNum},${lineCount}p">>"${t20s_tmpFile}"
    fi
    echo  "###ultrapath end, please don't change this!" >> "${t20s_tmpFile}"
    return 0

}

function t20s_appendTitle()
{
    local sour
    local newDefaultNum


    sour=`cat "${t20s_file}"|grep -E "^[[:blank:]]*title[[:blank:]]+"`
    g10s_getEqualLines "${sour}" "${t20s_title}" "" "#"
    if [ $? -ne 0 ]
    then
        print_warning "1016" "the title has been exist[${t20s_title},${t20s_file}]"
        return 0
    fi

    t20s_createNewImage "${t20s_file}"

    newDefaultNum=`cat "${t20s_tmpFile}"|grep -c "^[[:blank:]]*title[[:blank:]]"`
    let "newDefaultNum=newDefaultNum-1"
    print_debug "newDefaultNum=${newDefaultNum}"
    t20s_changeDefault "${t20s_tmpFile}" "${newDefaultNum}"


    t20s_checkMultiDefault "${t20s_tmpFile}"
    t20s_getDefaultTitle "${t20s_tmpFile}"
    if [ "${t20s_titleNameResult}" != "${t20s_title}" ]
    then
        print_error "1017" "check failed[${t20s_tmpFile}]"
        exit 1
    else

        print_debug "backup file [${t20s_file} to ${t20s_bakFileAppend}]"
        g10s_cp "${t20s_file}" "${t20s_bakFileAppend}"
        g10s_cp "${t20s_tmpFile}" "${t20s_file}"
        rm -f "${t20s_tmpFile}"
    fi
    return 0
}

function t20s_removeTitle()
{
    local ret
    local theNum
    local nextNum
    local lineCount


    ret=`cat "${t20s_file}"|grep -E -c "^[[:blank:]]*${t20s_title}[[:blank:]]*(#.*)?$"`
    if [ $ret -eq 0 ]
    then
        print_warning "1018" "the item has been removed.[${t20s_file},${t20s_title}]"
        return 0
    elif [ $ret -gt 1 ]
    then
        print_error "1019" "There's same item exist.[${t20s_file},${t20s_title}]"
        exit 1
    fi


    theNum=`grep -E -n "^[[:blank:]]*${t20s_title}[[:blank:]]*(#.*)?$" "${t20s_file}"|awk -F ":" '{print $1}'`
    print_debug "theNum=${theNum}"
    if ! [ ${theNum} -ge 0 ] 2>/dev/null
    then
        print_error "1020" "get line number failed[${t20s_title},${t20s_file}]"
        exit 1
    fi

    rm -f "${t20s_tmpFile}"
    touch "${t20s_tmpFile}"

    nextNum=`grep -E -n "^[[:blank:]]*title[[:blank:]]" "${t20s_file}"|awk -F ":" -v col=${theNum} '{if($1 > col){print $1;exit 0}}'`
    print_debug "nextNum=${nextNum}"
    if [ "${nextNum}" = "" ]
    then
        let "theNum=theNum-1"
        if [ ${theNum} -ge 1 ]
        then
            sed -n "1,${theNum}p" "${t20s_file}">>"${t20s_tmpFile}"
        fi
    else
        let "theNum=theNum-1"
        if ! [ ${nextNum} -ge 0 ] 2>/dev/null
        then
            print_error "1021" "get next number failed[${t20s_title},${t20s_file}]"
            exit 1
        fi
        lineCount=`cat ${t20s_file}|wc -l`
        print_debug "lineCount=${lineCount}"
        if [ ${theNum} -ge 1 ]
        then
            sed -n "1,${theNum}p" "${t20s_file}">>"${t20s_tmpFile}"
        fi
        if [ ${lineCount} -ge ${nextNum} ]
        then
            sed -n "${nextNum},${lineCount}p" "${t20s_file}">>"${t20s_tmpFile}"
        fi
    fi



    t20s_recoverDefault "${t20s_tmpFile}"


    t20s_checkMultiDefault "${t20s_tmpFile}"
    t20s_getDefaultTitle "${t20s_tmpFile}"
    if [ $? -ne 0 ]
    then
        print_error "1022" "check failed[${t20s_tmpFile}]"
        exit 1
    else

        print_debug "backup file [${t20s_file} to ${t20s_bakFileRemove}]"
        g10s_cp "${t20s_file}" "${t20s_bakFileRemove}"
        g10s_cp "${t20s_tmpFile}" "${t20s_file}"
        rm -f "${t20s_tmpFile}"
    fi
    return 0
}


function t20s_paramCheck()
{
    if [ "${t20s_action}" != "append" ] && [ "${t20s_action}" != "remove" ]
    then
        print_error "1025" "unknown parameter[action:${t20s_action}]"
        t20s_useage
    fi

    [ "${t20s_file}" = "" ] && t20s_file="/boot/grub/menu.lst"
    if [ ! -f "${t20s_file}" ]
    then
        print_error "1026" "file not exist[${t20s_file}]"
        exit 1
    fi

    [ "${t20s_bakfile}" = "" ] && t20s_bakfile="auto"
    if [ "${t20s_bakfile}" != "auto" ] && [ "${t20s_bakfile}" != "no" ] && [ ! -f "${t20s_bakfile}" ]
    then
        print_error "1027" "file not exist[${t20s_bakfile}]"
        exit 1
    fi

    if [ "${t20s_defaultTitle}" = "" ] && [ "${t20s_bakfile}" = "no" ]
    then
        print_error "1028" "must set defaultTitle=XXXX"
        t20s_useage
    fi
    return 0
}

function t20s_domain()
{
    print_debug "begin param check"
    t20s_paramCheck
    print_debug "check default item"
    t20s_checkMultiDefault "${t20s_file}"
    if [ "${t20s_action}" = "append" ]
    then
        if [ "${t20s_title}" = "" ]
        then
            t20s_title="title Linux with ultrapath"
        fi
        print_debug "begin appending UltraPath title"
        t20s_appendTitle
        print_event "1023" "append UltraPath title to bootloader successfully.[${t20s_file}]"
    elif [ "${t20s_action}" = "remove" ]
    then
        if [ "${t20s_title}" = "" ]
        then
            t20s_title="title Linux with (UltraPath|UPforLinux|UpforLinux|ultrapath)"
        fi
        print_debug "begin removing UltraPath title"
        t20s_removeTitle
        print_event "1024" "The UltraPath startup guide items are deleted successfully."
    fi
    return 0
}




while [ $# -gt 0 ]
do
    t20s_param="$1"
    t20s_param1=${t20s_param%%=*}
    t20s_param2=${t20s_param##*=}
    print_debug "t20s_param=${t20s_param},t20s_param1=${t20s_param1},t20s_param2=${t20s_param2}"
    case "${t20s_param1}" in
        "action")
            t20s_action="${t20s_param2}"
            ;;
        "bakfile")
            t20s_bakfile="${t20s_param2}"
            ;;
        "file")
            t20s_file="${t20s_param2}"
            ;;
        "defaultTitle")
            t20s_defaultTitle="${t20s_param2}"
            ;;
        *)
            print_error "1029" "unknown parameter[${t20s_param}]"
            t20s_useage
            ;;
    esac
    shift
done

t20s_domain










