#!/bin/bash

g30s_LIBFILE=${g30s_LIBFILE="../globalLib/g30s_systemEnvironment"}
if [ -f "${g30s_LIBFILE}" ]
then
	. "${g30s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g30s_LIBFILE}]\033[0m"
	exit 1
fi

t240s_action=""


t240s_iscsiType=""

t240s_iscsiServName=""

t240s_hasWarning="0"

readonly t240s_allActions=",before,after,clear,changeConf,"


readonly t240s_originalOpenIscsiConf="${g10s_bakPath}/t240s_openIscsid.conf"
readonly t240s_originalLinuxIscsiConf="${g10s_bakPath}/t240s_linuxIscsid.conf"
readonly t240s_beforeRemoveOpenIscsiConf="${g10s_bakPath}/t240s_openIscsid.conf.beforeRemove"
readonly t240s_beforeRemoveLinuxIscsiConf="${g10s_bakPath}/t240s_linuxIscsid.conf.beforeRemove"

function t240s_useage()
{
    local allActions
    allActions=${t240s_allActions//,/|}
    allActions=${allActions#|}
    allActions=${allActions%|}
    echo "usage:$0 [action=${allActions}]"
    exit 1
}

function t240s_checkParam()
{
    if [ `echo "${t240s_allActions}"|grep -c ",${t240s_action},"` -eq 0 ]
    then
        print_error "1001" "unknown parameter[action=${t240s_action}]"
        t240s_useage
    fi
    return 0
}





function t240s_openIscsiVersion()
{
    local iscsiVersion
    local iscsiVer

    iscsiVersion=`iscsiadm -V 2>/dev/null|awk '{print $3}'|grep -E "^[0-9]+"`
    if [ "${iscsiVersion}" = "" ]
    then
        print_debug "get open-iscsi version failed,skip version checking."
        return 1
    else
        iscsiVer=`iscsiadm -V 2>/dev/null| grep "2.0-" | awk -F"-" '{print $2}' | awk -F"." '{print $1}'`
        if [ "${iscsiVer}" = "" ]
        then
            if [ `iscsiadm -V 2>/dev/null| grep -c "1.0-"` -ne 0 ]
            then
                print_warning "1027" "The open-iscsi version is too low,Please update the iSCSI driver to the version 2.0-870.3."
                t240s_hasWarning="1"
            	return 1
            fi
        else
		    if [ ${iscsiVer} -le 865 ]
		    then
		        print_warning "1024" "The open-iscsi version is too low,Please update the iSCSI driver to the version 2.0-870.3."
		        t240s_hasWarning="1"
		        return 1
		    fi
		fi
    fi
    return 0
}





function t240s_modiflyOpenIscsiConf()
{
    local confFile
    local ret1
    local ret2
    local isChangeConf=$1
    if [ -f /etc/iscsi/iscsid.conf ]
    then
        confFile="/etc/iscsi/iscsid.conf"
    elif [ -f /etc/iscsid.conf ]
    then
        confFile="/etc/iscsid.conf"
    else
        print_warning "1025" "unkown open-iscsi configuration file."
        t240s_hasWarning="1"
        return 1
    fi

    g10s_checkExist "-f" "${confFile}" 2
    if [ ! -f "${t240s_originalOpenIscsiConf}" ]
    then
        g10s_cp "${confFile}" "${t240s_originalOpenIscsiConf}"
    fi

    g10s_saveSysConf_confName="${confFile}"
    unset g10s_saveSysConf_itemAndValue

    ret1=`grep -E -c "^[[:blank:]]*node.startup[[:blank:]]*=[[:blank:]]*automatic([[:blank:]#]+|$)" "${confFile}"`
    ret2=`grep -E -c "^[[:blank:]]*node.session.timeo.replacement_timeout[[:blank:]]*=[[:blank:]]*1([[:blank:]#]+|$)" "${confFile}"`
    if [ ${ret1} -le 1 ]
    then
        if [ "${isChangeConf}" -eq "1" ]
        then
            g10s_saveSysConf_itemAndValue=("${g10s_saveSysConf_itemAndValue[@]}" "node.startup " "  automatic")
        fi
    else
        print_warning "1010" "there's more than one config item found.[${confFile},node.startup]"
        t240s_hasWarning="1"
    fi

    if [ ${ret2} -le 1 ]
    then
        if [ "${isChangeConf}" -eq "1" ]
        then
            g10s_saveSysConf_itemAndValue=("${g10s_saveSysConf_itemAndValue[@]}" "node.session.timeo.replacement_timeout " "  1")
        fi
    else
        print_warning "1011" "there's more than one config item found.[${confFile},node.session.timeo.replacement_timeout]"
        t240s_hasWarning="1"
    fi

    if [ "${isChangeConf}" -eq "1" ]
    then
        g10s_saveSysConf "${confFile}"
    fi

    return 0
}




function t240s_check_openIscsi()
{
    t240s_openIscsiVersion
    t240s_modiflyOpenIscsiConf 0
    return 0

}



function t240s_changeConf_openIscsi()
{
    t240s_modiflyOpenIscsiConf 1
    return 0

}


function t240s_linuxIscsiVersion()
{
    local iscsiVersion
    local spVersion
    local ret

    iscsiVersion=`iscsid -v 2>/dev/null|awk '{print $3}'|grep -E "^[0-9\.-]+"`
    if [ "${iscsiVersion}" = "" ]
    then
        print_debug "get iscsi version failed,skip version checking."
        return 1
    else

        g10s_getOSType
        if [ `echo "${g10s_OSVersion}"|grep -c "SLES9SP3"` -eq 1 ]
        then
            ret=`modinfo iscsi | grep -c "4.0.188.26-UP" `
            if [ ${ret} -eq 0 ]
            then
                print_warning "1023" "There are some detects of the iSCSI Initiator provided by SLES9 SP3.Therefore,you need to update the iSCSI Initator."
                t240s_hasWarning="1"
                return 1
            fi
        fi

        if [ `echo "${g10s_OSVersion}"|grep -c "SLES9SP2_201"` -eq 1 ]
        then
            print_warning "1023" "There are some detects of the iSCSI Initiator provided by SLES9 SP2.It is recommended to use FC initator type."
            t240s_hasWarning="1"
            return 1
        fi
    fi
    return 0

}

function t240s_checkLinuxIscsiConf()
{
    local confFile
    local ret1
    local ret2
    local isChangeConf=$1
    confFile="/etc/iscsi.conf"
    g10s_checkExist "-f" "${confFile}" 2
    if [ ! -f "${t240s_originalLinuxIscsiConf}" ]
    then
        g10s_cp "${confFile}" "${t240s_originalLinuxIscsiConf}"
    fi

    g10s_saveSysConf_confName="${confFile}"

    unset g10s_saveSysConf_itemAndValue

    ret1=`grep -E -c "^[[:blank:]]*Multipath[[:blank:]]*=[[:blank:]]*portal([[:blank:]#]+|$)" "${confFile}"`
    ret2=`grep -E -c "^[[:blank:]]*ConnFailTimeout[[:blank:]]*=[[:blank:]]*1([[:blank:]#]+|$)" "${confFile}"`

    if [ ${ret1} -le 1 ]
    then
        if [ "${isChangeConf}" -eq "1" ]
        then
            g10s_saveSysConf_itemAndValue=("${g10s_saveSysConf_itemAndValue[@]}" "Multipath" "portal")
        fi
    else
        print_warning "1016" "there's more than one config item found.[${confFile},Multipath]"
        t240s_hasWarning="1"
    fi

    if [ ${ret2} -le 1 ]
    then
        if [ "${isChangeConf}" -eq "1" ]
        then
            g10s_saveSysConf_itemAndValue=("${g10s_saveSysConf_itemAndValue[@]}" "ConnFailTimeout" "1")
        fi
    else
        print_warning "1017" "there's more than one config item found.[${confFile},ConnFailTimeout]"
        t240s_hasWarning="1"
    fi
    if [ "${isChangeConf}" -eq "1" ]
    then
        g10s_saveSysConf
    fi
    return 0
}



function t240s_check_linuxIscsi()
{
    t240s_iscsiServName="iscsi"
    t240s_linuxIscsiVersion
    t240s_checkLinuxIscsiConf 0
    return 0
}


function t240s_changeConf_linuxIscsi()
{
    t240s_iscsiServName="iscsi"
    t240s_checkLinuxIscsiConf 1
    return 0
}


function t240s_main_clear()
{
    if [ -f "/etc/iscsi.conf" ]
    then
        if [ ! -f "${t240s_beforeRemoveLinuxIscsiConf}" ]
        then
            g10s_cp "/etc/iscsi.conf" "${t240s_beforeRemoveLinuxIscsiConf}"
        fi
        print_debug "reback configs in /etc/iscsi.conf"
        g10s_rebackFromSysConf "/etc/iscsi.conf" "ALL"
    fi
    if [ -f "/etc/iscsi/iscsid.conf" ]
    then
        if [ ! -f "${t240s_beforeRemoveOpenIscsiConf}" ]
        then
            g10s_cp "/etc/iscsi/iscsid.conf" "${t240s_beforeRemoveOpenIscsiConf}"
        fi
        print_debug "reback configs in /etc/iscsi/iscsid.conf"
        g10s_rebackFromSysConf "/etc/iscsi/iscsid.conf" "ALL"
    fi
    return 0
}


function t240s_main_before()
{
    if [ `which "iscsiadm" 2>/dev/null|grep -c "/iscsiadm"` -eq 1 ]
    then
        t240s_iscsiType="openIscsi"
    else
        t240s_iscsiType="linuxIscsi"
    fi

    g10s_eval "t240s_check_${t240s_iscsiType}"
    if [ "${t240s_hasWarning}" = "1" ]
    then

        exit 2
    fi
    print_event "1022" "complete iscsi checking."
    return 0
}

function t240s_main_after()
{
    if [ `which "iscsiadm" 2>/dev/null|grep -c "/iscsiadm"` -eq 1 ]
    then
        t240s_iscsiType="openIscsi"
    else
        t240s_iscsiType="linuxIscsi"
    fi

    g10s_eval "t240s_check_${t240s_iscsiType}"
    g10s_eval "t240s_changeConf_${t240s_iscsiType}"
    if [ "${t240s_hasWarning}" = "1" ]
    then

        exit 3
    fi
    print_event "1022" "complete iscsi checking."
    return 0
}


function t240s_main_changeConf()
{
    if [ `which "iscsiadm" 2>/dev/null|grep -c "/iscsiadm"` -eq 1 ]
    then
        t240s_iscsiType="openIscsi"
    else
        t240s_iscsiType="linuxIscsi"
    fi

    g10s_eval "t240s_changeConf_${t240s_iscsiType}"
    if [ "${t240s_hasWarning}" = "1" ]
    then

        exit 3
    fi
    return 0
}

g10s_rootCheck
while [ $# -gt 0 ]
do
    t240s_param="$1"
    t240s_param1=${t240s_param%%=*}
    t240s_param2=${t240s_param##*=}
    print_debug "t240s_param=${t240s_param},t240s_param1=${t240s_param1},t240s_param2=${t240s_param2}"
    case "${t240s_param1}" in
        "action")
            t240s_action="${t240s_param2}"
            ;;
        *)
            print_error "1020" "unknown parameter[${t240s_param}]"
            t240s_useage
            ;;
    esac
    shift
done

t240s_checkParam
if [ "${t240s_action}" = "clear" ]
then
    t240s_main_clear
    exit 0
fi
if [ `which iscsid 2>/dev/null|grep -c "/iscsid"` -eq 0 ]
then
    print_event "1019" "iscsi is not installed."
    exit 0
fi
g10s_eval "t240s_main_${t240s_action}"








