#!/bin/bash

g30s_LIBFILE=${g30s_LIBFILE="../globalLib/g30s_systemEnvironment"}
if [ -f "${g30s_LIBFILE}" ]
then
	. "${g30s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g30s_LIBFILE}]\033[0m"
	exit 1
fi

t250s_action=""

t250s_needReboot="0"

t250s_needUpdateImage="0"
t250s_hasWarning="0"
t250s_modprobeConfName=""
t250s_curModules=""

readonly t250s_allActions=",before,after,clear,changeConf,"

t250s_checkModules=("qla2xxx" )

readonly t250s_modprobeConfTmp="${g10s_tmpPath}/t250s_modprobe.conf.tmp.$$"
readonly t250s_flagBegin="###UltraPath append options begin,don't change this###"
readonly t250s_flagEnd="###UltraPath append options end,don't change this###"

readonly t250s_bakModprobeConf="${g10s_bakPath}/t250s_modprobeConf.bak"
readonly t250s_beforeRemoveModprobeConf="${g10s_bakPath}/t250s_modprobeConf.beforeRemove"

function t250s_useage()
{
    local allActions
    allActions=${t250s_allActions//,/|}
    allActions=${allActions#|}
    allActions=${allActions%|}
    echo "usage:$0 [action=${allActions}]"
    exit 1
}

function t250s_checkParam()
{
    if [ `echo "${t250s_allActions}"|grep -c ",${t250s_action},"` -eq 0 ]
    then
        print_error "1001" "unknown parameter[action=${t250s_action}]"
        exit 1
    fi
    return 0
}

function t250s_getModprobeConfName()
{
    t250s_modprobeConfName=""

    g10s_getOSType

    if [ -f /sbin/dracut ] || [ "${g10s_OSVersion}" = "ROCKY_2.6.32.41" ] || [ "${g10s_OSVersion}" = "ROCKY6SP4" ] || [ "${g10s_OSVersion}" = "ROCKY_4.2.39" ] || [ "${g10s_OSVersion}" = "ROCKY_4.2.40" ] || [ "${g10s_OSVersion}" = "ROCKY6SP6" ] || [ "${g10s_OSVersion}" = "DEEPIN15" ] || [ -f /usr/bin/dracut ]
    then
        t250s_modprobeConfName="/etc/modprobe.d/nxupmodules.conf"
        [ ! -f "${t250s_modprobeConfName}" ] && touch "${t250s_modprobeConfName}"
        g10s_chmodExec 600 "${t250s_modprobeConfName}"
    elif [ -f /etc/modprobe.conf.local ]
    then
        t250s_modprobeConfName="/etc/modprobe.conf.local"
    elif [ -f /etc/modprobe.conf ]
    then
        t250s_modprobeConfName="/etc/modprobe.conf"
    else
        print_warning "1002" "not found modprobe config."
        return 1
    fi
    return 0
}


t250s_optionsAndValue=""
function t250s_addOptions()
{
    local i
    local iCount
    local tmpModule
	local befModule=""
    local tmpItem
    local tmpValue
    local ret3
    local ret2
    local ret
    local hasModify="0"

    if [ "${t250s_optionsAndValue}" = "" ] || [ ${#t250s_optionsAndValue[@]} -le 1 ]
	then
		print_error "2" "param error."
		t250s_optionsAndValue=""
		exit 1
	fi

	t250s_getModprobeConfName

	if [ ! -f "${t250s_bakModprobeConf}" ]
	then
	    g10s_cp "${t250s_modprobeConfName}" "${t250s_bakModprobeConf}"
	fi

	iCount=${#t250s_optionsAndValue[@]}

	if [ "${g10s_modifySysConfAuto}" = "0" ]
	then
            print_yesOrNo "1003" "Is allowed to modify the system configuration?[${t250s_modprobeConfName}]"
            if [ $? -ne 0 ]
            then
                print_warning "1004" "you'd better modify those configuration by yourself."

        	i=0
        	for (( i=0; i<${iCount}; i=i+1))
        	do
        	    tmpModule="${t250s_optionsAndValue[${i}]}"
        	    if [ "${tmpModule}" = "" ]
        	    then

        	        continue
        	    fi
        	    let "i=i+1"
        	    tmpItem="${t250s_optionsAndValue[${i}]}"
        	    let "i=i+1"
        	    tmpValue="${t250s_optionsAndValue[${i}]}"
        	    print_event "1005" "configuration.[file:${t250s_modprobeConfName},options ${tmpModule} ${tmpItem}=${tmpValue}]"
        	done
            t250s_optionsAndValue=""
            return 1
        fi
	fi

	g10s_cp "${t250s_modprobeConfName}" "${t250s_modprobeConfTmp}"
	echo -n "${t250s_flagBegin}" >> "${t250s_modprobeConfTmp}"
    if [ $? -ne 0 ]
    then
        print_error "1006" "append begin flag failed."
        t250s_optionsAndValue=""
        rm -f "${t250s_modprobeConfTmp}"
        exit 1
    fi

    for (( i=0; i<${iCount}; i=i+1 ))
	do
	    tmpModule="${t250s_optionsAndValue[${i}]}"
	    if [ "${tmpModule}" = "" ]
	    then

	        continue
	    fi
	    let "i=i+1"
	    tmpItem="${t250s_optionsAndValue[${i}]}"
	    let "i=i+1"
	    tmpValue="${t250s_optionsAndValue[${i}]}"

        if [ `modinfo ${tmpModule} 2>/dev/null|grep -c "^[[:blank:]]*parm:[[:blank:]]*${tmpItem}"` -eq 0 ]
        then
            print_debug "${tmpModule} is not support param ${tmpItem}"
            continue
        fi




        ret=`grep -E -c "^[[:blank:]]*options[[:blank:]]+${tmpModule}[[:blank:]]+.*${tmpItem}[[:blank:]]*=" "${t250s_modprobeConfTmp}"`
        if [ ${ret} -eq 1 ]
        then

            ret2=`grep -E -c "^[[:blank:]]*options[[:blank:]]+${tmpModule}[[:blank:]]+.*${tmpItem}[[:blank:]]*=[[:blank:]]*${tmpValue}([[:blank:]#]+|$)" "${t250s_modprobeConfTmp}"`
            if [ ${ret2} -eq 0 ]
            then

                ret3=`grep -E -c "^[[:blank:]]*options[[:blank:]]+${tmpModule}[[:blank:]]+.*${tmpItem}[[:blank:]]*=[^=]+([[:blank:]]*$|[[:blank:]]*#.*$)" "${t250s_modprobeConfTmp}"`
                if [ ${ret3} -eq 1 ]
                then
                    print_debug "comment file:${t250s_modprobeConfName},options ${tmpModule} ${tmpItem}="

                    sed -i "/^[[:blank:]]*options[[:blank:]][[:blank:]]*${tmpModule}[[:blank:]][[:blank:]]*.*${tmpItem}[[:blank:]]*=/s/\(^.*$\)/###NXUP-COMMENT###\1/" "${t250s_modprobeConfTmp}"
                    if [ $? -ne 0 ]
                    then
                        print_error "1024" "comment line failed.[file:${t250s_modprobeConfName},options ${tmpModule} ${tmpItem}]"
                        exit 1
                    fi
                else
                    print_warning "1007" "the \"options\" is exist,please modify it by yourself.[file:${t250s_modprobeConfName},options ${tmpModule} ${tmpItem}=${tmpValue}]"
                    t250s_hasWarning="1"
                    continue
                fi
            else

                print_debug "[file:${t250s_modprobeConfName},module:${tmpModule},item:${tmpItem},value:${tmpValue}] exist."
                continue
            fi
       elif [ ${ret} -gt 1 ]
       then

            print_warning "1008" "there's repeat \"option\" exist,please modify it by yourself.[file:${t250s_modprobeConfName},options ${tmpModule} ${tmpItem}=${tmpValue}]"
            t250s_hasWarning="1"
            continue
       fi

       print_event "1009" "Modify system configuration.[file:${t250s_modprobeConfName},module:${tmpModule},item:${tmpItem},value:${tmpValue}]"

	   if [ "${befModule}" != "${tmpModule}" ]
	   then
			echo "" >> "${t250s_modprobeConfTmp}"
			echo -n "options ${tmpModule}" >> "${t250s_modprobeConfTmp}"
			befModule="${tmpModule}"
	   fi
	   echo -n " ${tmpItem}=${tmpValue}" >> "${t250s_modprobeConfTmp}"
       if [ $? -ne 0 ]
       then
            print_error "1010" "append options failed.[file:${t250s_modprobeConfName},options ${tmpModule} ${tmpItem}=${tmpValue}]"
            t250s_optionsAndValue=""
            rm -f "${t250s_modprobeConfTmp}"
            exit 1
        fi
        hasModify="1"
   done

   t250s_optionsAndValue=""
   echo "" >> "${t250s_modprobeConfTmp}"
   echo "${t250s_flagEnd}" >> "${t250s_modprobeConfTmp}"
   if [ $? -ne 0 ]
   then
        print_error "1012" "append end flag failed."
        exit 1
   fi
   [ "${hasModify}" = "1" ] && g10s_cp "${t250s_modprobeConfTmp}" "${t250s_modprobeConfName}"
   rm -f "${t250s_modprobeConfTmp}"
   return 0
}

function t250s_removeOptions()
{
    local i
    local iCount
    local findBegin
    local hasLineRmoved
    local lineMsg

    t250s_getModprobeConfName
    if [ ! -f "${t250s_beforeRemoveModprobeConf}" ]
	then
	    g10s_cp "${t250s_modprobeConfName}" "${t250s_beforeRemoveModprobeConf}"
	fi

    iCount=`cat "${t250s_modprobeConfName}"|wc -l`
    i=1
    findBegin="0"
    hasLineRmoved="0"
    rm -f "${t250s_modprobeConfTmp}"
    touch "${t250s_modprobeConfTmp}"
    for (( i=1; i<=${iCount}; i=i+1 ))
    do
        lineMsg=`sed -n "${i}p" "${t250s_modprobeConfName}"`
        if [ "${findBegin}" = "0" ] && [ `echo "${lineMsg}"|grep -c "${t250s_flagBegin}"` -eq 1 ]
        then
            findBegin="1"
            hasLineRmoved="1"
            continue
        fi

        if [ "${findBegin}" = "1" ]
        then
            if [ `echo "${lineMsg}"|grep -c "${t250s_flagEnd}"` -eq 1 ]
            then
                findBegin="0"
            fi
            continue
       fi
       echo "${lineMsg}" >> "${t250s_modprobeConfTmp}"
    done


    if [ "${hasLineRmoved}" = "1" ]
    then
        if [ "${findBegin}" != "0" ]
        then
            print_error "1013" "not found end flag.[${t250s_modprobeConfName}]"
            exit 1
        fi
        print_debug "t250s_modprobeConfTmp line count=`cat ${t250s_modprobeConfTmp}|wc -l`"
        g10s_cp "${t250s_modprobeConfTmp}" "${t250s_modprobeConfName}"
    fi
    
    rm -f "${t250s_modprobeConfTmp}"
    if [ "${t250s_modprobeConfName}" = "/etc/modprobe.d/nxupmodules.conf" ] && [ -f "/etc/modprobe.d/nxupmodules.conf" ]
    then
	rm -f "${t250s_modprobeConfName}"
    fi
    return 0
}

function t250s_checkQlaVersion_need_update()
{
    local old1
    local old2
    local old3
    local new1
    local new2
    local new3
    local qlaversion=""
    qlaversion=`modinfo qla2xxx 2>/dev/null|grep "^version:"|awk '{print $2}'|grep -E "^[0-9\.-]+"` 
    if [ "${qlaversion}" = "" ]
    then
    	return 0
    fi
    
    qlaversion=`echo "${qlaversion}" | awk -F "-" '{print $1}'`
    
    standardVsersion="8.02.23"
    old1=`echo "${standardVsersion}"|awk -F "." '{print $1}'`
    old2=`echo "${standardVsersion}"|awk -F "." '{print $2}'`
    old3=`echo "${standardVsersion}"|awk -F "." '{print $3}'`
    new1=`echo "${qlaversion}"|awk -F "." '{print $1}' | grep -v -E "[a-z]|[A-Z]|-"`
    if [ "${new1}" = "" ]
    then
    	new1=0
    fi
    
    new2=`echo "${qlaversion}"|awk -F "." '{print $2}' | grep -v -E "[a-z]|[A-Z]|-"`
    if [ "${new2}" = "" ]
    then
    	new2=0
    fi
    
    new3=`echo "${qlaversion}"|awk -F "." '{print $3}' | grep -v -E "[a-z]|[A-Z]|-"`
    if [ "${new3}" = "" ]
    then
    	new3=0
    fi
    
    
    if [ ${old1} -gt ${new1} ] 
    then 
        return 1
    elif [ ${old1} -eq ${new1} ] 
    then
        if [ ${old2} -gt ${new2} ]
        then
            return 1
        elif [ ${old2} -eq ${new2} ] 
        then
            if [ ${old3} -gt ${new3} ]
            then
                return 1
            elif [ ${old3} -eq ${new3} ]
            then
                return 3
            fi
        fi
    fi
    return 0
}

function t250s_checkRH4QlaVersion()
{
    local ret
    t250s_OSVersion=""
    g10s_getOSType
    t250s_OSVersion=${g10s_OSVersion}
    
    t250s_checkQlaVersion_need_update
    ret=$?
    
    if [ "${t250s_OSVersion}" = "" ]
    then
        print_warning "1014" "get OS version failed.skip version checking."
        return 1
    elif [ "${t250s_OSVersion}" = "RHEL4" ]
    then
        if [ ${ret} -eq 1 ]
        then
            print_warning "1015" "The default Qlogic driver in Red Hat AS4 has a defect for Using upRescan command provided by the UltraPath.Please update it to qla2xxx-v8.02.23"
            t250s_hasWarning="1"
            return 1            
        fi

    elif [ "${t250s_OSVersion}" = "ASIANUX2" ]
    then
        if [ ${ret} -eq 1 ]
        then
            print_warning "1015" "The default Qlogic driver in Asianux2 has a defect for Using upRescan command provided by the UltraPath.Please update it to qla2xxx-v8.02.23"
            t250s_hasWarning="1"
            return 1            
        fi    
    fi
    return 0
}
function t250s_checkQlaVersion()
{
    local qlaVersion

    qlaVersion=`modinfo qla2xxx 2>/dev/null|grep "^version:"|awk '{print $2}'|grep -E "^[0-9\.-]+"`
    if [ "${qlaVersion}" = "" ]
    then
        print_warning "1014" "get qlogic version failed,skip version checking."
        t240s_hasWarning="1"
        return 1 
    else     
        t250s_checkRH4QlaVersion
    fi
    return 0
    
}


function t250s_checkQlaFailover()
{
    local ret
    local result
    local hostnum

    ret=`modinfo qla2xxx 2>/dev/null| grep "^[[:blank:]]*parm:[[:blank:]]*ql2xfailover"| wc -l`
    if [ ${ret} -gt 0 ]
    then
        if [ -e /sys/module/qla2xxx/parameters/ql2xfailover ]
	then
	    ret=`cat /sys/module/qla2xxx/parameters/ql2xfailover 2>/dev/null | grep "1" | wc -l`
	elif [ -d /proc/scsi/qla2xxx/ ]
	then
	    result=`ls /proc/scsi/qla2xxx/|grep "[0-9][0-9]*"|wc -l`
	    if [ ${result} -eq 0 ]
	    then
		    print_error "1016" "Not find any host in[/proc/scsi/qla2xxx/]" "g30s"
		    exit 1
	    else
		    hostnum=`ls /proc/scsi/qla2xxx/|grep "[0-9][0-9]*"|sed -n '1p'|sed 's/[[:blank:]]//g'`
		    ret=`cat /proc/scsi/qla2xxx/${hostnum}|grep "version.*-fo[[:blank:]]*$"|wc -l` 
	   fi
	else
	     print_error "1017" "Not find /sys/module/ and /proc/scsi/qla2xxx/." "g30s"
	exit 1
	fi
		
        if [ ${ret} -ne 0 ]
        then
            print_warning "1018" "qlogic failover is on.you need reboot your system"
            t250s_hasWarning="1"
            return 1
        else
            return 0
        fi                                  
    fi
    return 0
}

function t250s_changeConf()
{
    unset t250s_optionsAndValue
    t250s_optionsAndValue=("${t250s_optionsAndValue[@]}" "qla2xxx" "ql2xfailover" "0")
    t250s_optionsAndValue=("${t250s_optionsAndValue[@]}" "qla2xxx" "qlport_down_retry" "5")
    t250s_optionsAndValue=("${t250s_optionsAndValue[@]}" "qla2xxx" "ql2xretrycount" "1")
    t250s_optionsAndValue=("${t250s_optionsAndValue[@]}" "lpfc" "lpfc_nodev_tmo" "5")
    t250s_addOptions
    return 0
}


function t250s_qla2xxx_before()
{
    if [ `lspci|grep "Fibre.*QLogic.*"|wc -l` -eq 0 ]
    then
	print_debug "Not find Qlogic HBA card,skip qla2xxx check."
	return 0
    fi
	
    t250s_checkQlaVersion
    
    t250s_checkQlaFailover

    return 0
}

function t250s_qla2xxx_after()
{
    if [ `lspci|grep "Fibre.*QLogic.*"|wc -l` -eq 0 ]
    then
	print_debug "Not find qlogic HBA card,skip qla2xxx check."
	return 0
    fi

    t250s_checkQlaVersion

    t250s_checkQlaFailover

    return 0
}

g10s_rootCheck
while [ $# -gt 0 ]
do
    t250s_param="$1"
    t250s_param1=${t250s_param%%=*}
    t250s_param2=${t250s_param##*=}
    print_debug "t250s_param=${t250s_param},t250s_param1=${t250s_param1},t250s_param2=${t250s_param2}"
    case "${t250s_param1}" in
        "action")
            t250s_action="${t250s_param2}"
            ;;
        *)
            print_error "1021" "unknown parameter[${t250s_param}]"
            t250s_useage
            ;;
    esac
    shift
done

t250s_checkParam
if [ "${t250s_action}" = "clear" ]
then
    t250s_removeOptions
    exit 0
fi
if [ "${t250s_action}" = "changeConf" ]
then
    t250s_changeConf
    if [ "${t250s_hasWarning}" = "1" ]
    then
        exit 3
    fi
    exit 0
fi
if [ "${t250s_action}" = "after" ]
then
    t250s_changeConf
fi
for t250s_curModules in "${t250s_checkModules[@]}"
do
    print_debug "check FC modules.[${t250s_curModules}]"
    g10s_eval "t250s_${t250s_curModules}_${t250s_action}"
done
if [ "${t250s_hasWarning}" = "1" ]
then
    exit 3
fi
print_event "1023" "complete FC checking."








