#!/bin/bash

g30s_LIBFILE=${g30s_LIBFILE="../globalLib/g30s_systemEnvironment"}
if [ -f "${g30s_LIBFILE}" ]
then
	. "${g30s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g30s_LIBFILE}]\033[0m"
	exit 1
fi


t260s_modprobeConfName=""
t260s_menulst="/boot/grub/menu.lst"
t260s_menulst_bak="/menu.lst.bak"
t260s_title="title Linux with (UltraPath|UPforLinux|UpforLinux)"
function t260s_check_Default()
{
    local ret
    if [ "$1" = "" ] || [ ! -f "$1" ]
    then
        print_error "2" "param error[$1]"
        exit 1
    fi
    ret=`grep -E -c "^[[:blank:]]*default[[:blank:]=]+[0-9]+[[:blank:]]*(#.*)?$" "$1"`
    if [ $ret -eq 0 ]
    then
        print_debug "not found \"default\" item in file.[${1}]"
        exit 1
    elif [ $ret -gt 1 ]
    then
        print_error "1002" "there's mutiple \"default\" exist.[$1]"
        exit 1
    fi

    return 0
}

function t260s_recover_defaut()
{
    local ret

    deftline=`cat $1 | grep -n "^[[:blank:]]*default[[:blank:]=][[:blank:]=]*" | tail -n 1 | awk -F ":" '{print $1}'`

    imageLine=`grep -E -n "(initrd|module)-$(uname -r)" "$1" |head -1 |awk -F ":" '{print $1}'`
    if [ -z "${imageLine}" ]
    then
        sed -i "${deftline}s/[0-9][0-9]*/0/g" $1 1>/dev/null 2>&1
        print_debug  "recover default [0]."
        return 0
    fi

    deftTitle=`grep -E -n "^[[:blank:]]*title" "$1" |awk -F ":" -v col=${imageLine} '{if($1 < col){print $2}}' |tail -1`
    if [ ! -z "${deftTitle}" ]
    then
        deftNum=`grep "^[[:blank:]]*title" "$1" | grep -n "^[[:blank:]]*${deftTitle}" |awk -F ":" '{print $1}'|head -1`
        let "deftNum=deftNum-1"
        sed -i "${deftline}s/[0-9][0-9]*/${deftNum}/g" $1 1>/dev/null 2>&1
        print_debug  "recover default [${deftNum}]."
    else
        sed -i "${deftline}s/[0-9][0-9]*/0/g" $1 1>/dev/null 2>&1
        print_debug  "recover default [0]."
    fi

    return 0
}

function t260s_clear_upTitle()
{
    local titleNum
    local upTitleLine
    local nxTitleLine

    titleNum=`grep -E -c "^[[:blank:]]*${t260s_title}" "$1"|awk -F ":" '{print $1}'`
    print_debug  "found [${titleNum}] Linux UltraPath titles."
    while [ ${titleNum} -gt 0 ]
    do
        upTitleLine=`grep -E -n "^[[:blank:]]*${t260s_title}" "$1"|awk -F ":" '{print $1}' |head -1`

        nxTitleLine=`grep -E -n "^[[:blank:]]*title" "$1" |awk -F ":" -v col=${upTitleLine} '{if($1 > col){print $1;exit 0}}' |head -1`
        if [ -z "${nxTitleLine}" ]
        then
            sed -i "${upTitleLine},\$d" "$1" 1>/dev/null 2>&1
        else
            let "nxTitleLine=nxTitleLine-1"
            sed -i "${upTitleLine},${nxTitleLine}d" "$1" 1>/dev/null 2>&1
        fi
        let "titleNum=titleNum-1"
    done

    return 0
}

function t260s_clear_menulst()
{
    if [ ! -f ${t260s_menulst} ]
    then
        return 0
    fi

    print_debug  "begin clear menu.lst Env."

    t260s_check_Default "${t260s_menulst}"

    ret=`cat "${t260s_menulst}"|grep -E -c "^[[:blank:]]*${t260s_title}"`
    if [ $ret -eq 0 ]
    then
        return 0
    fi

    g10s_cp "${t260s_menulst}" "${t260s_menulst_bak}"
    g10s_cp "${t260s_menulst}" "/opt/UltraPath/backup/menu.lst.old"

    t260s_clear_upTitle "${t260s_menulst_bak}"

    t260s_recover_defaut "${t260s_menulst_bak}"

    g10s_cp "${t260s_menulst_bak}" "${t260s_menulst}"
    rm -rf "${t260s_menulst_bak}"
    return 0
}

function t260s_comment_iscsiconf()
{
    sed -i "s/^[[:blank:]]*$1/#$1/" "${confFile}" 1>/dev/null 2>&1

    print_debug "comment [$1] success in [${confFile}]."

    return 0
}
function t260s_clear_openIscsi()
{
    local confFile
    local ret1
    local ret2

    print_debug  "begin clear open-iscsi Env."
    if [ -f /etc/iscsi/iscsid.conf ]
    then
        confFile="/etc/iscsi/iscsid.conf"
    elif [ -f /etc/iscsid.conf ]
    then
        confFile="/etc/iscsid.conf"
    else
        print_debug  "unkown open-iscsi configuration file."
        return 0
    fi
    g10s_checkExist "-f" "${confFile}" 2
    ret1=`grep -E -c "^[[:blank:]]*node.startup" "${confFile}"`
    ret2=`grep -E -c "^[[:blank:]]*node.session.timeo.replacement_timeout" "${confFile}"`
    if [ ${ret1} -gt 1 ]
    then
        t260s_comment_iscsiconf "node.startup"
    fi

    if [ ${ret2} -gt 1 ]
    then
        t260s_comment_iscsiconf "node.session.timeo.replacement_timeout"
    fi

    return 0
}


function t260s_clear_iscsi()
{
    local confFile
    local ret1
    local ret2

    print_debug  "begin clear iscsi Env."
    if [ -f /etc/iscsi.conf ]
    then
        confFile="/etc/iscsi.conf"
    else
        print_debug  "not found [/etc/iscsi.conf]."
        return 0
    fi

    g10s_checkExist "-f" "${confFile}" 2

    ret1=`grep -E -c "^[[:blank:]]*Multipath" "${confFile}"`
    ret2=`grep -E -c "^[[:blank:]]*ConnFailTimeout" "${confFile}"`
    if [ ${ret1} -gt 1 ]
    then
        t260s_comment_iscsiconf "Multipath"
    fi

    if [ ${ret2} -gt 1 ]
    then
	t260s_comment_iscsiconf "ConnFailTimeout"
    fi

    return 0
}


function t260s_clear_iscsiConf()
{
    t260s_iscsiType=""
    if [ `which "iscsiadm" 2>/dev/null|grep -c "/iscsiadm"` -eq 1 ]
    then
        t260s_iscsiType="openIscsi"
    else
        t260s_iscsiType="iscsi"
    fi

    g10s_eval "t260s_clear_${t260s_iscsiType}"

    return 0
}

function t260s_get_fcconfName()
{
    g10s_getOSType
    
    if [ -f /sbin/dracut ] || [ "${g10s_OSVersion}" = "ROCKY_2.6.32.41" ] || [ "${g10s_OSVersion}" = "ROCKY6SP4" ] || [ "${g10s_OSVersion}" = "ROCKY4SP2" ] || [ "${g10s_OSVersion}" = "ROCKY6SP6" ]
    then
        t260s_modprobeConfName="/etc/modprobe.d/mppmodules.conf"
        if [ -f "${t260s_modprobeConfName}" ]
        then
            rm -rf "${t260s_modprobeConfName}"
        fi
        return 1
    elif [ -f /etc/modprobe.conf.local ]
    then
        t260s_modprobeConfName="/etc/modprobe.conf.local"
    elif [ -f /etc/modprobe.conf ]
    then
        t260s_modprobeConfName="/etc/modprobe.conf"
    else
        print_debug  "not found modprobe config."
        return 1
    fi
    return 0
}

function t260s_comment_fcconf()
{

    sed -i "/^[[:blank:]]*options[[:blank:]]*$1[[:blank:]]*$2/d" "${t260s_modprobeConfName}"

    print_debug "comment [options $1 $2] success in [${t260s_modprobeConfName}]."

    return 0
}

function t260s_check_fcconf()
{
    local ret1

    ret1=`grep -E -c "^[[:blank:]]*options[[:blank:]]*$1[[:blank:]]*$2" "${t260s_modprobeConfName}"`
    if [ ${ret1} -gt 1 ]
    then
        t260s_comment_fcconf "$1" "$2"
    fi

    return 0
}


function t260s_clear_fcupinfo()
{
    local ret1
    local ret2
    local rtn
    ret1=`grep -c -E "(BEGIN|END) UltraPath Driver Comments" "${t260s_modprobeConfName}"`

    if [ ${ret1} -eq 2 ]
    then
        sed -i '/BEGIN UltraPath Driver Comments/,/END UltraPath Driver Comments/d' "${t260s_modprobeConfName}" 1>/dev/null 2>&1
    fi

    ret2=`grep -c -E "(BEGIN|END) OF UltraPath Driver Changes" "${t260s_modprobeConfName}"`
    if [ ${ret2} -eq 2 ]
    then
        sed -i '/BEGIN OF UltraPath Driver Changes/,/END OF UltraPath Driver Changes/d' "${t260s_modprobeConfName}" 1>/dev/null 2>&1
    fi

    ret=`cat ${t260s_modprobeConfName} | grep -c  "upUpper"`
    if [ ${ret} -ne 0 ]
    then
        sed -i '/upUpper/d' ${t260s_modprobeConfName} 1>/dev/null 2>&1
    fi

    ret=`cat ${t260s_modprobeConfName} | grep -c  "upVhba"`
    if [ ${ret} -ne 0 ]
    then
        sed -i '/upVhba/d' ${t260s_modprobeConfName} 1>/dev/null 2>&1
    fi

    ret=`cat ${t260s_modprobeConfName} | grep -c  "max_report_luns"`
    if [ ${ret} -ne 0 ]
    then
        sed -i '/max_report_luns/d' ${t260s_modprobeConfName} 1>/dev/null 2>&1
    fi

    return 0

}

function t260s_clear_fcConf()
{
    print_debug  "begin clear fc Env."
    t260s_get_fcconfName
    if [ $? -eq 1 ]
    then
        return 0
    fi

    t260s_check_fcconf "qla2xxx" "ql2xfailover" "0"
    t260s_check_fcconf "qla2xxx" "qlport_down_retry" "5"
    t260s_check_fcconf "qla2xxx" "ql2xretrycount" "1"
    t260s_check_fcconf "lpfc" "lpfc_nodev_tmo" "5"

    t260s_clear_fcupinfo
    return 0
}

function t260s_main()
{
    t260s_clear_iscsiConf

    t260s_clear_fcConf

    t260s_clear_menulst

    return 0

}

g10s_rootCheck

t260s_main

print_debug "finish clear Env."




