#!/bin/bash

g10s_LIBFILE=${g10s_LIBFILE="../globalLib/g10s_pub"}
if [ -f "${g10s_LIBFILE}" ]
then
	. "${g10s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g10s_LIBFILE}]\033[0m"
	exit 1
fi

t270s_type="normal"
t270s_action="insert"


t270s_targetFile=${t270s_targetFile="/etc/rc.d/rc.sysinit"}


readonly t270s_tmpFile="${g10s_tmpPath}/t270s_rc.sysinit.tmp"

readonly t270s_allTypes=",normal,"
readonly t270s_UltraPathFlagBegin="###UltraPath code,please don't change this###begin###"
readonly t270s_UltraPathFlagEnd="###UltraPath code,please don't change this###end###"
readonly t270s_bakFile="${g10s_tmpPath}/t270s_rc.sysinit.bak"
readonly t270s_beforeRemoveFile="${g10s_tmpPath}/t270s_rc.sysinit.beforeRemove"

function t270s_useage()
{
    local allTypes
    allTypes=${t270s_allTypes//,/|}
    allTypes=${allTypes#|}
    allTypes=${allTypes%|}
    echo "usage:$0 type=XXX action=[insert|remove]
    action=[insert|remove] insert or remove the service
    type=${allTypes}"
    return 0
}


function t270s_insert_normal()
{
    local ret
    
    if [ "$1" = "" ]
    then
        print_debug "begin insert UltraPath code by normal type."
    else
        print_debug "begin repair UltraPath code by normal type."
    fi
    
    ret=`grep -E -c "(${t270s_UltraPathFlagBegin}|${t270s_UltraPathFlagEnd})" "${t270s_targetFile}"`
    if [ ${ret} -eq 2 ]
    then
        if [ "$1" = "" ]
        then
            print_warning "1001" "UltraPath code has been exist.[${t270s_targetFile}]"
        else
            print_debug "The boot service not need to repair.[${t270s_targetFile}]"
        fi
        return 0
    elif [ ${ret} -eq 1 ]
    then
        print_error "1002" "UltraPath has been destroyed.[${t270s_targetFile}]"
        exit 1
    fi

    if [ `grep -c "/sbin/start_udev" "${t270s_targetFile}"` -ne 1 ]
    then
        print_error "1003" "not found the key words in target file.[/sbin/start_udev,${t270s_targetFile}]"
        exit 1
    fi

    g10s_cp "${t270s_targetFile}" "${t270s_tmpFile}"

    g10s_getOSType
    g10s_getSysBit
    if [ "${g10s_sysbit}" == "x86_64" ] && [ "${g10s_OSVersion}" == "OL6U8" ]
    then
        sed -i "/\/sbin\/start_udev/i ${t270s_UltraPathFlagBegin}\\
if [ -f \/UltraPath\/install\/linuxTools\/globalLib\/g10s_pub ]\\
then\\
    if [ \`cat \/proc\/cmdline 2>\/dev\/null|grep -E -c \"[[:blank:]]NoUltraPath([[:blank:]#]|\$)\"\` -eq 0 ]\\
    then\\
        g10s_exitFun=\"return\"\\
        . \/UltraPath\/install\/linuxTools\/globalLib\/g10s_pub\\
        g10s_getPathForInstall\\
        tmpModulePath=\"\${g10s_PathForInstall}\"\\
        if [ -f \/UltraPath\/install\/kernelModules\/\${tmpModulePath}\/nxup.ko ] && [ -f \/UltraPath\/install\/kernelModules\/\${tmpModulePath}\/nxupext_a.ko ]\\
        then\\
            modprobe scsi_mod 2>\/dev\/null\\
            modprobe sd_mod 2>\/dev\/null\\
            modprobe sg 2>\/dev\/null\\
            g10s_waitSdProbe\\
            echo \"begin load nxup\"\\
            insmod \/UltraPath\/install\/kernelModules\/\${tmpModulePath}\/nxup.ko\\
            if [ \$? -ne 0 ]\\
            then\\
                echo \"UltraPath--ERROR:load nxup failed\"\\
            fi\\
             insmod \/UltraPath\/install\/kernelModules\/\${tmpModulePath}\/nxupext_a.ko\\
            if [ \$? -ne 0 ]\\
            then\\
                rmmod nxup\\
                echo \"UltraPath--ERROR:load nxupext_a failed\"\\
            fi\\
            upRescan -m\\
        fi\\
    else\\
        echo \"UltraPath--No UltraPath mode.\"\\
    fi\\
    if [ \`cat \/proc\/cmdline 2>\/dev\/null|grep -E -c \"[[:blank:]]UltraPathDEBUG([[:blank:]#]|\$)\"\` -ne 0 ]\\
    then\\
        export g10s_debugon=\"1\"\\
        echo \"UltraPath--UltraPath debug mode\"\\
    fi\\
fi\\
${t270s_UltraPathFlagEnd}" "${t270s_tmpFile}"
    else
        sed -i "/\/sbin\/start_udev/i ${t270s_UltraPathFlagBegin}\\
if [ -f \/UltraPath\/install\/linuxTools\/globalLib\/g10s_pub ]\\
then\\
    if [ \`cat \/proc\/cmdline 2>\/dev\/null|grep -E -c \"[[:blank:]]NoUltraPath([[:blank:]#]|\$)\"\` -eq 0 ]\\
    then\\
        g10s_exitFun=\"return\"\\
        . \/UltraPath\/install\/linuxTools\/globalLib\/g10s_pub\\
        g10s_getPathForInstall\\
        tmpModulePath=\"\${g10s_PathForInstall}\"\\
        if [ -f \/UltraPath\/install\/kernelModules\/\${tmpModulePath}\/nxup.ko ] && [ -f \/UltraPath\/install\/kernelModules\/\${tmpModulePath}\/nxupext_a.ko ]\\
        then\\
            modprobe scsi_mod 2>\/dev\/null\\
            modprobe sd_mod 2>\/dev\/null\\
            modprobe sg 2>\/dev\/null\\
            echo \"begin load nxup\"\\
            insmod \/UltraPath\/install\/kernelModules\/\${tmpModulePath}\/nxup.ko\\
            if [ \$? -ne 0 ]\\
            then\\
                echo \"UltraPath--ERROR:load nxup failed\"\\
            fi\\
             insmod \/UltraPath\/install\/kernelModules\/\${tmpModulePath}\/nxupext_a.ko\\
            if [ \$? -ne 0 ]\\
            then\\
                rmmod nxup\\
                echo \"UltraPath--ERROR:load nxupext_a failed\"\\
            fi\\
            upRescan -m\\
        fi\\
    else\\
        echo \"UltraPath--No UltraPath mode.\"\\
    fi\\
    if [ \`cat \/proc\/cmdline 2>\/dev\/null|grep -E -c \"[[:blank:]]UltraPathDEBUG([[:blank:]#]|\$)\"\` -ne 0 ]\\
    then\\
        export g10s_debugon=\"1\"\\
        echo \"UltraPath--UltraPath debug mode\"\\
    fi\\
fi\\
${t270s_UltraPathFlagEnd}" "${t270s_tmpFile}"
    fi

    if [ `grep -E -c "(${t270s_UltraPathFlagBegin}|${t270s_UltraPathFlagEnd})" "${t270s_tmpFile}"` -ne 2 ]
    then
        if [ "$1" = "" ]
        then
            print_error "1004" "insert UltraPath code failed.[${t270s_targetFile}]"
        else
            print_error "1004" "repair UltraPath code failed.[${t270s_targetFile}]"
        fi
        rm -f "${t270s_tmpFile}"
        exit 1
    else
        g10s_cp "${t270s_targetFile}" "${t270s_bakFile}"
        g10s_cp "${t270s_tmpFile}" "${t270s_targetFile}"
        rm -f "${t270s_tmpFile}"
    fi

    if [ "$1" = "" ]
    then
        print_event "1005" "The boot service is installed successfully."
    else
        print_debug "The boot service is repaired successfully."
    fi
    return 0
}

function t270s_repair_normal()
{
    t270s_insert_normal "repair"
    return 0
}

function t270s_remove_normal()
{
    local ret
    print_debug "begin remove UltraPath code by normal type.[${t270s_targetFile}]"
    ret=`grep -E -c "(${t270s_UltraPathFlagBegin}|${t270s_UltraPathFlagEnd})" "${t270s_targetFile}"`
    if [ ${ret} -eq 0 ]
    then
        print_warning "1006" "UltraPath code has been removed.[${t270s_targetFile}]"
        return 0
    elif [ ${ret} -eq 1 ]
    then
        print_error "1007" "UltraPath has been destroyed.[${t270s_targetFile}]"
        exit 1
    fi

    g10s_cp "${t270s_targetFile}" "${t270s_tmpFile}"
    sed -i "/${t270s_UltraPathFlagBegin}/,/${t270s_UltraPathFlagEnd}/d" "${t270s_tmpFile}"


    if [ `grep -E -c "(${t270s_UltraPathFlagBegin}|${t270s_UltraPathFlagEnd})" "${t270s_tmpFile}"` -ne 0 ]
    then
        print_error "1008" "remove UltraPath code failed.[${t270s_targetFile}]"
        rm -f "${t270s_tmpFile}"
        exit 1
    else
        g10s_cp "${t270s_targetFile}" "${t270s_beforeRemoveFile}"
        g10s_cp "${t270s_tmpFile}" "${t270s_targetFile}"
        rm -f "${t270s_tmpFile}"
    fi

    print_event "1009" "The boot service is removed successfully."
    return 0
}

function t270s_main()
{
    if [ `echo "${t270s_allTypes}"|grep -c ",${t270s_type},"` -eq 0 ]
    then
        print_error "1010" "unknown type.[${t270s_type}]"
        exit 1
    fi
    if [ "${t270s_action}" != "insert" ] && [ "${t270s_action}" != "remove" ] && [ "${t270s_action}" != "repair" ]
    then
        print_error "1011" "unknown action.[${t270s_action}]"
        exit 1
    fi

    g10s_eval "t270s_${t270s_action}_${t270s_type}"
    return 0
}


g10s_rootCheck
while [ $# -gt 0 ]
do
    t270s_param="$1"
    t270s_param1=${t270s_param%%=*}
    t270s_param2=${t270s_param##*=}
    print_debug "t270s_param=${t270s_param},t270s_param1=${t270s_param1},t270s_param2=${t270s_param2}"
    case "${t270s_param1}" in
        "type")
            t270s_type="${t270s_param2}"
            ;;
        "action")
            t270s_action="${t270s_param2}"
            ;;
        *)
            print_error "1012" "unknown parameter[${t270s_param}]"
            t170s_useage
            exit 1
            ;;
    esac
    shift
done

t270s_main

