#!/bin/bash

g10s_LIBFILE=${g10s_LIBFILE="../globalLib/g10s_pub"}
if [ -f "${g10s_LIBFILE}" ]
then
	. "${g10s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g10s_LIBFILE}]\033[0m"
	exit 1
fi

t271s_type="normal"
t271s_action="insert"

t271s_targetFile=${t271s_targetFile="/etc/systemd/system/sysinit.target.wants/systemd-nxup.service"}
t271s_targetFileLnk1=${t271s_targetFileLnk1="/usr/lib/systemd/system/systemd-nxup.service"}
t271s_targetFileLnk2=${t271s_targetFileLnk2="/usr/lib/systemd/system/sysinit.target.wants/systemd-nxup.service"}
t271s_runFile=${t271s_runFile="/usr/sbin/nxupBoot"}

readonly t271s_sysCtl="/usr/bin/systemctl"
readonly t271s_allTypes=",normal,DEEPIN15,UVPV2R3,"
readonly t271s_UltraPathFlagBegin="###UltraPath code,please don't change this###begin###"
readonly t271s_UltraPathFlagEnd="###UltraPath code,please don't change this###end###"

function t271s_useage()
{
    local allTypes
    allTypes=${t271s_allTypes//,/|}
    allTypes=${allTypes#|}
    allTypes=${allTypes%|}
    echo "usage:$0 type=XXX action=[insert|remove]
    action=[insert|remove] insert or remove the service
    type=${allTypes}"
    return 0
}


function t271s_insert_normal()
{
    local tmpModulePath

    if [ ! -f ${t271s_targetFile} ]
    then
        /usr/bin/touch ${t271s_targetFile}
        g10s_chmodExec 600 "${t271s_targetFile}"
    fi

    if [ ! -f ${t271s_runFile} ]
    then
        /usr/bin/touch ${t271s_runFile}
    fi
	
    echo "[Unit]
Description=Start Nxup Service
DefaultDependencies=no
Conflicts=no
After=systemd-udevd.service
Before=local-fs-pre.target

[Service]
ExecStart="${t271s_runFile}"
Type=oneshot

[Install]
WantedBy=sysinit.target" > ${t271s_targetFile}

	[ -n "${t271s_targetFileLnk1}" ] && ln -s ${t271s_targetFile} ${t271s_targetFileLnk1}
	[ -n "${t271s_targetFileLnk2}" ] && ln -s ${t271s_targetFile} ${t271s_targetFileLnk2}

    echo "#!/bin/bash
${t271s_UltraPathFlagBegin}
if [ -f /UltraPath/install/linuxTools/globalLib/g10s_pub ]
then
    if [ \`cat /proc/cmdline 2>/dev/null|grep -E -c \"[[:blank:]]NoUltraPath([[:blank:]#]|\$)\"\` -eq 0 ]
    then
        g10s_exitFun=\"return\"
        . /UltraPath/install/linuxTools/globalLib/g10s_pub
        g10s_getPathForInstall
        tmpModulePath=\"\${g10s_PathForInstall}\"
        if [ -f /UltraPath/install/kernelModules/\${tmpModulePath}/nxup.ko ] && [ -f /UltraPath/install/kernelModules/\${tmpModulePath}/nxupext_a.ko ]
        then
          if [ \`lsmod|awk '{print \$1}'|grep -E -c \"^[[:blank:]]*(nxupext_a|nxupext_b)[[:blank:]]*\$\"\` -eq 0 ]
            then
            modprobe scsi_mod 2>/dev/null
            modprobe sd_mod 2>/dev/null
            modprobe sg 2>/dev/null
            echo \"begin load nxup\"
            insmod /UltraPath/install/kernelModules/\${tmpModulePath}/nxup.ko
            if [ \$? -ne 0 ]
            then
                echo \"UltraPath--ERROR:load nxup failed\"
            fi
             insmod /UltraPath/install/kernelModules/\${tmpModulePath}/nxupext_a.ko
            if [ \$? -ne 0 ]
            then
                rmmod nxup
                echo \"UltraPath--ERROR:load nxupext_a failed\"
            fi
            upRescan -m
            udevadm settle 2>/dev/null
            vgchange -an --ignorelockingfailure 2>/dev/null
            vgchange -ay --ignorelockingfailure 2>/dev/null 
          else
             echo \"UltraPath already exist.\"
          fi
        fi
    else
        echo \"UltraPath--No UltraPath mode.\"
    fi
    if [ \`cat /proc/cmdline 2>/dev/null|grep -E -c \"[[:blank:]]UltraPathDEBUG([[:blank:]#]|\$)\"\` -ne 0 ]
    then
        export g10s_debugon=\"1\"
        echo \"UltraPath--UltraPath debug mode\"
    fi
fi
${t271s_UltraPathFlagEnd}" > ${t271s_runFile}
    g10s_chmodExec 500 ${t271s_runFile}

    if [ "$1" = "" ]
    then    
        print_event "1005" "The boot service is installed successfully."
    fi
    return 0
}

function t271s_insert_UVPV2R3_special()
{
     local tmpModulePath
     local tmpPwd

    if [ ! -f ${t271s_targetFile} ]
    then
        /usr/bin/touch ${t271s_targetFile}
        g10s_chmodExec 600 "${t271s_targetFile}"
    fi

    if [ ! -f ${t271s_runFile} ]
    then
        /usr/bin/touch ${t271s_runFile}
    fi
	
    echo "[Unit]
Description=Start Nxup Service
DefaultDependencies=no
After=systemd-udevd.service
Before=local-fs-pre.target

[Service]
ExecStart="${t271s_runFile}"
Type=oneshot

[Install]
WantedBy=sysinit.target" > ${t271s_targetFile}

	[ -n "${t271s_targetFileLnk1}" ] && ln -s ${t271s_targetFile} ${t271s_targetFileLnk1}
	[ -n "${t271s_targetFileLnk2}" ] && ln -s ${t271s_targetFile} ${t271s_targetFileLnk2}

    echo "#!/bin/bash
${t271s_UltraPathFlagBegin}
if [ -f /UltraPath/install/linuxTools/globalLib/g10s_pub ]
then
    if [ \`cat /proc/cmdline 2>/dev/null|grep -E -c \"[[:blank:]]NoUltraPath([[:blank:]#]|\$)\"\` -eq 0 ]
    then
        g10s_exitFun=\"return\"
        . /UltraPath/install/linuxTools/globalLib/g10s_pub
        g10s_getPathForInstall
        tmpModulePath=\"\${g10s_PathForInstall}\"
        if [ -f /UltraPath/install/kernelModules/\${tmpModulePath}/nxup.ko ] && [ -f /UltraPath/install/kernelModules/\${tmpModulePath}/nxupext_a.ko ]
        then
            if [ \`lsmod|awk '{print \$1}'|grep -E -c \"^[[:blank:]]*(nxupext_a|nxupext_b)[[:blank:]]*\$\"\` -eq 0 ]
            then
                modprobe scsi_mod 2>/dev/null
                modprobe sd_mod 2>/dev/null
                modprobe sg 2>/dev/null
                echo \"begin load nxup\"
                insmod /UltraPath/install/kernelModules/\${tmpModulePath}/nxup.ko
                if [ \$? -ne 0 ]
                then
                    echo \"UltraPath--ERROR:load nxup failed\"
                fi
                 insmod /UltraPath/install/kernelModules/\${tmpModulePath}/nxupext_a.ko
                if [ \$? -ne 0 ]
                then
                    rmmod nxup
                    echo \"UltraPath--ERROR:load nxupext_a failed\"
                fi
                upRescan -m
                udevadm settle 2>/dev/null
                vgchange -an --ignorelockingfailure 2>/dev/null
                vgchange -ay --ignorelockingfailure 2>/dev/null 
            else
                echo \"UltraPath already exist.\"
            fi
        fi
    else
        echo \"UltraPath--No UltraPath mode.\"
    fi
    if [ \`cat /proc/cmdline 2>/dev/null|grep -E -c \"[[:blank:]]UltraPathDEBUG([[:blank:]#]|\$)\"\` -ne 0 ]
    then
        export g10s_debugon=\"1\"
        echo \"UltraPath--UltraPath debug mode\"
    fi
fi
${t271s_UltraPathFlagEnd}" > ${t271s_runFile}
    g10s_chmodExec 500 ${t271s_runFile}
    
    tmpPwd=`pwd`
    cd /UltraPath/install/linuxTools/toolLib/
    bash t390s_checkSystemdConf.sh action=changeConf
    cd "${tmpPwd}"
    
    if [ "$1" = "" ]
    then    
        print_event "1005" "The boot service is installed successfully."
    fi
       
    return 0
}

function t271s_repair_normal()
{
    if [ ! -s ${t271s_targetFile} ] && [ ! -s ${t271s_runFile} ]
    then
        print_debug "The boot service need to repair.not exist or empty [${t271s_targetFile},${t271s_runFile}]"
        t271s_insert_normal "repair"
        print_debug "The boot service is repaired successfully.[repair file Error]"
        return 0
    fi
    
    if [ ! -L ${t271s_targetFileLnk1} ] && [ `cat "${t271s_runFile}" | grep -E -c "(${t271s_UltraPathFlagBegin}|${t271s_UltraPathFlagEnd})"` -ne 2 ]
    then
       rm -f ${t271s_targetFileLnk1}
       t271s_insert_normal "repair"
       print_debug "The boot service is repaired successfully.[repair file contect Error]"
    fi
    return 0
}

function t271s_remove_normal()
{
    local ret
    print_debug "begin remove UltraPath code by normal type.[${t271s_targetFile}]"
    
    
	if [ -f ${t271s_targetFile} ]
	then
	    rm -f ${t271s_targetFile}
	fi

	if [ "${t271s_targetFileLnk1}" != "" ]
	then
	    rm -f ${t271s_targetFileLnk1}
	fi

	if [ "${t271s_targetFileLnk2}" != "" ]
	then
	    rm -f ${t271s_targetFileLnk2}
	fi
	
	if [ -f ${t271s_runFile} ]
	then
	    rm -f ${t271s_runFile}
	fi

    print_event "1009" "The boot service is removed successfully."
    return 0
}

function t271s_insert_DEEPIN15()
{
    t271s_targetFile="/lib/systemd/system/systemd-nxup.service"
    t271s_targetFileLnk1="/lib/systemd/system/sysinit.target.wants/systemd-nxup.service"
    t271s_targetFileLnk2=""
    t271s_runFile="/usr/sbin/nxupBoot"
    t271s_insert_normal
    lineNum=`cat ${t271s_targetFile} | grep -n "Conflicts" | awk -F ":" '{print $1}'`
    sed -i "${lineNum}d" ${t271s_targetFile}
}

function t271s_repair_DEEPIN15()
{
    t271s_targetFile="/lib/systemd/system/systemd-nxup.service"
    t271s_targetFileLnk1="/lib/systemd/system/sysinit.target.wants/systemd-nxup.service"
    t271s_targetFileLnk2=""
    t271s_runFile="/usr/sbin/nxupBoot"
    t271s_repair_normal
    lineNum=`cat ${t271s_targetFile} | grep -n "Conflicts" | awk -F ":" '{print $1}'`
    sed -i "${lineNum}d" ${t271s_targetFile}
}

function t271s_remove_DEEPIN15()
{
    t271s_targetFile="/lib/systemd/system/systemd-nxup.service"
    t271s_targetFileLnk1="/lib/systemd/system/sysinit.target.wants/systemd-nxup.service"
    t271s_runFile="/usr/sbin/nxupBoot"
    t271s_remove_normal
}


function t271s_insert_UVPV2R3()
{
    t271s_targetFile="/usr/lib/systemd/system/systemd-nxup.service"
    t271s_targetFileLnk1="/etc/systemd/system/sysinit.target.wants/systemd-nxup.service"
    t271s_targetFileLnk2=""
    t271s_runFile="/usr/sbin/nxupBoot"
    t271s_insert_UVPV2R3_special
}

function t271s_repair_UVPV2R3()
{
    return 0   
}

function t271s_remove_UVPV2R3()
{
    t271s_targetFile="/usr/lib/systemd/system/systemd-nxup.service"
    t271s_targetFileLnk1="/etc/systemd/system/sysinit.target.wants/systemd-nxup.service"
    t271s_runFile="/usr/sbin/nxupBoot"
    t271s_remove_normal
}

function t271s_main()
{
    if [ `echo "${t271s_allTypes}"|grep -c ",${t271s_type},"` -eq 0 ]
    then
        print_error "1010" "unknown type.[${t271s_type}]"
        exit 1
    fi
    if [ "${t271s_action}" != "insert" ] && [ "${t271s_action}" != "remove" ] && [ "${t271s_action}" != "repair" ]
    then
        print_error "1011" "unknown action.[${t271s_action}]"
        exit 1
    fi

    g10s_eval "t271s_${t271s_action}_${t271s_type}"
    return 0
}


g10s_rootCheck
while [ $# -gt 0 ]
do
    t271s_param="$1"
    t271s_param1=${t271s_param%%=*}
    t271s_param2=${t271s_param##*=}
    print_debug "t271s_param=${t271s_param},t271s_param1=${t271s_param1},t271s_param2=${t271s_param2}"
    case "${t271s_param1}" in
        "type")
            t271s_type="${t271s_param2}"
            ;;
        "action")
            t271s_action="${t271s_param2}"
            ;;
        *)
            print_error "1012" "unknown parameter[${t271s_param}]"
            t170s_useage
            exit 1
            ;;
    esac
    shift
done

t271s_main

