#!/bin/bash

g10s_LIBFILE=${g10s_LIBFILE="../globalLib/g10s_pub"}
if [ -f "${g10s_LIBFILE}" ]
then
	. "${g10s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g10s_LIBFILE}]\033[0m"
	exit 1
fi

t280s_action=""



readonly t280s_nxupDir="90nxup"
readonly t280s_ultrapathDir="99ultrapath"



function t280s_useage()
{
    echo "$0 [OPTIONS]
    action=append|remove|repair"
    exit 1
}

function t280s_append()
{
    local tmpNxupDir
    if [ `echo "${g10s_OSVersion}"|grep -c -E "(RHEL7|OL7|SLES12|KYLIN7SP2)"` -eq 1 ]
    then
        g10s_checkExist  -d "/usr/lib/dracut/modules.d" 2
        tmpNxupDir="/usr/lib/dracut/modules.d/${t280s_nxupDir}"
    else
        g10s_checkExist  -d "/usr/share/dracut/modules.d" 2
        tmpNxupDir="/usr/share/dracut/modules.d/${t280s_nxupDir}"
    fi
    [ "${t280s_nxupDir}" != "" ] && rm -rf "${tmpNxupDir}"
    
    g10s_cp ./t280c_dracutsetup "${tmpNxupDir}"
    g10s_chmodExec 500 "${tmpNxupDir}/install"
    g10s_chmodExec 500 "${tmpNxupDir}/installkernel"
    
    if [ `echo "${g10s_OSVersion}"|grep -c -E "SLES12"` -eq 1 ]
    then
        g10s_checkExist  -d "/usr/lib/dracut/modules.d" 2
        tmpNxupDir="/usr/lib/dracut/modules.d/${t280s_ultrapathDir}"
        [ "${t280s_ultrapathDir}" != "" ] && rm -rf "${tmpNxupDir}"
        g10s_cp ./t281c_systemdModify "${tmpNxupDir}"
        g10s_chmodExec 500 "${tmpNxupDir}/install"
        g10s_chmodExec 500 "${tmpNxupDir}/installkernel"
    fi
    
    print_event "1001" "append nxup to dracut."
    return 0
}
function t280s_remove()
{
    if [ `echo "${g10s_OSVersion}"|grep -c -E "(RHEL7|OL7|SLES12|KYLIN7SP2)"` -eq 1 ]
    then
        [ "${t280s_nxupDir}" != "" ] && rm -rf "/usr/lib/dracut/modules.d/${t280s_nxupDir}"
    else
        [ "${t280s_nxupDir}" != "" ] && rm -rf "/usr/share/dracut/modules.d/${t280s_nxupDir}"
    fi
    
    if [ `echo "${g10s_OSVersion}"|grep -c -E "SLES12"` -eq 1 ]
    then
        [ "${t280s_ultrapathDir}" != "" ] && rm -rf "/usr/lib/dracut/modules.d/${t280s_ultrapathDir}"
    fi
    print_event "1002" "remove nxup from dracut."
    return 0
}
function t280s_repair()
{
    local tmpNxupDir
    if [ `echo "${g10s_OSVersion}"|grep -c -E "(RHEL7|OL7|SLES12|KYLIN7SP2)"` -eq 1 ]
    then
        tmpNxupDir="/usr/lib/dracut/modules.d/${t280s_nxupDir}"
    else
        tmpNxupDir="/usr/share/dracut/modules.d/${t280s_nxupDir}"
    fi
    
    if [ ! -f "${tmpNxupDir}/install" ] || [ ! -f "${tmpNxupDir}/installkernel" ] || [ ! -f "${tmpNxupDir}/nxup_pre_udev.sh" ]
    then
        t280s_remove
        t280s_append
    else
       print_debug "dracut has UltraPath code."
    fi
    return 0
}


g10s_getOSType
if [ `echo "${g10s_OSVersion}"|grep -c "SLES12"` -eq 0 ]
then
	readonly t280s_dracutFile="/sbin/dracut"
else
	readonly t280s_dracutFile="/usr/bin/dracut"
fi
g10s_checkExist -f "${t280s_dracutFile}" 2

while [ $# -gt 0 ]
do
    t280s_param="$1"
    t280s_param1=${t280s_param%%=*}
    t280s_param2=${t280s_param##*=}
    print_debug "t280s_param=${t280s_param},t280s_param1=${t280s_param1},t280s_param2=${t280s_param2}"
    case "${t280s_param1}" in
        "action")
            t280s_action="${t280s_param2}"
            ;;
        *)
            print_error "1003" "unknown parameter[${t280s_param}]"
            t280s_useage
            ;;
    esac
    shift
done

if [ "${t280s_action}" = "append" ]
then
    t280s_append
elif [ "${t280s_action}" = "remove" ]
then
    t280s_remove
elif [ "${t280s_action}" = "repair" ]
then
    t280s_repair
else
    print_error "1004" "unknown parameter[action=${t280s_action}]"
    t280s_useage
fi
exit 0
