#!/bin/bash

g20s_LIBFILE=${g20s_LIBFILE="../globalLib/g20s_install"}
if [ -f "${g20s_LIBFILE}" ]
then
	. "${g20s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g20s_LIBFILE}]\033[0m"
	exit 1
fi


t290s_action="insert"
t290s_type=""
readonly t290s_allTypes=",suseService,suse12Service,redhatService,suseMkinitrd,suse12Mkinitrd,redhatMkinitrd,rocky6Service,rocky6Mkinitrd,DEEPIN15Service,DEEPIN15Mkinitrd,"

readonly t290s_serviceName="nxup"

t290s_targetFile="/usr/lib/systemd/system/nxup.service"
t290s_targetLnFile="/etc/systemd/system/nxup.service"
t290s_targetLn2File="/etc/systemd/system/multi-user.target.wants/nxup.service"

function t290s_useage()
{
    local allTypes
    allTypes=${t290s_allTypes//,/|}
    allTypes=${allTypes#|}
    allTypes=${allTypes%|}
    echo "$0 [action=insert|remove] [type=XXX]
    action  insert or remove run service.
    type    ${allTypes}"
    exit 1
}


function t290s_remove_normal()
{
    if [ "$1" = "" ] || [ "$2" = "" ]
    then
        print_error "2" "param error."
        exit 1
    fi
    g10s_checkExist -f "t330s_serviceMgr.sh" 2
    bash t330s_serviceMgr.sh action=del "serviceFile=${2}"
    if [ $? -ne 0 ]
    then
        print_warning "1001" "remove service failed.[${2}]"
    fi
    g10s_checkExist -d "$1" 2
    rm -f "${1}/${2}"
    return 0
}


function t290s_insert_normal()
{
    if [ "$1" = "" ] || [ "$2" = "" ] || [ "$3" = "" ]
    then
        print_error "2" "param error."
        exit 1
    fi
    g10s_checkExist -d "$2" 2
    g10s_cp "t300c_runServices/nxup.${1}" "${2}/${3}"
    g10s_chmodExec 500 "${2}/${3}"
    g10s_checkExist -f "t330s_serviceMgr.sh" 2
    bash t330s_serviceMgr.sh action=add "serviceFile=${3}"
    if [ $? -ne 0 ]
    then
        print_error "1002" "insert service failed.[${3}]"
        exit 1
    fi
    return 0
}

function t290s_insert_file()
{
     if [ "$1" = "" ] || [ "$2" = "" ] || [ "$3" = "" ]
    then
        print_error "2" "param error."
        exit 1
    fi
    g10s_checkExist -d "$2" 2
    g10s_cp "t300c_runServices/nxup.${1}" "${2}/${3}"
    g10s_chmodExec 500 "${2}/${3}"
    return 0
}
function t290s_remove_file()
{
    if [ "$1" = "" ] || [ "$2" = "" ]
    then
        print_error "2" "param error."
        exit 1
    fi
    g10s_checkExist -d "$1" 2
    rm -f "${1}/${2}"
    return 0
}
function t290s_insert_suseService()
{
    t290s_insert_normal "${t290s_type}" "/etc/init.d" "${t290s_serviceName}"
    return $?
}
function t290s_remove_suseService()
{
    t290s_remove_normal "/etc/init.d" "${t290s_serviceName}"
    return $?
}
function t290s_insert_suse12Service()
{
    t290s_insert_file "${t290s_type}Special" "/etc/init.d" "${t290s_serviceName}"
	g10s_cp "t300c_runServices/nxup.${t290s_type}" /usr/sbin/nxupServBoot
	g10s_chmodExec 500 /usr/sbin/nxupServBoot
	echo "[Unit]
	Description=Start Nxup Service
	DefaultDependencies=no
	Conflicts=no
	After=systemd-journald.service  systemd-nxup.service
	[Service]
	RemainAfterExit=yes
	ExecStart=/usr/sbin/nxupServBoot start
	ExecStop=/usr/sbin/nxupServBoot stop
	Type=oneshot 
	[Install]
	WantedBy=multi-user.target"> ${t290s_targetFile}
	ln -s ${t290s_targetFile}  ${t290s_targetLnFile}
	ln -s ${t290s_targetFile}  ${t290s_targetLn2File}
    return $?
}
function t290s_remove_suse12Service()
{
	rm -rf ${t290s_targetFile}
	rm -rf ${t290s_targetLnFile}
	rm -rf ${t290s_targetLn2File}
    t290s_remove_file "/etc/init.d" "${t290s_serviceName}"
    return $?
}
function t290s_insert_redhatService()
{
    t290s_insert_normal "${t290s_type}" "/etc/init.d" "${t290s_serviceName}"
    return $?
}
function t290s_remove_redhatService()
{
    t290s_remove_normal "/etc/init.d" "${t290s_serviceName}"
    return $?
}

function t290s_insert_suseMkinitrd()
{
    t290s_insert_normal "${t290s_type}" "/etc/init.d" "${t290s_serviceName}"
    return $?
}
function t290s_remove_suseMkinitrd()
{
    t290s_remove_normal "/etc/init.d" "${t290s_serviceName}"
    return $?
}
function t290s_insert_suse12Mkinitrd()
{
	g10s_cp "t300c_runServices/nxup.${t290s_type}" /usr/sbin/nxupServBoot
	g10s_chmodExec 500 /usr/sbin/nxupServBoot
	echo "[Unit]
	Description=Start Nxup Service
	DefaultDependencies=no
	Conflicts=no
	After=systemd-journald.service
	Before=systemd-udevd.service
	[Service]
	RemainAfterExit=yes
	ExecStart=/usr/sbin/nxupServBoot start
	ExecStop=/usr/sbin/nxupServBoot stop
	Type=oneshot 
	[Install]
	WantedBy=multi-user.target network-online.target"> ${t290s_targetFile}
	ln -s ${t290s_targetFile}  ${t290s_targetLnFile}
	ln -s ${t290s_targetFile}  ${t290s_targetLn2File}
    return $?
}
function t290s_remove_suse12Mkinitrd()
{
	rm -rf ${t290s_targetFile}
	rm -rf ${t290s_targetLnFile}
	rm -rf ${t290s_targetLn2File}
    return $?
}
function t290s_insert_redhatMkinitrd()
{
    t290s_insert_normal "${t290s_type}" "/etc/init.d" "${t290s_serviceName}"
    return $?
}
function t290s_remove_redhatMkinitrd()
{
    t290s_remove_normal "/etc/init.d" "${t290s_serviceName}"
    return $?
}

function t290s_insert_DEEPIN15Service()
{
    t290s_targetFile="/lib/systemd/system/nxup.service"
    t290s_targetLnFile="/etc/systemd/system/nxup.service"
    t290s_targetLn2File="/etc/systemd/system/multi-user.target.wants/nxup.service"
    
    t290s_insert_file ${t290s_type} "/etc/init.d" "${t290s_serviceName}"
    g10s_cp "t300c_runServices/nxup.${t290s_type}" /usr/sbin/nxupServBoot
    g10s_chmodExec 500 /usr/sbin/nxupServBoot
    echo "[Unit]
Description=Start Nxup Service
DefaultDependencies=no
After=network.target
[Service]
RemainAfterExit=yes
ExecStart=/usr/sbin/nxupServBoot start
ExecStop=/usr/sbin/nxupServBoot stop
Type=oneshot 
[Install]
WantedBy=multi-user.target"> ${t290s_targetFile}
    ln -s ${t290s_targetFile}  ${t290s_targetLnFile}
    ln -s ${t290s_targetFile}  ${t290s_targetLn2File}
    systemctl daemon-reload >/dev/null 2>&1
    if [ $? -ne 0 ]
    then
        print_debug "reload systemctl daemon failed."
    else
        print_debug "reload systemctl daemon succ."
    fi
    return $?
}
function t290s_remove_DEEPIN15Service()
{
    t290s_targetFile="/lib/systemd/system/nxup.service"
    t290s_targetLnFile="/etc/systemd/system/nxup.service"
    t290s_targetLn2File="/etc/systemd/system/multi-user.target.wants/nxup.service"
    rm -rf ${t290s_targetFile}
    rm -rf ${t290s_targetLnFile}
    rm -rf ${t290s_targetLn2File}
    t290s_remove_file "/etc/init.d" "${t290s_serviceName}"
    
    if [ -f /usr/sbin/nxupServBoot ]
    then
        rm -rf /usr/sbin/nxupServBoot
    fi
    
    systemctl daemon-reload >/dev/null 2>&1
    if [ $? -ne 0 ]
    then
        print_debug "reload systemctl daemon failed."
    else
        print_debug "reload systemctl daemon succ."
    fi
    return $?
}
function t290s_insert_DEEPIN15Mkinitrd()
{
    t290s_targetFile="/lib/systemd/system/nxup.service"
    t290s_targetLnFile="/etc/systemd/system/nxup.service"
    t290s_targetLn2File="/etc/systemd/system/multi-user.target.wants/nxup.service"
    
    g10s_cp "t300c_runServices/nxup.${t290s_type}" /usr/sbin/nxupServBoot
    g10s_chmodExec 500 /usr/sbin/nxupServBoot
    echo "[Unit]
Description=Start Nxup Service
DefaultDependencies=no
After=network.target
[Service]
RemainAfterExit=yes
ExecStart=/usr/sbin/nxupServBoot start
ExecStop=/usr/sbin/nxupServBoot stop
Type=oneshot 
[Install]
WantedBy=multi-user.target"> ${t290s_targetFile}
    ln -s ${t290s_targetFile}  ${t290s_targetLnFile}
    ln -s ${t290s_targetFile}  ${t290s_targetLn2File}
    systemctl daemon-reload >/dev/null 2>&1
    if [ $? -ne 0 ]
    then
        print_debug "reload systemctl daemon failed."
    else
        print_debug "reload systemctl daemon succ."
    fi
    return $?
}
function t290s_remove_DEEPIN15Mkinitrd()
{
    t290s_targetFile="/lib/systemd/system/nxup.service"
    t290s_targetLnFile="/etc/systemd/system/nxup.service"
    t290s_targetLn2File="/etc/systemd/system/multi-user.target.wants/nxup.service"
    rm -rf ${t290s_targetFile}
    rm -rf ${t290s_targetLnFile}
    rm -rf ${t290s_targetLn2File}
    
    if [ -f /usr/sbin/nxupServBoot ]
    then
        rm -rf /usr/sbin/nxupServBoot
    fi    
    
    systemctl daemon-reload >/dev/null 2>&1
    if [ $? -ne 0 ]
    then
        print_debug "reload systemctl daemon failed."
    else
        print_debug "reload systemctl daemon succ."
    fi
    return $?
}
function t290s_insert_rocky6Service()
{
    t290s_insert_normal "rocky6Service" "/etc/init.d" "${t290s_serviceName}"
    return $?
}
function t290s_remove_rocky6Service()
{
    t290s_remove_normal "/etc/init.d" "${t290s_serviceName}"
    return $?
}
function t290s_insert_rocky6Mkinitrd()
{
    t290s_insert_normal "rocky6Mkinitrd" "/etc/init.d" "${t290s_serviceName}"
    return $?
}
function t290s_remove_rocky6Mkinitrd()
{
    t290s_remove_normal "/etc/init.d" "${t290s_serviceName}"
    return $?
}

while [ $# -gt 0 ]
do
    t290s_param="$1"
    t290s_param1=${t290s_param%%=*}
    t290s_param2=${t290s_param##*=}
    print_debug "t290s_param=${t290s_param},t290s_param1=${t290s_param1},t290s_param2=${t290s_param2}"
    case "${t290s_param1}" in
        "action")
            t290s_action="${t290s_param2}"
            ;;
        "type")
            t290s_type="${t290s_param2}"
            ;;
        *)
            print_error "1005" "unknown parameter[${t290s_param}]"
            t290s_useage
            ;;
    esac
    shift
done

if [ `echo "${t290s_allTypes}"|grep -c ",${t290s_type},"` -eq 0 ]
then
    print_error "1006" "unknown parameter[type=${t290s_type}]"
    t290s_useage
fi

if [ "${t290s_action}" != "insert" ] && [ "${t290s_action}" != "remove" ]
then
    print_error "1007" "unknown parameter[action=${t300s_action}]"
    t290s_useage
    exit 1
fi


g10s_eval "t290s_${t290s_action}_${t290s_type}"
if [ "${t290s_action}" = "insert" ]
then
    g10s_saveConf "${g10s_ultraPathPackagesInfo}" "t290s_runServType" "${t290s_type}"
    print_event "1003" "The running service is installed successfully." 
else
    print_event "1004" "The running service is deleted successfully." 
fi    
 

