#!/bin/bash

g20s_LIBFILE=${g20s_LIBFILE="../globalLib/g20s_install"}
g30s_LIBFILE=${g30s_LIBFILE="../globalLib/g30s_systemEnvironment"}
if [ -f "${g20s_LIBFILE}" ] && [ -f "${g30s_LIBFILE}" ]
then

	. "${g20s_LIBFILE}"

    . "${g30s_LIBFILE}"

else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib\033[0m"
	exit 1
fi


t300s_action_start="start"
t300s_action_stop="stop"
t300s_type=""
t300s_OPER_TYPE=1396789565
t300s_hostInfo="/etc/up.hostinfo"
t300s_hasWarning="0"
t300s_filelockFile="/UltraPath/install/otherTools/t300s_shell.lock"
t300s_hasFilelockFile="false"
t300s_lockFile="/var/lock/t300s_shell.lock"
t100s_lockFile="/var/lock/t100s_shell.lock"
i10s_lockFile="/var/lock/i10s_install.shell.lock"
readonly t300s_nxupLunMap_a="/proc/nxup_lun_map_a"
readonly t300s_nxupLunMap_b="/proc/nxup_lun_map_b"
readonly t300s_nxupScsiDir="/sys/bus/scsi/drivers/NXUP"
readonly t300s_ScsiSdDir="/sys/bus/scsi/drivers/sd"
t300s_tmpLunMap="/nxup_lun_map"
t300s_LUNIDS=""
function t300s_lockShell()
{
    local thepid=""
    local ret=""

    if [ "$1" = "" ]
    then
        print_error "2" "need a param"
        exit 1
    fi
    print_debug "lock shell[${1}]"
    if [ -f "${1}" ]
    then
        thepid=`cat "${1}"`
        if [ "${thepid}" != "" ]
        then
            ret=`ps ax |awk '{print $1}'| grep -c "^[[:blank:]]*${thepid}[[:blank:]]*$"`
            if [ ${ret} -eq 1 ]
            then
                print_error "18" "this shell is already running, exit."
                exit 1
            fi
        fi
    fi
    echo $$ > "${1}"
    return 0
}


function t300s_unlockShell()
{
    if [ "$1" = "" ]
    then
        print_error "2" "need a param"
        exit 1
    fi
    if [ -f "${1}" ]
    then
    	print_debug "unlock shell[${1}]"
    	rm -f "${1}"
    fi
    return 0
}

function t300s_exit()
{
    t300s_unlockShell "${t300s_lockFile}"
    exit "$1"
}

function t300s_useage()
{
    echo "$0 [action=start|stop] [type=service|mkinitrd]"
    exit 1
}


function t300s_checkOldUltraPath()
{
    if [ `lsmod|awk '{print $1}'|grep -E -c "^[[:blank:]]*(upUpper|upVhba)[[:blank:]]*$"` -ne 0 ]
    then
        print_error "1001" "The old UltraPath driver has been detected.please reboot the system."
        exit 1
    fi
    return 0
}

function t300s_analyseCmdline()
{
    if [ `cat /proc/cmdline 2>/dev/null|grep -E -c "[[:blank:]]NoUltraPath([[:blank:]#]|$)"` -ne 0 ]
    then
        print_warning "1002" "No UltraPath mode"
        exit 0
    fi
    if [ `cat /proc/cmdline 2>/dev/null|grep -E -c "[[:blank:]]UltraPathDEBUG([[:blank:]#]|$)"` -ne 0 ]
    then
        export g10s_debugon="1"
        print_debug "UltraPath debug mode"
    fi
    return 0
}

function t300s_serviceNxupLock()
{
    [ ! -f "${t300s_filelockFile}" ] && touch "${t300s_filelockFile}" 1>/dev/null 2>&1 && chmod 600 "${t300s_filelockFile}"

    if [ -f "${t300s_filelockFile}" ] && [ -f  /UltraPath/install/otherTools/shell_flock ]
    then
        chmod 500 /UltraPath/install/otherTools/shell_flock
        exec 6<>"${t300s_filelockFile}"
        /UltraPath/install/otherTools/shell_flock -x 6 1>/dev/null 2>&1
        if [ $? -eq 0 ]
        then
            t300s_hasFilelockFile="true"
        fi
    fi
    return 0
}

function t300s_serviceNxupunLock()
{
    if [ ${t300s_hasFilelockFile} == "true" ]
    then
        /UltraPath/install/otherTools/shell_flock -u 6 1>/dev/null 2>&1
    fi
}

function t300s_startUpRescan()
{
    local number
    number=`ps x|grep "upservice" | grep -v "grep"|wc -l`
    if [ ${number} -lt 1 ]
    then
        upRescan -su
    fi
    return 0
}

function t300s_startBootService()
{
    local ret

    if [ `lsmod|awk '{print $1}'|grep -E -c "^[[:blank:]]*(nxupext_a|nxupext_b)[[:blank:]]*$"` -ne 0 ]
    then
        print_event "1021" "nxup module already exists."
    else
        export g10s_logDir
        export g10s_logName
        export g10s_logFile
        export g10s_tmpPath
        export g10s_bakPath
        export g10s_modifySysConfAuto
        export g10s_debugon
        export g10s_closeMsgHead
        export g10s_skipPubMain
        bash t160s_boot.ultrapath nxupstart
        ret=$?
        if [ ${ret} -ne 0 ]
        then
            if [ ${ret} -eq 124 ]
            then
               exit 1
            fi
			print_oper "[execute ${t300s_action_start} ${t300s_type}]" "[0]" "$g10s_LEVEL_ERROR" "[execute start UltraPath servcie] [·]" "[$t300s_OPER_TYPE]"
            print_error "1003" "execute boot service failed."
            rmmod nxupext_a
            rmmod nxup
            exit 1
        fi
    fi
    t300s_mknodNxup
    return $?
}

function t300s_startUpService()
{
    local number
    number=`ps x|grep "upservice" | grep -v "grep"|wc -l`
    if [ ${number} -lt 1 ]
    then
        chmod 500 /UltraPath/install/otherTools/upservice
        /UltraPath/install/otherTools/upservice
        if [ $? -ne 0 ]
        then
            print_warning "1005" "start upservice failed."
            print_oper "[execute ${t300s_action_start} ${t300s_type}]" "[0]" "$g10s_LEVEL_ERROR" "[execute start UltraPath servcie] [·]" "[$t300s_OPER_TYPE]"
            t300s_hasWarning="1"
            return 1
        fi
        print_event "1004" "start upservice"
    fi
    return 0
}

function t300s_killUpService()
{
    local number
    local upsrvpid
    number=`ps x|grep "upservice" | grep -c "UltraPath"`
    if [ ${number} -ge 1 ]
    then
        killall upservice 2>/dev/null
        if [ $? -ne 0 ]
        then
            upsrvpid=`ps x|grep "upservice" | grep  "UltraPath" | awk '{print $1}'`
            kill -9 ${upsrvpid} 2>/dev/null
            if [ $? -ne 0 ]
            then
                print_error "1018" "kill upservice failed."
				print_oper "[execute ${t300s_action_stop} ${t300s_type}]" "[0]" "$g10s_LEVEL_ERROR" "[execute stop UltraPath servcie] [ֹͣ·]" "[$t300s_OPER_TYPE]"
                t300s_exit 1
            fi
        fi
    else
        print_debug "upservice num=${number}"
    fi
    return 0
}

function t300s_updatetimezone()
{
    local ret
    ${g10s_R5upadm} updatetimezone >/dev/null
    ret=$?
    if [ ${ret} -ne 0 ]
    then
        print_warning "1005" "update timezone failed."
        t300s_hasWarning="1"
        return 1
    fi
    return 0
}

function t300s_createOsInfo()
{
    local ret1
    local ret2
    g10s_checkExist -f "t130s_getOSInfo.sh" 2
    bash t130s_getOSInfo.sh "${t300s_hostInfo}" >/dev/null
    ret1=$?
    if [ ${ret1} -ne 0 ]
    then
        print_warning "1017" "create OS info failed."
        t300s_hasWarning="1"
        return 1
    fi
    return 0
}

function t300s_mknodNxup()
{
    local major

    major=`cat /proc/devices |grep -E "[[:blank:]]nxup([[:blank:]]|$)"|awk '{print $1}'`
    major=$(echo ${major})
    if [ "${major}" = "" ]
    then
        print_error "1019" "get nxup major number failed."
        exit 1
    fi
    if [ -e /dev/nxup ]
    then
        [ `ls -l /dev/nxup |grep -c "${major},"` -ne 0 ] && return 0
        rm -f /dev/nxup
    fi
    mknod /dev/nxup c ${major} 0
    if [ $? -ne 0 ]
    then
        print_error "1020" "mknod /dev/nxup failed."
        exit 1
    fi
}

function t300s_rmnodNxup()
{
    [ -e /dev/nxup ] && rm -f /dev/nxup
}

function t300s_StartcheckRDAC()
{
	if [ `lsmod | grep -c mppUpper` -ne 0 ]
	then
	    print_event "1022" "Detect RDAC."
	    echo "RDAC_exsit=1" > /opt/UltraPath/backup/compatibility
	fi
}


function t300s_StopcheckRDAC()
{
	if [ ! -f /opt/UltraPath/backup/compatibility ]
	then
	    if [ `lsmod | grep -c mppUpper` -ne 0 ] &&  [ `lsmod|awk '{print $1}'|grep -c "^[[:blank:]]*nxup[[:blank:]]*$"` -ne 0 ]
	    then
	        return 1
	    fi
	else
	    rm /opt/UltraPath/backup/compatibility
	fi
	return 0
}

function t300s_start_udev()
{
    if [ `/bin/ps x | grep -c "udev"` -lt 2 ]
    then
        g10s_getOSType
        if [ "${g10s_OSVersion}" != "RHEL4" ] && [ `echo "${g10s_OSVersion}" | grep -c "SLES9"` -eq 0  ]
        then
            if [ `echo "${g10s_OSVersion}" | grep -c "SLES"` -eq 1 ] || [ `echo "${g10s_OSVersion}" | grep -c "UVP"` -eq 1 ]
            then
                /etc/init.d/boot.udev restart 1>/dev/null 2>&1
            elif [ `echo "${g10s_OSVersion}" | grep -c "ROCKY"` -eq 1 ]
            then
                /etc/init.d/udev start 1>/dev/null 2>&1
            else
                start_udev 1>/dev/null 2>&1
            fi
        fi
    fi
    print_debug "Start udev successfully."
    return 0
}
function t300s_start_service()
{
    t300s_analyseCmdline
    t300s_checkOldUltraPath
    t300s_serviceNxupLock
    t300s_startBootService
    t300s_startUpRescan
    t300s_startUpService
    t300s_StartcheckRDAC

    g20s_repairService
    t300s_serviceNxupunLock
    t300s_start_udev
    print_event "1006" "The UltraPath starts successfully."
	print_oper "[execute ${t300s_action_start} ${t300s_type}]" "[1]" "$g10s_LEVEL_INFO" "[execute start UltraPath servcie] [·]" "[$t300s_OPER_TYPE]"
    return 0
}

function t300s_check_upRescan()
{
    local thepid=""
    local ret=""

    if [ -f "${t100s_lockFile}" ]
    then
    	thepid=`cat "${t100s_lockFile}" 2>/dev/null`
    if [ "${thepid}" != "" ]
    then
    	ret=`ps ax |awk '{print $1}'| grep -c "^[[:blank:]]*${thepid}[[:blank:]]*$"`
    	if [ ${ret} -eq 1 ]
    	then
			print_oper "[execute ${t300s_action_stop} ${t300s_type}]" "[0]" "$g10s_LEVEL_ERROR" "[execute stop UltraPath servcie] [ֹͣ·]" "[$t300s_OPER_TYPE]"
        	print_warning "1027" "upRescan is running, exit."
        	t300s_exit 1
    	fi
    fi
    	rm -f "${t100s_lockFile}"
    fi
}
function t300s_check_install()
{
    local thepid=""
    local ret=""

    if [ -f "${i10s_lockFile}" ]
    then
    	thepid=`cat "${i10s_lockFile}" 2>/dev/null`
    if [ "${thepid}" != "" ]
    then
    	ret=`ps ax |awk '{print $1}'| grep -c "^[[:blank:]]*${thepid}[[:blank:]]*$"`
    	if [ ${ret} -eq 1 ]
    	then
			print_oper "[execute ${t300s_action_stop} ${t300s_type}]" "[0]" "$g10s_LEVEL_ERROR" "[execute stop UltraPath servcie] [ֹͣ·]" "[$t300s_OPER_TYPE]"
        	print_warning "1027" "install is running, exit."
        	t300s_exit 1
    	fi
    fi
    	rm -f "${t100s_lockFile}"
    fi
}

function t100s_checkExist()
{
    if [ -z "$1" ] || [ -z "$2" ] || [ -z "$3" ]
    then
        print_error "1" "param error[$1,$2,$3]"
        t100s_exit 1
    fi

    if [ "$1" "$2" ]
    then

        if [ "$3" = "1" ]
        then
            print_error "4" "file or directory is exist[$2]"
            t100s_exit 1
        fi
    else

        if [ "$3" = "2" ]
        then
            print_error "5" "file or directory is not exist[$2]"
            t100s_exit 1
        fi
    fi
    return 0
}
function t100s_labelToIDS()
{
    local label=`echo "$1"`
    if [ `echo "${label}"|grep -E -c "^(sd[a-z]+|-[0-9]+)$"` -ne 0 ]
    then
        t300s_LUNIDS=`ls -l /sys/block/${label}/device|sed  "s/^.*\/\([0-9][0-9]*:[0-9][0-9]*:[0-9][0-9]*:[0-9][0-9]*\).*$/\1/"`
    else
        t300s_LUNIDS="$1"
    fi


    print_debug "t300s_LUNIDS=${t300s_LUNIDS}"
    if [ `echo "${t300s_LUNIDS}"|grep -E -c "^[0-9]+:[0-9]+:[0-9]+:[0-9]+$"` -ne 1 ]
    then
        print_error "1001" "LUNIDS must be H:C:T:L or sd*."
        t300s_exit 1
    fi
    return 0
}
function t300s_testExist()
{
    local host=""
    local channel=""
    local id=""
    local lun=""
    local grepstr=""

    t100s_labelToIDS "$1"
    t100s_checkExist -d "/sys/class/scsi_device/" 2

    if [ -d /sys/class/scsi_device/${t300s_LUNIDS} ]
    then
        return 1
    else
        return 0
    fi
    return 0
}
function t300s_removePHYLUN()
{


    local LunId=""
    local wait_all_sec=99
    local waited=0
    local wait_sec=0.1

    t100s_checkExist -d "${t300s_nxupScsiDir}" 2

        LunId=`ls ${t300s_nxupScsiDir} | grep -E "^[0-9]+:[0-9]+:[0-9]+:[0-9]+$"`
        LunId=$(echo ${LunId})

        for tempLUNid in ${LunId}
        do
           t300s_testExist ${tempLUNid}
           ret=$?
           if [ ${ret} -eq 0 ]
           then
              return 0
           fi

           if [ -f /sys/class/scsi_device/${tempLUNid}/device/delete ]
           then
              echo 1 > /sys/class/scsi_device/${tempLUNid}/device/delete
          fi

          while [ 1 -eq 1 ]
          do
             t300s_testExist ${tempLUNid}
             ret=$?
             if [ ${waited} -ge ${wait_all_sec} ] || [ ${ret} -eq 0 ]
             then
                 break
             fi

             sleep ${wait_sec}
             let "waited=${waited} + 1"
          done
          waited=0


           t300s_testExist ${tempLUNid}
          ret=$?
           if [ ${ret} -ne 0 ]
           then
               print_debug "1004" "delete lun failed.[${tempLUNid}]"
               return 1
           else
               print_debug "1005" "delete lun successfully.[${tempLUNid}]"
            fi
        done
    return 0
}

function t300s_removeAllNotManagePhyLuns
{
   local ret
   local ext_mod_name
   if [ `lsmod|awk '{print $1}'|grep -c "^[[:blank:]]*nxupext_a[[:blank:]]*$"` -ne 0 ]
   then
        ext_mod_name=nxupext_a
   else
        ext_mod_name=nxupext_b
   fi

   if [ `lsmod|awk '{print $1}'|grep -c "^[[:blank:]]*${ext_mod_name}[[:blank:]]*$"` -gt 0 ]
   then
      if [ `lsmod |grep "^[[:blank:]]*${ext_mod_name}" | awk '{print $3}'` -gt 1 ]
      then
          return 1
      fi
   fi
   if [ `lsmod |grep "^[[:blank:]]*nxup[[:blank:]]" | awk '{print $3}'` -gt 1 ]
   then
       return 1
   fi
   t300s_removePHYLUN
   ret=$?
   return $ret
}

function t300s_stop_service()
{
    t300s_serviceNxupLock
    t300s_lockShell "${t300s_lockFile}"

    local ext_mod_name

    t300s_check_upRescan
    t300s_check_install
    t300s_StopcheckRDAC
    if [ $? -ne 0 ]
    then
        print_warning "1008" "Stop failed! Please uninstall RDAC first."
		print_oper "[execute ${t300s_action_stop} ${t300s_type}]" "[0]" "$g10s_LEVEL_ERROR" "[execute stop UltraPath servcie] [ֹͣ·]" "[$t300s_OPER_TYPE]"
        t300s_exit 1
    fi

    t300s_createOsInfo
    t300s_killUpService
    g20s_repairService
    t300s_rmnodNxup
    if [ `lsmod|awk '{print $1}'|grep -c "^[[:blank:]]*nxupext_a[[:blank:]]*$"` -ne 0 ]
    then
        ext_mod_name=nxupext_a
    else
        ext_mod_name=nxupext_b
    fi
    if [ `lsmod|awk '{print $1}'|grep -c "^[[:blank:]]*${ext_mod_name}[[:blank:]]*$"` -ne 0 ] || [ `lsmod|awk '{print $1}'|grep -c "^[[:blank:]]*nxup[[:blank:]]*$"` -ne 0 ]
    then
        t300s_removeAllNotManagePhyLuns
        if [ $? -ne 0 ]
        then
            print_debug "delete physic lun failed."
        fi

        if [ `lsmod |grep "^[[:blank:]]*nxup[[:blank:]]" | awk '{print $3}'` -gt 1 ]
        then
            print_warning "1008" "Stop failed!UltraPath nxup is being used!"
            t300s_unlockShell "${t300s_lockFile}"
            t300s_mknodNxup
            t300s_startUpService
            t300s_hasWarning="1"
        else
            if [ `lsmod|awk '{print $1}'|grep -c "^[[:blank:]]*${ext_mod_name}[[:blank:]]*$"` -ne 0 ]
            then
                rmmod ${ext_mod_name} 2>/dev/null
                if [ $? -ne 0 ]
                then
                    print_warning "1008" "Stop failed!UltraPath nxupext is being used!"
                    t300s_unlockShell "${t300s_lockFile}"
                    t300s_mknodNxup
                    t300s_startUpService
                    t300s_hasWarning="1"
                else
                    rmmod nxup 2>/dev/null
                    if [ $? -ne 0 ]
                    then
                	print_warning "1008" "rmmod nxup.ko failed!"
                    fi
                fi
            else
                rmmod nxup 2>/dev/null
                if [ $? -ne 0 ]
                then
                	print_warning "1008" "rmmod nxup.ko failed!"
                fi
            fi
        fi
    fi
    if [ "${t300s_hasWarning}" = "1" ]
    then
        print_oper "[execute ${t300s_action_stop} ${t300s_type}]" "[0]" "$g10s_LEVEL_ERROR" "[execute stop UltraPath servcie] [ֹͣ·]" "[$t300s_OPER_TYPE]"
        t300s_exit "${t300s_hasWarning}"
    else
        print_oper "[execute ${t300s_action_stop} ${t300s_type}]" "[1]" "$g10s_LEVEL_INFO" "[execute stop UltraPath servcie] [ֹͣ·]" "[$t300s_OPER_TYPE]"
        print_event "1022" "The UltraPath service is stopped successfully."
        t300s_exit 0
    fi
}


function t300s_start_mkinitrd()
{
    t300s_analyseCmdline
    t300s_checkOldUltraPath
    if [ `lsmod|awk '{print $1}'|grep -c "^[[:blank:]]*nxup[[:blank:]]*$"` -eq 0 ]
    then

        print_warning "1013" "Please reboot OS to enable UltraPath."
        exit 1
    fi
    t300s_mknodNxup
    t300s_start_udev
    t300s_startUpService

    print_event "1010" "Start UltraPath service successfully."
    return 0
}

function t300s_stop_mkinitrd()
{
    t300s_createOsInfo

    print_warning "1011" "We don't disable UltraPath in this environment."
    exit 1
}


while [ $# -gt 0 ]
do
    t300s_param="$1"
    t300s_param1=${t300s_param%%=*}
    t300s_param2=${t300s_param##*=}
    print_debug "t300s_param=${t300s_param},t300s_param1=${t300s_param1},t300s_param2=${t300s_param2}"
    case "${t300s_param1}" in
        "action")
            t300s_action="${t300s_param2}"
            ;;
        "type")
            t300s_type="${t300s_param2}"
            ;;
        *)
            print_error "1014" "unknown parameter[${t300s_param}]"
            t300s_useage
            ;;
    esac
    shift
done

if [ "${t300s_type}" != "service" ] &&  [ "${t300s_type}" != "mkinitrd" ]
then
    print_error "1015" "unknown parameter[type=${t300s_type}]"
    t300s_useage
fi
if [ "${t300s_action}" != "start" ] && [ "${t300s_action}" != "stop" ]
then
    print_error "1016" "unknown parameter[action=${t300s_action}]"
    t300s_useage
fi

g10s_eval "t300s_${t300s_action}_${t300s_type}"

t300s_exit $?
