#!/bin/bash

g10s_LIBFILE=${g10s_LIBFILE="../globalLib/g10s_pub"}
if [ -f "${g10s_LIBFILE}" ]
then
    . "${g10s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
    echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g10s_LIBFILE}]\033[0m"
    exit 1
fi

function t301c_devportParse ()
{
    g_device=`echo $1 | cut -d: -f1`
    g_port=`echo $1 | cut -d: -f2`
} 

function t301c_opensmstart()
{
    local port_guid
    if smpquery -C ${g_device} -P ${g_port} -D portinfo 0 | grep -q IsSM; then
        print_debug "$g_device:$g_port already connected to SM, skipping opensm start." "t301c" 
        return
    fi

    port_guid=`ibstat ${g_device} ${g_port} | grep 'Port GUID' | cut -d: -f2`
    if [ "$port_guid" != "" ]; then
        print_debug "starting opensm on $g_device:$g_port [guid=$port_guid]" "t301c"
        opensm -B -g ${port_guid} -s 30 > /dev/null
        if [ $? -ne 0 ]
        then
            print_debug "Failed to start opensm $port_guid." "t301c"
        fi
    else
        print_debug "unable to find port_guid for $g_device:$g_port" "t301c"
    fi
}

function t301c_checksrp_linkstatus()
{
    local timeout=15
    local curtime=1
    while [ ${curtime} -le ${timeout} ]; do
        for g_device in `ibstat -l`; do
            num_ports=`ibstat -s ${g_device} | grep 'Number of ports' | awk -F: '{print $2}'`
            for g_port in `seq 1 ${num_ports}`; do
                if ibstat ${g_device} ${g_port} | grep -q Down; then
                    continue
                else
                    print_debug "IB status is not Down delay($curtime)" "t301c"
                    return 0
                fi
            done
        done
        sleep 1
        let curtime=curtime+1
    done
    return 1
}

SRP_DEVICES=""
function t301c_findsrp_devices()
{   
    t301c_checksrp_linkstatus
    if [ $? -ne 0 ]; then
        print_debug "IB disconneting is timeout" "t301c"
    fi
    for g_device in `ibstat -l`; do
        num_ports=`ibstat -s ${g_device} | grep 'Number of ports' | awk -F: '{print $2}'`
        for g_port in `seq 1 ${num_ports}`; do
        
            if smpquery -C ${g_device} -P ${g_port} -D nodeinfo 0,${g_port} \
                    | grep "NodeType" | grep -vq "Channel Adapter"; then
                print_event "1050" "IB service configurations do not support switch-based networking."
                return 1
            fi
            
            if ibstat ${g_device} ${g_port} | grep -q 'Link layer: Ethernet'; then
                print_debug "$g_device:$g_port is Ethernet, skipping opensm start." "t301c"
                continue
            fi
        
            stat=`ibstat ${g_device} ${g_port} | grep "State" | awk -F: '{print $2}'`
            if [ ${stat} == "Active" ]; then
                if smpquery -C ${g_device} -P ${g_port} -D portinfo 0 | grep -q IsSM; then
                    ps -ef | grep "run_srp_daemon" | grep -v "grep" | grep "$g_device" | grep -q "$g_port$"
                    if [ $? -ne 0 ]; then
                        cat /etc/issue | grep -q "Red Hat Enterprise Linux Server release 6.1 (Santiago)"
                        if [ $? -eq 0 ]; then
                             print_debug "$g_device:$g_port is Active, only startting run_srp_daemon." "t301c"
                             run_srp_daemon -e -c -n -i $g_device -p $g_port > /dev/null 2> /dev/null &
                        else
                            print_debug "$g_device:$g_port is Active, only startting run_srp_daemon." "t301c"
                            run_srp_daemon -f "a max_sect=65535,max_cmd_per_lun=16" -R 30 -T 10 -t 7000 -ance -i ${g_device} -p ${g_port} > /dev/null 2> /dev/null &
                        fi
                    fi
                    continue
                else
                    SRP_DEVICES="$SRP_DEVICES $g_device:$g_port"
                fi
            else    
                SRP_DEVICES="$SRP_DEVICES $g_device:$g_port"  
            fi
        done
    done
    return 0
}


function t301c_srp_daemonStart()
{
    ps -ef | grep "run_srp_daemon" | grep -v "grep" | grep "$g_device" | grep -q "$g_port$"
    if [ $? -ne 0 ]; then
        cat /etc/issue | grep -q "Red Hat Enterprise Linux Server release 6.1 (Santiago)"
        if [ $? -eq 0 ]; then
            print_debug "Starting srp_daemon on $g_device:$g_port" "t301c"
            run_srp_daemon -e -c -n -i $g_device -p $g_port > /dev/null 2> /dev/null &
        else
            print_debug "Starting srp_daemon on $g_device:$g_port" "t301c"
            run_srp_daemon -f "a max_sect=65535,max_cmd_per_lun=16" -R 30 -T 10 -t 7000 -ance -i $g_device -p $g_port > /dev/null 2> /dev/null &
        fi
    else
        print_debug "$g_device:$g_port existing run_srp_daemon" "t301c"
        return
    fi
}

function t301c_startsrp()
{
    print_debug "Loading IB_SRP start:" "t301c"
    
    /sbin/modprobe ib_srp
    /sbin/modprobe ib_umad
    /sbin/modprobe ib_uverbs
    
    sleep 5
    
    t301c_findsrp_devices
    if [ $? -ne 0 ]; then
        SRP_DEVICES=""
        killall run_srp_daemon >/dev/null 2>/dev/null
        exit 1
    fi
    
    
    if [ ${command} == 'cli' ]; then
        print_yesOrNo "1050" "Warning: This command can only be used in direct connection mode, verify the networking environment before running this command. Do you want to go on?"
        if [ $? -ne 0 ]
        then
            exit 1
        fi
    fi
    
    if [ ${SRP_DEVICES} == "" ]; then
        print_debug "Cannot find srpDevice need to start opensm and daemon" "t301c"
    else
    
        for devport in ${SRP_DEVICES}; do
            t301c_devportParse ${devport}
            t301c_opensmstart
        done
    
        sleep 1

        for devport in ${SRP_DEVICES}; do
            t301c_devportParse ${devport}
            t301c_srp_daemonStart
        done    
    fi
    
    print_debug "Loading IB_SRP finished." "t301c"
}
command=$1
t301c_startsrp 
exit 0



