#!/bin/bash

g10s_LIBFILE=${g10s_LIBFILE="../globalLib/g10s_pub"}
if [ -f "${g10s_LIBFILE}" ]
then
	. "${g10s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g10s_LIBFILE}]\033[0m"
	exit 1
fi


t30s_label=${t30s_label:="ultrapath"}
t30s_initrd=${t30s_initrd:="${g10s_isspImageName}-`g10s_curUnamer`.img"}

t30s_tmpFile="${g10s_tmpPath}/t30s_tmplilo.conf"
t30s_tmpFile_newImage="${g10s_tmpPath}/t30s_tmplilo.conf.newImage"

t30s_action=""
t30s_bakfile="auto"
t30s_file=""
t30s_defaultLabel=""
t30s_type=""

t30s_bakFileAppend=${t30s_bakFileAppend=""}
t30s_bakFileRemove=${t30s_bakFileRemove=""}


function t30s_useage()
{
    echo "USAGE:$0 [option]
        action=append
               remove
        type=elilo
             lilo
        file=filename
        bakfile=filename
        defaultLabel=defaultLabel"
   exit 1
}


function t30s_checkMultiDefault()
{
    local ret
    if [ "$1" = "" ] || [ ! -f "$1" ]
    then
        print_error "2" "param error[$1]"
        exit 1
    fi
    ret=`grep -E -c "^[[:blank:]]*default[[:blank:]=]+.*$" "$1"`
    if [ ${ret} -eq 0 ]
    then
        print_error "1001" "not found \"default\" item.[$1]"
        exit 1
    elif [ ${ret} -gt 1 ]
    then
        print_error "1002" "there's mutiple \"default\" exist.[$1]"
        exit 1
    fi
    return 0

}


function t30s_changeDefault()
{
    local paramItem
    local defaultValue

    if [ "$1" = "" ] || [ ! -f "$1" ] || [ "$2" = "" ]
    then
        print_error "2" "param error[$1,$2]"
        exit 1
    fi
    paramItem="$2"

    defaultItem=`grep -E "^[[:blank:]]*default[[:blank:]=]+.*$" "$1"`
    print_debug "defaultItem=${defaultItem}"
    if [ `echo "${defaultItem}"|grep -v "^[[:blank:]]*$"|wc -l` -ne 1 ]
    then
        print_error "1003" "get default value failed in file[$1]"
        exit 1
    fi

    defaultValue=${defaultItem%%#*}
    if [ `echo "${defaultValue}"|grep -c "="` -eq 0 ]
    then
        paramItem="default ${paramItem}"
    else
        paramItem="default=${paramItem}"
    fi


    if [ `echo "${defaultItem}"|grep -c "#"` -ne 0 ]
    then
        defaultItem=${defaultItem#*#}
        defaultItem="${paramItem} #${defaultItem}"
    else
        defaultItem="${paramItem}"
    fi
    g10s_replaceSingleLine "$1" "^[[:blank:]]*default[[:blank:]=][[:blank:]=]*.*$" "${defaultItem}"
    return 0
}



function t30s_remove()
{
    local sour
    local defaultLabel
    local defaultValue
    local defaultItem
    local labelNum
    local image_linenum
    local image_nextlinenum
    local result1
    local result2
    local ret


    sour=`cat "${t30s_file}"|grep -E  "^[[:blank:]]*label[[:blank:]]*=[[:blank:]]*"`
    g10s_getEqualLines "${sour}" "${t30s_label}" "=" "#"
    ret=$?
    if [ ${ret} -eq 0 ]
    then
        print_warning "1004" "the item has been removed[${t30s_label},${t30s_file}]"
        return 0
    elif [ ${ret} -gt 1 ]
    then
        print_error "1005" "there's ${ret} item exist in file[${t30s_label},${t30s_file}]"
        exit 1
    fi

    defaultLabel=""

    if [ "${t30s_bakfile}" != "no" ]
    then
        if [ "${t30s_bakfile}" = "" ] || [ "${t30s_bakfile}" = "auto" ]
        then
            if [ -f "${t30s_bakFileAppend}" ]
            then

                defaultItem=`cat "${t30s_bakFileAppend}"|grep -E "^[[:blank:]]*default[[:blank:]=]+.*$"|sed -n '$p'`
                defaultItem=${defaultItem%%#*}
                print_debug "defaultItem=${defaultItem}"
                defaultValue=${defaultItem#*default}

                defaultValue=${defaultValue#*=}

                defaultValue=$(echo ${defaultValue})
                print_debug "defaultValue=${defaultValue}"
                if [ "${defaultItem}" = "" ] || [ "${defaultValue}" = "" ]
                then
                    print_warning "1006" "get default value failed[${t30s_bakFileAppend}]"
                fi
                defaultLabel="${defaultValue}"
                rm -f "${t30s_bakFileAppend}"
            fi

            if [ "${defaultLabel}" = "" ] && [ "${t30s_type}" = "elilo" ] && [ -f "/opt/UltraPath/bak/elilo.conf" ]
            then

                defaultItem=`cat "/opt/UltraPath/bak/elilo.conf"|grep -E "^[[:blank:]]*default[[:blank:]=]+.*$"|sed -n '$p'`
                defaultItem=${defaultItem%%#*}
                print_debug "defaultItem=${defaultItem}"
                defaultValue=${defaultItem#*default}

                defaultValue=${defaultValue#*=}

                defaultValue=$(echo ${defaultValue})
                print_debug "defaultValue=${defaultValue}"
                if [ "${defaultItem}" = "" ] || [ "${defaultValue}" = "" ]
                then
                    print_warning "1007" "get default value failed[/opt/UltraPath/bak/elilo.conf]"
                fi
                defaultLabel="${defaultValue}"
            fi
        else
            g10s_checkExist -f "${t30s_bakfile}" 2

            defaultItem=`cat "${t30s_bakfile}"|grep -E "^[[:blank:]]*default[[:blank:]=]+.*$"|sed -n '$p'`
            defaultItem=${defaultItem%%#*}
            print_debug "defaultItem=${defaultItem}"
            defaultValue=${defaultItem#*default}

            defaultValue=${defaultValue#*=}

            defaultValue=$(echo ${defaultValue})
            print_debug "defaultValue=${defaultValue}"
            if [ "${defaultItem}" = "" ] || [ "${defaultValue}" = "" ]
            then
                print_error "1008" "get default value failed[${t30s_bakfile}]"
                exit 1
            fi
            defaultLabel="${defaultValue}"
        fi

        if [ "${defaultLabel}" = "" ]
        then
            print_error "1009" "get default label failed."
            exit 1
        fi
    else
        if [ "${t30s_defaultLabel}" != ""  ]
        then
            print_debug "set defaultLabel=[${t30s_defaultLabel}]"
            defaultLabel="${t30s_defaultLabel}"
        else
            print_error "1010" "must set  defaultLabel=XXX."
            exit 1
        fi
    fi


    print_debug "begin removing label[${t30s_file},${defaultLabel}]"
    sour=`grep -E -n "^[[:blank:]]*label[[:blank:]]*=[[:blank:]]*" "${t30s_file}"`
    g10s_getEqualLines "${sour}" "${t30s_label}" "=" "#"
    if [ $? -ne 1 ]
    then
        print_error "1011" "the label repeat or not exist[${t30s_label},${t30s_file}]"
        exit 1
    fi


    labelNum=`echo "${g10s_equalLines}"|awk -F ":" '{print $1}'`
    print_debug "labelNum=${labelNum}"
    if ! [ ${labelNum} -ge 0 ] 2>/dev/null
    then

        print_error "1012" "get label line number failed[${labelNum}]."
        exit 1
    fi


    image_linenum=`cat "${t30s_file}"|grep -E -n "^[[:blank:]]*(image|other)[[:blank:]]*=..*"|awk -F ":" -v col=${labelNum} '{if($1 <= col) print $1}'|sed -n '$p'`
    print_debug "image_linenum=${image_linenum}"
    if ! [ ${image_linenum} -ge 0 ] 2>/dev/null
    then
        print_error "1013" "get image_linenum failed[${image_linenum}]."
        exit 1
    fi
    image_nextlinenum=`cat "${t30s_file}"|grep -E -n "^[[:blank:]]*(image|other)[[:blank:]]*=..*"|awk -F ":" -v col=${labelNum} '{if($1 > col) print $1}'|sed -n '1p'`
    print_debug "image_nextlinenum=${image_nextlinenum}"



    if [ "${image_nextlinenum}" = "" ]
    then
       let "image_linenum=image_linenum-1"
       sed -n "1,${image_linenum}p" "${t30s_file}">"${t30s_tmpFile}"
    else
       if ! [ ${image_nextlinenum} -ge 0 ] 2>/dev/null
       then
           print_error "1014" "get image_nextlinenum failed[${image_nextlinenum}]"
           exit 1
       fi
       let "image_linenum=image_linenum-1"
       lineCount=`cat "${t30s_file}"|wc -l`
       sed -n "1,${image_linenum}p;${image_nextlinenum},${lineCount}p" "${t30s_file}">"${t30s_tmpFile}"
    fi


    t30s_changeDefault "${t30s_tmpFile}" "${defaultLabel}"


    t30s_checkMultiDefault "${t30s_tmpFile}"
    sour=`cat "${t30s_tmpFile}"|grep -E "^[[:blank:]]*label[[:blank:]]*=[[:blank:]]*"`
    g10s_getEqualLines "${sour}" "${defaultLabel}" "=" "#"
    result1=$?
    sour=`cat "${t30s_tmpFile}"|grep -E "^[[:blank:]]*default[[:blank:]]*=[[:blank:]]*"`
    g10s_getEqualLines "${sour}" "${defaultLabel}" "=" "#"
    result2=$?
    print_debug "result1=${result1},result2=${result2}"
    if [ ${result1} -eq 1 ] && [ ${result2} -eq 1 ]
    then
        print_debug "backup file[${t30s_file} to ${t30s_bakFileRemove}]"
        g10s_cp  "${t30s_file}" "${t30s_bakFileRemove}"
        g10s_cp "${t30s_tmpFile}" "${t30s_file}"
        rm -rf "${t30s_tmpFile}"
        if [ "${t30s_type}" = "elilo" ]
        then
            if  [ `which elilo 2>/dev/null|grep -c "/elilo"` -ne 0 ]
            then
                elilo 1> /dev/null 2>&1
                ret=$?
            else
                print_debug "not found elilo."
                ret=0
            fi
        else
            if  [ `which lilo 2>/dev/null|grep -c "/lilo"` -ne 0 ]
            then
                lilo 1> /dev/null 2>&1
                ret=$?
            else
                print_debug "not found lilo."
                ret=0
            fi
        fi
        if [ ${ret} -ne 0 ]
        then
            print_error "1015" "run elilo or lilo failed."
            g10s_cp  "${t30s_bakFileRemove}" "${t30s_file}"
            exit 1
        fi
        print_event "1016" "remove UltraPath label from bootloader successfully[${t30s_file},${defaultLabel}]"
    else
        print_error "1017" "check failed[${t30s_tmpFile}]"
        exit 1
    fi
    return 0
}


function t30s_createNewImage()
{
    local defaultItem
    local defaultValue
    local sour
    local labelNum
    local tabKey
    local newLabelLine
    local image_linenum
    local image_nextlinenum
    local initrdLine
    local initrdCommentary
    local newInitrdLine
    local lineNum

    if [ "$1" = "" ]
    then
        print_error "2" "param error."
        exit 1
    fi


    defaultItem=`cat "$1"|grep -E "^[[:blank:]]*default[[:blank:]=]+.*$"|sed -n '$p'`
    defaultItem=${defaultItem%%#*}
    print_debug "defaultItem=${defaultItem}"
    defaultValue=${defaultItem#*default}
    defaultValue=${defaultValue#*=}
    defaultValue=$(echo ${defaultValue})
    print_debug "defaultValue=${defaultValue}"
    if [ "${defaultItem}" = "" ] || [ "${defaultValue}" = "" ]
    then
        print_error "1018" "get default value failed[$1]"
        exit 1
    fi



    sour=`grep -E -n "^[[:blank:]]*label[[:blank:]]*=[[:blank:]]+" "$1"`
    g10s_getEqualLines "${sour}" "${defaultValue}" "=" "#"
    if [ $? -ne 1 ]
    then
        print_error "1019" "there's no \"label=${defaultValue}\" or repeated in file [$1]"
        exit 1
    fi


    labelNum=`echo "${g10s_equalLines}"|awk -F ":" '{print $1}'`
    print_debug "labelNum=${labelNum}"
    if ! [ ${labelNum} -ge 0 ] 2>/dev/null
    then

        print_error "1020" "get label line number failed[${labelNum}]"
        exit 1
    fi


    tabKey=${g10s_equalLines#*:}
    tabKey=${tabKey%%label*}
    if [ `echo "${g10s_equalLines}"|grep -c "#"` -ne 0 ]
    then
        newLabelLine=${g10s_equalLines#*#}
        newLabelLine="${tabKey}label=${t30s_label} #${newLabelLine}"
    else
        newLabelLine="${tabKey}label=${t30s_label}"
    fi
    print_debug "tabKey=${tabKey},newLabelLine=${newLabelLine}"


    image_linenum=`cat "$1"|grep -E -n "^[[:blank:]]*(image|other)[[:blank:]]*=..*"|awk -F ":" -v col=${labelNum} '{if($1 <= col) print $1}'|sed -n '$p'`
    print_debug "image_linenum=${image_linenum}"
    if ! [ ${image_linenum} -ge 0 ] 2>/dev/null
    then
        print_error "1021" "get label image line number failed[${image_linenum}]"
        exit 1
    fi
    image_nextlinenum=`cat "$1"|grep -E -n "^[[:blank:]]*(image|other)[[:blank:]]*=..*"|awk -F ":" -v col=${labelNum} '{if($1 > col) print $1}'|sed -n '1p'`
    print_debug "image_nextlinenum=${image_nextlinenum}"


    rm -f "${t30s_tmpFile_newImage}"
    touch "${t30s_tmpFile_newImage}"
    if [ "${image_nextlinenum}" = "" ]
    then
       sed -n "${image_linenum},\$p" "$1" > "${t30s_tmpFile_newImage}"
    else
       if ! [ ${image_nextlinenum} -ge 0 ] 2>/dev/null
       then
           print_error "1022" "get image_nextlinenum failed[${image_nextlinenum}]"
           rm -f "${t30s_tmpFile_newImage}"
           exit 1
       fi
       let "image_nextlinenum=image_nextlinenum-1"
       print_debug "[image_nextlinenum=image_nextlinenum-1]image_nextlinenum=${image_nextlinenum}"
       sed -n "${image_linenum},${image_nextlinenum}p" "$1" > "${t30s_tmpFile_newImage}"
    fi


    g10s_replaceSingleLine "${t30s_tmpFile_newImage}" "^[[:blank:]]*label[[:blank:]]*=[[:blank:]][[:blank:]]*" "${newLabelLine}"
    lineNum=`cat ${t30s_tmpFile_newImage} | grep -n "^[[:blank:]]append" | awk -F ":" '{print $1}'`
    sed -i "${lineNum} s/\"$/ UltraPath\"/g" ${t30s_tmpFile_newImage}


    if [ `cat  "${t30s_tmpFile_newImage}"|grep -c "^[[:blank:]]*initrd[[:blank:]=][[:blank:]=]*"` -ne 1 ]
    then
        print_error "1023" "defaultImage format error."
        rm -f "${t30s_tmpFile_newImage}"
        exit 1
    fi
    initrdLine=`cat  "${t30s_tmpFile_newImage}"|grep "^[[:blank:]]*initrd[[:blank:]=][[:blank:]=]*"`
    print_debug "initrdLine=${initrdLine}"
    initrdCommentary=${initrdLine#*#}
    print_debug "initrdCommentary=${initrdCommentary}"
    initrdLine=${initrdLine%%#*}
    if [ `echo "${initrdLine}"|grep -c "/"` -eq 0 ]
    then
        newInitrdLine="${initrdLine%%=*}="
    else
        newInitrdLine=${initrdLine%/*}
    fi
    print_debug "newInitrdLine=${newInitrdLine}"

    if [ `cat  "${t30s_tmpFile_newImage}"|grep -c "^[[:blank:]]*initrd[[:blank:]=][[:blank:]=]*.*#"` -ne 0 ]
    then
        newInitrdLine="${newInitrdLine}/${t30s_initrd} #${initrdCommentary}"
    else
        newInitrdLine="${newInitrdLine}/${t30s_initrd}"
    fi

    g10s_replaceSingleLine "${t30s_tmpFile_newImage}" "^[[:blank:]]*initrd[[:blank:]=][[:blank:]=]*" "${newInitrdLine}"


    g10s_cp "$1" "${t30s_tmpFile}"
    if [ `cat "${t30s_tmpFile}"|sed -n '$p'|grep -c "^[[:blank:]]*$"` -ne 0 ]
    then
        echo "">>"${t30s_tmpFile}"
    fi


    sed -i '/^[[:blank:]]*#/'d "${t30s_tmpFile_newImage}"

    cat "${t30s_tmpFile_newImage}">>"${t30s_tmpFile}"
    echo "###ultrapath end, please don't change this!" >> "${t30s_tmpFile}"
    rm -f "${t30s_tmpFile_newImage}"
    return 0
}

function t30s_append()
{
    local sour
    local result1
    local result2
    local ret

    sour=`cat "${t30s_file}"|grep -E  "^[[:blank:]]*label[[:blank:]]*=[[:blank:]]*"`
    g10s_getEqualLines "${sour}" "${t30s_label}" "=" "#"
    if [ $? -ge 1 ]
    then
        print_warning "1024" "the item has been exist in file[${t30s_label},${t30s_file}]"
        return 0
    fi

    print_debug "begin appending image"
    t30s_createNewImage "${t30s_file}"
    t30s_changeDefault "${t30s_tmpFile}" "${t30s_label}"

    
    t30s_checkMultiDefault "${t30s_tmpFile}"
    sour=`cat "${t30s_tmpFile}"|grep -E "^[[:blank:]]*label[[:blank:]]*=[[:blank:]]*"`
    g10s_getEqualLines "${sour}" "${t30s_label}" "=" "#"
    result1=$?
    sour=`cat "${t30s_tmpFile}"|grep -E "^[[:blank:]]*default[[:blank:]]*=[[:blank:]]*"`
    g10s_getEqualLines "${sour}" "${t30s_label}" "=" "#"
    result2=$?
    print_debug "result1=${result1},result2=${result2}"
    if [ ${result1} -eq 1 ] && [ ${result2} -eq 1 ]
    then
        print_debug "backup file[${t30s_file} to ${t30s_bakFileAppend}]"
        g10s_cp  "${t30s_file}" "${t30s_bakFileAppend}"
        g10s_cp "${t30s_tmpFile}" "${t30s_file}"
        rm -rf "${t30s_tmpFile}"
        if [ "${t30s_type}" = "elilo" ]
        then
            if  [ `which elilo 2>/dev/null|grep -c "/elilo"` -ne 0 ]
            then
                elilo 1> /dev/null 2>&1
                ret=$?
            else
                print_debug "not found elilo."
                ret=0
            fi
        else
            if  [ `which lilo 2>/dev/null|grep -c "/lilo"` -ne 0 ]
            then
                lilo 1> /dev/null 2>&1
                ret=$?
            else
                print_debug "not found lilo."
                ret=0
            fi
        fi
        if [ ${ret} -ne 0 ]
        then
            print_error "1025" "run elilo or lilo failed."
            g10s_cp  "${t30s_bakFileAppend}" "${t30s_file}"
            exit 1
        fi
        print_event "1026" "append UltraPath label to bootloder successfully[${t30s_file},${t30s_label}]"
    else
        print_error "1027" "check failed[${t30s_tmpFile}]"
        exit 1
    fi
    return 0
}

function t30s_domain()
{
    if [ "${t30s_file}" = "" ]
    then
        print_error "1028" "must set the file=XXX"
        t30s_useage
    fi
    if [ "${t30s_type}" = "elilo" ]
    then
        if [ "${t30s_bakFileAppend}" = "" ]
        then
            t30s_bakFileAppend="${g10s_isspBakPath}/elilo.conf.bak"
        fi
        if [ "${t30s_bakFileRemove}" = "" ]
        then
            t30s_bakFileRemove="${g10s_bakPath}/t30s_elilo.conf.beforeremove"
        fi
    elif [ "${t30s_type}" = "lilo" ]
    then
        if [ "${t30s_bakFileAppend}" = "" ]
        then
            t30s_bakFileAppend="${g10s_isspBakPath}/lilo.conf.bak"
        fi
        if [ "${t30s_bakFileRemove}" = "" ]
        then
            t30s_bakFileRemove="${g10s_bakPath}/t30s_lilo.conf.beforeremove"
        fi
    else
        print_error "1029" "parametr error:[type:${t30s_type}]"
        t30s_useage
    fi
    g10s_checkExist -f "${t30s_file}" 2
    t30s_checkMultiDefault "${t30s_file}"


    if [ "${t30s_action}" = "append" ]
    then
        t30s_append
    elif [ "${t30s_action}" = "remove" ]
    then
        t30s_remove
    else
        print_error "1030" "unknown parameter[action:${t30s_action}]"
        t30s_useage
    fi
    return 0
}


while [ $# -gt 0 ]
do
    t30s_param="$1"
    t30s_param1=${t30s_param%%=*}
    t30s_param2=${t30s_param##*=}
    print_debug "t30s_param=${t30s_param},t30s_param1=${t30s_param1},t30s_param2=${t30s_param2}"
    case "${t30s_param1}" in
        "action")
            t30s_action="${t30s_param2}"
            ;;
        "bakfile")
            t30s_bakfile="${t30s_param2}"
            ;;
        "file")
            t30s_file="${t30s_param2}"
            ;;
        "defaultLabel")
            t30s_defaultLabel="${t30s_param2}"
            ;;
        "type")
            t30s_type="${t30s_param2}"
            ;;
        *)
            print_error "1031" "unknown parameter[${t30s_param}]"
            t30s_useage
            ;;
    esac
    shift
done

t30s_domain


