#!/bin/bash

g10s_LIBFILE=${g10s_LIBFILE="../globalLib/g10s_pub"}
if [ -f "${g10s_LIBFILE}" ]
then
    . "${g10s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
    echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib\033[0m"
    exit 1
fi

t320s_sourUpConf="/etc/up.conf"
t320s_sourUPConfTmp="${g10s_tmpPath}/t320s_sourUp.conf"
t320s_tmpFile="${g10s_tmpPath}/t320s_tmpFile"
t320s_sourUPConfCommon="${g10s_tmpPath}/t320s_sourUp.common"
t320s_sourUPConfVendors="${g10s_tmpPath}/t320s_sourUp.vendors"
t320s_bakUpConf="$1"
t320s_bakUpConfTmp="${g10s_tmpPath}/t320s_bakUp.conf"
t320s_lineNum=1
t320s_lineMsg=""

function t320s_clearTmpFiles()
{
    rm -f "${t320s_tmpFile}"
    rm -f "${t320s_sourUPConfCommon}"
    rm -f "${t320s_sourUPConfVendors}"
    rm -f "${t320s_bakUpConfTmp}"
    rm -f "${t320s_sourUPConfTmp}"
    return 0
}

function t320s_getLineFromBakUpConf()
{
    g10s_checkExist -f "${t320s_bakUpConfTmp}" 2
    t320s_lineMsg=`sed -n "${t320s_lineNum}p" "${t320s_bakUpConfTmp}"`
    let "t320s_lineNum=t320s_lineNum+1"
}

function t320s_resolveUPConf()
{
    local iCount=0


    sed -n '1,/^[[:blank:]]*\[common\]/p'  "${t320s_sourUpConf}" > "${t320s_sourUPConfTmp}"

    sed -n '/^[[:blank:]]*\[common\]/,/^[[:blank:]]*\[.*\]/p' "${t320s_sourUpConf}" > "${t320s_tmpFile}"
    iCount=`cat "${t320s_tmpFile}"|wc -l`
    if [ ${iCount} -le 2 ]
    then
        print_error "1001" "get common serction failed."
        exit 1
    fi
    let "iCount=iCount-1"
    sed -n "2,${iCount}p"  "${t320s_tmpFile}" > "${t320s_sourUPConfCommon}"


    sed -n '/^[[:blank:]]*\[vendors\]/,/^[[:blank:]]*\[.*\]/p' "${t320s_sourUpConf}" > "${t320s_tmpFile}"
    iCount=`cat "${t320s_tmpFile}"|wc -l`
    if [ ${iCount} -le 2 ]
    then
        print_error "1002" "get vendors serction failed."
        exit 1
    fi
    if [ `cat "${t320s_tmpFile}"|tail -n 1|grep -c "^[[:blank:]]*\[.*\]"` -ne 0 ]
    then
        print_error "1003" "there's some serction not support."
        exit 1
    fi
    sed -n "2,${iCount}p"  "${t320s_tmpFile}" > "${t320s_sourUPConfVendors}"
}

function t320s_parse_common()
{
    if [ `echo "${t320s_lineMsg}"|grep -E -c "^[^=]+=[^=]+;"` -eq 0 ]
    then
        print_error "1004" "format error in common.[${t320s_lineMsg}]"
        exit 1
    fi
    local item
    local value
    local ret

    item=`echo "${t320s_lineMsg}"|awk -F "=" '{print $1}'`
    item=`echo ${item}`
    value=`echo "${t320s_lineMsg}"|awk -F "=" '{print $2}'`
    value=`echo ${value}`

    if [ "${item}" = "" ]
    then
        print_error "1005" "get item failed."
        exit 1
    fi
    ret=`cat "${t320s_sourUPConfCommon}"|grep -c "^[[:blank:]]*${item}[[:blank:]]*="`
    if [ ${ret} -eq 1 ]
    then

        if [ `cat "${t320s_sourUPConfCommon}"|grep -c "^[[:blank:]]*${item}[[:blank:]]*=[[:blank:]]*${value%;*}[[:blank:]]*;"` -eq 0 ]
        then

            if [ `echo ${item}` = "usi_step_size" ]
            then
                print_debug "do not modify ${item}"
            else
                print_debug "modify common config.[${item}=${value}]"
                value=`echo ${value}`
                sed -i "/^[[:blank:]]*${item}[[:blank:]]*=/s/=[^#]*;/= ${value}/" "${t320s_sourUPConfCommon}"
                if [ $? -ne 0 ]
                then
                    print_error "1006" "modify common config failed."
                    exit 1
                fi
            fi
        fi
    elif [ ${ret} -eq 0 ]
    then
        print_debug "append common config.[${item}=${value}]"
        sed -i "1a ${item}=${value}" "${t320s_sourUPConfCommon}"
        if [ $? -ne 0 ]
        then
            print_error "1007" "append common config failed."
            exit 1
        fi
    else
        print_error "1008" "there's repeat item exist.[${item}]"
        exit 1
    fi
}

function t320s_parseItemName()
{
    if [ "$2" = "" ] || [ "$1" = "" ]
    then
        print_error "2" "param error"
        exit 1
    fi

    t320s_lineMsg=`echo ${t320s_lineMsg}`


    if [ `echo "${t320s_lineMsg}"|grep -c "^[[:blank:]]*${1}[[:blank:]]*="` -eq 0 ]
    then
        print_error "1009" "not found \"${1}\""
        exit 1
    fi


    local itemName=""
    local tmpLineMsg="${t320s_lineMsg#*,}"
    local newFile="$2"
    local item
    local value
    local newItemMsg=""
    local ret=0

    local vendorNameOld=""
    local productNameOld=""
    local ruleNameOld=""
    local ruleNameNew=""

    g10s_checkExist -f "${newFile}" 2

    if [ `cat "${newFile}"|grep -c "^[[:blank:]]*${1}[[:blank:]]*="` -eq 0 ]
    then
        print_error "1010"  "not found \"${1}\" in ${newFile}"
        exit 1
    fi


    itemName=${t320s_lineMsg#*=}
    itemName=${itemName%%,*}
    itemName=${itemName#*\"}
    itemName=${itemName%\"*}
    itemName=$(echo ${itemName})
    print_debug "itemName=${itemName}"


    ruleNameOld=`echo ${itemName}`

    vendorNameOld=`cat "${t320s_bakUpConf}" | sed -n "/\"${ruleNameOld}\"/{n;p}" | awk -F "," '{for(i=1;i<NF+1;i++)print $i}' | grep "vendor_id" | sed 's/\"/@/g' | awk -F "@" '{print $2}'`
    productNameOld=`cat "${t320s_bakUpConf}" | sed -n "/\"${ruleNameOld}\"/{n;p}" |  awk -F "," '{for(i=1;i<NF+1;i++)print $i}' | grep "product_id" | sed 's/\"/@/g' | awk -F "@" '{print $2}'`
    if [ "$productNameOld" != ""  ] && [ "$vendorNameOld" != "" ] && [ `cat "${newFile}" | grep "\"${vendorNameOld}\"" | grep "\"${productNameOld}\"" -c ` -gt 0 ]
    then
    	ruleNameNew=`cat "${newFile}" | sed -n "/^[[:blank:]]*vendor_id[[:blank:]]*=[[:blank:]]*\"${vendorNameOld}\"[[:blank:]]*,[[:blank:]]*product_id[[:blank:]]*=[[:blank:]]*\"${productNameOld}\"/{x;p};h" | sed 's/\"/@/g' | awk -F "@" '{print $2}'`
	if [ "$ruleNameNew" != "$ruleNameOld"  ]
	then
	    sed -i "s/\"${ruleNameNew}\"/\"${ruleNameOld}\"/g" ${newFile}
        fi
    fi


    if [ `cat "${newFile}"|grep -E -c "^[[:blank:]]*${1}[[:blank:]]*=[[:blank:]\"]*${itemName}[[:blank:]\",]"` -eq 0 ]
    then
        print_debug "append big item.[${1}=${itemName}]"

        echo "" > "${t320s_tmpFile}"
        echo "${t320s_lineMsg}" >> "${t320s_tmpFile}"
        while [ 1 -eq 1 ]
        do
            t320s_getLineFromBakUpConf
            tmpLineMsg="${t320s_lineMsg}"
            tmpLineMsg=`echo ${tmpLineMsg}`

            if [ `echo "${tmpLineMsg}"|grep -E -c "^[[:blank:]]*${1}[[:blank:]]*="` -ne 0 ]
            then
                print_error "1011" "not found endLine flag.[${t320s_lineMsg}]"
                exit 1
            fi
            echo "${tmpLineMsg}" >> "${t320s_tmpFile}"
            if [ `echo "${tmpLineMsg}"|grep -E -c "^[^#]*;"` -ne 0 ]
            then
                cat "${newFile}" >> "${t320s_tmpFile}"
                cat "${t320s_tmpFile}" > "${newFile}"
                rm -f "${t320s_tmpFile}"
                break
            fi
        done
        return 0
    fi


    if [ `sed -n "/^[[:blank:]]*${1}[[:blank:]]*=[[:blank:]\"]*${itemName}[[:blank:]\",]/,/^[[:blank:]]*${1}[[:blank:]]*=/p" "${newFile}"|grep -c "^[^#]*;"` -eq 0 ]
    then
        print_error "1012" "not found endLine flag.[${1}=${itemName}]"
        exit 1
    fi


    while [ 1 -eq 1 ]
    do
        if [ `echo "${tmpLineMsg}"|grep -E -c "^[[:blank:]]*${1}[[:blank:]]*="` -ne 0 ]
        then
            print_error "1013" "not found endLine flag.[${1}=${itemName}]"
            exit 1
        fi

        while [ 1 -eq 1 ]
        do
            if [ `echo "${tmpLineMsg}"|grep -c "="` -eq 0 ]
            then
                break
            fi
            item=${tmpLineMsg%%=*}
            item=`echo ${item}`
            value=${tmpLineMsg#*=}
            value=${value%%\\*}
            value=${value%%,*}
            value=${value%%;*}
            value=`echo ${value}`

            if [ `echo "${tmpLineMsg}"|grep -c ","` -ne 0 ]
            then
                tmpLineMsg=${tmpLineMsg#*,}
                tmpLineMsg=`echo ${tmpLineMsg}`
            else
                if [ `echo "${tmpLineMsg}"|grep -c ";"` -eq 0 ]
                then

                    print_error "1014" "up.conf format error.[${tmpLineMsg}]"
                    exit 1
                else
                    tmpLineMsg=";"
                fi
            fi
            if [ "${item}" = "" ]
            then
                print_error "1015" "get item failed."
                exit 1
            fi

            newItemMsg=$(sed -n "/^[[:blank:]]*${1}[[:blank:]]*=[[:blank:]\"]*${itemName}[[:blank:]\",]/,/^[^#]*;/p" "${newFile}")
            newItemMsg=`echo ${newItemMsg}`
            ret=`echo "${newItemMsg}"|grep -c "${item}[[:blank:]]*="`
            if [ ${ret} -eq 0 ]
            then
                print_debug "append litter item.[${1}=${itemName},${item} = ${value}]"

                sed -i "/^[[:blank:]]*${1}[[:blank:]]*=[[:blank:]\"]*${itemName}[[:blank:]\",]/,/^[^#]*;/s/\(^[^#]*\);/\1, ${item} = ${value};/" "${newFile}"
                if [ $? -ne 0 ]
                then
                    print_error "1016" "append litter item failed.[${1}=${itemName},${item} = ${value}] "
                    exit 1
                fi
            elif [ ${ret} -eq 1 ]
            then

                if [ `echo "${newItemMsg}"|grep -c "${item}[[:blank:]]*=[[:blank:]\"]*${value}[[:blank:]\",;]"` -eq 0 ]
                then
                    print_debug "modify litter item.[${1}=${itemName},${item} = ${value}]"

                    sed -i "/^[[:blank:]]*${1}[[:blank:]]*=[[:blank:]\"]*${itemName}[[:blank:]\",]/,/^[^#]*;/s/${item}[[:blank:]]*=[^,;]*/${item} = ${value}/" "${newFile}"
                    if [ $? -ne 0 ]
                    then
                        print_error "1017" "modify litter item failed.[${1}=${itemName},${item} = ${value}] "
                        exit 1
                    fi
                fi
            else
                print_error "1018" "there's repeat item exist.[$1,${itemName},${item}]"
                exit 1
            fi
        done

        if [ `echo "${tmpLineMsg}"|grep -E -c "^[^#]*;"` -ne 0 ]
        then
            break
        fi
        t320s_getLineFromBakUpConf
        tmpLineMsg="${t320s_lineMsg}"
        tmpLineMsg=`echo ${tmpLineMsg}`
    done
    return 0
}

function t320s_parse_vendors()
{
   t320s_parseItemName "ruler_name" "${t320s_sourUPConfVendors}"
   return 0
}

function t320s_createNewUPConf()
{
    set -e
    cat "${t320s_sourUPConfCommon}" >> "${t320s_sourUPConfTmp}"
    echo "[vendors]" >> "${t320s_sourUPConfTmp}"
    cat "${t320s_sourUPConfVendors}" >> "${t320s_sourUPConfTmp}"
    set +e
    g10s_cp "${t320s_sourUPConfTmp}" "${t320s_sourUpConf}"
    return 0
}

function t320s_main()
{
    local iCount=0
    local tmpSection=""

    t320s_lineNum=0
    t320s_lineMsg=""
    t320s_resolveUPConf
    g10s_removeBlank "${t320s_bakUpConf}" "${t320s_bakUpConfTmp}"
    iCount=`cat "${t320s_bakUpConfTmp}"|wc -l`
    t320s_lineNum=1
    while [ ${t320s_lineNum} -le ${iCount} ]
    do
        t320s_getLineFromBakUpConf
        if [ `echo "${t320s_lineMsg}"|grep -c "^[[:blank:]]*$"` -ne 0 ]
        then
            continue
        fi
        if [ `echo "${t320s_lineMsg}"|grep -c "^[[:blank:]]*\[.*\][[:blank:]]*$"` -ne 0 ]
        then
            tmpSection="${t320s_lineMsg#*[}"
            tmpSection="${tmpSection%]*}"
            continue
        fi
        if [ "${tmpSection}" = "" ]
        then
            print_error "1019" "section is null."
            exit 1
        fi
        g10s_eval "t320s_parse_${tmpSection}"
    done

    t320s_createNewUPConf
    t320s_clearTmpFiles
    return 0
}

t320s_clearTmpFiles
g10s_checkExist "-f" "${t320s_sourUpConf}" 2
if [ "${t320s_bakUpConf}" = "" ]
then
    print_error "1020" "please input the bak file(up.conf) path name."
    exit 1
fi
g10s_checkExist "-f" "${t320s_bakUpConf}" 2
t320s_main
print_event "1021" "User configuration is saved successfully."
