#!/bin/bash

g20s_LIBFILE=${g20s_LIBFILE="../globalLib/g20s_install"}
if [ -f "${g20s_LIBFILE}" ]
then
	. "${g20s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g20s_LIBFILE}]\033[0m"
	exit 1
fi



t330s_action=""
t330s_type="auto"
t330s_serviceFile=""
readonly t330s_allTypes=",auto,suse,redhat,insserv,lnForRocky,"

function t330s_useage()
{
    local allTypes
    allTypes=${t330s_allTypes//,/|}
    allTypes=${allTypes#|}
    allTypes=${allTypes%|}
    echo "$0 action=add|del [type=XXX] serviceFile=XXX
    action  insert or remove run service.
    type    ${allTypes}
    serviceFile  the service file name."
    exit 1
}

function t330s_chooseType()
{
    local type
    print_debug "choose type..."
    g20s_getActionListBySystem
    type=`cat "${g20s_actionList}"|grep "^[[:blank:]]*210_serviceMgr=="|awk -F "==" '{print $2}'`
    if [ "${type}" = "" ]
    then
        print_error "1001" "get 210_serviceMgr from ${g20s_actionList} failed."
        exit 1
    fi
    g10s_checkExist -f "t330s_serviceMgr.sh" 2
    bash t330s_serviceMgr.sh "action=${t330s_action}" "type=${type}" "serviceFile=${t330s_serviceFile}"
    return $?

}

function t330s_suse_add()
{
    local ret

    ret=0
    if [ "${g10s_logFile}" != "" ]
    then
        chkconfig --add "${t330s_serviceFile}" 1>>"${g10s_logFile}" 2>&1
        ret=$?
    else
        chkconfig --add "${t330s_serviceFile}" 1>/dev/null 2>&1
        ret=$?
    fi
    return ${ret}
}

function t330s_suse_del()
{
    local ret

    ret=0
    if [ "${g10s_logFile}" != "" ]
    then
        chkconfig --del "${t330s_serviceFile}" 1>>"${g10s_logFile}" 2>&1
        ret=$?
    else
        chkconfig --del "${t330s_serviceFile}" 1>/dev/null 2>&1
        ret=$?
    fi
    return ${ret}
}

function t330s_redhat_add()
{
    local ret

    ret=0
    if [ "${g10s_logFile}" != "" ]
    then
        chkconfig --add "${t330s_serviceFile}" 1>>"${g10s_logFile}" 2>&1
        ret=$?
    else
        chkconfig --add "${t330s_serviceFile}" 1>/dev/null 2>&1
        ret=$?
    fi

    if [ ${ret} -eq 0 ]
    then
        if [ "${g10s_logFile}" != "" ]
        then
            chkconfig --level 35 "${t330s_serviceFile}" on 1>>"${g10s_logFile}" 2>&1
            ret=$?
        else
            chkconfig --level 35 "${t330s_serviceFile}" on 1>/dev/null 2>&1
            ret=$?
        fi
    fi
    return ${ret}
}

function t330s_redhat_del()
{
    local ret

    ret=0
    if [ "${g10s_logFile}" != "" ]
    then
        chkconfig --del "${t330s_serviceFile}" 1>>"${g10s_logFile}" 2>&1
        ret=$?
    else
        chkconfig --del "${t330s_serviceFile}" 1>/dev/null 2>&1
        ret=$?
    fi
    return ${ret}
}


function t330s_insserv_add()
{
    local ret

    ret=0
    if [ "${g10s_logFile}" != "" ]
    then
        insserv "${t330s_serviceFile}" 1>>"${g10s_logFile}" 2>&1
        ret=$?
    else
        insserv "${t330s_serviceFile}" 1>/dev/null 2>&1
        ret=$?
    fi
    return ${ret}
}

function t330s_insserv_del()
{
    local ret

    ret=0
    if [ "${g10s_logFile}" != "" ]
    then
        insserv -r "${t330s_serviceFile}" 1>>"${g10s_logFile}" 2>&1
        ret=$?
    else
        insserv -r "${t330s_serviceFile}" 1>/dev/null 2>&1
        ret=$?
    fi
    return ${ret}
}

function t330s_lnForRocky_add()
{
    local ret

    if [ -d /etc/rc.d/rc3.d ]
    then
        [ -f /etc/rc.d/rc3.d/S911${t330s_serviceFile} ] && rm -rf /etc/rc.d/rc3.d/S911${t330s_serviceFile}
        ln -s  /etc/init.d/${t330s_serviceFile} /etc/rc.d/rc3.d/S911${t330s_serviceFile}
        ret=$?
    fi

    if [ -d /etc/rc.d/rc5.d ]
    then
        [ -f /etc/rc.d/rc5.d/S911${t330s_serviceFile} ] && rm -rf /etc/rc.d/rc5.d/S911${t330s_serviceFile}
        ln -s  /etc/init.d/${t330s_serviceFile} /etc/rc.d/rc5.d/S911${t330s_serviceFile}
        ret=$?
    fi
    return ${ret}
}

function t330s_lnForRocky_del()
{
    local ret

    if [ -f /etc/rc.d/rc5.d/S911${t330s_serviceFile} ]
    then
        rm -rf /etc/rc.d/rc5.d/S911${t330s_serviceFile}
        ret=$?
    fi

    if [ -f /etc/rc.d/rc3.d/S911${t330s_serviceFile} ]
    then
        rm -rf /etc/rc.d/rc3.d/S911${t330s_serviceFile}
        ret=$?
    fi
    return ${ret}
}


function t330s_main()
{
    if [ "${t330s_type}" = "auto" ]
    then
        t330s_chooseType
        exit $?
    else
        print_debug "type=${t330s_type},action=${t330s_action}"
        g10s_eval "t330s_${t330s_type}_${t330s_action}"
        exit $?
    fi
}


while [ $# -gt 0 ]
do
    t330s_param="$1"
    t330s_param1=${t330s_param%%=*}
    t330s_param2=${t330s_param##*=}
    print_debug "t330s_param=${t330s_param},t330s_param1=${t330s_param1},t330s_param2=${t330s_param2}"
    case "${t330s_param1}" in
        "action")
            t330s_action="${t330s_param2}"
            ;;
        "type")
            t330s_type="${t330s_param2}"
            ;;
        "serviceFile")
            t330s_serviceFile="${t330s_param2}"
            ;;
        *)
            print_error "1002" "unknown parameter[${t330s_param}]"
            t330s_useage
            ;;
    esac
    shift
done

if [ `echo "${t330s_allTypes}"|grep -c ",${t330s_type},"` -eq 0 ]
then
    print_error "1003" "unknown parameter.[type=${t330s_type}]"
    t330s_useage
fi

if [ `echo ",add,del,"|grep -c ",${t330s_action},"` -eq 0 ]
then
    print_error "1004" "unknown parameter.[action=${t330s_action}]"
    t330s_useage
fi

if [ "${t330s_serviceFile}" = "" ]
then
    print_error "1005" "please set serviceFile."
    t330s_useage
fi

t330s_main

