#!/bin/bash


t340s_toolsDir="../toolLib"
g20s_LIBFILE=${g20s_LIBFILE="../globalLib/g20s_install"}
g30s_LIBFILE=${g30s_LIBFILE="../globalLib/g30s_systemEnvironment"}
if [ -f "${g30s_LIBFILE}" ]
then
	. "${g30s_LIBFILE}"

elif [ -f "/UltraPath/install/linuxTools/globalLib/g30s_systemEnvironment" ]
then
    pwdPath=`pwd`
    t190s_toolsDir="/UltraPath/install/linuxTools/toolLib"
    cd "${t190s_toolsDir}"

    . ../globalLib/g30s_systemEnvironment

    cd ${pwdPath}
    unset pwdPath
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib file.\033[0m"
	exit 1
fi

t340s_bootLoaderType=""
t340s_defaultSysImage=""

function t340s_getsysimage_grub()
{
    local menLstFile
    local defaultNum
    local lineNum
    local nextNum
    local nextLineNum
    local defaultImage=""

    local ret


    if [ "$1" != "" ]
    then
        menLstFile="$1"
    else
       	menLstFile="/boot/grub/menu.lst"
    fi
    g10s_checkExist "-f" "${menLstFile}" 2


    ret=`grep -E -c "^[[:blank:]]*default[[:blank:]=]+[0-9]+[[:blank:]]*(#.*)?$" "${menLstFile}"`
    if [ ${ret} -eq 0 ]
    then
        print_error "1019" "not found \"default\" item in file.[${menLstFile}]" "g30s"
        eval "${g10s_exitFun} 1"
    elif [ ${ret} -gt 1 ]
    then
        print_error "1020" "there's mutiple \"default\" exist.[${menLstFile}]" "g30s"
        eval "${g10s_exitFun} 1"
    fi


    defaultNum=`sed -n "s/^[[:blank:]]*default[[:blank:]=][[:blank:]=]*\([0-9][0-9]*\).*$/\1/p" "${menLstFile}"|sed "s/[[:blank:]]//g"`
    print_debug "defaultNum=${defaultNum}" "g30s"
    if ! [ ${defaultNum} -ge 0 ] 2>/dev/null
    then
        print_error "1013" "get default number failed[$1]" "g30s"
        eval "${g10s_exitFun} 1"
    fi

    let "defaultNum=defaultNum+1"
    lineNum=`grep -n "^[[:blank:]]*title[[:blank:]]" "${menLstFile}"|sed -n "${defaultNum}p"|awk -F ":" '{print $1}'`
    print_debug "lineNum=${lineNum}" "g30s"
    if ! [ ${lineNum} -ge 0 ] 2>/dev/null
    then
        print_error "1014" "get line number failed[$1]" "g30s"
        eval "${g10s_exitFun} 1"
    fi

    nextNum="${defaultNum}"
    let "nextNum=nextNum+1"
    nextLineNum=`grep -n "^[[:blank:]]*title[[:blank:]]" "${menLstFile}"|sed -n "${nextNum}p"|awk -F ":" '{print $1}'`
    if [ "${nextLineNum}" = "" ]
    then
        nextLineNum=`cat "${menLstFile}"|wc -l`
    else
        let "nextLineNum=nextLineNum-1"
    fi
    print_debug "nextLineNum=${nextLineNum}" "g30s"
    if ! [ ${nextLineNum} -ge ${lineNum} ] 2>/dev/null
    then
        print_error "1015" "get next line number failed[${menLstFile}]" "g30s"
        eval "${g10s_exitFun} 1"
    fi


    defaultImage=`sed -n "${lineNum},${nextLineNum}p" "${menLstFile}"`
    print_debug "defaultImage=${defaultImage}" "g30s"
    if [ `echo   "${defaultImage}"|grep -c -E "^[[:blank:]]*(initrd|module)[[:blank:]][[:blank:]]*"` -eq 0 ] \
        || [ `echo   "${defaultImage}"|grep -c -E "^[[:blank:]]*title[[:blank:]]"` -ne 1 ]
    then
        print_error "1016" "defaultImage format error." "g30s"
        eval "${g10s_exitFun} 1"
    fi


    defaultImage=`echo "${defaultImage}" | grep -E "^[[:blank:]]*(initrd|module)[[:blank:]]*"`


    defaultImage=$(echo ${defaultImage})
    if [ -z "${defaultImage}" ]
    then
        print_error "1017" "get default system image failed." "g30s"
        eval "${g10s_exitFun} 1"
    fi


    defaultImage=${defaultImage##*/}
    if [ -f /boot/${defaultImage} ]
    then
        if [ -L /boot/${defaultImage} ]
        then
            defaultImage=`ls -l /boot/${defaultImage} | awk -F "->" '{print $2}'`
            defaultImage=$(echo ${defaultImage})
        fi
        t340s_defaultSysImage=/boot/${defaultImage}
    else
        print_error "1018" "[/boot/${defaultImage}] not found." "g30s"
        eval "${g10s_exitFun} 1"
    fi

    echo "t340s_defaultSysImage=${t340s_defaultSysImage}"
    print_debug "get defaut image name is [${t340s_defaultSysImage}]" "g30s"

    return 0
}

function t340s_getsysimage_lilo()
{
    local defaultCount
    local cmpret
    local fileName=""
    local deft=""
    local sour
    local defaultImage=""

    if [ $# -eq 1 ] && [ ! -z "$1" ]
    then

        fileName="$1"
    else
        fileName="/etc/lilo.conf"
    fi

    defaultCount=`grep -E -c "^[[:blank:]]*default[[:blank:]=]+.*$" "${fileName}"`
    if [ ${defaultCount} -eq 0 ]
    then
        print_error "1021" "not found \"default\" item in file.[${menLstFile}]" "g30s"
        eval "${g10s_exitFun} 1"
    elif [ ${defaultCount} -gt 1 ]
    then
        print_error "1022" "there's mutiple \"default\" exist.[${menLstFile}]" "g30s"
        eval "${g10s_exitFun} 1"
    fi

    deft=`cat "${fileName}"|grep -n "^[[:blank:]]*default[[:blank:]]*=..*"|awk -F "=" '{print $2}'`
    if [ -z "${deft}" ]
    then
        deft=`cat "${fileName}"|grep -n "^[[:blank:]]*default[[:blank:]]* ..*"|awk -F " " '{print $2}'`
    fi
    if [ -z "${deft}" ]
    then
        print_error "1023" "get default value failed.[${fileName}]" "g30s"
        eval "${g10s_exitFun} 1"
    fi
    print_debug "${fileName} default=${deft}" "g30s"

    deft=$(echo ${deft})

    defaultImage=`sed -n "/^[[:blank:]]*label[[:blank:]]*=[[:blank:]]*${deft}/,/^[[:blank:]]*label[[:blank:]]*=[[:blank:]]*/p" ${fileName} | grep "initrd"`
    if [ -z "${defaultImage}" ]
    then
        print_error "1023" "get default initrd failed.[${fileName}]" "g30s"
        eval "${g10s_exitFun} 1"
    fi


    defaultImage=$(echo ${defaultImage})
    if [ -z "${defaultImage}" ]
    then
        print_error "1017" "get default system image failed." "g30s"
        eval "${g10s_exitFun} 1"
    fi


    defaultImage=${defaultImage##*/}
    if [ -f /boot/${defaultImage} ]
    then
        if [ -L /boot/${defaultImage} ]
        then
            defaultImage=`ls -l /boot/${defaultImage} | awk -F "->" '{print $2}'`
            defaultImage=$(echo ${defaultImage})
        fi
        t340s_defaultSysImage=/boot/${defaultImage}
    else
        print_error "1018" "[/boot/${defaultImage}] not found." "g30s"
        eval "${g10s_exitFun} 1"
    fi

    echo "t340s_defaultSysImage=${t340s_defaultSysImage}"
    print_debug "get defaut image name is [${t340s_defaultSysImage}]" "g30s"

    return 0
}


function t340s_checkMultiDefault_grubcfg()
{
    local ret
    if [ "$1" = "" ] || [ ! -f "$1" ]
    then
        print_error "2" "param error[$1]"
        exit 1
    fi
    ret=`grep -E -c "^[[:blank:]]*set[[:blank:]]*default[[:blank:]=]+[0-9'\"]+[[:blank:]]*(#.*)?$" "$1"`
    if [ ${ret} -eq 0 ]
    then
        print_error "1001" "not found \"default\" item.[$1]"
        exit 1
    elif [ ${ret} -gt 1 ]
    then
        print_error "1002" "there's mutiple \"default\" exist.[$1]"
        exit 1
    fi
    return 0
}


function t340s_dogetsysimage_grub2()
{
    local fileName=""
    local deftTitleLine=""
    local deftNum=""
    local defaultImage=""

    if [ $# -eq 1 ] && [ ! -z "$1" ]
    then
        fileName="$1"
    else
        fileName="/boot/grub/grub.cfg"
    fi

    if [ ! -f "${fileName}" ]
    then
        print_error "1018" "not found [${fileName}]" "g30s"
        return 0
    fi

    t340s_checkMultiDefault_grubcfg "${fileName}"


    defaultNum=`sed -n "s/^[[:blank:]]*set[[:blank:]]*default[[:blank:]=][[:blank:]='\"]*\([0-9][0-9]*\).*$/\1/p" "${fileName}"|sed "s/[[:blank:]]//g"`
    if ! [ ${defaultNum} -ge 0 ] 2>/dev/null
    then
        print_error "1003" "get default number failed[$1]"
        return 1
    fi


    let "defaultNum=defaultNum+1"
    deftTitleLine=`grep -n "^[[:blank:]]*menuentry[[:blank:]]" "${fileName}"|sed -n "${defaultNum}p"|awk -F ":" '{print $1}'`


    defaultImage=`grep -n "^[[:blank:]]*initrd" "${fileName}" | awk -F ":" -v col=${deftTitleLine} '{if($1 > col){print $2; exit 0}}'`
    if [ -z "${defaultImage}" ]
    then
        print_error "1003" "get default Image failed."
        return 1
    fi
    defaultImage=${defaultImage##*/}
    if [ -f /boot/${defaultImage} ]
    then
        if [ -L /boot/${defaultImage} ]
        then
            defaultImage=`ls -l /boot/${defaultImage} | awk -F "->" '{print $2}'`
            defaultImage=$(echo ${defaultImage})
        fi
        t340s_defaultSysImage=/boot/${defaultImage}
    else
        print_error "1018" "[/boot/${defaultImage}] not found." "g30s"
        eval "${g10s_exitFun} 1"
    fi

    echo "t340s_defaultSysImage=${t340s_defaultSysImage}"
    print_debug "get defaut image name is [${t340s_defaultSysImage}]" "g30s"

    return 0
}

function t340s_getsysimage_elilo()
{
    t340s_getsysimage_lilo "/etc/elilo.conf"
    return $?
}

function t340s_getsysimage_egrub()
{
    t340s_getsysimage_grub "/etc/grub.conf"
    return $?
}

function t340s_getsysimage_grub2()
{
    local t340s_file=""

    if [ -f /boot/grub/grub.cfg ]
    then
        t340s_file="/boot/grub/grub.cfg"
    elif [ -f /boot/grub2/grub.cfg ]
    then
        t340s_file="/boot/grub2/grub.cfg"
    fi

    t340s_dogetsysimage_grub2 "${t340s_file}"
    return $?
}

function t340s_getsysimageforrh7_grub2()
{
    local menuentryNum
    local menuentryNumForOldDefaultTitle
    local DefaultString
    local oldDefault
    local flag
    local n
    local allIsNum
    local defaultImage
    
    DefaultString=`cat /etc/default/grub | grep GRUB_DEFAULT | awk -F"=" '{print $2}'`
    if [ "${DefaultString}" != "saved" ]
    then
        print_error "1015" "grub default string is not saved"
        return 1
    fi

    oldDefault=`cat /boot/grub2/grubenv | grep saved_entry | awk -F"=" '{print $2}'`
    menuentryNum=`cat /boot/grub2/grub.cfg | grep -c -E "^[[:blank:]]*menuentry[[:blank:]]+"`
    if [ ${menuentryNum} -le 0 ]
    then
        print_error "1016" "menuentryNum is [${menuentryNum}]"
    fi
    
    allIsNum=`expr match "${oldDefault}" '[a-zA-Z]*'`
    if [ ${allIsNum} -ne 0 ]
    then
        menuentryNumForOldDefault=`cat /boot/grub2/grub.cfg | grep -c -E "${oldDefault}"`
        if [ ${menuentryNumForOldDefault} -ne 0 ]
        then
	        flag=0
			while read line
			do
				TMP=`echo ${line} | grep -E "^[[:blank:]]*menuentry[[:blank:]]+" | grep -E "${oldDefault}"`
				if [ ${flag} -eq 0 ]
				then
					if [ "${TMP}" = "" ]
					then
					    continue
					fi
					flag=1
				fi
				
				TMP2=`echo ${line} | grep -E "initrd16"`
				if [ "${TMP2}" != "" ]
				then
					defaultImage=$(echo ${TMP2} | grep "initrd16")
					break
				fi
				
				if [ "$line" = "}" ]
				then
					break
				fi
			done < /boot/grub2/grub.cfg
		else
		    oldDefault=0
		    allIsNum=0
		fi
	fi
    
    if [ ${allIsNum} -eq 0 ]
    then
		flag=0
		n=0
		while read line
		do
			if [ ${flag} -eq 0 ]
			then
			    TMP=`echo ${line} | grep -E "^[[:blank:]]*menuentry[[:blank:]]+"`
				if [ "${TMP}" = "" ]
				then
				    continue
				fi
				
				if [ ${n} -lt ${oldDefault} ]
				then
				    n=`expr $n + 1`
				    continue
				fi
				
				flag=1
			fi
			
			TMP2=`echo ${line} | grep -E "initrd16"`
			if [ "${TMP2}" != "" ]
			then
				defaultImage=$(echo "${TMP2}" | grep "initrd16")
				break
			fi
			
			if [ "$line" = "}" ]
			then
				break
			fi
		done < /boot/grub2/grub.cfg
	fi
	
    defaultImage=$(echo ${defaultImage})
    if [ -z "${defaultImage}" ]
    then
        print_error "1017" "get default system image failed." "g30s"
        eval "${g10s_exitFun} 1"
    fi
    
    defaultImage=${defaultImage##*/}
    if [ -f /boot/${defaultImage} ]
    then
        if [ -L /boot/${defaultImage} ]
        then
            defaultImage=`ls -l /boot/${defaultImage} | awk -F "->" '{print $2}'`
            defaultImage=$(echo ${defaultImage})
        fi
        t340s_defaultSysImage=/boot/${defaultImage}
    else
        print_error "1018" "[/boot/${defaultImage}] not found." "g30s"
        eval "${g10s_exitFun} 1"
    fi

    echo "t340s_defaultSysImage=${t340s_defaultSysImage}"
    print_debug "get defaut image name is [${t340s_defaultSysImage}]" "g30s"
    return 0
}

function t340s_getsysimageforSLES12_grub2()  
{     
    local menuentryNum     
    local menuentryNumForOldDefaultTitle     
    local DefaultString     
    local oldDefault     
    local flag     
    local n     
    local allIsNum     
    DefaultString=`cat /etc/default/grub | grep GRUB_DEFAULT | awk -F"=" '{print $2}'` 
    if [ "${DefaultString}" != "saved" ]     
    then     
        print_error "1015" "grub default string is not saved"     
        return 1     
    fi     
    oldDefault=`cat /boot/grub2/grubenv | grep saved_entry | awk -F"=" '{print $2}'` 
    g10s_getOSType
    if [ "${oldDefault}" = "" ] && [ "${g10s_OSVersion}" = "SLES12SP2" ]
    then
        oldDefault=0 
    elif [ "${oldDefault}" = "" ]
    then
          print_error "1001" "defaut image name is not exsist in grubenv."
	  return 0
    fi
    menuentryNum=`cat /boot/grub2/grub.cfg | grep -c -E "^[[:blank:]]*menuentry[[:blank:]]+"`     
    if [ ${menuentryNum} -le 0 ]     
    then     
        print_error "1016" "menuentryNum is [${menuentryNum}]"     
    fi     
    allIsNum=`expr match "${oldDefault}" '[a-zA-Z]*'`   
    if [ ${allIsNum} -ne 0 ]     
    then     
        menuentryNumForOldDefault=`cat /boot/grub2/grub.cfg | grep -c -E "${oldDefault}"`     
        if [ ${menuentryNumForOldDefault} -ne 0 ]     
        then     
                flag=0     
                        while read line     
                        do     
                                TMP=`echo $line | grep -E "${oldDefault}"`     
                                if [ ${flag} -eq 0 ]     
                                then     
                                        if [ "${TMP}" = "" ]     
                                        then     
                                            continue     
                                        fi     
                                        flag=1     
                                fi     
                                TMP2=`echo $line | grep -E "initrd"`
                                if [ "${TMP2}" != "" ]     
                                then     
                                        t340s_defaultSysImage=`echo ${TMP2} | grep initrd | awk -F" /" '{print $2}'`     
                                        t340s_defaultSysImage="/${t340s_defaultSysImage}"
                                        break     
                                fi     
                                if [ "$line" = "}" ]
                                then     
                                        break     
                                fi     
                        done < /boot/grub2/grub.cfg     
                else     
                    oldDefault=0     
                    allIsNum=0     
                fi     
        fi     
    if [ ${allIsNum} -eq 0 ]
    then     
                flag=0     
                n=0     
                while read line     
                do     
                        if [ ${flag} -eq 0 ]     
                        then     
                            TMP=`echo $line | grep -E "^[[:blank:]]*menuentry[[:blank:]]+"`  
                                if [ "${TMP}" = "" ]     
                                then     
                                    continue     
                                fi     
                                    
                                flag=1
                        fi     
                        TMP2=`echo $line | grep -E "initrd"`
                        if [ "${TMP2}" != "" ]  && [ ${flag} -eq 1 ]  
                        then     
                                if [ ${oldDefault}  -eq ${n} ]
                                then
                                    t340s_defaultSysImage=`echo "${TMP2}" | grep initrd | awk -F" /" '{print $2}'`     
                                    t340s_defaultSysImage="/${t340s_defaultSysImage}"
                                     break     
                                fi     
                                n=`expr $n + 1`     
                        fi
                        if [ "$line" = "}" ] && [ ${flag} -eq 1 ]
                        then     
                                flag=0 
       
                        fi     
                done < /boot/grub2/grub.cfg     
        fi       
    print_debug "get defaut image name is [${t340s_defaultSysImage}]" "g30s" 
    return 0     
} 
function t340_main()
{
    if [ "${t340s_bootLoaderType}" = "" ]
    then
        g30s_getBootLoaderType
        if [ $? -ne 0 ]
        then
            print_error "1001" "get boot loader type failed."
            exit 1
        fi
        t340s_bootLoaderType=$(echo ${g30s_bootLoaderType})
    fi
    print_debug "get system boot loader type[${t340s_bootLoaderType}]."

    if [ `echo "${g10s_OSVersion}"|grep -c -E "(RHEL7|OL7|KYLIN7SP2)"` -eq 1 ]
    then
        g10s_eval "t340s_getsysimageforrh7_${t340s_bootLoaderType}"
    elif [ `echo "${g10s_OSVersion}"|grep -c "SLES12"` -eq 1 ]
    then
    	g10s_eval "t340s_getsysimageforSLES12_${t340s_bootLoaderType}"
    else
        g10s_eval "t340s_getsysimage_${t340s_bootLoaderType}"   
    fi

    return 0
}

g10s_getOSType
t340_main