#!/bin/bash

g10s_LIBFILE=${g10s_LIBFILE="../globalLib/g10s_pub"}
if [ -f "${g10s_LIBFILE}" ]
then
	. "${g10s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g10s_LIBFILE}]\033[0m"
	exit 1
fi

t350s_r3SourUPCfg="/opt/UltraPath/backup/up.conf.r3"
t350s_r5sourUpCfg="/etc/up.conf"
t350s_r5tmpUpCfg="${g10s_tmpPath}/t350s_tmpFile"


t350s_UPCfgItems=("R3failover" "R5lun_trespass" \
                "R3FailBackDelayTime" "R5path_failback_time" \
                "R3lbcontroller" "R5lb_controller" \
                "R3HoldIoTimeout" "R5holdon_timeout" \
                "R3S2ToS3Key" "R5pr_key")

t350s_R3UPCfgValue=""

function t350s_useage()
{
    echo "$0 [action=update] [type=upcfg|fccfg|iscsicfg]"
    exit 1
}


function t350s_getR3UPCfgValue()
{
    if [ `cat ${t350s_r3SourUPCfg} | grep -c "^[[:blank:]]*${1}"` -eq 0 ]
    then
        t350s_R3UPCfgValue="null"
        return 0
    fi

    t350s_R3UPCfgValue=`cat ${t350s_r3SourUPCfg} | grep "^[[:blank:]]*${1}" | awk -F "=" '{print $2}'`
    t350s_R3UPCfgValue=$(echo ${t350s_R3UPCfgValue})
    if [ -z "${t350s_R3UPCfgValue}" ]
    then
        print_error "1000" "get the value of [${1}] failed."
        exit 1
    fi
    print_debug "get upconfig success [$1=${t350s_R3UPCfgValue}]."
    return 0
}


function t350s_modifyR5UPCfgValue()
{
    local r5UPCfgItem=""
    local r5UPCfgItemValue=""

    r5UPCfgItem=$1
    r5UPCfgItemValue=$2

    sed -i "/^[[:blank:]]*${r5UPCfgItem}[[:blank:]]*=/s/=[^#]*;/= \"${r5UPCfgItemValue}\"\;/" ${t350s_r5tmpUpCfg}
    if [ $? -ne 0 ]
    then
        print_error "1001" "modify the value of [${1}] failed."
        exit 1
    fi
    print_debug "modify r5upconfig success [$1=$2]."
    return 0
}


function t350s_updateR5UPCfgByR3()
{
   local r5CfgItem=""
   r5CfgItem="$1"

   if [ "${t350s_R3UPCfgValue}" = "null" ]
   then
       return 0
   fi
   case "${r5CfgItem}" in
       "lun_trespass")
           if [ "${t350s_R3UPCfgValue}" = "0" ]
           then
               t350s_modifyR5UPCfgValue "${r5CfgItem}" "off"
           elif [ "${t350s_R3UPCfgValue}" = "1" ]
           then
               t350s_modifyR5UPCfgValue "${r5CfgItem}" "on"
           fi
           ;;

       "path_failback_time")
           t350s_modifyR5UPCfgValue "${r5CfgItem}" "${t350s_R3UPCfgValue}"
           ;;
           

       "lb_controller")
           if [ "${t350s_R3UPCfgValue}" = "0" ]
           then
               
               t350s_modifyR5UPCfgValue "${r5CfgItem}" "1"
           else
               
               t350s_modifyR5UPCfgValue "${r5CfgItem}" "0"
           fi
           ;;

       "holdon_timeout")
           t350s_modifyR5UPCfgValue "${r5CfgItem}" "${t350s_R3UPCfgValue}"
           ;;

       "pr_key")
           t350s_modifyR5UPCfgValue "${r5CfgItem}" "${t350s_R3UPCfgValue}"
           ;;

       *)
           print_error "1003" "unknown parameter[${r5CfgItem}]"
           exit 1
           ;;
   esac
   return 0
}


function t350s_update_upcfg()
{
    local r3UPCfgItem=""
    local r5UPCfgItem=""

    if [ ! -f "${t350s_r3SourUPCfg}" ]
    then
        print_debug "No need update upconfig since [${t350s_r3SourUPCfg}] is not found."
        return 1
    fi
    rm -rf /etc/up.conf
    cp /UltraPath/install/up.conf.r5 /etc/up.conf
    g10s_checkExist -f "${t350s_r5sourUpCfg}" 2
    g10s_cp "${t350s_r5sourUpCfg}" "${t350s_r5tmpUpCfg}"

    for (( i=0; i<${#t350s_UPCfgItems[@]}; i=i+1 ))
    do

       r3UPCfgItem="${t350s_UPCfgItems[$[i]]}"
       r3UPCfgItem=${r3UPCfgItem#R3}


       if [ "${r3UPCfgItem}" = "HoldIoTimeout" ]
       then
           t350s_getR3UPCfgValue "HoldIO"
           if [ "${t350s_R3UPCfgValue}" = "null" ] || [ "${t350s_R3UPCfgValue}" = "0" ]
           then
               t350s_R3UPCfgValue="0"
           else
               t350s_getR3UPCfgValue "${r3UPCfgItem}"
           fi
       else
           t350s_getR3UPCfgValue "${r3UPCfgItem}"
       fi


       let "i=i+1"
       r5UPCfgItem="${t350s_UPCfgItems[$[i]]}"
       r5UPCfgItem=${r5UPCfgItem#R5}
       t350s_updateR5UPCfgByR3 "${r5UPCfgItem}"
    done
    g10s_cp "${t350s_r5tmpUpCfg}" "${t350s_r5sourUpCfg}"
    return 0
}


function t350s_update_fccfg()
{
    return 0
}


function t350s_update_iscsicfg()
{
    return 0
}


while [ $# -gt 0 ]
do
    t350s_param="$1"
    t350s_param1=${t350s_param%%=*}
    t350s_param2=${t350s_param##*=}
    print_debug "t350s_param=${t350s_param},t350s_param1=${t350s_param1},t350s_param2=${t350s_param2}"

    case "${t350s_param1}" in
        "action")
            t350s_action="${t350s_param2}"
            ;;
        "type")
            t350s_type="${t350s_param2}"
            ;;
        *)
            print_error "1014" "unknown parameter[${t350s_param}]"
            t350s_useage
            ;;
    esac
    shift
done

if [ "${t350s_action}" != "update" ]
then
    print_error "1016" "unknown parameter[action=${t350s_action}]"
    t350s_useage
fi

if [ "${t350s_type}" != "upcfg" ] && [ "${t350s_type}" != "fccfg" ] && [ "${t350s_type}" != "iscsicfg" ]
then
    print_error "1015" "unknown parameter[type=${t350s_type}]"
    t350s_useage
fi


g10s_eval "t350s_${t350s_action}_${t350s_type}"

exit 0



