#!/bin/bash


g20s_LIBFILE=${g20s_LIBFILE="../globalLib/g20s_install"}
if [ -f "${g20s_LIBFILE}" ]
then
	. "${g20s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib\033[0m"
	exit 1
fi

t360s_image="$1"
t360s_tmpDir="${g10s_tmpPath}/t360s_initrd"
t360s_sysImageName=""
curPath=`pwd`

function t360s_exit()
{
    g10s_unlockShell "${t190s_lockFile}"
    chmod -R 600 "${t360s_tmpDir}"
    exit "$1"
}

function t360s_decompressionSysImage()
{
    local ret=""

    g10s_checkExist -f "t340s_getsysimage.sh" 2
    t360s_sysImageName=`bash t340s_getsysimage.sh | grep "^[[:blank:]]*t340s_defaultSysImage[[:blank:]]*=[[:blank:]]*" | awk -F "=" '{print $2}'`
    if [ -z "${t360s_sysImageName}" ]
    then
        print_error "1000" "get system imageName failed."
        t360s_exit 1
    fi
    print_debug "get sysImage of ROCKY success.[${t360s_sysImageName}]."
    g10s_cp "${t360s_sysImageName}" "${t360s_tmpDir}"

    t360s_pwd="`pwd`"
    cd ${t360s_tmpDir}
    gunzip -cd ${t360s_sysImageName} | cpio -iudm 2> /dev/null
    ret=$?
    cd ${t360s_pwd}
    return ${ret}
}

function t360s_copymodprobeconfile()
{
    
    if [ "${g10s_OSVersion}" = "ROCKY_2.6.32.41" ] || [ "${g10s_OSVersion}" = "ROCKY_4.2.39" ] || [ "${g10s_OSVersion}" = "ROCKY_4.2.40" ]
    then
        if [ -f /etc/modprobe.d/nxupmodules.conf ]
        then
            [ ! -d "${t360s_tmpDir}/etc/modprobe.d" ] && mkdir "${t360s_tmpDir}/etc/modprobe.d"
            g10s_cp "/etc/modprobe.d/nxupmodules.conf" "${t360s_tmpDir}/etc/modprobe.d/nxupmodules.conf"
        else
            print_debug "not found /etc/modprobe.d/nxupmodules.conf."
        fi
    fi


    if [ "${g10s_OSVersion}" = "ROCKY_2.6.27.54" ]
    then
        if [ -f /etc/modprobe.conf ]
        then
            g10s_cp "/etc/modprobe.conf" "${t360s_tmpDir}/etc/modprobe.conf"
        else
            print_debug "not found /etc/modprobe.conf."
        fi
    fi

    return 0
}

function t360s_copyNxupInfo()
{
    local tmpModulePath

    g10s_getPathForInstall
    tmpModulePath="${g10s_PathForInstall}"

    print_debug "get modules path for [`uname -r`,${tmpModulePath}]"
    g10s_checkExist -f "${g20s_modulesBaseDir}/${tmpModulePath}/nxupext_a.ko" 2
    g10s_checkExist -f "${g20s_modulesBaseDir}/${tmpModulePath}/nxup.ko" 2
    g10s_cp "${g20s_modulesBaseDir}/${tmpModulePath}/nxupext_a.ko" ${t360s_tmpDir}/lib/nxupext_a.ko
    g10s_cp "${g20s_modulesBaseDir}/${tmpModulePath}/nxup.ko" ${t360s_tmpDir}/lib/nxup.ko

    g10s_cp "/etc/up.conf" "${t360s_tmpDir}/etc/up.conf"
    g10s_cp "/etc/up.hostinfo" "${t360s_tmpDir}/etc/up.hostinfo"

    return 0
}

function t360s_modifyinitFile_32()
{
    addfilename="./scripts/init-top/insert_ultrapath"
    echo '#!/bin/sh' > ${addfilename}
    echo 'PREREQ=""' >> ${addfilename}
    echo 'prereqs()' >> ${addfilename}
    echo '{' >> ${addfilename}
    echo 'echo "$PREREQ"' >> ${addfilename}
    echo '}' >> ${addfilename}
    echo 'case $1 in' >> ${addfilename}
    echo '# get pre-requisites' >> ${addfilename}
    echo 'prereqs)' >> ${addfilename}
    echo 'prereqs' >> ${addfilename}
    echo 'exit 0' >> ${addfilename}
    echo ';;' >> ${addfilename}
    echo 'esac' >> ${addfilename}

    echo 'modprobe sg' >> ${addfilename}
    echo 'modprobe sd_mod' >> ${addfilename}
    echo 'echo "Loading nxup.ko"' >> ${addfilename}
    echo 'insmod /lib/nxup.ko' >> ${addfilename}
    echo 'echo "Loading nxupext.ko"' >> ${addfilename}
    echo 'insmod /lib/nxupext_a.ko' >> ${addfilename}


    g10s_chmodExec 700 ${addfilename}

    tmpfile="./tmpfile"
    orderfile="./scripts/init-top/ORDER"
    cat "${orderfile}" > ${tmpfile}
    echo '/scripts/init-top/insert_ultrapath' > ${orderfile}
    echo '[ -e /conf/param.conf ] && . /conf/param.conf' >> ${orderfile}
    cat "${tmpfile}" >> ${orderfile}
    rm -rf ${tmpfile}

    return 0
}

function t360s_modifyinitFile_27()
{

   str="echo \"Loading nxup.ko module\"""\n""insmod /lib/nxup.ko""\n""echo \"Loading nxup.ko module\"""\n""insmod /lib/nxupext_a.ko"
   start=`cat -n init | grep jbd.ko | /bin/sed -n 1p | awk -F " " '{print $1}'`
   sed -i "$start i$str" init
   return 0
}

function t360s_modifyinitFile_default()
{
   
   str="echo \"Loading nxup.ko module\"""\n""insmod /lib/nxup.ko""\n""echo \"Loading nxup.ko module\"""\n""insmod /lib/nxupext_a.ko"
   sed -i "2 i$str" init 
   
   return 0
}

function t360s_modifyinitFile()
{
   cd ${t360s_tmpDir}
   if [ "${g10s_OSVersion}" = "ROCKY_2.6.32.41" ]
   then
       t360s_modifyinitFile_32
   elif [ "${g10s_OSVersion}" = "ROCKY_2.6.27.54" ] || [ "${g10s_OSVersion}" = "ROCKY_4.2.39" ] || [ "${g10s_OSVersion}" = "ROCKY_4.2.40" ]
   then

       if [ -d scripts/init-top ]
       then
           t360s_modifyinitFile_32
       elif [ `cat init | grep -c "jbd.ko"` -ne 0 ]
       then
           t360s_modifyinitFile_27
       else
			t360s_modifyinitFile_default
       fi
   else
       print_error "1018" "get incorrect OSType [${g10s_OSVersion}]."
       t360s_exit 1
   fi

   return 0
}

function t360s_mkinitrdRocky()
{
    cd ${t360s_tmpDir}

    find . | cpio -H newc --create 2> /dev/null | gzip -9 > initrd.gz
    mv initrd.gz ${t360s_image}

    cd ${t360s_pwd}
    return 0
}

function t360s_main()
{
    [ ! -d ${t360s_tmpDir} ] && mkdir -p ${t360s_tmpDir}

    g10s_getOSType
    if [ -z "${g10s_OSVersion}" ]
    then
        print_error "1017" "get OSType failed."
        t360s_exit 1
    fi

    g10s_getConf "${g10s_ultraPathPackagesInfo}" "t140s_solutionType"
    if [ "${g10s_confValue}" != "mkinitrd" ]
    then
        t360s_decompressionSysImage
        t360s_copymodprobeconfile
        t360s_mkinitrdRocky
    else
        t360s_decompressionSysImage
        t360s_copyNxupInfo
        t360s_copymodprobeconfile
        t360s_modifyinitFile
        t360s_mkinitrdRocky
    fi
    cd /lib/modules/`g10s_curUnamer`/kernel/drivers
    depmod -a
    cd  ${curPath}

    chmod -R 600 "${t360s_tmpDir}"
    print_event "1020" "mkinitrd Image of ROCKY4 successfully."

    return 0
}

t360s_main

exit 0
