#!/bin/bash


g10s_LIBFILE=${g10s_LIBFILE="/UltraPath/install/linuxTools/globalLib/g10s_pub"}
if [ -f "${g10s_LIBFILE}" ]
then
	. "${g10s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g10s_LIBFILE}]\033[0m"
	exit 1
fi

t370s_notsupportvruleVersion=( "SLES9SP3" "SLES9SP2_201" "SLES9SP4" "RHEL4" )
t370s_notsupportfastiofailtmoVersion=( "SLES9SP3" "SLES9SP2_201" "SLES9SP4" "SLES10SP0" "SLES10SP1" "SLES10SP2" "SLES10SP3" "SLES10SP4" \ 
                            "RHEL4" "RHEL5" \
                            "ASIANUX2" "ASIANUX3"  \
                            "KYLIN3SP0" \
                            "ROCKY_2.6.27.54" \
                            "OL5U6" "OL5U7" "OL5U8" "OL5U9" "OL5U10" )
         		   
udev_rule_file_name="99-ultrapath.rules"
udev_rule_file_path="/etc/udev/rules.d"

readonly t370s_readProcInfoLockFile="/UltraPath/install/readprocinfo.lock"
t370s_isEventLocked=1

function t370s_createlockfile()
{
    touch ${t370s_readProcInfoLockFile} 1>/dev/null 2>&1
}

function t370s_lock_event()
{
    if [ ! -f ${t370s_readProcInfoLockFile} ]
    then
        return
    fi
    
    which flock > /dev/null 2>&1
    ret="$?"
    if [ "${ret}" -eq "0" ]
    then
        exec 6<>"${t370s_readProcInfoLockFile}"
        flock -w 120 -x 6 2>/dev/null
        t370s_isEventLocked=0
    fi
}

function t370s_unlock_event()
{
    if [ ${t370s_isEventLocked} -eq 0 ]
    then
        flock -u 6 2>/dev/null
        t370s_isEventLocked=1
    fi
}

function t370s_vstorequeueattr()
{
    local directory
    local temp_dir
    local attrvalue=""
	local vendorpath
	local vendor=""
    

	vendorpath="/sys/block/$1/device/vendor"
	if [ -f "$vendorpath" ]
	then
	    vendor=`cat $vendorpath`
	fi

	if [ `echo "$vendor" | grep -c "up"` -eq 0 ]
	then
	    return 0
	fi
	
	sleep 2

    directory="/sys/block/$1/queue"

    for dir in `ls $directory`
    do
        cd $directory
        temp_dir=`ls -l | grep $dir |grep "^-rw-.*$"`
        if [ ! -z "$temp_dir" -a $dir != "scheduler" -a ! -d "$dir" -a $dir != "rq_timeout_secs" ]
        then
            attrvalue=`cat $dir`
            echo "${attrvalue}" >$dir 2>/dev/null

        elif [ "$dir" = "scheduler" ]
        then
            attrvalue=`sed -r 's/.*\[(.*)\].*/\1/' $dir`
            echo "${attrvalue}" >$dir
        fi
    done

    return 0
}

function t370s_pstorequeueattr()
{
    local lun_map_path
    local disk
    local directory
    local temp_dir
    local attrvalue=""

    if [ -f /proc/nxup_lun_map_a ] && [ -f /proc/nxup_lun_map_b ]
    then
        exit 1
    fi

    sleep 2
    t370s_lock_event
    lun_map_path="/proc/nxup_lun_map_a"
    disk=`grep ".*=$1=" $lun_map_path | sed -r 's/.*=.*=.*=.*=(.*)=.*=.*/\1/'`
    if [ -z "${disk}" ]
    then
         lun_map_path="/proc/nxup_lun_map_b"
         disk=`grep ".*=$1=" $lun_map_path | sed -r 's/.*=.*=.*=.*=(.*)=.*=.*/\1/'`
         if [ -z "${disk}" ]
         then
               t370s_unlock_event
               exit 1
         fi
    fi
    t370s_unlock_event
    directory="/sys/block/$disk/queue"

    for dir in `ls ${directory}`
    do
        cd ${directory}
        temp_dir=`ls -l | grep $dir |grep "^-rw-.*$"`
        if [ ! -z "${temp_dir}" -a ${dir} != "scheduler" -a ! -d "$dir" -a $dir != "rq_timeout_secs" ]
        then
            attrvalue=`cat $dir`
            echo "${attrvalue}" >/sys/block/$1/queue/$dir 2>/dev/null

        elif [ "${dir}" = "scheduler" ]
        then
            attrvalue=`sed -r 's/.*\[(.*)\].*/\1/' $dir`
            echo "${attrvalue}" >/sys/block/$1/queue/${dir} 2>/dev/null
        fi
    done

    return 0
}

function t370s_modifyudevcfg()
{
    local udev_cfg_path="/etc/udev"
    local udev_cfg_file_name="udev.conf"
    local udev_cfg_string="udev_rules=\"/etc/udev/rules.d\""
    local ret

    cd $udev_cfg_path
    if [ -f "$udev_cfg_file_name" ]
    then
        ret=`grep -c "^udev_rules" ${udev_cfg_file_name}`
        if [ ${ret} -ne 0 ]
        then
            sed -i '/udev_rules/d' ${udev_cfg_file_name} 1>/dev/null 2>&1
        fi
        echo "${udev_cfg_string}" >>${udev_cfg_file_name}
        print_debug "add [${udev_cfg_string}] to file [${udev_cfg_file_name}]"
    fi

    return 0
}

function t370s_makeudevrule()
{
    local udev_rule_file_path="/etc/udev/rules.d"
    local udev_rule_file_name="56-nxup.rules"
    local udev_rule_string="ACTION==\"add\", KERNEL==\"up-*|sd*\", RUN+=\"/bin/sh /UltraPath/install/linuxTools/toolLib/t370s_udevnxup.sh \$kernel\""

    cd ${udev_rule_file_path}
    echo "${udev_rule_string}" >${udev_rule_file_name}
    g10s_chmodExec 600 ${udev_rule_file_name}
    return 0
}

function t370_checkSupportOSTimeoutType()
{
   local tmpVersion=""
   if [ "$1" = "" ]
   then
       echo "param error." 
       exit 1
   fi
   for tmpVersion in "${t370s_notsupportvruleVersion[@]}"
   do
        if [ "${tmpVersion}" = "$1" ]
        then
                return 1
        fi
   done
   return 0 
}

function t370_checkSupportOSFastIOTimeType()
{
   local tmpVersion=""
   if [ "$1" = "" ]
   then
       echo "param error." 
       exit 1
   fi
   for tmpVersion in "${t370s_notsupportfastiofailtmoVersion[@]}"
   do
        if [ "${tmpVersion}" = "$1" ]
        then
             return 1
        fi
   done
   return 0 
}

function t370_modifyFastIOFailTmo()
{
   local udev_rule_string_fast_io_tmo="KERNEL==\"rport-*\", SUBSYSTEM==\"fc_remote_ports\", ACTION==\"add\", RUN+=\"/bin/sh -c 'grep -q off /sys/class/fc_remote_ports/%k/fast_io_fail_tmo;if [ $? -eq 0 ];then echo 4 > /sys/class/fc_remote_ports/%k/fast_io_fail_tmo 2> /dev/null;fi;'\""
   local ret
   
   g10s_getOSType
   t370_checkSupportOSFastIOTimeType ${g10s_OSVersion}
   ret=$?
   if [ ${ret} -eq 0  ]
   then
       echo "${udev_rule_string_fast_io_tmo}" >> ${udev_rule_file_name}
   fi
}

function t370s_modifytimeout()
{
   local udev_rule_string_version="KERNEL==\"*\",SUBSYSTEM==\"scsi_device\", ACTION==\"add\", RUN+=\"/bin/sh -c 'if  [ -d /sys/bus/scsi/drivers/NXUP/%k ] && [ -f /sys/class/scsi_device/%k/device/timeout ];then echo 30 > /sys/class/scsi_device/%k/device/timeout 2> /dev/null;fi;'\""
   local tmpType
   g10s_getOSType
   
    cd ${udev_rule_file_path}
    
    
    t370_checkSupportOSTimeoutType ${g10s_OSVersion}
    tmpType=$?
    if [ ${tmpType} -eq 0  ]
    then
        echo "${udev_rule_string_version}" >${udev_rule_file_name}
    fi
    t370_modifyFastIOFailTmo
    g10s_chmodExec 600 ${udev_rule_file_name}
    return 0
}
function t370s_createsymlink()
{
    local udev_rule_for_disk="KERNEL==\"sd*[!0-9]\", ENV{ID_VENDOR}==\"up\", ENV{ID_MODEL}=\"updisk\", ENV{ID_SERIAL}==\"?*\", SYMLINK+=\"disk/by-id/\$env{ID_BUS}-\$env{ID_SERIAL}\""
    local udev_rule_for_diskpart="KERNEL==\"sd*[0-9]\", ENV{ID_VENDOR}==\"up\", ENV{ID_MODEL}=\"updisk\", ENV{ID_SERIAL}==\"?*\", SYMLINK+=\"disk/by-id/\$env{ID_BUS}-\$env{ID_SERIAL}-part%n\""
    local udev_rule_for_uuid="ENV{ID_FS_USAGE}==\"filesystem|other|crypto\", ENV{ID_VENDOR}==\"up\", ENV{ID_MODEL}=\"updisk\", ENV{ID_FS_UUID}==\"?*\", SYMLINK+=\"disk/by-uuid/\$env{ID_FS_UUID}\""
    local udev_rule_for_label="ENV{ID_FS_USAGE}==\"filesystem|other\", ENV{ID_VENDOR}==\"up\", ENV{ID_MODEL}=\"updisk\", ENV{ID_FS_LABEL_SAFE}==\"?*\", SYMLINK+=\"disk/by-label/\$env{ID_FS_LABEL_SAFE}\""
    
    cd ${udev_rule_file_path}
    echo ${udev_rule_for_disk} >> ${udev_rule_file_name}
    echo ${udev_rule_for_diskpart} >> ${udev_rule_file_name}
    echo ${udev_rule_for_uuid} >> ${udev_rule_file_name}
    echo ${udev_rule_for_label} >> ${udev_rule_file_name}
}

function t370s_set_devlosstmo()
{
    local rport=""
    if [ -z "$1" ] || [ -z "$2" ]
    then
        return 1
    fi
    
    if [ `echo $2 | grep -c -E "^[0-9]+$"` -ne 1 ]
    then
        return 1
    fi
    
    rport=`echo $1 | awk -F "/" '{print $7}'`

    g10s_checkAndSetValue /sys/class/fc_remote_ports/${rport}/dev_loss_tmo ${2}
}

if [ "$1" = "modifycfg" ]
then
    t370s_makeudevrule 
elif [ "$1" = "modifytimeout" ]
then
    t370s_modifytimeout
elif [ "$1" = "createsymlink" ]
then
    t370s_createsymlink
elif [ "$1" = "createlock" ]
then
    t370s_createlockfile
elif [ `echo $1 | grep -c "fc_transport"` -ne 0 ]
then
    t370s_set_devlosstmo "$1" "$2"
elif [  `echo "$1" | grep -c "up-"` -eq 1 ]
then
    t370s_pstorequeueattr "$1"
else
    t370s_vstorequeueattr "$1"
fi

