#!/bin/bash


g10s_LIBFILE=${g10s_LIBFILE="/UltraPath/install/linuxTools/globalLib/g10s_pub"}
if [ -f "${g10s_LIBFILE}" ]
then
	. "${g10s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g10s_LIBFILE}]\033[0m"
	exit 1
fi

function t380s_add()
{
    print_debug "start insmod ${t380s_name} mod"
    if [ `lsmod|awk '{print $1}'|grep -c "^[[:blank:]]*${t380s_name}[[:blank:]]*$"` -ne 0 ]
    then
        print_debug "${t380s_name} mod already exists."
        return 0
    fi

    g10s_getPathForInstall

    tmpModulePath="${g10s_PathForInstall}"

    if [ ! -f "/UltraPath/install/kernelModules/${tmpModulePath}/${t380s_name}.ko" ]
    then
        print_debug "ERROR:[/UltraPath/install/kernelModules/${tmpModulePath}/${t380s_name}.ko] is not exist."
        exit 4
    fi

    insmod /UltraPath/install/kernelModules/${tmpModulePath}/${t380s_name}.ko
    if [ $? -ne 0 ]
    then
        print_debug "load ${t380s_name} failed"
        exit 5
    fi
    print_debug "insmod ${t380s_name} mod success"

    return 0
}

function t380s_remove()
{
    local time_for_wait=60
    local time_waited=0
    
    print_debug "start rmmod ${t380s_name} mod"
    if [ `lsmod|awk '{print $1}'|grep -c "^[[:blank:]]*${t380s_name}[[:blank:]]*$"` -eq 0 ]
    then
        print_debug "${t380s_name} mod already not exists."
        return 0
    fi

    if [ `ps -ef | grep "rmmod ${t380s_name}" | wc -l` -le 1 ]
    then
        rmmod ${t380s_name} 2>/dev/null &
    fi
    
    while [ `lsmod|awk '{print $1}'|grep -c "^[[:blank:]]*${t380s_name}[[:blank:]]*$"` -gt 0 ]
    do    
        if [ `ps -ef | grep "rmmod ${t380s_name}" | wc -l` -le 1 ]
        then
            rmmod ${t380s_name} 2>/dev/null &
        fi
        
        if [ ${time_waited} -ge ${time_for_wait} ]
        then
            print_debug "unload ${t380s_name} failed, because rmmod cmd timeout"
            exit 6
        fi
        
        let time_waited=time_waited+5
        sleep 5
    done
    
    print_debug "rmmod ${t380s_name} mod success"

    return 0
}



while [ $# -gt 0 ]
do
    t380s_param="$1"
    t380s_param1=${t380s_param%%=*}
    t380s_param2=${t380s_param##*=}
    print_debug "t380s_param=${t380s_param},t380s_param1=${t380s_param1},t380s_param2=${t380s_param2}"
    case "${t380s_param1}" in
        "action")
            t380s_action="${t380s_param2}"
            ;;
        "name")
            t380s_name="${t380s_param2}"
            ;;
        *)
            print_error "1014" "unknown parameter[${t380s_param}]"
            t380s_useage
            ;;
    esac
    shift
done

if [ "${t380s_name}" != "nxupext_a" ] &&  [ "${t380s_name}" != "nxupext_b" ]
then
    print_debug "1015" "unknown parameter[name=${t380s_name}]"
    exit 2
fi
if [ "${t380s_action}" != "add" ] && [ "${t380s_action}" != "remove" ]
then
    print_debug "1016" "unknown parameter[action=${t380s_action}]"
    exit 3
fi

g10s_eval "t380s_${t380s_action}"

