#!/bin/bash

g30s_LIBFILE=${g30s_LIBFILE="../globalLib/g30s_systemEnvironment"}
if [ -f "${g30s_LIBFILE}" ]
then
	. "${g30s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g30s_LIBFILE}]\033[0m"
	exit 1
fi


t390s_action=""




t390s_hasWarning="0"

readonly t390s_allActions=",clear,changeConf,"


readonly t390s_originalSystemdConf="${g10s_bakPath}/t390s_system.conf"
readonly t390s_beforeRemoveSystemdConf="${g10s_bakPath}/t390s_system.conf.beforeRemove"


function t390s_useage()
{
    local allActions
    allActions=${t390s_allActions//,/|}
    allActions=${allActions#|}
    allActions=${allActions%|}
    echo "usage:$0 [action=${allActions}]"
    exit 1
}

function t390s_checkParam()
{
    if [ `echo "${t390s_allActions}"|grep -c ",${t390s_action},"` -eq 0 ]
    then
        print_error "1001" "unknown parameter[action=${t390s_action}]"
        t390s_useage
    fi
    return 0
}



function t390s_modiflySystemconf()
{
    local confFile
    local ret1
    local isChangeConf=$1
    if [ -f /etc/systemd/system.conf ]
    then
        confFile="/etc/systemd/system.conf"
    else
        print_warning "1025" "system.conf is not existed."
        t390s_hasWarning="1"
        return 1
    fi

    g10s_checkExist "-f" "${confFile}" 2
    if [ ! -f "${t390s_originalSystemdConf}" ]
    then
        g10s_cp "${confFile}" "${t390s_originalSystemdConf}"
    fi

    g10s_saveSysConf_confName="${confFile}"
    unset g10s_saveSysConf_itemAndValue
    ret1=`grep -E -c "^[[:blank:]]*DefaultTimeoutStartSec[[:blank:]]*=[[:blank:]]*600s([[:blank:]#]+|$)" "${confFile}"`
    if [ ${ret1} -le 1 ]
    then
            g10s_saveSysConf_itemAndValue=("${g10s_saveSysConf_itemAndValue[@]}" "DefaultTimeoutStartSec" "600s")
    else
       print_warning "1010" "there's more than one config item found.[${confFile},DefaultTimeoutStartSec]"
       t390s_hasWarning="1"
       return 0
    fi
    g10s_saveSysConf "${confFile}"

    return 0
}



function t390s_main_clear()
{
    if [ -f "/etc/systemd/system.conf" ]
    then
        if [ ! -f "${t390s_beforeRemoveSystemdConf}" ]
        then
            g10s_cp "/etc/systemd/system.conf" "${t390s_beforeRemoveSystemdConf}"
        fi
        
        g10s_rebackFromSysConf "/etc/systemd/system.conf" "ALL"
    fi
   
    return 0
}



function t390s_main_changeConf()
{
    t390s_modiflySystemconf
    if [ "${t390s_hasWarning}" = "1" ]
    then
        exit 3
    fi
    return 0
}

g10s_rootCheck
while [ $# -gt 0 ]
do
    t390s_param="$1"
    t390s_param1=${t390s_param%%=*}
    t390s_param2=${t390s_param##*=}
    print_debug "t390s_param=${t390s_param},t390s_param1=${t390s_param1},t390s_param2=${t390s_param2}"
    case "${t390s_param1}" in
        "action")
            t390s_action="${t390s_param2}"
            ;;
        *)
            print_error "1020" "unknown parameter[${t390s_param}]"
            t390s_useage
            ;;
    esac
    shift
done

t390s_checkParam

if [ -f /etc/systemd/system.conf ]
then 
    g10s_eval "t390s_main_${t390s_action}"
else
    print_debug "system.conf is not existed."
    exit 0
fi








