#!/bin/bash

g10s_LIBFILE=${g10s_LIBFILE="../globalLib/g10s_pub"}
g20s_LIBFILE=${g20s_LIBFILE="../globalLib/g20s_install"}
g30s_LIBFILE=${g30s_LIBFILE="../globalLib/g30s_systemEnvironment"}
if [ -f "${g10s_LIBFILE}" ] && [ -f "${g20s_LIBFILE}" ] && [ -f "${g30s_LIBFILE}" ]
then
    . "${g10s_LIBFILE}"
    . "${g20s_LIBFILE}"
    . "${g30s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
    echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g10s_LIBFILE}]\033[0m"
    echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g20s_LIBFILE}]\033[0m"
    echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g30s_LIBFILE}]\033[0m"
    exit 1
fi

t400s_ultrapathtitle="title Linux with (UltraPath|UPforLinux|UpforLinux|ultrapath)"
t400s_ultrapathlabel=${t400s_ultrapathlabel:="ultrapath"}

t400s_ultrapathEndtitle="(###ultrapath end, please don't change this!|###ultrapath-t51s end,don't change this!###)"

t400s_toolDir="/UltraPath/install/linuxTools/toolLib"
t400s_curDir=`pwd`
t400s_updateKernel=""

t400s_curMenulst="/boot/grub/menu.lst"
t400s_curLilo="/etc/lilo.conf"
t400s_curElilo="/etc/elilo.conf"
t400s_curEgrub="/etc/grub.conf"
t400s_curGrubenv="/boot/grub2/grubenv"
t400s_curGrub2cfg="/boot/grub2/grub.cfg"

t400s_bakFileMenulst=${t400s_bakFileMenulst="${g10s_isspBakPath}/menu.lst.bak"}
t400s_bakFileElilo=${t400s_bakFileElilo="${g10s_isspBakPath}/elilo.conf.bak"}
t400s_bakFileLilo=${t400s_bakFileLilo="${g10s_isspBakPath}/lilo.conf.bak"}
t400s_bakFileGrubenv=${t400s_bakFileGrubenv="${g10s_isspBakPath}/grubEnvAfter.bak"}
t400s_bakfileforgrub2=""
readonly t400s_grubSetDefault="/usr/sbin/grub2-set-default"

function t400s_helpinfo()
{

    echo -e "*************************************************************************************************"
    echo -e "*                                           Notice                                              *"  
    echo -e "*************************************************************************************************"
    echo -e "*UltraPath detects that the default startup item of BootLoader do not point to the upgraded     *"
    echo -e "*new kernel.                                                                                    *" 
    echo -e "*The BootLoader type is ${t400s_bootloadtype}.                                                                  *"  
    if [ "${t400s_bootloadtype}" = "grub" ]
    then
        echo -e "*<Step> Edit /boot/grub/menu.lst and make the entry that contains the initramfs image without   *"
        echo -e "*the ultraPath modules as the default entry.                                                    *"
    elif [ "${t400s_bootloadtype}" = "grub2" ]
    then
        echo -e "*<Step> Run grub2-set-default and make the entry that contains the initramfs image without      *"
        echo -e "*the ultraPath modules as the default entry.                                                    *"
    elif [ "${t400s_bootloadtype}" = "lilo" ]
    then
        echo -e "*<Step> Edit /etc/lilo.conf and make the entry that contains the initramfs image without        *"
        echo -e "*the ultraPath modules as the default entry.                                                    *"
    elif [ "${t400s_bootloadtype}" = "elilo" ]
    then
        echo -e "*<Step> Edit /etc/elilo.conf and make the entry that contains the initramfs image without       *"
        echo -e "*the ultraPath modules as the default entry.                                                    *"
    elif [ "${t400s_bootloadtype}" = "egrub" ]
    then
        echo -e "*<Step> Edit /etc/grub.conf and make the entry that contains the initramfs image without        *"
        echo -e "*the ultraPath modules as the default entry.                                                    *"
    fi        
        
    echo -e "*For more information, see the UltraPath User Guide.                                            *"
    echo -e "*Chapter in [Upgrading Kernel Without Re-Installing UltraPath]                                  *"
    echo -e "*************************************************************************************************"  
    print_yesOrNo "1026" "Whether to go on? "
    if [ $? -eq 0 ]
    then
        return 0
    else
        return 1
    fi
}

function t400s_getKernel()
{
    local t400s_kernelList
    g10s_getOSType
    if [ `echo "${g10s_OSVersion}"|grep -c -E "(RHEL7|OL7)"` -eq 1 ]
    then
        t400s_kernelList=(`ls /usr/lib/modules/`)
    else
        t400s_kernelList=(`ls /lib/modules/`)
    fi
    
    for tmpkernel in ${t400s_kernelList[@]}
    do
        g30s_checkBootFrom_${t400s_bootloadtype} "" "${tmpkernel}"
        if [ $? -eq 1 ]
        then
            t400s_updateKernel=${tmpkernel}
            break
        fi
    done
    
    if [ "${t400s_updateKernel}" = "" ]
    then
        print_event "1005" "System kernel that the system BootLoader's default startup item point to is incorrect."
        return 1
    else
        print_event "1005" "The new kernel is ${t400s_updateKernel}"
        return 0
    fi
}



function t400s_checkKernel_grub()
{

    local t400s_defaultValue
    local t400s_defaultName
    local t400s_recheck
    
    if [ "$1" = "" ]
    then
        t400s_recheck=0
    else
        t400s_recheck="$1"
    fi
    print_debug "recheck ${t400s_recheck}."
    
    if [ ! -f "${t400s_curMenulst}" ]
    then
        print_debug "menu.lst is not exist.[${t400s_curMenulst}]"
        exit 1
    fi
    
    t400s_defaultValue=`sed -n "s/^[[:blank:]]*default[[:blank:]=][[:blank:]=]*\([0-9][0-9]*\).*$/\1/p" "${t400s_curMenulst}"|sed "s/[[:blank:]]//g"`
    print_debug "default_value=${t400s_defaultValue}"
    if [ ${t400s_defaultValue} -lt 0 ]
    then
        print_debug "get default_value failed.[${t400s_curMenulst}]"
        exit 1
    fi
    let t400s_defaultValue=t400s_defaultValue+1
    
    t400s_defaultName=`grep "^[[:blank:]]*title[[:blank:]]" "${t400s_curMenulst}"|sed -n "${t400s_defaultValue}p"`
    print_debug "default_title=${t400s_defaultName}"
    t400s_defaultName=${t400s_defaultName%%#*}
    if [ -z "${t400s_defaultName}" ]
    then
        print_debug "get default_title failed.[${t400s_curMenulst}]"
        exit 1
    fi
    
    ret=`echo "${t400s_defaultName}"|grep -E -c "^[[:blank:]]*${t400s_ultrapathtitle}[[:blank:]]*(#.*)?$"`
    if [ $ret -eq 1 ]
    then  
        if [ "${t400s_recheck}" -eq 0 ]
        then
            print_debug "the default item is not changed in file[${t400s_curMenulst}]"
            t400s_helpinfo 
            if [ $? -eq 0 ]
            then
                t400s_checkKernel_grub "1"
                return $?
            else
                return 1
            fi
        else
            return 2
        fi
    else
        return 0
    fi
}

function t400s_checkKernel_egrub()
{
    local t400s_defaultValue
    local t400s_defaultName
    local t400s_recheck
    
    if [ "$1" = "" ]
    then
        t400s_recheck=0
    else
        t400s_recheck="$1"
    fi
    print_debug "recheck ${t400s_recheck}."
    
    if [ ! -f "${t400s_curEgrub}" ]
    then
        print_debug "menu.lst is not exist.[${t400s_curEgrub}]"
        exit 1
    fi
    
    t400s_defaultValue=`sed -n "s/^[[:blank:]]*default[[:blank:]=][[:blank:]=]*\([0-9][0-9]*\).*$/\1/p" "${t400s_curEgrub}"|sed "s/[[:blank:]]//g"`
    print_debug "default_value=${t400s_defaultValue}"
    if [ ${t400s_defaultValue} -lt 0 ]
    then
        print_debug "get default_value failed.[${t400s_curEgrub}]"
        exit 1
    fi
    let t400s_defaultValue=t400s_defaultValue+1
    
    t400s_defaultName=`grep "^[[:blank:]]*title[[:blank:]]" "${t400s_curEgrub}"|sed -n "${t400s_defaultValue}p"`
    print_debug "default_title=${t400s_defaultName}"
    t400s_defaultName=${t400s_defaultName%%#*}
    if [ -z "${t400s_defaultName}" ]
    then
        print_debug "get default_title failed.[${t400s_curEgrub}]"
        exit 1
    fi
    
    ret=`echo "${t400s_defaultName}"|grep -E -c "^[[:blank:]]*${t400s_ultrapathtitle}[[:blank:]]*(#.*)?$"`
    if [ $ret -eq 1 ]
    then  
        if [ "${t400s_recheck}" -eq 0 ]
        then
            print_debug "the default item is not changed in file[${t400s_curEgrub}]"
            t400s_helpinfo 
            if [ $? -eq 0 ]
            then
                t400s_checkKernel_egrub "1"
                return $?
            else
                return 1
            fi
        else
            return 2
        fi
    else
        return 0
    fi
}

function t400s_getgrub2defaultItem()
{
    local defaultItem=""
    local t400s_defaultName=""
    local tmpGrubcfg=${tmpGrubcfg="${g10s_isspBakPath}/grub2.cfg.bak"}
    if [ -f "${tmpGrubcfg}" ]
    then
        rm -f "${tmpGrubcfg}"
    else
        touch "${tmpGrubcfg}"
    fi
    g10s_getOSType
    g10s_cp "${t400s_curGrub2cfg}" "${tmpGrubcfg}"
       
    defaultItem=`cat ${t400s_curGrubenv} | grep saved_entry | awk -F"=" '{print $2}'`
    allIsNum=`expr match "${defaultItem}" '[a-zA-Z]*'`
    if [ ${allIsNum} -eq 0 ]
    then
        n=0
        
        if [ `echo "${g10s_OSVersion}"|grep -c -E "(SLES12)"` -eq 1 ]
        then    
            menuentrylist=(`cat "${tmpGrubcfg}" | grep -E -n "^menuentry[[:blank:]]+" | awk -F ":" '{print $1}'`)             
            print_debug "get the default item is ${defaultItem}." "t400s"
            if [ "${defaultItem}" -ge "${#menuentrylist[@]}" ] || [ "${defaultItem}" = "" ] 2>/dev/null
            then
                print_error "1014" "not found image in [${grubcfgFile}]" "t400s"
                eval "${g10s_exitFun} 1"
            fi    
            deletstart=${menuentrylist[${defaultItem}]}
            let deletstart=deletstart-1
            print_debug "begin to delete :${deletstart}" "t400s"
            sed -i "1,${deletstart}d" "${tmpGrubcfg}"
            n=${defaultItem}
        fi
    
        while read line
        do
            TMP=`echo $line | grep -E "^[[:blank:]]*menuentry[[:blank:]]+"` 
            if [ "${TMP}" = "" ]
            then
                continue
            fi
            
            if [ ${n} -lt ${defaultItem} ]
            then
                n=`expr $n + 1`
                continue
            elif [ `echo "${line}"|grep -E -c "${t400s_ultrapathtitle}"` -ne 0 ]
            then
                t400s_defaultName=${TMP}
                t400s_defaultName=${t400s_defaultName%\'*}
                t400s_defaultName=${t400s_defaultName#*\'}
                print_debug "the default item is digital[${defaultItem}]. convert to string [${t400s_ultrapathtitle}]"
                break
            else
                t400s_defaultName=${TMP}
                t400s_defaultName=${t400s_defaultName%\'*}
                t400s_defaultName=${t400s_defaultName%\'*}
                t400s_defaultName=${t400s_defaultName%\'*}
                t400s_defaultName=${t400s_defaultName#*\'}
                print_debug "the default item is digital[${defaultItem}]. convert to string [${t400s_defaultName}]"
                break
            fi           
        done < ${tmpGrubcfg}
    else
        t400s_defaultName=${defaultItem}
    fi
    
    if [ -f "${tmpGrubcfg}" ]
    then
        rm -r "${tmpGrubcfg}"
    fi
    
    if [ -z "${t400s_defaultName}" ]
    then
        print_debug "digital convert to string failed."
        eval "${g10s_exitFun} 1"
    else
        echo "${t400s_defaultName}"
    fi
}

function t400s_checkKernel_grub2()
{
    local t400s_defaultValue
    local t400s_defaultName
    local t400s_tmpFile
    local t400s_recheck
    
    if [ "$1" = "" ]
    then
        t400s_recheck=0
    else
        t400s_recheck="$1"
    fi
    print_debug "recheck ${t400s_recheck}."
    
    g10s_getOSType
    if [ `echo "${g10s_OSVersion}"|grep -c -E "(RHEL7|OL7|SLES12)"` -eq 1 ]
    then   
        t400s_defaultName=`t400s_getgrub2defaultItem`
        ret=`echo "${t400s_defaultName}"|grep -E -c "^[[:blank:]]*${t400s_ultrapathtitle}[[:blank:]]*(#.*)?$"`
        if [ $ret -eq 1 ]
        then  
            if [ "${t400s_recheck}" -eq 0 ]
            then
                print_debug "the default item is not changed in file[${t400s_curGrubenv}]"
                t400s_helpinfo 
                if [ $? -eq 0 ]
                then
                    t400s_checkKernel_grub2 "1"
                    return $?
                else
                    return 1
                fi
            else
                return 2
            fi
        else
            return 0
        fi
    else 
        if [ -f /boot/grub/grub.cfg ]
        then
            t400s_tmpFile="/boot/grub/grub.cfg"
        elif [ -f /boot/grub2/grub.cfg ]
        then
            t400s_tmpFile="/boot/grub2/grub.cfg"
        fi
        
        t400s_defaultValue=`sed -n "s/^[[:blank:]]*set[[:blank:]]*default[[:blank:]=][[:blank:]='\"]*\([0-9][0-9]*\).*$/\1/p" "${t400s_tmpFile}"|sed "s/[[:blank:]]//g"`
        print_debug "default_value=${t400s_defaultValue}"
        if ! [ ${t400s_defaultValue} -ge 0 ] 2>/dev/null
        then
            print_error "1003" "get default number failed[${t400s_tmpFile}]"
            return 1
        fi

        let "t400s_defaultValue=t400s_defaultValue+1"

        t400s_defaultName=`grep "^[[:blank:]]*menuentry[[:blank:]]" "${t400s_tmpFile}"|sed -n "${t400s_defaultValue}p"|awk -F "['\"]" '{print $2}'`
        print_debug "default_title=${t400s_defaultName}"
        if [ -z "${t400s_defaultName}" ]
        then
            print_debug "get default_title failed.[${t400s_tmpFile}]"
            exit 1
        fi
        
        ret=`echo "${t400s_defaultName}"|grep -E -c "^[[:blank:]]*${t400s_ultrapathtitle}[[:blank:]]*(#.*)?$"`
        if [ $ret -eq 1 ]
        then  
            if [ "${t400s_recheck}" -eq 0 ]
            then
                print_debug "the default item is not changed in file[${t400s_tmpFile}]"
                t400s_helpinfo 
                if [ $? -eq 0 ]
                then
                    t400s_checkKernel_grub2 "1"
                    return $?
                else
                    return 1
                fi
            else
                return 2
            fi
        else
            return 0
        fi
    fi
}

function t400s_checkKernel_lilo()
{
    local t400s_defaultValue
    local t400s_defaultName
    local t400s_label=${t400s_label:="ultrapath"}
    local t400s_recheck
    
    if [ "$1" = "" ]
    then
        t400s_recheck=0
    else
        t400s_recheck="$1"
    fi
    print_debug "recheck ${t400s_recheck}."
    
    t400s_defaultValue=`cat "${t400s_curLilo}"|grep -E "^[[:blank:]]*default[[:blank:]=]+.*$"|sed -n '$p'`
    t400s_defaultValue=${t400s_defaultValue%%#*}
    print_debug "default_value=${t400s_defaultValue}"
    t400s_defaultName=${t400s_defaultValue#*default}
    t400s_defaultName=${t400s_defaultName#*=}

    t400s_defaultName=$(echo ${t400s_defaultName})
    print_debug "default_title=${t400s_defaultName}"
    
    if [ "${t400s_defaultValue}" = "" ] || [ "${t400s_defaultName}" = "" ]
    then
        print_debug "1006" "get default value failed[${t400s_curLilo}]"
    fi

    ret=`echo "${t400s_defaultName}"|grep -E -c "^[[:blank:]]*${t400s_ultrapathlabel}[[:blank:]]*(#.*)?$"`
    if [ $ret -eq 1 ]
    then  
        if [ "${t400s_recheck}" -eq 0 ]
        then
            print_debug "the default item is not changed in file[${t400s_curLilo}]"
            t400s_helpinfo 
            if [ $? -eq 0 ]
            then
                t400s_checkKernel_lilo "1"
                return $?
            else
                return 1
            fi
        else
            return 2
        fi
    else
        return 0
    fi
}

function t400s_checkKernel_elilo()
{
    local t400s_defaultValue
    local t400s_defaultName
    local t400s_label=${t400s_label:="ultrapath"}
    local t400s_recheck
    
    if [ "$1" = "" ]
    then
        t400s_recheck=0
    else
        t400s_recheck="$1"
    fi
    print_debug "recheck ${t400s_recheck}."
    
    t400s_defaultValue=`cat "${t400s_curElilo}"|grep -E "^[[:blank:]]*default[[:blank:]=]+.*$"|sed -n '$p'`
    t400s_defaultValue=${t400s_defaultValue%%#*}
    print_debug "default_value=${t400s_defaultValue}"
    t400s_defaultName=${t400s_defaultValue#*default}
    t400s_defaultName=${t400s_defaultName#*=}

    t400s_defaultName=$(echo ${t400s_defaultName})
    print_debug "default_title=${t400s_defaultName}"
    
    if [ "${t400s_defaultValue}" = "" ] || [ "${t400s_defaultName}" = "" ]
    then
        print_debug "1006" "get default value failed[${t400s_curElilo}]"
    fi

    ret=`echo "${t400s_defaultName}"|grep -E -c "^[[:blank:]]*${t400s_ultrapathlabel}[[:blank:]]*(#.*)?$"`
    if [ $ret -eq 1 ]
    then  
        if [ "${t400s_recheck}" -eq 0 ]
        then
            print_debug "the default item is not changed in file[${t400s_curElilo}]"
            t400s_helpinfo 
            if [ $? -eq 0 ]
            then
                t400s_checkKernel_elilo "1"
                return $?
            else
                return 1
            fi
        else
            return 2
        fi
    else
        return 0
    fi
}

function t400s_cleanBootloaderBeforeAppend()
{
    if [ "$1" = "" ] || [ "$2" = "" ] || [ "$3" = "" ]
    then
        print_debug "para error in [t400s_cleanBootloaderBeforeAppend]" "t400s"
        return 0
    fi
    startline="$1"
    endline="$2"
    fileDir="$3"
    
    if [ "$4" != "" ]
    then
        range=`grep -E -n "[[:blank:]]*UltraPath[[:blank:]]*\"$" "${fileDir}" | awk -F ":" '{print $1}'`
        for curline in ${range[@]}
        do
            if [ ${curline} -lt ${startline} ] || [ ${curline} -gt ${endline} ]
            then
                sed -i "${curline} s/[[:blank:]]*UltraPath[[:blank:]]*\"$/\"/" "${fileDir}"
            fi
        done
    else
        range=`grep -E -n "[[:blank:]]*UltraPath[[:blank:]]*$" "${fileDir}" | awk -F ":" '{print $1}'`
        for curline in ${range[@]}
        do
            if [ ${curline} -lt ${startline} ] || [ ${curline} -gt ${endline} ]
            then
                sed -i "${curline} s/[[:blank:]]*UltraPath[[:blank:]]*$//" "${fileDir}"
            fi
        done
    fi
    
    return 0
}

function t400s_clean_grub()
{
    local t400s_tmpMenulst="/opt/UltraPath/backup/tmpMenu.lst"
    local t400s_startLine=""
    if [ ! -f "${t400s_curMenulst}" ]
    then
        print_debug "menu.lst is not exist.[${t400s_curMenulst}]"
        exit 1
    fi
    g10s_cp "${t400s_curMenulst}" "${t400s_tmpMenulst}"
    
    t400s_startLine=`cat "${t400s_tmpMenulst}" | grep -E -n "^[[:blank:]]*${t400s_ultrapathtitle}" | awk -F ":" '{print $1}'`
    print_debug "update menulst start Line from ${t400s_startLine}"
    if [ ${t400s_startLine} -lt 0 ] || [ "${t400s_startLine}" = "" ]
    then
        print_debug "update menu.lst failed.[${t400s_curMenulst}]"
    fi     
    
    t400s_endLine=`cat "${t400s_tmpMenulst}" | grep -E -n "^[[:blank:]]*${t400s_ultrapathEndtitle}" | awk -F ":" '{print $1}'`
    print_debug "update menulst end Line from ${t400s_endLine}"
    if [ ${t400s_endLine} -lt 0 ] || [ "${t400s_endLine}" = "" ]
    then
        print_debug "update menu.lst failed.[${t400s_tmpMenulst}]"
    fi
    t400s_cleanBootloaderBeforeAppend "${t400s_startLine}" "${t400s_endLine}" "${t400s_tmpMenulst}"
    g10s_cp "${t400s_tmpMenulst}" "${t400s_curMenulst}"
    
    sed -i "${t400s_startLine},$"d "${t400s_tmpMenulst}"
    g10s_cp "${t400s_tmpMenulst}" "${t400s_bakFileMenulst}"
    print_debug "update backup menulst.[${t400s_bakFileMenulst}]"    
    rm -rf "${t400s_tmpMenulst}" 
    bash t80s_prerm.sh "debprerm"
    return $?
}

function t400s_clean_egrub()
{
    local t400s_tmpMenulst="/opt/UltraPath/backup/tmpMenu.lst"
    local t400s_startLine=""
    if [ ! -f "${t400s_curEgrub}" ]
    then
        print_debug "menu.lst is not exist.[${t400s_curEgrub}]"
        exit 1
    fi
    g10s_cp "${t400s_curEgrub}" "${t400s_tmpMenulst}"
    
    t400s_startLine=`cat "${t400s_tmpMenulst}" | grep -E -n "^[[:blank:]]*${t400s_ultrapathtitle}" | awk -F ":" '{print $1}'`
    print_debug "update menulst start Line from ${t400s_startLine}"
    if [ ${t400s_startLine} -lt 0 ] || [ "${t400s_startLine}" = "" ]
    then
        print_debug "update menu.lst failed.[${t400s_curEgrub}]"
    fi
        
    t400s_endLine=`cat "${t400s_tmpMenulst}" | grep -E -n "^[[:blank:]]*${t400s_ultrapathEndtitle}" | awk -F ":" '{print $1}'`
    print_debug "update menulst end Line from ${t400s_endLine}"
    if [ ${t400s_endLine} -lt 0 ] || [ "${t400s_endLine}" = "" ]
    then
        print_debug "update menu.lst failed.[${t400s_tmpMenulst}]"
    fi
    t400s_cleanBootloaderBeforeAppend "${t400s_startLine}" "${t400s_endLine}" "${t400s_tmpMenulst}"
    g10s_cp "${t400s_tmpMenulst}" "${t400s_curEgrub}"
    
    sed -i "${t400s_startLine},$"d "${t400s_tmpMenulst}"
    g10s_cp "${t400s_tmpMenulst}" "${t400s_bakFileMenulst}"
    print_debug "update backup menulst.[${t400s_bakFileMenulst}]"
    rm -rf "${t400s_tmpMenulst}"
    bash t80s_prerm.sh "debprerm"
    return $?
}

function t400s_clean_recheck_grub2()
{
    local tmpGrubenv
    if [ `echo "${g10s_OSVersion}"|grep -c -E "(RHEL7|OL7|SLES12)"` -eq 1 ] 
    then
        tmpGrubenv=`t400s_getgrub2defaultItem`    
        if [ "${t400s_bakfileforgrub2}" = "${tmpGrubenv}" ]
        then
            if [ `cat "${t400s_curGrub2cfg}" | grep -c "${t400s_bakfileforgrub2}"` -eq 0 ]
            then
                print_debug "${t400s_bakfileforgrub2} is not in [${t400s_curGrub2cfg}]"
                return 1
            fi
        fi
        
        if [ "${t400s_bakfileforgrub2}" != "${tmpGrubenv}" ] && [ `cat "${t400s_curGrub2cfg}" | grep -c "${t400s_bakfileforgrub2}"` -eq 0 ] 
        then
            t400s_bakfileforgrub2=`cat "${t400s_curGrub2cfg}" | grep -E "^[[:blank:]]*menuentry" | grep "$t400s_updateKernel"`
            if [ $? -ne 0 ]
            then
                print_debug "grub2 default item not found in ${t400s_curGrub2cfg}"
                return 1
            fi
            t400s_bakfileforgrub2=${t400s_bakfileforgrub2%\'*}
            t400s_bakfileforgrub2=${t400s_bakfileforgrub2%\'*}
            t400s_bakfileforgrub2=${t400s_bakfileforgrub2%\'*}
            t400s_bakfileforgrub2=${t400s_bakfileforgrub2#*\'}
            "${t400s_grubSetDefault}" "${t400s_bakfileforgrub2}"
            print_debug "change ${tmpGrubenv} to ${t400s_grubSetDefault} [not found in $t400s_curGrub2cfg]"
        fi
        
        if [ "${t400s_bakfileforgrub2}" != "${tmpGrubenv}" ] && [ `cat "${t400s_curGrub2cfg}" | grep -c "${t400s_bakfileforgrub2}"` -ne 0 ] 
        then
            "${t400s_grubSetDefault}" "${t400s_bakfileforgrub2}"
            print_debug "change ${tmpGrubenv} to ${t400s_grubSetDefault}"
        fi
    fi
    return 0
}

function t400s_clean_grub2()
{
    t400s_cleanTargetLabel="GRUB_CMDLINE_LINUX"
    t400s_cleanKernelPara="/etc/default/grub"
    
    g10s_getOSType
    if [ `echo "${g10s_OSVersion}"|grep -c -E "(RHEL7|OL7|SLES12)"` -eq 1 ]
    then
        t400s_bakfileforgrub2=`t400s_getgrub2defaultItem`
        if [ "${t400s_bakfileforgrub2}"  = "" ]
        then 
            return 1
        fi
        g10s_cp "${t400s_curGrubenv}" "${t400s_bakFileGrubenv}"
        print_debug "update backup Grubenv.[${t400s_bakFileGrubenv},${t400s_bakfileforgrub2}]"
        
        if [ -f "${t400s_cleanKernelPara}" ]
        then
            targetline=(`cat "${t400s_cleanKernelPara}" | grep -E -n "^[[:blank:]]*${t400s_cleanTargetLabel}" | awk -F ":" '{print $1}'`)
            numarray=${#targetline[@]}
            if [ ${numarray} -ne 0 ]
            then
                for tmpline in ${targetline[@]}
                do
                    sed -i "${tmpline} s/UltraPath[[:blank:]]*\"/\"/g" "${t400s_cleanKernelPara}"
                done
            fi
        else
            print_debug "can not find ${t400s_cleanKernelPara}"
        fi
        
        bash t80s_prerm.sh "debprerm"
        if [ $? -ne 0 ]
        then
            return 1
        fi
        t400s_clean_recheck_grub2
        return $?
    else 
        if [ -f /boot/grub/grub.cfg ]
        then
            t400s_tmpFile="/boot/grub/grub.cfg"
        elif [ -f /boot/grub2/grub.cfg ]
        then
            t400s_tmpFile="/boot/grub2/grub.cfg"
        fi
        g10s_cp "${t400s_tmpFile}" "${t400s_bakfile}"
        print_debug "update backup grub.cfg.[${t400s_bakfile}]"
        bash t80s_prerm.sh "debprerm"
        return $?
    fi
}

function t400s_clean_lilo()
{
    local t400s_tmplilo="/opt/UltraPath/backup/tmplilo.conf"
    if [ ! -f "${t400s_curLilo}" ]
    then
        print_debug "lilo.conf is not exist.[${t400s_curLilo}]"
        exit 1
    fi
    g10s_cp "${t400s_curLilo}" "${t400s_tmplilo}"
    
    
    t400s_startLine=`cat "${t400s_tmplilo}" | grep -E -n "^[[:blank:]]*label[[:blank:]]*=[[:blank:]]*${t400s_ultrapathlabel}" | awk -F ":" '{print $1}'`
    print_debug "update menulst start Line from ${t400s_startLine}"
    t400s_endLine=`cat "${t400s_tmplilo}" | grep -E -n "^[[:blank:]]*${t400s_ultrapathEndtitle}" | awk -F ":" '{print $1}'`
    print_debug "update menulst end Line from ${t400s_startLine}"
    t400s_cleanBootloaderBeforeAppend "${t400s_startLine}" "${t400s_endLine}" "${t400s_tmplilo}" "lilo"
    
    lilo 1>/dev/null 2>&1
    g10s_cp "${t400s_tmplilo}" "${t400s_curLilo}"
    g10s_cp "${t400s_tmplilo}" "${t400s_bakFileLilo}"
    rm -rf "${t400s_tmplilo}"
    bash t80s_prerm.sh "debprerm"
    return $?
}

function t400s_clean_elilo()
{
    local t400s_tmpelilo="/opt/UltraPath/backup/tmplilo.conf"
    if [ ! -f "${t400s_curElilo}" ]
    then
        print_debug "elilo.conf is not exist.[${t400s_curElilo}]"
        exit 1
    fi
    g10s_cp "${t400s_curElilo}" "${t400s_tmpelilo}"
    
    t400s_startLine=`cat "${t400s_tmpelilo}" | grep -E -n "^[[:blank:]]*label[[:blank:]]*=[[:blank:]]*${t400s_ultrapathlabel}" | awk -F ":" '{print $1}'`
    print_debug "update menulst start Line from ${t400s_startLine}"
    t400s_endLine=`cat "${t400s_tmpelilo}" | grep -E -n "^[[:blank:]]*${t400s_ultrapathEndtitle}" | awk -F ":" '{print $1}'`
    print_debug "update menulst end Line from ${t400s_startLine}"
    t400s_cleanBootloaderBeforeAppend "${t400s_startLine}" "${t400s_endLine}" "${t400s_tmpelilo}" "elilo"
    
    elilo 1>/dev/null 2>&1
    g10s_cp "${t400s_tmpelilo}" "${t400s_curElilo}"
    g10s_cp "${t400s_tmpelilo}" "${t400s_bakFileElilo}"
    rm -rf "${t400s_tmpelilo}"
    bash t80s_prerm.sh "debprerm"
    return $?
}


function t400s_create_update()
{
    local t400s_solutionType="mkinitrd"

    g10s_saveConf "${g10s_ultraPathPackagesInfo}" "g30s_bootLoaderType" "${t400s_bootloadtype}"
    g10s_saveConf "${g10s_ultraPathPackagesInfo}" "t140s_solutionType" "${t400s_solutionType}"
    g10s_saveConf "${g10s_ultraPathPackagesInfo}" "t400s_updateKernel" "${t400s_updateKernel}"
    bash t70s_postinst.sh "debpostinst"
    return $?
}



function t400s_main()
{
    local ret
    print_debug "begin update new kernel for ultraPath."
    print_debug "BootLoader type [${t400s_bootloadtype}]."
    "t400s_checkKernel_${t400s_bootloadtype}"
    ret=$?
    if [ $ret -eq 1 ]
    then
        print_debug "Do not continue update new Kernel by selsect [N] in helpinfo."
        exit 1
    elif [ $ret -eq 2 ]
    then
        print_event "1005" "System mirrors that the system BootLoader's default startup items point to contain ultrapath modules."
        print_event "1005" "Please check the bootloader."
        exit 1
    fi
    t400s_getKernel
    if [ $? -ne 0 ]
    then
        print_debug "get new kernel failed exit."
        exit 1
    fi
    cd "${t400s_toolDir}"
    g10s_eval "t400s_clean_${t400s_bootloadtype}"
    if [ $? -ne 0 ]
    then
        print_debug "clean environment failed exit."
        cd "${t400s_curDir}"
        exit 1
    fi
    cd "${t400s_toolDir}"
    g10s_eval "t400s_create_update"
    if [ $? -ne 0 ]
    then
        print_debug "update new kernel failed exit."
        cd "${t400s_curDir}"
        exit 1
    fi
    cd "${t400s_curDir}"
    exit 0
}

g10s_rootCheck
t400s_bootloadtype=""
g10s_getConf "${g10s_ultraPathPackagesInfo}" "g30s_bootLoaderType"
t400s_bootloadtype="${g10s_confValue}"
if [ "${t400s_bootloadtype}" = "" ]
then
    print_debug "can not find bootloader type."
    exit 1
else
    t400s_main
fi
