#!/bin/bash

g10s_LIBFILE=${g10s_LIBFILE="../globalLib/g10s_pub"}
if [ -f "${g10s_LIBFILE}" ]
then
	. "${g10s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g10s_LIBFILE}]\033[0m"
	exit 1
fi

t40s_elilofile="${g10s_tmpPath}/t40s_elilo.tmp.conf"
t40s_grubfile="${g10s_tmpPath}/t40s_menulst.tmp"

readonly t40s_bakLogFile="${g10s_tmpPath}/t40s_logFile.bak.tmp"

function t40s_clean_elilo()
{
    print_debug "begin cleaning elilo.conf"
    g10s_cp "/etc/elilo.conf" "${t40s_elilofile}"
    bash t30s_liloModify.sh "action=remove" "type=elilo" "file=${t40s_elilofile}"
    if [ $? -ne 0 ]
    then
        print_error "1001" "modify elilo.conf failed."
        rm -f "${t40s_elilofile}"
        exit 1
    fi
    return 0
}

function t40s_clean_grub()
{
    print_debug "begin cleaning menu.lst"
    g10s_cp "/boot/grub/menu.lst" "${t40s_grubfile}"
    bash t20s_menulstModify.sh "action=remove" "file=${t40s_grubfile}"
    if [ $? -ne 0 ]
    then
        print_error "1002" "modify menu.lst failed."
        rm -f "${t40s_grubfile}"
        exit 1
    fi
    return 0
}

print_event "1003" "The UltraPath uninstallation starts"
oldRpm=`rpm -qa | grep -E "UltraPath|[Uu][Pp][Ff]orLinux"`
if [ "${oldRpm}" != "" ]
then
	if [ -f /opt/UltraPath/bak/elilo.conf ] && [ -f /etc/elilo.conf ]
	then
	    t40s_clean_elilo
	fi
	if [ -f "/boot/grub/menu.lst" ]
	then
	    if [ -f "/opt/UpForLinux/menu.lst" ] || [ -f "/opt/UltraPath/bak/menu.lst" ]
	    then
	        t40s_clean_grub
	    fi
	fi



	rm -f "${t40s_bakLogFile}"
	if [ -f "${g10s_logFile}" ]
	then
	    g10s_cp "${g10s_logFile}" "${t40s_bakLogFile}"

            rpm -e ${oldRpm}
            if [ $? -ne 0 ]
            then
                print_error "1004" "unistall UltraPath failed"
                exit 1
            fi

	    mkdir -p ${g10s_logDir}
	    chmod 600 ${g10s_logDir}
	    g10s_cp "${t40s_bakLogFile}" "${g10s_logFile}"
	else
	    print_error "1005" "Not found log file.[${g10s_logFile}]"
	    exit 1
	fi

	if [ -f "/usr/sbin/upTools" ]
	then
	    print_error "1006" "upTools is already exist,unistall UltraPath failed"
            exit 1
        fi

	if [ -f "${t40s_elilofile}" ]
	then
	    print_debug "recover /etc/elilo.conf"
	    g10s_cp "${t40s_elilofile}" "/etc/elilo.conf"
	    rm -f "${t40s_elilofile}"
	    if  [ `which elilo 2>/dev/null|grep -c "/elilo"` -ne 0 ]
            then
                elilo
	    fi
	fi
	if [ -f "${t40s_grubfile}" ]
	then
	    print_debug "recover /boot/grub/menu.lst"
		g10s_cp "${t40s_grubfile}" "/boot/grub/menu.lst"
		rm -f "${t40s_grubfile}"
	fi
	print_event "1007" "UltraPath driver uninstalled successfully"
else
	print_event "1008" "not find UltraPath driver"
fi




if [ -e /etc/init.d/boot.ultrapath ] && [ -f /etc/SuSE-release ]
then
    print_debug "uninstall UltraPath patch[up_vlun_patch]"
    chkconfig --del boot.ultrapath 2>/dev/null
    if [ $? -ne 0 ]
    then
        print_error "1010" "remove sevice boot.ultrapath failed[chkconfig --del boot.ultrapath]"
        exit 1
    fi
    rm -f /etc/init.d/boot.ultrapath
    print_event "1009" "uninstall UltraPath patch successfully[up_vlun_patch]"
fi

if [ ! -d /opt/UltraPath/backup ]
then
    mkdir -p /opt/UltraPath/backup
fi

[ -f /etc/up.conf ] && g10s_cp "/etc/up.conf" "/opt/UltraPath/backup/up.conf.r3"

exit 0
