#!/bin/bash

g10s_LIBFILE=${g10s_LIBFILE="../globalLib/g10s_pub"}
if [ -f "${g10s_LIBFILE}" ]
then
	. "${g10s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e -n "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}]\033[0m ";echo "not found the lib[${g10s_LIBFILE}]"
	exit 1
fi


t50s_title=${t50s_title:="title Linux with ultrapath"}

t50s_initrd=${t50s_initrd:="${g10s_isspImageName}-`g10s_curUnamer`.img"}

t50s_action=""
t50s_file=""
if [ -f /boot/grub/grub.cfg ]
then
    t50s_file="/boot/grub/grub.cfg"
elif [ -f /boot/grub2/grub.cfg ]
then
    t50s_file="/boot/grub2/grub.cfg"
fi
t50s_bakfile="auto"
t50s_defaultTitle=""

t50s_bakFileAppend=${t50s_bakFileAppend="${g10s_isspBakPath}/grub.cfg.bak"}
t50s_bakFileRemove=${t50s_bakFileRemove="${g10s_bakPath}/t50s_grub.cfg.beforeremove"}

t50s_tmpFile="${g10s_tmpPath}/t50s_tmpGrubcfg"
t50s_oldDefaultTitle=""
t50s_newDefaultNum=""
t50s_newImage=""
t50s_regExpr=""

readonly t50s_grubMkconfig="/usr/sbin/grub-mkconfig"
readonly t50s_flagBegin="###ultrapath-t50s begin,don't change this!###"
readonly t50s_flagEnd="###ultrapath-t50s end,don't change this!###"

t50s_shellDir=""
if [ `echo "$0"|grep -c "/"` -ne 0 ]
then
    t50s_shellDir=`echo ${0%/*}`
fi
print_debug "pwd=`pwd`,\$0=$0,t50s_shellDir=${t50s_shellDir}"
if [ "${t50s_shellDir}" = "." ] || [ "${t50s_shellDir}" = "" ]
then
    t50s_shellDir="`pwd`"
else
    t50s_shellDir="`pwd`/${shellDir}"
fi
t50s_shellName=`echo ${0##*/}`

function t50s_useage()
{
    echo "USAGE:$0 [option]
       action=append
              remove
       file=XXXX
       bakfile=XXXX
               auto
               no
       defaultTitle=XXXX"
    exit 1
}

function t50s_checkMultiDefault()
{
    local ret
    if [ "$1" = "" ] || [ ! -f "$1" ]
    then
        print_error "2" "param error[$1]"
        exit 1
    fi
    ret=`grep -E -c "^[[:blank:]]*set[[:blank:]]*default[[:blank:]=]+[0-9'\"]+[[:blank:]]*(#.*)?$" "$1"`
    if [ ${ret} -eq 0 ]
    then
        print_error "1001" "not found \"default\" item.[$1]"
        exit 1
    elif [ ${ret} -gt 1 ]
    then
        print_error "1002" "there's mutiple \"default\" exist.[$1]"
        exit 1
    fi
    return 0
}

function t50s_getOldDefaultTitle()
{
    local defaultNum

    t50s_oldDefaultTitle=""
    if [ "$1" = "" ] || [ ! -f "$1" ]
    then
        print_error "2" "param error[$1]"
        exit 1
    fi

    print_debug "get default title from file[$1]"

    defaultNum=`sed -n "s/^[[:blank:]]*set[[:blank:]]*default[[:blank:]=][[:blank:]='\"]*\([0-9][0-9]*\).*$/\1/p" "$1"|sed "s/[[:blank:]]//g"`
    print_debug "defaultNum=${defaultNum}"
    if ! [ ${defaultNum} -ge 0 ] 2>/dev/null
    then
        print_error "1003" "get default number failed[$1]"
        return 1
    fi

    let "defaultNum=defaultNum+1"

    t50s_oldDefaultTitle=`grep -E "${t50s_regExpr}" "$1"|sed -n "${defaultNum}p"|awk -F "['\"]" '{print $2}'`
    print_debug "t50s_oldDefaultTitle=${t50s_oldDefaultTitle}"
    if [ "${t50s_oldDefaultTitle}" = "" ]
    then
        return 1
    else
        return 0
    fi
}


function t50s_getNewDefaultNum()
{
    local sour
    local ret

    if [ "$1" = "" ] || [ "$2" = "" ]
    then
        print_error "2" "param error[$1,$2]"
        exit 1
    fi
    t50s_newDefaultNum=""
    t50s_getOldDefaultTitle "$1"
    if [ "${t50s_oldDefaultTitle}" = "" ]
    then
        return 1
    fi

    sour=`grep -E "^[[:blank:]]*menuentry[[:blank:]]+" "${2}"|grep -E -n "^[[:blank:]]*menuentry[[:blank:]]+"`
    g10s_getEqualLines2 "${sour}" "${t50s_oldDefaultTitle}" "[\'\"]" "[\'\"]"
    ret=$?
    if [ ${ret} -ne 1 ]
    then
        t50s_newDefaultNum=""
        return 1
    else
        t50s_newDefaultNum=`echo "${g10s_equalLines}"|awk -F ":" '{print $1}'`
        if ! [ ${t50s_newDefaultNum} -ge 1 ] 2>/dev/null
        then
            t50s_newDefaultNum=""
            return 1
        fi
        let "t50s_newDefaultNum=t50s_newDefaultNum-1"
        print_debug "get new default number[t50s_newDefaultNum=${t50s_newDefaultNum}]"
        return 0
    fi
    return 0
}


function t50s_changeDefault()
{
    local paramItem
    local newdefaultItem
    if [ "$1" = "" ] || [ ! -f "$1" ] || [ "$2" = "" ]
    then
        print_error "2" "param error[$1,$2]"
        exit 1
    fi
    paramItem="$2"

    newdefaultItem=`grep -E "^[[:blank:]]*set[[:blank:]]*default[[:blank:]=]+\"[[:digit:]].*$" "$1"`
    print_debug "newdefaultItem=${newdefaultItem}"
    if [ `echo "${newdefaultItem}"|grep -v "^[[:blank:]]*$"|wc -l` -ne 1 ]
    then
        print_error "1004" "get default value failed in file[$1]"
        exit 1
    fi
    paramItem="set default=\"${paramItem}\""


    if [ `echo "${newdefaultItem}"|grep -c "#"` -ne 0 ]
    then
        newdefaultItem=${newdefaultItem#*#}
        newdefaultItem="${paramItem} #${newdefaultItem}"
    else
        newdefaultItem="${paramItem}"
    fi
    g10s_replaceSingleLine "$1" "^[[:blank:]]*set[[:blank:]]*default[[:blank:]=][[:blank:]=]*.*$" "${newdefaultItem}"
    return 0
}


function t50s_recoverDefault()
{
    local bakfile
    local sour
    if [ "$1" = "" ] || [ ! -f "$1" ]
    then
        print_error "2" "param error[$1]"
        exit 1
    fi
    print_debug "recover default value[$1]"

    bakfile=""
    t50s_newDefaultNum=""

    if [ "${t50s_bakfile}" != "no" ] && [ "${t50s_bakfile}" != "auto" ]
    then
        g10s_checkExist "-f" "${t50s_bakfile}" 2
        t50s_getNewDefaultNum "${t50s_bakfile}" "$1"
        if [ $? -ne 0 ]
        then
            print_error "1005" "get new default value failed[${t50s_bakfile},$1]"
            exit 1
        fi
    elif [ "${t50s_bakfile}" = "auto" ]
    then
        if [ -f "${t50s_bakFileAppend}" ]
        then
            bakfile="${t50s_bakFileAppend}"
            t50s_getNewDefaultNum "${bakfile}" "$1"
            rm -f "${t50s_bakFileAppend}"
        fi

        if [ "${t50s_newDefaultNum}" = "" ] && [ "${t50s_defaultTitle}" != "" ]
        then
            t50s_newDefaultNum=`grep -E "${t50s_regExpr}" "$1"|grep -E -n "${t50s_regExpr}['\"].*${t50s_defaultTitle}.*['\"]"|awk -F ":" '{print $1}'`
            if [ ${t50s_newDefaultNum} -ge 1 ] 2>/dev/null
            then
                let "t50s_newDefaultNum=t50s_newDefaultNum-1"
            else
                print_error "1006" "get default number by \"defaultTitle\" failed."
                exit 1
            fi
        fi
    elif [ "${t50s_bakfile}" = "no" ]
    then
        if [ "${t50s_newDefaultNum}" = "" ] && [ "${t50s_defaultTitle}" != "" ]
        then
            t50s_newDefaultNum=`grep -E "${t50s_regExpr}" "$1"|grep -E -n "${t50s_regExpr}['\"].*${t50s_defaultTitle}.*['\"]"|awk -F ":" '{print $1}'`
            if [ ${t50s_newDefaultNum} -ge 1 ] 2>/dev/null
            then
                let "t50s_newDefaultNum=t50s_newDefaultNum-1"
            else
                print_error "1007" "get default number by \"defaultTitle\" failed."
                exit 1
            fi
        fi
    fi

    if ! [ ${t50s_newDefaultNum} -ge 0 ] 2>/dev/null
    then
        print_error "1008" "get default number failed[${t50s_newDefaultNum}]"
        exit 1
    fi
    print_debug "before change:t50s_newDefaultNum=${t50s_newDefaultNum}"

    t50s_changeDefault "$1" "${t50s_newDefaultNum}"
    return 0
}

function t50s_createNewImage()
{
    local defaultNum
    local lineNum
    local newMenuentry
    local newMenuentryHead
    local newMenuentryTail
    local nextNum
    local nextLineNum
    local defaultImage
    local initrdLine
    local initrdCommentary
    local newInitrdLine
    local preLineNum
    local lineCount

    if [ "$1" = "" ]
    then
        print_error "2" "param error[$1]"
        exit 1
    fi


    defaultNum=`sed -n "s/^[[:blank:]]*set[[:blank:]]*default[[:blank:]='\"][[:blank:]='\"]*\([0-9][0-9]*\).*$/\1/p" "$1"|sed "s/[[:blank:]]//g"`
    print_debug "defaultNum=${defaultNum}"
    if ! [ ${defaultNum} -ge 0 ] 2>/dev/null
    then
        print_error "1009" "get default number failed[$1]"
        return 1
    fi

    let "defaultNum=defaultNum+1"
    lineNum=`grep -n -E "${t50s_regExpr}" "$1"|sed -n "${defaultNum}p"|awk -F ":" '{print $1}'`
    print_debug "lineNum=${lineNum}"
    if ! [ ${lineNum} -ge 0 ] 2>/dev/null
    then
        print_error "1010" "get line number failed[$1]"
        exit 1
    fi


    newMenuentry=`sed -n "${lineNum}p" "$1"`
    if [ `echo "${newMenuentry}"|grep -c -E "^[[:blank:]]*menuentry[[:blank:]]*['\"][^'\"]+['\"]"` -ne 1 ]
    then
        print_error "1011" "get default menuentry line failed.[newMenuentry=${newMenuentry}]"
        exit 1
    fi
    newMenuentryHead=${newMenuentry%%[\'\"]*}
    newMenuentryTail=${newMenuentry#*[\'\"]}
    newMenuentryTail=${newMenuentryTail#*[\'\"]}
    newMenuentry="${newMenuentryHead}'${t50s_title}'${newMenuentryTail}"


    nextNum="${defaultNum}"
    let "nextNum=nextNum+1"
    nextLineNum=`grep -n -E "${t50s_regExpr}" "$1"|sed -n "${nextNum}p"|awk -F ":" '{print $1}'`
    if [ "${nextLineNum}" = "" ]
    then
        nextLineNum=`cat "$1"|wc -l`
    else
        let "nextLineNum=nextLineNum-1"
    fi
    let "lineNum=lineNum+1"
    print_debug "lineNum=${lineNum},nextLineNum=${nextLineNum}"

    if ! [ ${nextLineNum} -ge ${lineNum} ] 2>/dev/null
    then
        print_error "1012" "get next line number failed[$1]"
        exit 1
    fi


    defaultImage=`sed -n "${lineNum},${nextLineNum}p" "$1"`

    defaultImage=`echo "${defaultImage}" | grep -v "^[[:blank:]]*#"`

    if [ `echo  "${defaultImage}"|grep -c -E "^[[:blank:]]*initrd[[:blank:]][[:blank:]]*"` -ne 1 ]
    then
        print_error "1013" "defaultImage format error"
        exit 1
    fi

    
    initrdLine=`echo  "${defaultImage}"|grep -E "^[[:blank:]]*initrd[[:blank:]][[:blank:]]*"`
    print_debug "initrdLine=${initrdLine}"
    initrdCommentary=${initrdLine#*#}
    print_debug "initrdCommentary=${initrdCommentary}"
    initrdLine=${initrdLine%%#*}
    newInitrdLine=${initrdLine%/*}
    print_debug "newInitrdLine=${newInitrdLine}"
    if [ `echo   "${defaultImage}"|grep -E -c "^[[:blank:]]*initrd[[:blank:]][[:blank:]]*.*#"` -ne 0 ]
    then
        newInitrdLine="${newInitrdLine}/${t50s_initrd} #${initrdCommentary}"
    else
        newInitrdLine="${newInitrdLine}/${t50s_initrd}"
    fi

    g10s_cp "$1" "${t50s_tmpFile}"
    if [ `cat "${t50s_tmpFile}"|sed -n '$p'|grep -c "^[[:blank:]]*$"` -ne 0 ]
    then
        echo "">>"${t50s_tmpFile}"
    fi
    echo "${newMenuentry}" >> "${t50s_tmpFile}"

    lineNum=`echo  "${defaultImage}"|grep -E -n "^[[:blank:]]*initrd[[:blank:]][[:blank:]]*"|awk -F ":" '{print $1}'`
    print_debug "lineNum=${lineNum}"
    if ! [ ${lineNum} -ge 1 ] 2>/dev/null
    then
        print_error "1014" "get lineNum failed[${lineNum}]"
        exit 1
    fi
    preLineNum=${lineNum}
    nextLineNum=${lineNum}
    let "preLineNum=preLineNum-1"
    let "nextLineNum=nextLineNum+1"
    kernelLine=`echo "${defaultImage}" | grep -n "^[[:blank:]]*linux" | awk -F ":" '{print $1}'`
    defaultImage=`echo "${defaultImage}" | sed "${kernelLine} s/$/ UltraPath/g"`
    if [ ${preLineNum} -ge 1 ]
    then
        echo  "${defaultImage}"|sed -n "1,${preLineNum}p">>"${t50s_tmpFile}"
    fi
    
    echo "${newInitrdLine}">>"${t50s_tmpFile}"
    lineCount=`echo  "${defaultImage}"|wc -l`
    print_debug "lineCount=${lineCount}"
    if [ ${lineCount} -ge ${nextLineNum} ]
    then
        echo  "${defaultImage}"|sed -n "${nextLineNum},${lineCount}p">>"${t50s_tmpFile}"
    fi
    echo  "###ultrapath end, please don't change this!" >> "${t50s_tmpFile}"
    return 0
}

function t50s_appendTitle()
{
    local sour
    local newDefaultNum

    sour=`cat "${t50s_file}"|grep -E "^[[:blank:]]*menuentry[[:blank:]]+"`
    g10s_getEqualLines2 "${sour}" "${t50s_title}" "[\'\"]" "[\'\"]"
    if [ $? -ne 0 ]
    then
        print_warning "1015" "the title has been exist[${t50s_title},${t50s_file}]"
        return 0
    fi

    t50s_createNewImage "${t50s_file}"

    newDefaultNum=`cat "${t50s_tmpFile}"|grep -c -E "${t50s_regExpr}"`
    
    let "newDefaultNum=newDefaultNum-1"
    print_debug "newDefaultNum=${newDefaultNum}"
    t50s_changeDefault "${t50s_tmpFile}" "${newDefaultNum}"


    t50s_checkMultiDefault "${t50s_tmpFile}"
    t50s_getOldDefaultTitle "${t50s_tmpFile}"
    if [ "${t50s_oldDefaultTitle}" != "${t50s_title}" ]
    then
        print_error "1016" "check failed[${t50s_tmpFile}]"
        exit 1
    else
        print_debug "backup file[${t50s_file} to ${t50s_bakFileAppend}]"
        g10s_cp "${t50s_file}" "${t50s_bakFileAppend}"
        g10s_cp "${t50s_tmpFile}" "${t50s_file}"
        rm -f "${t50s_tmpFile}"
    fi
    return 0
}

function t50s_removeTitle()
{
    local sour
    local theNum
    local nextNum
    local lineCount

    sour=`cat "${t50s_file}"|grep -E -n "^[[:blank:]]*menuentry[[:blank:]]+"`
    g10s_getEqualLines2 "${sour}" "${t50s_title}" "[\'\"]" "[\'\"]"
    ret=$?
    if [ ${ret} -eq 0 ]
    then
        print_warning "1017" "the item has been removed.[${t50s_file},${t50s_title}]"
        return 0
    elif [ ${ret} -gt 1 ]
    then
        print_error "1018" "There's same item exist.[${t50s_file},${t50s_title}]"
        exit 1
    fi


    theNum=`echo "${g10s_equalLines}"|awk -F ":" '{print $1}'`
    print_debug "theNum=${theNum}"
    if ! [ ${theNum} -gt 1 ] 2>/dev/null
    then
        print_error "1019" "get line number failed[${t50s_title},${t50s_file}]"
        exit 1
    fi

    nextNum=`grep -E -n "^[[:blank:]]*menuentry[[:blank:]]" "${t50s_file}"|awk -F ":" -v col=${theNum} '{if($1 > col){print $1;exit 0}}'`
    print_debug "nextNum=${nextNum}"
    if [ "${nextNum}" = "" ]
    then
        let "theNum=theNum-1"
        sed -n "1,${theNum}p" "${t50s_file}">"${t50s_tmpFile}"
    else
        let "theNum=theNum-1"
        if ! [ ${nextNum} -gt ${theNum} ] 2>/dev/null
        then
            print_error "1020" "get next number failed[${t50s_title},${t50s_file}]"
            exit 1
        fi
        lineCount=`cat ${t50s_file}|wc -l`
        print_debug "lineCount=${lineCount}"
        sed -n "1,${theNum}p" "${t50s_file}">"${t50s_tmpFile}"
        if [ ${lineCount} -ge ${nextNum} ]
        then
            sed -n "${nextNum},${lineCount}p" "${t50s_file}">>"${t50s_tmpFile}"
        fi
    fi



    t50s_recoverDefault "${t50s_tmpFile}"


    t50s_checkMultiDefault "${t50s_tmpFile}"
    t50s_getOldDefaultTitle "${t50s_tmpFile}"
    if [ $? -ne 0 ]
    then
        print_error "1021" "check failed[${t50s_tmpFile}]"
        exit 1
    else
        print_debug "backup file[${t50s_file} to ${t50s_bakFileRemove}]"
        g10s_cp "${t50s_file}" "${t50s_bakFileRemove}"
        g10s_cp "${t50s_tmpFile}" "${t50s_file}"
        rm -f "${t50s_tmpFile}"
    fi
    return 0
}



function t50s_paramCheck()
{
    if [ "${t50s_action}" != "append" ] && [ "${t50s_action}" != "remove" ]
    then
        print_error "1027" "unknown parameter[action:${t50s_action}]"
        t50s_useage
    fi

    [ "${t50s_file}" = "" ] && t50s_file="/boot/grub/grub.cfg"
    if [ ! -f "${t50s_file}" ]
    then
        print_error "1028" "file not exist[${t50s_file}]"
        exit 1
    fi

    [ "${t50s_bakfile}" = "" ] && t50s_bakfile="auto"
    if [ "${t50s_bakfile}" != "auto" ] && [ "${t50s_bakfile}" != "no" ] && [ ! -f "${t50s_bakfile}" ]
    then
        print_error "1029" "file not exist[${t50s_bakfile}]"
        exit 1
    fi

    if [ "${t50s_defaultTitle}" = "" ] && [ "${t50s_bakfile}" = "no" ]
    then
        print_error "1030" "must set defaultNum=XXXX"
        t50s_useage
    fi
    return 0
}


function t50s_domain()
{
    print_debug "begin param check..."
    t50s_paramCheck
    print_debug "check default item..."
    t50s_checkMultiDefault "${t50s_file}"
    if [ `cat ${t50s_file} | grep -c -E "^submenu[[:blank:]]"` -ne 0 ]
    then
        t50s_regExpr="^menuentry[[:blank:]]*|^submenu[[:blank:]]*"
    else
        t50s_regExpr="^[[:blank:]]*menuentry[[:blank:]]*"
    fi
    if [ "${t50s_action}" = "append" ]
    then
        print_debug "begin appending title[${t50s_file},${t50s_title},${t50s_initrd}]"
        t50s_appendTitle
        print_event "1025" "append UltraPath title to bootloader successfully.[${t50s_file}]"
    elif [ "${t50s_action}" = "remove" ]
    then
        print_debug "begin removing title[${t50s_file},${t50s_title},${t50s_initrd}]"
        t50s_removeTitle
        print_event "1026" "remove UltraPath title from bootloader successfully.[${t50s_file}]"
    fi
    return 0
}



while [ $# -gt 0 ]
do
    t50s_param="$1"
    t50s_param1=${t50s_param%%=*}
    t50s_param2=${t50s_param##*=}
    print_debug "t50s_param=${t50s_param},t50s_param1=${t50s_param1},t50s_param2=${t50s_param2}"
    case "${t50s_param1}" in
        "action")
            t50s_action="${t50s_param2}"
            ;;
        "bakfile")
            t50s_bakfile="${t50s_param2}"
            ;;
        "file")
            t50s_file="${t50s_param2}"
            ;;
        "defaultTitle")
            t50s_defaultTitle="${t50s_param2}"
            ;;
        *)
            print_error "1031" "unknown parameter[${t50s_param}]"
            t50s_useage
            ;;
    esac
    shift
done

t50s_domain










