#!/bin/bash

g10s_LIBFILE=${g10s_LIBFILE="../globalLib/g10s_pub"}
if [ -f "${g10s_LIBFILE}" ]
then
        . "${g10s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
    echo -e -n "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}]\033[0m ";echo "not found the lib[${g10s_LIBFILE}]"
    exit 1
fi

t51s_title=${t51s_title:="title Linux with ultrapath"}
t51s_initrd=${t51s_initrd:="${g10s_isspImageName}-`g10s_curUnamer`.img"}

t51s_action=""
t51s_fileForGrubCfg=""
t51s_fileForGrubEnv=""
t51s_fileForCustom=""
if [ -f /boot/grub2/grub.cfg ] && [ -f /etc/default/grub ] && [ -f /etc/grub.d/40_custom ]
then
    t51s_fileForGrubCfg="/boot/grub2/grub.cfg"
    t51s_fileForGrub="/etc/default/grub"
    t51s_fileForGrubEnv="/boot/grub2/grubenv"
    t51s_fileForCustom="/etc/grub.d/40_custom"
fi

t51s_bakfile="auto"
t51s_defaultTitle=""

t51s_bakFileAppendForGrubCfg=${t51s_bakFileAppendForGrubCfg="${g10s_isspBakPath}/grub.cfg.bak"}
t51s_bakFileAppendForGrubEnv=${t51s_bakFileAppendForGrubEnv="${g10s_isspBakPath}/grubenv.bak"}
t51s_bakFileAfterAppendForGrubEnv=${t51s_bakFileAfterAppendForGrubEnv="${g10s_isspBakPath}/grubEnvAfter.bak"}
t51s_bakFileAppendForCustom=${t51s_bakFileAppendForCustom="${g10s_isspBakPath}/40_custom.bak"}

t51s_bakFileRemoveForGrubCfg=${t51s_bakFileRemoveForGrubCfg="${g10s_bakPath}/t51s_grub.cfg.beforeremove"}
t51s_bakFileRemoveForGrubEnv=${t51s_bakFileRemoveForGrub="${g10s_bakPath}/t51s_grubEnv.beforeremove"}
t51s_bakFileRemoveForCustom=${t51s_bakFileRemoveForCustom="${g10s_bakPath}/t51s_custom.beforeremove"}

t51s_tmpFileForCustom="${g10s_tmpPath}/t51s_tmpCustom"

t51s_oldDefaultTitle=""
t51s_newDefaultNum=""
t51s_newImage=""

readonly t51s_grubMkconfig="/usr/sbin/grub2-mkconfig"
readonly t51s_grubSetDefault="/usr/sbin/grub2-set-default"
readonly t51s_flagBegin="###ultrapath-t51s begin,don't change this!###"
readonly t51s_flagEnd="###ultrapath-t51s end,don't change this!###"

t51s_shellDir=""
if [ `echo "$0"|grep -c "/"` -ne 0 ]
then
    t51s_shellDir=`echo ${0%/*}`
fi
print_debug "pwd=`pwd`,\$0=$0,t51s_shellDir=${t51s_shellDir}"
if [ "${t51s_shellDir}" = "." ] || [ "${t51s_shellDir}" = "" ]
then
    t51s_shellDir="`pwd`"
else
    t51s_shellDir="`pwd`/${shellDir}"
fi
t51s_shellName=`echo ${0##*/}`

function t51s_useage()
{
    echo "USAGE:$0 [option]
       action=append
              remove
       file=XXXX
       bakfile=XXXX
               auto
               no
       defaultTitle=XXXX"
    exit 1
}

function t51s_appendTitle()
{
    local menuentryNum
    local menuentryNumForOldDefaultTitle
    local DefaultString
    local oldDefault
    local flag
    local n
    local allIsNum
    local kernelLine
    local kernelType
    local newinitrd
    local tmpGrubcfg=${tmpGrubcfg="${g10s_isspBakPath}/grub2.cfg.bak"} 
    
    DefaultString=`cat ${t51s_fileForGrub} | grep GRUB_DEFAULT | awk -F"=" '{print $2}'` 
    if [ "${DefaultString}" != "saved" ]
    then
        print_error "1015" "grub default string is not saved"
        return 1
    fi
    
    oldDefault=`cat ${t51s_fileForGrubEnv} | grep saved_entry | awk -F"=" '{print $2}'`         
    g10s_getOSType	
    if [ "${oldDefault}" = "" ] && [ "${g10s_OSVersion}" = "SLES12SP2" ]
    then
        oldDefault="0"
    fi
    menuentryNum=`cat ${t51s_fileForGrubCfg} | grep -c -E "^[[:blank:]]*menuentry[[:blank:]]+"`
    if [ ${menuentryNum} -le 0 ]
    then
        print_error "1016" "menuentryNum is [${menuentryNum}]"
    fi
	
	let menuentryNum-=2
	
    g10s_cp "${t51s_fileForGrubEnv}" "${t51s_bakFileAppendForGrubEnv}"
	${t51s_grubSetDefault} "${t51s_title}"
	g10s_cp "${t51s_fileForCustom}" "${t51s_tmpFileForCustom}"
    g10s_cp "${t51s_fileForGrubEnv}" "${t51s_bakFileAfterAppendForGrubEnv}"
    
    echo ${t51s_flagBegin} >> ${t51s_tmpFileForCustom}
    
    allIsNum=`expr match "${oldDefault}" '[a-zA-Z]*'`
    if [ ${allIsNum} -ne 0 ]
    then
        menuentryNumForOldDefault=`cat ${t51s_fileForGrubCfg} | grep -c -E "${oldDefault}"`
        if [ ${menuentryNumForOldDefault} -ne 0 ]
        then
	        flag=0
			while read line
			do
				TMP=`echo $line | grep -E "${oldDefault}"`
				if [ ${flag} -eq 0 ]
				then
					if [ "${TMP}" = "" ]
					then
					    continue
					fi
					flag=1
				fi
				
				echo $line >> ${t51s_tmpFileForCustom}
				
				if [ "$line" = "}" ]
				then
					break
				fi
			done < ${t51s_fileForGrubCfg}
		else
		    oldDefault=0
		    allIsNum=0
		fi
	fi
    
    if [ ${allIsNum} -eq 0 ]
    then
        g10s_cp "${t51s_fileForGrubCfg}" "${tmpGrubcfg}"
        menuentrylist=(`cat "${tmpGrubcfg}" | grep -E -n "^menuentry[[:blank:]]+" | awk -F ":" '{print $1}'`)             
        print_debug "get the default item is ${oldDefault}." "t51s"
        deletstart=${menuentrylist[${oldDefault}]}
        let deletstart=deletstart-1
        print_debug "begin to delete :${deletstart}" "t51s"
        sed -i "1,${deletstart}d" "${tmpGrubcfg}"
		flag=0
		n=${oldDefault}
		while read line
		do
			if [ ${flag} -eq 0 ]
			then
			    TMP=`echo $line | grep -E "^[[:blank:]]*menuentry[[:blank:]]+"`
				if [ "${TMP}" = "" ]
				then
				    continue
				fi
				
				if [ ${n} -lt ${oldDefault} ]
				then
				    n=`expr $n + 1`
				    continue
				fi
				
				flag=1
			fi
			
			echo $line >> ${t51s_tmpFileForCustom}
			
			if [ "$line" = "}" ]
			then
				break
			fi
		done < ${tmpGrubcfg}
	fi
    if [ -f "${tmpGrubcfg}" ]
    then
        rm -r ${tmpGrubcfg}
    fi
	echo ${t51s_flagEnd} >> ${t51s_tmpFileForCustom}
	
	flag=0
	n=0
	while read line
	do
		n=`expr $n + 1`
		if [ ${flag} -eq 0 ]
		then
			TMP=`echo $line | grep -E "${t51s_flagBegin}"`
			if [ "${TMP}" != "" ]
			then
				flag=1
			fi
			continue
		fi
	
	    TMP=`echo $line | grep -E "^[[:blank:]]*menuentry[[:blank:]]+"`
		if [ "${TMP}" != "" ]
		then
		    sed -i "${n}s/${TMP}/menuentry \'${t51s_title}\' {/g" ${t51s_tmpFileForCustom}
		fi
		
		TMP=`echo $line | grep -E "^[[:blank:]]*initrd[[:blank:]]+"`
		if [ "${TMP}" != "" ]
		then
            newinitrd="${TMP}"
            print_debug "initrdLine=${newinitrd}"
            newinitrd=${newinitrd%/*}
            newinitrd=`echo ${newinitrd}/${t51s_initrd}`
		    sed -i "s:${TMP}:${newinitrd}:g" ${t51s_tmpFileForCustom}
            kernelType="linux.*"
		fi
		
        TMP=`echo $line | grep -E "^[[:blank:]]*module[[:blank:]]+"`
		if [ "${TMP}" != "" ]
		then
            if [ `echo ${TMP} | grep -c "nounzip"` -eq 0 ]
            then
                continue
            fi
            newinitrd="${TMP}"
            print_debug "initrdLine=${newinitrd}"
            newinitrd=${newinitrd%/*}
            newinitrd=`echo ${newinitrd}/${t51s_initrd}`
		    sed -i "s:${TMP}:${newinitrd}:g" ${t51s_tmpFileForCustom}
            kernelType="module.*vmlinuz."
		fi
        
		TMP=`echo $line | grep -E "${t51s_flagEnd}"`
		if [ "${TMP}" != "" ]
		then
			break
		fi
	done < ${t51s_tmpFileForCustom}
	
    
	g10s_cp "${t51s_fileForGrubCfg}" "${t51s_bakFileAppendForGrubCfg}"
	g10s_cp "${t51s_fileForCustom}" "${t51s_bakFileAppendForCustom}"
	
	chattr + i "${t51s_bakFileAppendForGrubCfg}" 1>/dev/null 2>&1
	chattr + i "${t51s_bakFileAfterAppendForGrubEnv}" 1>/dev/null 2>&1
	chattr + i "${t51s_bakFileAppendForGrubEnv}" 1>/dev/null 2>&1
	chattr + i "${t51s_bakFileAppendForCustom}" 1>/dev/null 2>&1

    kernelLine=`cat "${t51s_tmpFileForCustom}" | grep -n "${kernelType}" | awk -F ":" '{print $1}'`
    sed -i "${kernelLine} s/$/ UltraPath/g" ${t51s_tmpFileForCustom}
    
	g10s_cp "${t51s_tmpFileForCustom}" "${t51s_fileForCustom}"
   
	${t51s_grubMkconfig} -o ${t51s_fileForGrubCfg} 1>/dev/null 2>&1

	rm -f "${t51s_tmpFileForCustom}"
	return 0
}

function t51s_removeTitle()
{
    local curDefaultString
    local curDefault
    local bakDefault
    local flag
    local startLine
    local endLine
    local n
    local menuentryNumForUltraPath

    DefaultString=`cat ${t51s_fileForGrub} | grep GRUB_DEFAULT | awk -F"=" '{print $2}'`
    if [ "${DefaultString}" != "saved" ]
    then
        print_warning "grub default string is not saved"
    fi
    
    curDefault=`cat ${t51s_fileForGrubEnv} | grep saved_entry | awk -F"=" '{print $2}'`
    bakDefault=`cat ${t51s_bakFileAfterAppendForGrubEnv} | grep saved_entry | awk -F"=" '{print $2}'`
    if [ "${curDefault}" = "${bakDefault}" ]
    then
        g10s_cp "${t51s_bakFileAppendForGrubEnv}" "${t51s_fileForGrubEnv}"
    fi
    
    g10s_cp "${t51s_fileForCustom}" "${t51s_tmpFileForCustom}"
    menuentryNumForUltraPath=`cat ${t51s_tmpFileForCustom} | grep -c -E "${t51s_title}"`
    startLine=0
    endLine=0
    n=0
    flag=0
    if [ ${menuentryNumForUltraPath} -ne 0 ]
    then
	    while read line
		do
		    n=`expr $n + 1`
			if [ ${flag} -eq 0 ]
			then
			    TMP=`echo $line | grep -E "${t51s_flagBegin}"`
				if [ "${TMP}" = "" ]
				then
				    continue
				fi
				flag=1
				startLine=${n}
			fi
			
			TMP=`echo $line | grep -E "${t51s_flagEnd}"`
			if [ "${TMP}" != "" ]
			then
				endLine=${n}
				break
			fi
		done < ${t51s_tmpFileForCustom}
		sed -i "${startLine},${endLine}d" ${t51s_tmpFileForCustom}
	fi
	
    g10s_cp "${t51s_fileForGrubCfg}" "${t51s_bakFileRemoveForGrubCfg}"
	g10s_cp "${t51s_fileForGrubEnv}" "${t51s_bakFileRemoveForGrub}"
	g10s_cp "${t51s_fileForCustom}" "${t51s_bakFileRemoveForCustom}"
	
	chattr + i "${t51s_bakFileRemoveForGrubCfg}" 1>/dev/null 2>&1
	chattr + i "${t51s_bakFileRemoveForGrub}" 1>/dev/null 2>&1
	chattr + i "${t51s_bakFileRemoveForCustom}" 1>/dev/null 2>&1
	
	g10s_cp "${t51s_tmpFileForCustom}" "${t51s_fileForCustom}"
	
    ${t51s_grubMkconfig} -o ${t51s_fileForGrubCfg} 1>/dev/null 2>&1
    
    rm -f "${t51s_bakFileAppendForGrubCfg}"
    rm -f "${t51s_bakFileAppendForGrubEnv}"
    rm -f "${t51s_bakFileAfterAppendForGrubEnv}"
	rm -f "${t51s_bakFileAppendForCustom}"

    return 0
}



function t51s_paramCheck()
{
    if [ "${t51s_action}" != "append" ] && [ "${t51s_action}" != "remove" ]
    then
        print_error "1027" "unknown parameter[action:${t51s_action}]"
        t51s_useage
    fi

    if [ ! -f "${t51s_fileForGrubCfg}" ] || [ ! -f "${t51s_fileForGrubEnv}" ] || [ ! -f "${t51s_fileForCustom}" ]
    then
        print_error "1028" "file not exist[${t51s_fileForGrubCfg},${t51s_fileForGrubEnv},${t51s_fileForCustom}]"
        exit 1
    fi

    [ "${t51s_bakfile}" = "" ] && t51s_bakfile="auto"
    if [ "${t51s_bakfile}" != "auto" ] && [ "${t51s_bakfile}" != "no" ] && [ ! -f "${t51s_bakfile}" ]
    then
        print_error "1029" "file not exist[${t51s_bakfile}]"
        exit 1
    fi

    if [ "${t51s_defaultTitle}" = "" ] && [ "${t51s_bakfile}" = "no" ]
    then
        print_error "1030" "must set defaultNum=XXXX"
        t51s_useage
    fi
    return 0
}


function t51s_domain()
{
    print_debug "begin param check..."
    t51s_paramCheck
    print_debug "check default item..."
    if [ "${t51s_action}" = "append" ]
    then
        print_debug "begin appending title[${t51s_fileForGrubCfg},${t51s_fileForGrub},${t51s_fileForGrubEnv},${t51s_fileForCustom},${t51s_title},${t51s_initrd}]"
        t51s_appendTitle
        print_event "1025" "append UltraPath title to bootloader successfully.[${t51s_fileForGrubCfg}]"
    elif [ "${t51s_action}" = "remove" ]
    then
        print_debug "begin removing title[${t51s_fileForGrubCfg},${t51s_fileForGrub},${t51s_fileForGrubEnv},${t51s_fileForCustom},${t51s_title},${t51s_initrd}]"
        t51s_removeTitle
        print_event "1026" "remove UltraPath title from bootloader successfully.[${t51s_fileForGrubCfg}]"
    fi
    return 0
}



while [ $# -gt 0 ]
do
    t51s_param="$1"
    t51s_param1=${t51s_param%%=*}
    t51s_param2=${t51s_param##*=}
    print_debug "t51s_param=${t51s_param},t51s_param1=${t51s_param1},t51s_param2=${t51s_param2}"
    case "${t51s_param1}" in
        "action")
            t51s_action="${t51s_param2}"
            ;;
        "bakfile")
            t51s_bakfile="${t51s_param2}"
            ;;
        "file")
            t51s_file="${t51s_param2}"
            ;;
        "defaultTitle")
            t51s_defaultTitle="${t51s_param2}"
            ;;
        *)
            print_error "1031" "unknown parameter[${t51s_param}]"
            t51s_useage
            ;;
    esac
    shift
done

t51s_domain










