#!/bin/bash

g10s_LIBFILE=${g10s_LIBFILE="../globalLib/g10s_pub"}
if [ -f "${g10s_LIBFILE}" ]
then
	. "${g10s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
	echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g10s_LIBFILE}]\033[0m"
	exit 1
fi

t60s_mkinitFile=${t60s_mkinitFile:="/usr/sbin/mkinitramfs"}
t60s_bakInitFile=${t60s_bakInitFile="${g10s_bakPath}/t60s_init.bak"}
t60s_bakFileAppend=${t60s_bakFileAppend="${g10s_bakPath}/t60s_mkinitramfs.append"}


readonly t60s_bakFileRemove="${g10s_bakPath}/t60s_mkinitramfs.remove"
readonly t60s_tmpFile="${g10s_tmpPath}/t60s_mkinitramfs"
readonly t60s_flagBegin1="###UltraPath-t60s-1 begin,don't change this!###"
readonly t60s_flagEnd1="###UltraPath-t60s-1 end,don't change this!###"
readonly t60s_flagBegin2="###UltraPath-t60s-2 begin,don't change this!###"
readonly t60s_flagEnd2="###UltraPath-t60s-2 end,don't change this!###"
readonly t60s_flagBegin3="###UltraPath-t60s-3 begin,don't change this!###"
readonly t60s_flagEnd3="###UltraPath-t60s-3 end,don't change this!###"

function t60s_useage()
{
	echo "USAGE: $0 [append|remove]"
	exit 1
}


function t60s_modifyInit()
{
    local ret

    g10s_checkExist -f "/usr/share/initramfs-tools/init" 2
    if [ -L "/usr/share/initramfs-tools/init" ]
    then
        print_error "1001" "/usr/share/initramfs-tools/init is a link file."
        exit 1
    fi


    ret=$(cat "/usr/share/initramfs-tools/init" 2>/dev/null|grep -E -c "(${t60s_flagBegin1}|${t60s_flagEnd1}|${t60s_flagBegin2}|${t60s_flagEnd2})")
    print_debug "before append code to init shell,ret=${ret}"
    if [ ${ret} -ne 0 ]
    then
        print_warning "1002" "UltraPath code has been exist in file[/usr/share/initramfs-tools/init]"
        return 0
    fi
    ret=$(cat "/usr/share/initramfs-tools/init" 2>/dev/null|grep -c "^[[:blank:]]*\[ -n \"\${netconsole}\" \] && modprobe netconsole netconsole=\"\${netconsole}\"[[:blank:]]*$")
    if [ ${ret} -ne 1 ]
    then
        print_error "1003" "not found the key words in file[/usr/share/initramfs-tools/init]"
        exit 1
    fi
    ret=$(cat "/usr/share/initramfs-tools/init" 2>/dev/null|grep -c "^[[:blank:]]*maybe_break[[:blank:]]*premount[[:blank:]]*$")
    if [ ${ret} -ne 1 ]
    then
        print_error "1004" "not found the key words in file[/usr/share/initramfs-tools/init]"
        exit 1
    fi

    g10s_cp "/usr/share/initramfs-tools/init" "${t60s_bakInitFile}"
    
    g10s_getOSType
    if [ "${g10s_OSVersion}" = "DEEPIN15" ]
    then
    sed -i "/^[[:blank:]]*maybe_break[[:blank:]]*top[[:blank:]]*$/i \
${t60s_flagBegin1}\\
modprobe scsi_mod\\
modprobe sd_mod\\
modprobe sg\\
modprobe nxup\\
modprobe nxupext_a\\
${t60s_flagEnd1}\n" "${t60s_bakInitFile}"
    else
    sed -i "/^[[:blank:]]*\[ -n \"\${netconsole}\" \] && modprobe netconsole netconsole=\"\${netconsole}\"[[:blank:]]*$/i \
${t60s_flagBegin1}\\
modprobe scsi_mod\\
modprobe sd_mod\\
modprobe sg\\
modprobe nxup\\
modprobe nxupext_a\\
${t60s_flagEnd1}\n" "${t60s_bakInitFile}"
    fi

    sed -i "/^[[:blank:]]*maybe_break[[:blank:]]*premount[[:blank:]]*$/i \
${t60s_flagBegin2}\\
${t60s_flagEnd2}\n" "${t60s_bakInitFile}"


    ret=$(cat "${t60s_bakInitFile}" 2>/dev/null|grep -E -c "(${t60s_flagBegin1}|${t60s_flagEnd1}|${t60s_flagBegin2}|${t60s_flagEnd2})")
    if [ ${ret} -ne 4 ]
    then
        print_error "1005" "append UltraPath code failed[${t60s_bakInitFile}]"
        exit 1
    fi
    return 0
}

function t60s_appendCode()
{
    local ret
    local tmpinitFile

    ret=$(cat ${t60s_mkinitFile}|grep -E -c "(${t60s_flagBegin1}|${t60s_flagEnd1}|${t60s_flagBegin2}|${t60s_flagEnd2}|${t60s_flagBegin3}|${t60s_flagEnd3})")
    print_debug "before append code to mkinitramfs,ret=${ret}"
    if [ ${ret} -ne 0 ]
    then
        print_warning "1006" "UltraPath code has been exist in file[${t60s_mkinitFile}]"
        return 0
    fi
    ret=$(cat "${t60s_mkinitFile}" 2>/dev/null|grep -c "^[[:blank:]]*cp[[:blank:]]*-p[[:blank:]]*/usr/share/initramfs-tools/init[[:blank:]]*\${DESTDIR}/init[[:blank:]]*$")
    if [ ${ret} -ne 1 ]
    then
        print_error "1007" "not found the key words in file[${t60s_mkinitFile}]"
        exit 1
    fi
    ret=$(cat "${t60s_mkinitFile}" 2>/dev/null|grep -c "^[[:blank:]]*cp -a /etc/modprobe\.d/\* \"\${DESTDIR}/etc/modprobe\.d/\"[[:blank:]]*$")
    if [ ${ret} -ne 1 ]
    then
        print_error "1008" "not found the key words in file[${t60s_mkinitFile}]"
        exit 1
    fi

    print_debug "begin appending UltraPath to file[${t60s_mkinitFile}]"

    if [ `sed -n "1p" "${t60s_mkinitFile}" 2>/dev/null|grep -c "^[[:blank:]]*#\![[:blank:]]*/bin/"` -eq 1 ]
    then
        g10s_cp "${t60s_mkinitFile}" "${t60s_tmpFile}"
        sed -n "1p" "${t60s_mkinitFile}">"${t60s_tmpFile}"
        echo "${t60s_flagBegin1}
if [ \`echo \"\${outfile}\" | grep -E -c \"(/nxup|/ultrapath)\"\` -ne 0 ]
then
    if [ ! -f /etc/up.conf ]
    then
        echo \"UltraPath--ERROR:not found /etc/up.conf\"
        exit 1
    fi
    if [ ! -f /etc/up.hostinfo ]
    then
        echo \"UltraPath--ERROR:not found /etc/up.hostinfo\"
        exit 1
    fi
    if [ ! -f \"${t60s_bakInitFile}\" ]
    then
        echo \"UltraPath--ERROR:not found ${t60s_bakInitFile}\"
        exit 1
    fi
    echo \"UltraPath--make image for UltraPath...\"
    shift
fi
${t60s_flagEnd1}">>"${t60s_tmpFile}"
        sed -n '2,$p' "${t60s_mkinitFile}">>"${t60s_tmpFile}"
    else
        print_error "1009" "not found \"^[[:blank:]]*#\![[:blank:]]*/bin/\" in file[${t60s_mkinitFile}]"
        exit 1
    fi


    tmpinitFile=$(echo "${t60s_bakInitFile}"|sed 's/\//\\\//g')
    sed -i "/^[[:blank:]]*cp[[:blank:]]*-p[[:blank:]]*\/usr\/share\/initramfs-tools\/init[[:blank:]]*\${DESTDIR}\/init[[:blank:]]*$/a \
${t60s_flagBegin2}\\
if [ \`echo \"\${outfile}\" | grep -E -c \"\(/nxup\|/ultrapath\)\"\` -ne 0 ]\\
then\\
    echo \"UltraPath--recover init shell\"\\
    cp -apf \"${tmpinitFile}\" \"\${DESTDIR}\/init\"\\
fi\\
${t60s_flagEnd2}\n" "${t60s_tmpFile}"


    sed -i "/^[[:blank:]]*cp -a \/etc\/modprobe\.d\/\* \"\${DESTDIR}\/etc\/modprobe\.d\/\"[[:blank:]]*$/a \
${t60s_flagBegin3}\\
if [ \`echo \"\${outfile}\" | grep -E -c \"\(/nxup\|/ultrapath\)\"\` -ne 0 ]\\
then\\
    echo \"UltraPath--copy \/etc\/up.conf...\"\\
    cp -apf \/etc\/up.conf \"\${DESTDIR}\/etc\/up.conf\"\\
    cp -apf \/etc\/up.hostinfo \"\${DESTDIR}\/etc\/up.hostinfo\"\\
fi\\
${t60s_flagEnd3}" "${t60s_tmpFile}"


    ret=$(cat ${t60s_tmpFile}|grep -E -c "(${t60s_flagBegin1}|${t60s_flagEnd1}|${t60s_flagBegin2}|${t60s_flagEnd2}|${t60s_flagBegin3}|${t60s_flagEnd3})")
    if [ ${ret} -ne 6 ]
    then
        print_error "1010" "check failed after appending UltraPath code[${t60s_tmpFile}]"
        exit 1
    else
        print_debug "backup file [${t60s_mkinitFile} to ${t60s_bakFileAppend}]"
        g10s_cp "${t60s_mkinitFile}" "${t60s_bakFileAppend}"
        g10s_cp "${t60s_tmpFile}" "${t60s_mkinitFile}"
        rm -rf "${t60s_tmpFile}"
        print_event "1011" "successfully appended UltraPath code[${t60s_mkinitFile}]"
    fi
    return 0
}

function t60s_removeCode()
{
    local ret

    print_debug "begin removing UltraPath code[${t60s_mkinitFile}]"
    g10s_cp "${t60s_mkinitFile}" "${t60s_tmpFile}"
    if [ $? -ne 0 ]
    then
        print_error "1012" "copy file failed[${t60s_mkinitFile} to ${t60s_tmpFile}]"
        exit 1
    fi
    ret=$(cat ${t60s_tmpFile} 2>/dev/null|grep -E -c "(${t60s_flagBegin1}|${t60s_flagEnd1})")
    if [ $ret -ne 2 ]
    then
        print_debug "skip removing UltraPath code 1..."
    else
        sed -i "/${t60s_flagBegin1}/,/${t60s_flagEnd1}/d" "${t60s_tmpFile}"
    fi
    ret=$(cat ${t60s_tmpFile} 2>/dev/null|grep -E -c "(${t60s_flagBegin2}|${t60s_flagEnd2})")
    if [ $ret -ne 2 ]
    then
        print_debug "skip removing UltraPath code 2..."
    else
        sed -i "/${t60s_flagBegin2}/,/${t60s_flagEnd2}/d" "${t60s_tmpFile}"
    fi
    ret=$(cat ${t60s_tmpFile} 2>/dev/null|grep -E -c "(${t60s_flagBegin3}|${t60s_flagEnd3})")
    if [ $ret -ne 2 ]
    then
        print_debug "skip removing UltraPath code 3..."
    else
        sed -i "/${t60s_flagBegin3}/,/${t60s_flagEnd3}/d" "${t60s_tmpFile}"
    fi


    ret=$(cat ${t60s_tmpFile}|grep -E -c "(${t60s_flagBegin1}|${t60s_flagEnd1}|${t60s_flagBegin2}|${t60s_flagEnd2}|${t60s_flagBegin3}|${t60s_flagEnd3})")
    if [ ${ret} -ne 0 ]
    then
        print_error "1013" "check failed after removing UltraPath code[${t60s_tmpFile}]"
        exit 1
    else
        print_debug "backup file [${t60s_mkinitFile} to ${t60s_bakFileRemove}]"
        g10s_cp "${t60s_mkinitFile}" "${t60s_bakFileRemove}"
        g10s_cp "${t60s_tmpFile}" "${t60s_mkinitFile}"
        rm -rf "${t60s_tmpFile}"
        print_event "1014" "successfully removed UltraPath code[${t60s_mkinitFile}]"
    fi
    return 0
}


function t60s_repairCode()
{
    local ret
    local ret2

    ret=$(cat "${t60s_mkinitFile}"|grep -E -c "(${t60s_flagBegin1}|${t60s_flagEnd1}|${t60s_flagBegin2}|${t60s_flagEnd2}|${t60s_flagBegin3}|${t60s_flagEnd3})")

    if [ ${ret} -ne 0 ] && [ ${ret} -ne 6 ]
    then
        print_debug "remove UltraPath code from file[${t60s_mkinitFile}]"
        t60s_removeCode
    fi

    ret2=$(cat "${t60s_mkinitFile}"|grep -E -c "(${t60s_flagBegin1}|${t60s_flagEnd1}|${t60s_flagBegin2}|${t60s_flagEnd2}|${t60s_flagBegin3}|${t60s_flagEnd3})")
    if [ ${ret2} -eq 0 ]
    then
        print_debug "append UltraPath code to file[${t60s_mkinitFile}]"

        t60s_modifyInit
        t60s_appendCode
    fi
    return 0
}

g10s_checkExist -f "${t60s_mkinitFile}" 2

if [ "$1" = "append" ]
then

    t60s_modifyInit
    t60s_appendCode
elif [ "$1" = "remove" ]
then
    t60s_removeCode
elif [ "$1" = "repair" ]
then
    t60s_repairCode
else
    print_error "1001" "unknown parameter[$1]"
    t60s_useage
fi

