#!/bin/bash

g20s_LIBFILE=${g20s_LIBFILE="../globalLib/g20s_install"}
if [ -f "${g20s_LIBFILE}" ]
then
    . "${g20s_LIBFILE}"
else
    fileFlag=${0##*/}
    fileFlag=${fileFlag%%_*}
    echo -e "\033[31m[ERROR:15][${fileFlag}][`date +%Y%m%d%H%M%S`][$0,${FUNCNAME[1]},${BASH_LINENO[0]}] not found the lib[${g20s_LIBFILE}]\033[0m"
    exit 1
fi

t90s_r3bakUpConf="/opt/UltraPath/backup/up.conf.r3"
t90s_bakUpConf="/etc/up.conf.save_999"
t90s_r8bakUpConf="/etc/up.conf.save_r8_999"
t90s_r8bakCFGXML="/etc/user_cfg.xml.old"
t90s_upConf="/etc/up.conf"
t90_bakoldConf="/etc/up.conf.old"
t90s_allcmdforbootservice=("grep" "uname" "sed" "awk" "wc" "cat" "eval" "echo" "tr" "getconf" "head" "tail" "vgchange" \
                           "ls" "lsmod" "date")
t90s_upDir=/UltraPath/bin

function t90s_useage()
{
    echo "USAGE:$0 [install|remove]"
    exit 1
}

function t90s_cpcmd2upDir()
{

    if [ ! -d ${t90s_upDir} ]
    then
        mkdir ${t90s_upDir}
    fi

    for tmpcmd in "${t90s_allcmdforbootservice[@]}"
    do
        tmpcmd=`echo "${tmpcmd}"`
        if [ `which ${tmpcmd} 2>/dev/null|grep -c "/${tmpcmd}"` -ne 0 ]
        then
            g10s_cp "`which ${tmpcmd}`" "${t90s_upDir}"
        fi
    done

    return 0
}

function t90s_killUpService()
{
    local number
    local upsrvpid
    number=`ps x|grep "upservice" | grep -c "UltraPath"`
    if [ ${number} -ge 1 ]
    then
        killall upservice 2>/dev/null
        if [ $? -ne 0 ]
        then
            upsrvpid=`ps x|grep "upservice" | grep  "UltraPath" | awk '{print $1}'`
            kill -9 ${upsrvpid} 2>/dev/null
            if [ $? -ne 0 ]
            then
                print_error "1018" "kill upservice failed."
                exit 1
            fi
        fi
    else
        print_debug "upservice num=${number}"
    fi
    return 0
}

function t90s_updateTimeZone()
{
	pwdPath=`pwd`
	cd /UltraPath/install/otherTools
	print_debug  "Update timeZone to configure file start."
	./cfg_upgrade -updatetime
	if [ $? -ne 0 ]
	then
	  print_debug  "Update timeZone to configure file failed."
	fi
	cd "${pwdPath}"

	return 0
}

function t90s_install()
{
    local OLD_IFS
    local iCount
    local tmpKernel
    local tmpModulePath
    local tmpUnamer
    local pr_key
    local isGenerateKey
    local installKernel

    isGenerateKey=0
    print_event "1001" "User configuration is saved successfully."

    g10s_checkExist -f "${g20s_modulesBaseDir}/../otherTools/${g10s_R3upadm}" 2
    g10s_checkExist -f "${g20s_modulesBaseDir}/../otherTools/${g10s_R5upadm}" 2
    set -e
    install -o root -g root -m 0500 -D "${g20s_modulesBaseDir}/../otherTools/${g10s_R5upadm}" /usr/sbin/${g10s_R5upadm}
    install -o root -g root -m 0500 -D "${g20s_modulesBaseDir}/../otherTools/${g10s_R3upadm}" /usr/sbin/${g10s_R3upadm}

    [ -f /usr/sbin/upTools ] && rm -rf /usr/sbin/upTools

    ln -s /usr/sbin/${g10s_R3upadm} /usr/sbin/upTools
    install -o root -g root -m 0500 -D "${g20s_modulesBaseDir}/../otherTools/slTools" /sbin/slTools
    set +e

    g10s_getConf "${g10s_ultraPathPackagesInfo}" "t400s_updateKernel"
    if [ "${g10s_confValue}" = "" ] || [ `cat "${g10s_ultraPathPackagesInfo}" | grep -c "^[[:blank:]]*t400s_updateKernel"` -eq 0 ]
    then
        installKernel=`uname -r`
    else
        installKernel=${g10s_confValue}
    fi
    
    g10s_getOSType
    g10s_getConf "${g10s_ultraPathPackagesInfo}" "t140s_solutionType"
    if [ "${g10s_confValue}" = "mkinitrd" ]
    then
        g10s_getPathForInstall

        tmpModulePath="${g10s_PathForInstall}"
        print_debug "get modules path for [${installKernel},${tmpModulePath}]"

        g10s_checkExist -f "${g20s_modulesBaseDir}/${tmpModulePath}/nxup.ko" 2
        g10s_checkExist -f "${g20s_modulesBaseDir}/${tmpModulePath}/nxupext_a.ko" 2
        g10s_cp "${g20s_modulesBaseDir}/${tmpModulePath}/nxup.ko" /lib/modules/${installKernel}/kernel/drivers/scsi/nxup.ko
        g10s_cp "${g20s_modulesBaseDir}/${tmpModulePath}/nxupext_a.ko" /lib/modules/${installKernel}/kernel/drivers/scsi/nxupext_a.ko
        if [ `echo "${g10s_OSVersion}"|grep -c -E "(RHEL7|OL7|KYLIN7SP2)"` -eq 1 ]
        then
            g10s_cp "${g20s_modulesBaseDir}/${tmpModulePath}/nxup.ko" /usr/lib/modules/${installKernel}/kernel/drivers/scsi/nxup.ko
            g10s_cp "${g20s_modulesBaseDir}/${tmpModulePath}/nxupext_a.ko" /usr/lib/modules/${installKernel}/kernel/drivers/scsi/nxupext_a.ko
        fi
    fi

    pwdPath=`pwd`
    set -e
    cd /UltraPath/install
    install -o root -g root -m 0600 -D up.conf /etc/up.conf
    install -o root -g root -m 0600 -D user_cfg.xml /etc/user_cfg.xml
    install -o root -g root -m 0500 -D linuxTools/toolLib/t190s_upLinux.sh /usr/sbin/upLinux
    install -o root -g root -m 0500 -D linuxTools/toolLib/t100s_upBusRescan.sh /sbin/upRescan
    install -o root -g root -m 0500 -D linuxTools/toolLib/updd /sbin/updd

    [ -f /usr/sbin/hot_add ] && rm -rf /usr/sbin/hot_add
    ln -s /sbin/upRescan /usr/sbin/hot_add
    set +e
    cd "${pwdPath}"

    if [ `echo "${g10s_OSVersion}" | grep -c -E "(UVPV1R2|UVPV1R3|UVPSP3|UVPV2R3)"` -ne 0 ]
    then
    	if [ ! -d /var/log/UltraPath ]
    	then
    	    mkdir -p "/var/log/UltraPath"
    	fi
    fi



    if [ -f "${t90s_r8bakUpConf}" ] && [ -f "${t90s_r8bakCFGXML}" ]
    then
         pwdPath=`pwd`
         cp "${t90s_r8bakUpConf}" "${t90_bakoldConf}"
         cd /UltraPath/install/otherTools
         ./cfg_upgrade -r8
         if [ $? -ne 0 ]
         then
              rm -rf "${t90_bakoldConf}"
              g10s_saveConf "${g10s_ultraPathPackagesInfo}"  "t320s_recoverUpConfFailed" "1"
              exit 1
         fi
         print_debug  "update R8 config files."
         rm -rf "${t90_bakoldConf}"
         cd "${pwdPath}"
         isGenerateKey=1
    fi


    if [ -f "${t90s_bakUpConf}" ] && [ `cat "${t90s_bakUpConf}"|grep -c "common"` -ne 0 ]
    then
         pwdPath=`pwd`
         cp "${t90s_bakUpConf}" "${t90_bakoldConf}"
         cd /UltraPath/install/otherTools
         ./cfg_upgrade -r5
         if [ $? -ne 0 ]
         then
              rm -rf "${t90_bakoldConf}"
              g10s_saveConf "${g10s_ultraPathPackagesInfo}" "t320s_recoverUpConfFailed" "1"
              exit 1
         fi
         rm -rf "${t90_bakoldConf}"
         rm -rf "${t90s_bakUpConf}"
         print_debug  "upgrade config files from R5 to R8."
         cd "${pwdPath}"
         isGenerateKey=1
    fi


    if [ "${g10s_OSVersion}" != "RHEL4" ] && [ `echo "${g10s_OSVersion}" | grep -c "SLES9"` -eq 0  ]
    then
        if [ -f t370s_udevnxup.sh ]
        then
            bash t370s_udevnxup.sh modifycfg
            bash t370s_udevnxup.sh modifytimeout
            g10s_getSysBit
            if [ `echo ${g10s_OSVersion} | grep -c -E "RHEL6|RHEL7|SLES11|SLES12|OL6|OL7"` -ne 0 ] && [ "${g10s_sysbit}" == "x86_64" ]
            then
                bash t370s_udevnxup.sh createlock
            else
                sed -i "/^[[:blank:]]*t370s_lock_event[[:blank:]]*$/d" t370s_udevnxup.sh 1>/dev/null 2>&1
                sed -i "/^[[:blank:]]*t370s_unlock_event[[:blank:]]*$/d" t370s_udevnxup.sh 1>/dev/null 2>&1
                if [ `cat t370s_udevnxup.sh | grep -c -E "^[[:blank:]]*t370s_lock_event[[:blank:]]*$|^[[:blank:]]*t370s_unlock_event[[:blank:]]*$"` -ne 0 ]
                then
                    print_debug "fail to remove lock function"
                fi
            fi
        fi        
    fi
    
    if [ "${g10s_OSVersion}" == "SLES10SP4" ]
    then
        bash t370s_udevnxup.sh createsymlink
    fi
    
    if [ `echo ${g10s_OSVersion} | grep -c -E "RHEL6|RHEL7|SLES11|SLES12|OL6|OL7"` -ne 0 ]
    then
        if [ -f "${g10s_bakPath}/../linkdowntmo.last.uninst" ]
        then
            print_debug "restore the linkdowntmo value"
            [ -f ${g10s_bakPath}/../linkdowntmo.default.uninst ] && mv ${g10s_bakPath}/../linkdowntmo.default.uninst ${g10s_bakPath}/linkdowntmo.default
            value=`cat ${g10s_bakPath}/../linkdowntmo.last.uninst`
            upLinux set linkdown_tmo=${value} >/dev/null 2>&1
            rm -f ${g10s_bakPath}/../linkdowntmo.last.uninst
        fi
    fi



    g10s_checkExist -f "t350s_updateR5Conf.sh" 2
    bash t350s_updateR5Conf.sh "action=update" "type=upcfg"
    if [ $? -ne 0 ]
    then
       print_debug  "not find R3 config files."
    else
       cp "${t90s_upConf}" "${t90s_bakUpConf}"
       rm -rf /etc/up.conf
       cp /UltraPath/install/up.conf "${t90s_upConf}"
       if [ -f "${t90s_upConf}" ] && [ `cat "${t90s_upConf}"|grep -c "common"` -ne 0 ]
       then
           pwdPath=`pwd`
           cp "${t90s_bakUpConf}" "${t90_bakoldConf}"
           cd /UltraPath/install/otherTools
           print_debug  "upgrade config files from R3 to R8."
           ./cfg_upgrade -r5
           if [ $? -ne 0 ]
           then
              rm -rf "${t90_bakoldConf}"
              g10s_saveConf "${g10s_ultraPathPackagesInfo}" "t320s_recoverUpConfFailed" "1"
              exit 1
           fi
           rm -rf "${t90_bakoldConf}"
           rm -rf "${t90s_bakUpConf}"
           cd "${pwdPath}"
           isGenerateKey=1
       fi
    fi
	t90s_updateTimeZone

    if [ $isGenerateKey -eq 0 ]
    then
        pr_key="\"`date +%s%N 2>/dev/null`\";"
        print_debug "create new pr_key[${pr_key}]"
        g10s_saveConf "/etc/up.conf" "pr_key " " ${pr_key}"
    fi


    g10s_checkExist -f "t130s_getOSInfo.sh" 2
    bash t130s_getOSInfo.sh "/etc/up.hostinfo" >/dev/null
    if [ $? -ne 0 ]
    then
        print_error "1006" "t130s_getOSInfo.sh failed."
        exit 1
    fi

    g10s_chmodExec -R 500 /UltraPath/
    [ -d /opt/UltraPath/ ] && g10s_chmodExec -R 600 /opt/UltraPath/
    [ -d /var/log/UltraPath ] && g10s_chmodExec -R 600 /var/log/UltraPath/

    t90s_cpcmd2upDir
    if [ -f "${t90s_bakUpConf}" ]
    then
        rm -rf "${t90_bakoldConf}"
    fi
    if [ -f "${t90s_r8bakUpConf}" ]
    then
        rm -rf "${t90s_r8bakUpConf}"
    fi
    if [ -f "${t90s_r3bakUpConf}" ]
    then
        rm -rf "${t90s_r3bakUpConf}"
    fi
    return 0
}

function t90s_remove()
{
    local tmpUnamer
    local tmpVersion
    print_event "1004" "The UltraPath files are deleted successfully."
    if [ -f /etc/up.conf ] || [ -f /etc/user_cfg.xml ]
    then
        g10s_getConf "${g10s_ultraPathPackagesInfo}" "t320s_recoverUpConfFailed"
        if [ "${g10s_confValue}" != "1" ]
        then
            cp /etc/up.conf ${t90s_r8bakUpConf}
        fi
        cp /etc/user_cfg.xml /etc/user_cfg.xml.old
    fi

    if [ -f /etc/udev/rules.d/56-nxup.rules ]
    then
        rm -f /etc/udev/rules.d/56-nxup.rules
    fi

    if [ -f /etc/udev/rules.d/99-ultrapath.rules ]
    then
        if [ -f "${g10s_bakPath}/linkdowntmo.last" ]
        then
            print_debug "backup the linkdowntmo"
            cp -f ${g10s_bakPath}/linkdowntmo.last ${g10s_bakPath}/../linkdowntmo.last.uninst
            cp -f ${g10s_bakPath}/linkdowntmo.default ${g10s_bakPath}/../linkdowntmo.default.uninst >/dev/null 2>&1
            upLinux unset linkdown_tmo >/dev/null 2>&1
            [ $? -eq 0 ] && rm -f ${g10s_bakPath}/../linkdowntmo.default.uninst
        fi
        rm -f /etc/udev/rules.d/99-ultrapath.rules
    fi

    rm -f /etc/up.conf
    rm -f /etc/user_cfg.xml
    rm -f /etc/up.conf.bak
    rm -f /usr/sbin/${g10s_R3upadm}
    rm -f /usr/sbin/upLinux
    rm -f /sbin/upRescan
    rm -f /sbin/updd
    rm -f /usr/sbin/${g10s_R5upadm}
    rm -f /usr/sbin/hot_add
    rm -f /usr/sbin/upTools
    rm -f /sbin/slTools
    rm -f /etc/up.hostinfo
    killall up_restart_warn 2>/dev/null
    rm -f /sbin/stop_up_restart_warn
    rm -f /opt/UltraPath/wall_msg 2>/dev/null
    [ -f ${g10s_ultraPathPackagesInfo} ] && rm -rf ${g10s_ultraPathPackagesInfo}

    tmpVersion=${installKernel}
    if [ -d /boot/efi ] && [ "`find /boot/efi/ -name ultrapath-${tmpVersion}.img`" != "" ]
    then
    	rm -f `find /boot/efi/ -name ultrapath-${tmpVersion}.img`
    fi

    t90s_killUpService

    for tmpUnamer in `ls /lib/modules/`
    do
        if [ -d /lib/modules/${tmpUnamer}/kernel/drivers/scsi/ ] && [ -f /lib/modules/${tmpUnamer}/kernel/drivers/scsi/nxup.ko ]
        then
            rm -rf /lib/modules/${tmpUnamer}/kernel/drivers/scsi/nxup.ko
            depmod "${tmpUnamer}"
        fi
        if [ -d /lib/modules/${tmpUnamer}/kernel/drivers/scsi/ ] && [ -f /lib/modules/${tmpUnamer}/kernel/drivers/scsi/nxupext_a.ko ]
        then
            rm -rf /lib/modules/${tmpUnamer}/kernel/drivers/scsi/nxupext_a.ko
            depmod "${tmpUnamer}"
        fi
    done

    return 0
}


if [ "$1" = "install" ]
then
    t90s_install
elif [ "$1" = "remove" ]
then
    t90s_remove
else
    print_error "1005" "unknown parameter[$1]"
    t90s_useage
fi
